Package: ThSQCA
Type: Package
Title: Threshold-Sweep QCA
Version: 2.0.0
Authors@R: c(
    person("Yuki", "Toyoda", role = c("aut", "cre"),
           email = "yuki.toyoda.ds@hosei.ac.jp"),
    person("Japan Society for the Promotion of Science", role = "fnd",
           comment = "KAKENHI Grant Number JP20K01998"))
Description: Provides threshold sweep methods for Qualitative Comparative
    Analysis ('QCA'). Implements Condition Threshold Sweep-Single (CTS-S),
    Condition Threshold Sweep-Multiple (CTS-M), Outcome Threshold Sweep
    (OTS), and Dual Threshold Sweep (DTS) for systematic exploration of
    threshold calibration effects on crisp-set 'QCA' results. These methods
    extend traditional robustness approaches by treating threshold variation
    as an exploratory tool for discovering causal structures. Also provides
    Fiss (2011) <doi:10.5465/amj.2011.60263120> core/peripheral condition
    classification via compute_fiss_core() and generate_fiss_chart(),
    enabling four-symbol configuration charts that distinguish core conditions
    (present in both parsimonious and intermediate solutions) from peripheral
    conditions (intermediate only). Built on top of the 'QCA' package by
    Dusa (2019) <doi:10.1007/978-3-319-75668-4>, with function arguments
    following 'QCA' conventions. Based on set-theoretic methods by Ragin
    (2008) <doi:10.7208/chicago/9780226702797.001.0001> and established
    robustness protocols by Rubinson et al. (2019)
    <doi:10.1177/00491241211036158>. This package supersedes 'TSQCA'; see
    the NEWS file for migration guidance.
Depends: R (>= 4.0)
Imports: QCA
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
License: MIT + file LICENSE
URL: https://github.com/im-research-yt/ThSQCA,
        https://doi.org/10.5281/zenodo.17899390
BugReports: https://github.com/im-research-yt/ThSQCA/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-27 14:10:22 UTC; yukit
Author: Yuki Toyoda [aut, cre],
  Japan Society for the Promotion of Science [fnd] (KAKENHI Grant Number
    JP20K01998)
Maintainer: Yuki Toyoda <yuki.toyoda.ds@hosei.ac.jp>
Repository: CRAN
Date/Publication: 2026-05-30 09:00:08 UTC
Built: R 4.5.2; ; 2026-05-30 12:10:44 UTC; unix

Package: ThSQCA
Type: Package
Title: Threshold-Sweep QCA
Version: 2.0.6
Authors@R: c(
    person("Yuki", "Toyoda", role = c("aut", "cre"),
           email = "yuki.toyoda.ds@hosei.ac.jp"),
    person("Japan Society for the Promotion of Science", role = "fnd",
           comment = "KAKENHI Grant Number JP20K01998"))
Description: Provides threshold sweep methods for Qualitative Comparative 
    Analysis (QCA). Implements Condition Threshold Sweep-Single (CTS-S), 
    Condition Threshold Sweep-Multiple (CTS-M), Outcome Threshold Sweep (OTS), 
    and Dual Threshold Sweep (DTS) for systematic exploration of threshold 
    calibration effects on crisp-set QCA results. These methods extend 
    traditional robustness approaches by treating threshold variation as an 
    exploratory tool for discovering causal structures. Also provides Fiss 
    (2011) <doi:10.5465/amj.2011.60263120> core/peripheral condition 
    classification via compute_fiss_core() and generate_fiss_chart(), enabling 
    four-symbol configuration charts that distinguish core conditions (present 
    in both parsimonious and intermediate solutions) from peripheral conditions 
    (intermediate only). Built on top of the 'QCA' package by Dusa (2019) 
    <doi:10.1007/978-3-319-75668-4>, with function arguments following 'QCA' 
    conventions. Based on set-theoretic methods by Ragin (2008) 
    <doi:10.7208/chicago/9780226702797.001.0001> and established robustness 
    protocols by Oana and Schneider (2024) <doi:10.1177/00491241211036158>.
    This package supersedes 'TSQCA'; see the NEWS file for migration guidance.
Depends: R (>= 4.0)
Imports: QCA
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
License: MIT + file LICENSE
URL: https://github.com/im-research-yt/ThSQCA,
        https://doi.org/10.5281/zenodo.17899390
BugReports: https://github.com/im-research-yt/ThSQCA/issues
Date: 2026-07-23
Encoding: UTF-8
LazyData: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Maintainer: Yuki Toyoda <yuki.toyoda.ds@hosei.ac.jp>
Packaged: 2026-07-31 14:52:36 UTC; yukit
Author: Yuki Toyoda [aut, cre],
  Japan Society for the Promotion of Science [fnd] (KAKENHI Grant Number
    JP20K01998)
Repository: CRAN
Date/Publication: 2026-07-31 16:10:12 UTC
Built: R 4.5.2; ; 2026-07-31 17:43:07 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,ctSweepM_result)
S3method(print,ctSweepS_result)
S3method(print,dtSweep_result)
S3method(print,otSweep_result)
S3method(print,tsqca_result)
S3method(summary,ctSweepM_result)
S3method(summary,ctSweepS_result)
S3method(summary,dtSweep_result)
S3method(summary,otSweep_result)
S3method(summary,tsqca_result)
export(compute_fiss_core)
export(config_chart_from_paths)
export(config_chart_multi_solutions)
export(ctSweepM)
export(ctSweepS)
export(dtSweep)
export(extract_terms)
export(format_qca_solution)
export(format_qca_solutions)
export(format_qca_term)
export(generate_config_chart)
export(generate_cross_threshold_chart)
export(generate_fiss_chart)
export(generate_report)
export(generate_solution_note)
export(identify_epi)
export(otSweep)
export(print_fiss_summary)
importFrom(QCA,minimize)
importFrom(QCA,truthTable)
importFrom(utils,capture.output)

# ThSQCA

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.17899390.svg)](https://doi.org/10.5281/zenodo.17899390)

ThSQCA is an R package implementing **Threshold-Sweep QCA (TS-QCA)**,  
a framework for systematically varying the thresholds used to binarize  
the outcome and conditions in crisp-set QCA.

After calibration, QCA results may change depending on how thresholds are set.  
TS-QCA evaluates this sensitivity by automatically:

- binarizing the data using many threshold candidates  
- generating truth tables  
- applying `QCA::minimize()`  
- extracting the solution, consistency, and coverage  

Implemented sweep types:

- **CTS-QCA (ctSweepS)**: Sweep the threshold of one X  
- **MCTS-QCA (ctSweepM)**: Sweep thresholds of multiple X conditions  
- **OTS-QCA (otSweep)**: Sweep the threshold of Y only  
- **DTS-QCA (dtSweep)**: Sweep X and Y thresholds simultaneously (2D sweep)

> **Scope:** ThSQCA focuses on **sufficiency analysis**. Necessity analysis is planned for future versions.

---

## Features

### Bug Fix for Intermediate Solutions (v1.2.0)

**Important**: v1.2.0 fixes a critical bug where intermediate solutions were incorrectly extracted when using `dir.exp`. If you used intermediate solutions in previous versions, please re-run your analyses.

### Fit-measure fixes for fuzzy multiple-solution and intermediate cells (v2.0.3–v2.0.4)

**Important**: v2.0.3 and v2.0.4 correct the consistency and coverage (`inclS`/`covS`) reported next to a solution in two situations: when a cell has multiple minimal solutions (which occurs on fuzzy data), and when a cell reports an intermediate solution (`dir.exp`). In earlier versions the displayed formula could be paired with the aggregate fit of all minimal solutions, or with the parsimonious fit, rather than with the fit of the formula actually shown. The affected measure is mainly `covS` (`inclS` differs only marginally). Crisp-set results, single-solution cells, and the solution formulas themselves are unaffected, and these fixes do not change any calculation, only which fit value is read. If you have reported ThSQCA's `covS` for fuzzy multiple-solution or intermediate cells, please re-check those values against the NEWS entries.

### Verification Recommendation for Publications

**For academic publications**, we strongly recommend verifying ThSQCA results directly with the QCA package:

```r
library(QCA)

# Run QCA directly to verify ThSQCA results
tt <- truthTable(dat, outcome = "Y", 
                 conditions = c("X1", "X2", "X3"),
                 incl.cut = 0.8)
sol <- minimize(tt, include = "?", dir.exp = c(1, 1, 1))
print(sol)  # Compare with ThSQCA report output
```

ThSQCA reports include a "QCA Package Output" section for this purpose. Always ensure the solution expressions match before publishing.

### Solution Type Display and Verification (New in v1.2.0)

Reports now explicitly show the **Solution Type** (Complex/Parsimonious/Intermediate) and include optional **QCA package output** for verification:

```r
# Reports now show Solution Type in Analysis Overview
generate_report(result, "report.md", dat = mydata)

# Disable QCA raw output if not needed
generate_report(result, "report.md", dat = mydata, include_raw_output = FALSE)
```

### Three Types of QCA Solutions (New in v1.1.0)

As of v1.1.0, ThSQCA uses the **same defaults** as `QCA::minimize()`:

| Solution Type | `include` | `dir.exp` | Logical Remainders | Description |
|--------------|-----------|-----------|-------------------|-------------|
| **Complex** (default) | `""` | `NULL` | Not used | Most conservative |
| **Parsimonious** | `"?"` | `NULL` | All used | Most simplified |
| **Intermediate** | `"?"` | `c(1,1,...)` | Theory-guided | Most common in publications |

```r
data(sample_data)
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

# 1. Complex Solution (default)
result_comp <- otSweep(dat = sample_data, outcome = "Y", 
                       conditions = c("X1", "X2", "X3"),
                       sweep_range = 7, thrX = thrX)

# 2. Parsimonious Solution
result_pars <- otSweep(dat = sample_data, outcome = "Y",
                       conditions = c("X1", "X2", "X3"),
                       sweep_range = 7, thrX = thrX,
                       include = "?")

# 3. Intermediate Solution (most common in publications)
result_int <- otSweep(dat = sample_data, outcome = "Y",
                      conditions = c("X1", "X2", "X3"),
                      sweep_range = 7, thrX = thrX,
                      include = "?", dir.exp = c(1, 1, 1))
```

> **Migration from v1.0.0**: The default has changed from intermediate to complex solution. To reproduce v1.0.0 behavior, explicitly specify `include = "?"` and `dir.exp = c(1, 1, 1)`.

### Multiple Solution Detection

QCA minimization can produce multiple equivalent intermediate solutions. ThSQCA detects and reports these cases, allowing researchers to identify robust essential prime implicants versus solution-specific selective prime implicants.

Use the `extract_mode` parameter to control output:

| Mode | Description | Use Case |
|------|-------------|----------|
| `"first"` | Returns only the first solution (M1) | Default, backward compatible |
| `"all"` | Returns all solutions concatenated | See all equivalent solutions |
| `"essential"` | Returns essential prime implicants common to all solutions | Identify robust findings |

```r
# Detect and show all solutions
result <- otSweep(
  dat = mydata,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7),
  extract_mode = "all"  # Show all solutions
)

# Extract essential prime implicants only
result_essential <- otSweep(
  dat = mydata,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7),
  extract_mode = "essential"  # Show essential prime implicants
)
```

### Automatic Report Generation

The `generate_report()` function creates comprehensive markdown reports:

```r
# Run analysis with return_details = TRUE (the default)
result <- otSweep(
  dat = mydata,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)

# Generate full report
generate_report(result, "my_analysis.md", dat = mydata, format = "full")

# Generate simple report (for journal manuscripts)
generate_report(result, "my_analysis_simple.md", dat = mydata, format = "simple")
```

Reports include:
- Analysis settings (for reproducibility)
- Solution formulas with essential/selective prime implicants
- Fit measures (consistency, coverage, PRI)
- Cross-threshold comparison tables

### Configuration Charts

Configuration charts are automatically included in reports. Generate Fiss-style configuration charts (Table 5 format):

```r
# Reports include configuration charts by default
generate_report(result, "my_report.md", dat = mydata, format = "full")

# Disable charts if needed
generate_report(result, "my_report.md", dat = mydata, include_chart = FALSE)

# Use LaTeX symbols for academic papers
generate_report(result, "my_report.md", dat = mydata, chart_symbol_set = "latex")
```

Standalone chart functions are also available:

```r
# From path strings
paths <- c("A*B*~C", "A*D")
chart <- config_chart_from_paths(paths)
cat(chart)
```

Output:
```
| Condition | M1 | M2 |
|:--:|:--:|:--:|
| A | ● | ● |
| B | ● |   |
| C | ⊗ |   |
| D |   | ● |
```

Three symbol sets available: `"unicode"` (● / ⊗), `"ascii"` (O / X), `"latex"` ($\bullet$ / $\otimes$)

### S3 Methods

All sweep functions return objects supporting S3 class methods:

- `print()`: Display analysis overview with solution statistics
- `summary()`: Display full results table with parameters

```r
result <- otSweep(...)
print(result)    # Quick overview
summary(result)  # Full details
```

### Negated Outcome Support

Analyze conditions sufficient for the **absence** of an outcome:

```r
# Analyze when Y is low (below threshold)
result <- otSweep(
  dat = mydata,
  outcome = "~Y",  # Tilde prefix for negation
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:9,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
```

### Terminology Note

ThSQCA uses precise Boolean algebra terminology:

| Term | Meaning |
|------|---------|
| **Essential Prime Implicants (EPI)** | Terms in ALL equivalent solutions (M1, M2...) |
| **Selective Prime Implicants (SPI)** | Terms in SOME solutions only |

> **Note**: This is distinct from Fiss (2011) "Core Conditions" which compares parsimonious vs. intermediate solutions. See `docs/TSQCA_Terminology_Guide_EN.md` for details.

---

## Installation

```r
install.packages("devtools")
devtools::install_github("im-research-yt/ThSQCA")
```

## Relationship with QCA Package

ThSQCA is built on top of the [QCA package](https://cran.r-project.org/package=QCA) (Duşa, 2019). All function arguments follow QCA conventions:

- **`incl.cut`, `n.cut`, `pri.cut`** → See `QCA::truthTable()`
  - Consistency, frequency, and PRI cutoffs for truth table construction
- **`include`, `dir.exp`** → See `QCA::minimize()`
  - Inclusion rule and directional expectations for logical minimization

### Why This Matters

- **Familiar Workflow**: If you know QCA, you already know ThSQCA's parameters
- **Detailed Documentation**: For in-depth parameter explanations, refer to QCA package documentation
- **Seamless Integration**: ThSQCA extends QCA without replacing it

### Quick Reference

```r
# Check QCA parameter documentation
?QCA::truthTable  # For incl.cut, n.cut, pri.cut
?QCA::minimize    # For include, dir.exp

# ThSQCA uses these same parameters
result <- dtSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2"),
  sweep_list_X = list(X1 = 6:7, X2 = 6:7),
  sweep_range_Y = 6:7,
  incl.cut = 0.8,   # QCA parameter
  n.cut = 1,        # QCA parameter
  pri.cut = 0       # QCA parameter
)
```

## Basic Setup

```r
library(QCA)
library(ThSQCA)

dat <- read.csv("sample_data.csv", fileEncoding = "UTF-8")

outcome  <- "Y"
conditions <- c("X1", "X2", "X3")

str(dat)
```

## Working with Mixed Data Types

### Handling Binary Variables (0/1)

When your dataset contains **both continuous and binary (0/1) variables**, special attention is needed:

- **Do NOT use threshold sweep for binary variables** — they are already binarized
- **Specify threshold = 1** for binary variables to preserve their original values
- **Explicitly define thresholds** for each variable in `sweep_list`

#### Why Threshold = 1 for Binary Variables?

The `qca_bin()` function uses `x >= thr` for binarization:
- If `x = 0`: `0 >= 1` → FALSE → **0** (preserved)
- If `x = 1`: `1 >= 1` → TRUE → **1** (preserved)

#### Example: Mixed Data

```r
# Suppose X1 is binary (0/1), while X2 and X3 are continuous (0-10)
sweep_list <- list(
  X1 = 1,      # Binary variable: use threshold 1 to preserve values
  X2 = 6:8,    # Continuous: sweep thresholds
  X3 = 6:8     # Continuous: sweep thresholds
)

res_mcts <- ctSweepM(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_list = sweep_list,
  thrY = 7
)
```

This explores 1 × 3 × 3 = 9 threshold combinations, treating X1 as a fixed binary condition.

#### Common Mistake

```r
# WRONG: Using sweep range for binary variables
sweep_list <- list(
  X1 = 6:8,    # All values become 0 (since 0 < 6 and 1 < 6)
  X2 = 6:8,
  X3 = 6:8
)
```

**Best Practice**: Always specify thresholds explicitly for each variable based on its data type.

## Usage Examples

### 1. CTS-QCA: single-condition X sweep (ctSweepS)

```r
sweep_var <- "X3"      # Condition (X) whose threshold will be varied
sweep_range <- 6:9     # Candidate threshold values

thrY <- 7              # Fixed threshold for Y
thrX_default <- 7      # Fixed thresholds for other X's

res_cts <- ctSweepS(
  dat            = dat,
  outcome        = outcome,
  conditions     = conditions,
  sweep_var      = sweep_var,      # X to sweep
  sweep_range    = sweep_range,    # Threshold candidates for X
  thrY           = thrY,           # Fixed Y threshold
  thrX_default   = thrX_default    # Fixed thresholds for other X's
)

summary(res_cts)
```

### 2. MCTS-QCA: multi-condition X sweep (ctSweepM)

```r
# Threshold candidates for each X
sweep_list <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

res_mcts <- ctSweepM(
  dat            = dat,
  outcome        = outcome,
  conditions     = conditions,
  sweep_list     = sweep_list,     # Threshold candidates for each X
  thrY           = 7               # Fixed Y threshold
)

summary(res_mcts)
```

### 3. OTS-QCA: outcome Y sweep (otSweep)

```r
thrX <- c(X1 = 7, X2 = 7, X3 = 7)  # Fixed thresholds for X
sweep_range_Y <- 6:8               # Candidate thresholds for Y

res_ots <- otSweep(
  dat            = dat,
  outcome        = outcome,
  conditions     = conditions,
  sweep_range    = sweep_range_Y,  # Y threshold candidates
  thrX           = thrX            # Fixed X thresholds
)

summary(res_ots)

# Generate report for detailed analysis
generate_report(res_ots, "ots_report.md", dat = dat, format = "full")
```

### 4. DTS-QCA: 2D sweep of X and Y (dtSweep)

```r
# X-side threshold candidates (multiple conditions)
sweep_list_X <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

# Y-side threshold candidates
sweep_range_Y <- 6:7

res_dts <- dtSweep(
  dat            = dat,
  outcome        = outcome,
  conditions     = conditions,
  sweep_list_X   = sweep_list_X,   # X threshold candidates
  sweep_range_Y  = sweep_range_Y   # Y threshold candidates
)

summary(res_dts)
```

## Sample Data

```r
data(sample_data)
str(sample_data)
```

## References

### Core QCA Methodology

- Ragin, C. C. (2008). *Redesigning Social Inquiry: Fuzzy Sets and Beyond*. University of Chicago Press. [DOI: 10.7208/chicago/9780226702797.001.0001](https://doi.org/10.7208/chicago/9780226702797.001.0001)
- Schneider, C. Q., & Wagemann, C. (2012). *Set-Theoretic Methods for the Social Sciences: A Guide to Qualitative Comparative Analysis*. Cambridge University Press. [DOI: 10.1017/CBO9781139004244](https://doi.org/10.1017/CBO9781139004244)

### QCA R Package

- Duşa, A. (2019). *QCA with R: A Comprehensive Resource*. Springer. [DOI: 10.1007/978-3-319-75668-4](https://doi.org/10.1007/978-3-319-75668-4)
- Duşa, A. (2018). Consistency Cubes: A Fast, Efficient Method for Exact Boolean Minimization. *The R Journal*, 10(2), 357–370. [DOI: 10.32614/RJ-2018-080](https://doi.org/10.32614/RJ-2018-080)
- Duşa, A. (2024). *QCA: Qualitative Comparative Analysis*. R package version 3.25. https://CRAN.R-project.org/package=QCA

### Robustness and Threshold Sensitivity

- Oana, I.-E., & Schneider, C. Q. (2024). A Robustness Test Protocol for Applied QCA: Theory and R Software Application. *Sociological Methods & Research*, 53(1), 57–88. [DOI: 10.1177/00491241211036158](https://doi.org/10.1177/00491241211036158)
- Oana, I.-E., & Schneider, C. Q. (2018). SetMethods: An Add-on R Package for Advanced QCA. *The R Journal*, 10(1), 507–533. [DOI: 10.32614/RJ-2018-031](https://doi.org/10.32614/RJ-2018-031)
- Skaaning, S.-E. (2011). Assessing the Robustness of Crisp-Set and Fuzzy-Set QCA Results. *Sociological Methods & Research*, 40(2), 391–408. [DOI: 10.1177/0049124111404818](https://doi.org/10.1177/0049124111404818)
- Thiem, A., Spöhel, R., & Duşa, A. (2016). Enhancing Sensitivity Diagnostics for Qualitative Comparative Analysis: A Combinatorial Approach. *Political Analysis*, 24(1), 104–120. [DOI: 10.1093/pan/mpv028](https://doi.org/10.1093/pan/mpv028)

## Acknowledgements

This package implements methods developed in research supported by JSPS KAKENHI Grant Number JP20K01998.

## License

MIT License.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(ThSQCA)
data(sample_data)
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

# 1. Complex Solution (default, most conservative)
result_comp <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX
  # include = "" (default), dir.exp = NULL (default)
)
cat("Complex Solution:", result_comp$summary$expression, "\n")

# 2. Parsimonious Solution (uses all logical remainders)
result_pars <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?"  # Include logical remainders
)
cat("Parsimonious Solution:", result_pars$summary$expression, "\n")

# 3. Intermediate Solution (theory-guided remainders)
result_int <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?",
  dir.exp = c(1, 1, 1)  # All conditions expected to contribute positively
)
cat("Intermediate Solution:", result_int$summary$expression, "\n")

## ----eval=FALSE---------------------------------------------------------------
# # v1.0.0 (intermediate solution by default due to bug)
# result <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX)
# 
# # v1.1.0: To get the same result as v1.0.0, explicitly specify:
# result <- otSweep(
#   dat = sample_data,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 7,
#   thrX = thrX,
#   include = "?",
#   dir.exp = c(1, 1, 1)  # Explicit intermediate solution
# )

## -----------------------------------------------------------------------------
library(ThSQCA)
data("sample_data")
dat <- sample_data
str(dat)

## -----------------------------------------------------------------------------
outcome  <- "Y"
conditions <- c("X1", "X2", "X3")

## ----eval=FALSE---------------------------------------------------------------
# # CORRECT: Specify threshold explicitly for each variable
# sweep_list <- list(
#   X1 = 1,      # Binary variable: use threshold 1
#   X2 = 6:8,    # Continuous: sweep thresholds
#   X3 = 6:8     # Continuous: sweep thresholds
# )
# 
# # Using intermediate solution (most common in publications)
# res_mixed <- ctSweepM(
#   dat            = dat,
#   outcome        = "Y",
#   conditions     = c("X1", "X2", "X3"),
#   sweep_list     = sweep_list,
#   thrY           = 7,
#   include        = "?",           # Include logical remainders
#   dir.exp        = c(1, 1, 1)     # Directional expectations (intermediate)
# )

## ----eval=FALSE---------------------------------------------------------------
# # WRONG: Using sweep range for binary variables
# sweep_list <- list(
#   X1 = 6:8,    # All values become 0 (since 0 < 6 and 1 < 6)
#   X2 = 6:8,
#   X3 = 6:8
# )

## ----eval=FALSE---------------------------------------------------------------
# # Check variable ranges
# summary(dat[, c("X1", "X2", "X3")])
# 
# # Identify binary variables (only 0 and 1)
# sapply(dat[, c("X1", "X2", "X3")], function(x) {
#   unique_vals <- sort(unique(x))
#   if (length(unique_vals) == 2 && all(unique_vals == c(0, 1))) {
#     "Binary (use threshold = 1)"
#   } else {
#     paste("Continuous (range:", min(x), "-", max(x), ")")
#   }
# })

## ----error=TRUE---------------------------------------------------------------
try({
sweep_var   <- "X3"   # Condition (X) whose threshold is swept
sweep_range <- 6:9    # Candidate threshold values to evaluate

thrY         <- 7     # Outcome (Y) threshold (fixed)
thrX_default <- 7     # Threshold for other X conditions (fixed)

# Default: Complex solution (include = "", dir.exp = NULL)
res_cts <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = sweep_var,
  sweep_range    = sweep_range,
  thrY           = thrY,
  thrX_default   = thrX_default,
  return_details = TRUE
)

summary(res_cts)
})

## ----error=TRUE---------------------------------------------------------------
try({
# Intermediate solution: specify include = "?" and dir.exp
res_cts_int <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = sweep_var,
  sweep_range    = sweep_range,
  thrY           = thrY,
  thrX_default   = thrX_default,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1),    # Directional expectations
  return_details = TRUE
)

summary(res_cts_int)
})

## ----error=TRUE---------------------------------------------------------------
try({
# Create a sweep list specifying thresholds for each condition
sweep_list <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

# Default: Complex solution
res_mcts <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  return_details = TRUE
)

summary(res_mcts)
})

## ----error=TRUE---------------------------------------------------------------
try({
# Intermediate solution: specify include = "?" and dir.exp
res_mcts_int <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_mcts_int)
})

## -----------------------------------------------------------------------------
# Default: Complex solution
res_ots <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = c(X1 = 7, X2 = 7, X3 = 7),
  return_details = TRUE
)

summary(res_ots)

## -----------------------------------------------------------------------------
# Intermediate solution: specify include = "?" and dir.exp
res_ots_int <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = c(X1 = 7, X2 = 7, X3 = 7),
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_ots_int)

## -----------------------------------------------------------------------------
sweep_list_dts_X <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

sweep_range_dts_Y <- 6:7

# Default: Complex solution
res_dts <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  return_details = TRUE
)

summary(res_dts)

## -----------------------------------------------------------------------------
# Intermediate solution: specify include = "?" and dir.exp
res_dts_int <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_dts_int)

## ----eval=FALSE---------------------------------------------------------------
# # Returns only the first solution (M1)
# # Backward compatible with v0.1.x
# result <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = c(X1 = 7, X2 = 7, X3 = 7),
#   extract_mode = "first"  # Default
# )

## ----eval=FALSE---------------------------------------------------------------
# # Returns all solutions concatenated
# # Useful for seeing all equivalent solutions
# result_all <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = c(X1 = 7, X2 = 7, X3 = 7),
#   extract_mode = "all"
# )
# 
# # Output includes n_solutions column
# head(result_all$summary)
# # expression column shows: "M1: A*B + C; M2: A*B + D; M3: ..."

## ----eval=FALSE---------------------------------------------------------------
# # Returns essential prime implicants (terms common to all solutions)
# # Best for identifying robust findings
# result_essential <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = c(X1 = 7, X2 = 7, X3 = 7),
#   extract_mode = "essential"
# )
# 
# # Output includes:
# # - expression: essential prime implicants
# # - selective_terms: terms in some but not all solutions
# # - unique_terms: solution-specific terms
# # - n_solutions: number of equivalent solutions

## ----eval=FALSE---------------------------------------------------------------
# # Generate full report
# generate_report(res_ots, "my_analysis_full.md", dat = dat, format = "full")

## ----eval=FALSE---------------------------------------------------------------
# # Generate simple report
# generate_report(res_ots, "my_analysis_simple.md", dat = dat, format = "simple")

## ----eval=FALSE---------------------------------------------------------------
# # Complete workflow
# result <- otSweep(
#   dat = mydata,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = c(X1 = 7, X2 = 7, X3 = 7)
# )
# 
# # For main text
# generate_report(result, "manuscript_results.md", dat = mydata, format = "simple")
# 
# # For supplementary materials
# generate_report(result, "supplementary_full.md", dat = mydata, format = "full")

## ----eval=FALSE---------------------------------------------------------------
# # View stored parameters
# result$params
# 
# # Includes:
# # - outcome, conditions: variable names
# # - thrX, thrY: threshold values
# # - incl.cut, n.cut, pri.cut: QCA parameters
# # - dir.exp, include: minimization settings

## ----eval=FALSE---------------------------------------------------------------
# # Test with a single threshold first
# result <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 7,  # Single value
#   thrX = c(X1 = 7, X2 = 7, X3 = 7)
# )

## ----eval=FALSE---------------------------------------------------------------
# # Expand to a small range
# result <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:7,  # Small range
#   thrX = c(X1 = 7, X2 = 7, X3 = 7)
# )

## ----eval=FALSE---------------------------------------------------------------
# # Run full analysis only after confirming it works
# result <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 5:9,  # Full range
#   thrX = c(X1 = 7, X2 = 7, X3 = 7)
# )

## ----eval=FALSE---------------------------------------------------------------
# # Manageable: 2 × 2 × 2 = 8 combinations
# sweep_list <- list(X1 = 6:7, X2 = 6:7, X3 = 6:7)
# 
# # Caution: 5 × 5 × 5 = 125 combinations
# sweep_list <- list(X1 = 5:9, X2 = 5:9, X3 = 5:9)
# 
# # Avoid: 5 × 5 × 5 × 5 × 5 = 3125 combinations!
# sweep_list <- list(X1 = 5:9, X2 = 5:9, X3 = 5:9, X4 = 5:9, X5 = 5:9)

## ----eval=FALSE---------------------------------------------------------------
# # Analyze conditions for Y >= threshold (standard)
# result_Y <- otSweep(
#   dat = dat,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = c(X1 = 7, X2 = 7, X3 = 7)
# )
# 
# # Analyze conditions for Y < threshold (negated)
# result_negY <- otSweep(
#   dat = dat,
#   outcome = "~Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = c(X1 = 7, X2 = 7, X3 = 7)
# )

## ----eval=FALSE---------------------------------------------------------------
# # ctSweepS with negated outcome
# result <- ctSweepS(
#   dat = dat,
#   outcome = "~Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_var = "X3",
#   sweep_range = 6:8,
#   thrY = 7,
#   thrX_default = 7
# )
# 
# # ctSweepM with negated outcome
# result <- ctSweepM(
#   dat = dat,
#   outcome = "~Y",
#   conditions = c("X1", "X2"),
#   sweep_list = list(X1 = 6:7, X2 = 6:7),
#   thrY = 7
# )
# 
# # dtSweep with negated outcome
# result <- dtSweep(
#   dat = dat,
#   outcome = "~Y",
#   conditions = c("X1", "X2"),
#   sweep_list_X = list(X1 = 6:7, X2 = 7),
#   sweep_range_Y = 6:8
# )

## ----eval=FALSE---------------------------------------------------------------
# result <- otSweep(dat = dat, outcome = "~Y", ...)
# 
# # Check if negated
# result$params$negate_outcome
# # [1] TRUE
# 
# result$params$outcome
# # [1] "~Y"

## ----eval=FALSE---------------------------------------------------------------
# library(ThSQCA)
# data(sample_data)
# 
# # Step 1: Run intermediate sweep (all three conditions above are met)
# res <- otSweep(
#   dat        = sample_data,
#   outcome    = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX       = c(X1 = 7, X2 = 7, X3 = 7),
#   include    = "?",           # required
#   dir.exp    = c(1, 1, 1),   # required
#   return_details = TRUE       # required
# )
# 
# # Step 2: Compute Fiss core/peripheral classification
# # For each threshold, compute_fiss_core() re-runs QCA::minimize()
# # with dir.exp = NULL to obtain the parsimonious solution,
# # then compares it to the stored intermediate solution.
# res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))
# 
# # Step 3: Inspect a specific threshold
# print_fiss_summary(res_fiss, thr_key = "7")
# 
# # Step 4: Generate Fiss-style four-symbol chart
# cat(generate_fiss_chart(res_fiss, symbol_set = "unicode"))
# 
# # For academic papers (LaTeX)
# cat(generate_fiss_chart(res_fiss, symbol_set = "latex"))
# 
# # Step 5: Include in report
# generate_report(
#   res_fiss,
#   output_file      = "fiss_report.md",
#   format           = "full",
#   include_fiss_core = TRUE,          # activates four-symbol charts
#   chart_symbol_set  = "unicode"
# )

## ----eval=FALSE---------------------------------------------------------------
# # Generate report with configuration charts (default)
# generate_report(result, "my_report.md", dat = dat, format = "full")
# 
# # Disable charts if needed
# generate_report(result, "my_report.md", dat = dat, include_chart = FALSE)
# 
# # Use LaTeX symbols for academic papers
# generate_report(result, "my_report.md", dat = dat, chart_symbol_set = "latex")
# 
# # Include QCA package raw output for verification (default is TRUE)
# generate_report(result, "my_report.md", dat = dat, include_raw_output = TRUE)
# 
# # Disable raw output if not needed
# generate_report(result, "my_report.md", dat = dat, include_raw_output = FALSE)

## -----------------------------------------------------------------------------
# From path strings
paths <- c("A*B*~C", "A*D", "B*E")
chart <- config_chart_from_paths(paths)
cat(chart)

## -----------------------------------------------------------------------------
# ASCII (maximum compatibility)
cat(config_chart_from_paths(paths, symbol_set = "ascii"))

## ----eval=FALSE---------------------------------------------------------------
# # LaTeX (for PDF/academic papers)
# cat(config_chart_from_paths(paths, symbol_set = "latex"))
# # Output: $\bullet$ for presence, $\otimes$ for absence

## -----------------------------------------------------------------------------
solutions <- list(
  c("A*B", "C*D"),
  c("A*B", "C*E")
)
chart <- config_chart_multi_solutions(solutions)
cat(chart)

## ----eval=FALSE---------------------------------------------------------------
# library(QCA)
# library(ThSQCA)
# 
# # Step 1: Run ThSQCA analysis
# data(sample_data)
# thrX <- c(X1 = 7, X2 = 7, X3 = 7)
# 
# result <- otSweep(
#   dat = sample_data,
#   outcome = "Y",
#   conditions = c("X1", "X2", "X3"),
#   sweep_range = 6:8,
#   thrX = thrX,
#   include = "?",
#   dir.exp = c(1, 1, 1),
#   return_details = TRUE
# )
# 
# # Step 2: Generate report with QCA output for verification
# generate_report(result, "my_analysis.md", dat = sample_data,
#                 include_raw_output = TRUE)  # Default is TRUE
# 
# # Step 3: Verify key results directly with QCA package
# # For each important threshold, run QCA::minimize() directly
# dat_bin <- sample_data
# dat_bin$Y_bin <- ifelse(sample_data$Y >= 7, 1, 0)
# dat_bin$X1_bin <- ifelse(sample_data$X1 >= 7, 1, 0)
# dat_bin$X2_bin <- ifelse(sample_data$X2 >= 7, 1, 0)
# dat_bin$X3_bin <- ifelse(sample_data$X3 >= 7, 1, 0)
# 
# tt <- truthTable(dat_bin, outcome = "Y_bin",
#                  conditions = c("X1_bin", "X2_bin", "X3_bin"),
#                  incl.cut = 0.8)
# sol <- minimize(tt, include = "?", dir.exp = c(1, 1, 1))
# print(sol)  # Compare with ThSQCA report

## -----------------------------------------------------------------------------
sessionInfo()

## ----include=FALSE------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(ThSQCA)
library(QCA)

## -----------------------------------------------------------------------------
# Adjust the file name as needed
library(ThSQCA)
data("sample_data")
dat <- sample_data

# Outcome and conditions
outcome  <- "Y"
conditions <- c("X1", "X2", "X3")

# Quick inspection
str(dat)
summary(dat)

## -----------------------------------------------------------------------------
thrY_base <- 7
thrX_base <- 7

# Fixed X thresholds (for OTS–QCA)
thrX_vec <- c(
  X1 = thrX_base,
  X2 = thrX_base,
  X3 = thrX_base
)
thrX_vec

## -----------------------------------------------------------------------------
# Example: Three solution types at single threshold
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

# Complex (default)
res_comp <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX)
cat("Complex:", res_comp$summary$expression, "\n")

# Parsimonious
res_pars <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX,
                    include = "?")
cat("Parsimonious:", res_pars$summary$expression, "\n")

# Intermediate
res_int <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX,
                   include = "?", dir.exp = c(1, 1, 1))
cat("Intermediate:", res_int$summary$expression, "\n")

## ----error=TRUE---------------------------------------------------------------
try({
sweep_var   <- "X3"   # Condition (X) whose threshold is swept
sweep_range <- 6:9    # Candidate threshold values to evaluate
thrY         <- 7     # Outcome (Y) threshold (fixed)
thrX_default <- 7     # Threshold for other X conditions (fixed)

# Default: Complex solution (include = "", dir.exp = NULL)
res_cts <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = "X3",
  sweep_range    = 6:9,
  thrY           = 7,
  thrX_default   = 7,
  return_details = TRUE
)

summary(res_cts)
})

## ----error=TRUE---------------------------------------------------------------
try({
# Intermediate solution: specify include = "?" and dir.exp
res_cts_int <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = "X3",
  sweep_range    = 6:9,
  thrY           = 7,
  thrX_default   = 7,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_cts_int)
})

## ----eval=FALSE---------------------------------------------------------------
# write.csv(res_cts$summary, file = "TSQCA_CTS_results.csv", row.names = FALSE)

## ----error=TRUE---------------------------------------------------------------
try({
# Create a sweep list specifying thresholds for each condition
sweep_list <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

# Default: Complex solution
res_mcts <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  return_details = TRUE
)

summary(res_mcts)
})

## ----error=TRUE---------------------------------------------------------------
try({
# Intermediate solution: specify include = "?" and dir.exp
res_mcts_int <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_mcts_int)
})

## ----eval=FALSE---------------------------------------------------------------
# write.csv(res_mcts$summary, file = "TSQCA_MCTS_results.csv", row.names = FALSE)

## -----------------------------------------------------------------------------
sweep_range_ots <- 6:8

# Default: Complex solution
res_ots <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = sweep_range_ots,
  thrX           = thrX_vec,
  return_details = TRUE
)

summary(res_ots)

## -----------------------------------------------------------------------------
# Intermediate solution: specify include = "?" and dir.exp
res_ots_int <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = sweep_range_ots,
  thrX           = thrX_vec,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_ots_int)

## ----eval=FALSE---------------------------------------------------------------
# write.csv(res_ots$summary, file = "TSQCA_OTS_results.csv", row.names = FALSE)

## -----------------------------------------------------------------------------
sweep_list_dts_X <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

sweep_range_dts_Y <- 6:7

# Default: Complex solution
res_dts <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_dts)

## -----------------------------------------------------------------------------
# Intermediate solution: specify include = "?" and dir.exp
res_dts_int <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_dts_int)

## ----eval=FALSE---------------------------------------------------------------
# write.csv(res_dts$summary, file = "TSQCA_DTS_results.csv", row.names = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# res_all <- otSweep(
#   dat            = dat,
#   outcome        = "Y",
#   conditions     = c("X1", "X2", "X3"),
#   sweep_range    = 6:8,
#   thrX           = thrX_vec,
#   include        = "?",           # Include logical remainders
#   dir.exp        = c(1, 1, 1),    # Intermediate solution
#   extract_mode   = "all",
#   return_details = TRUE
# )
# 
# # View results with n_solutions column
# head(res_all$summary)

## ----eval=FALSE---------------------------------------------------------------
# res_essential <- otSweep(
#   dat            = dat,
#   outcome        = "Y",
#   conditions     = c("X1", "X2", "X3"),
#   sweep_range    = 6:8,
#   thrX           = thrX_vec,
#   include        = "?",           # Include logical remainders
#   dir.exp        = c(1, 1, 1),    # Intermediate solution
#   extract_mode   = "essential",
#   return_details = TRUE
# )
# 
# # View results with essential prime implicants, selective terms, and unique terms
# head(res_essential$summary)

## ----eval=FALSE---------------------------------------------------------------
# generate_report(res_ots, "TSQCA_OTS_report_full.md", dat = dat, format = "full")

## ----eval=FALSE---------------------------------------------------------------
# generate_report(res_ots, "TSQCA_OTS_report_simple.md", dat = dat, format = "simple")

## ----eval=FALSE---------------------------------------------------------------
# # Standard: conditions for Y >= threshold (intermediate solution)
# res_Y <- otSweep(
#   dat            = dat,
#   outcome        = "Y",
#   conditions     = c("X1", "X2", "X3"),
#   sweep_range    = 6:8,
#   thrX           = thrX_vec,
#   include        = "?",           # Include logical remainders
#   dir.exp        = c(1, 1, 1)     # Intermediate solution
# )
# 
# # Negated: conditions for Y < threshold (intermediate solution)
# res_negY <- otSweep(
#   dat            = dat,
#   outcome        = "~Y",
#   conditions     = c("X1", "X2", "X3"),
#   sweep_range    = 6:8,
#   thrX           = thrX_vec,
#   include        = "?",           # Include logical remainders
#   dir.exp        = c(1, 1, 1)     # Intermediate solution
# )
# 
# # Compare results
# res_Y$summary
# res_negY$summary
# 
# # Check negation flag
# res_negY$params$negate_outcome
# # [1] TRUE

## ----eval=FALSE---------------------------------------------------------------
# # View stored parameters
# res_ots$params
# 
# # Example output:
# # $outcome
# # [1] "Y"
# # $conditions
# # [1] "X1" "X2" "X3"
# # $thrX
# # X1 X2 X3
# #  7  7  7
# # $incl.cut
# # [1] 0.8
# # $n.cut
# # [1] 1
# # $pri.cut
# # [1] 0

## -----------------------------------------------------------------------------
# From path strings
paths <- c("A*B*~C", "A*D")
chart <- config_chart_from_paths(paths)
cat(chart)

## ----eval=FALSE---------------------------------------------------------------
# # Charts are included by default
# generate_report(result, "report.md", dat = dat, format = "full")
# 
# # Use LaTeX symbols for academic papers
# generate_report(result, "report.md", dat = dat, chart_symbol_set = "latex")

## ----eval=FALSE---------------------------------------------------------------
# # Requires: include = "?", dir.exp specified, return_details = TRUE
# # res_ots_int was created in Section 6 above.
# 
# res_fiss <- compute_fiss_core(
#   res_ots_int,
#   conditions = c("X1", "X2", "X3")
# )
# 
# # Inspect available thresholds
# names(res_fiss$fiss_core)

## ----eval=FALSE---------------------------------------------------------------
# # English
# print_fiss_summary(res_fiss, thr_key = "7")
# 
# # Japanese
# print_fiss_summary(res_fiss, thr_key = "7", language = "ja")

## ----eval=FALSE---------------------------------------------------------------
# # Unicode (Markdown / screen)
# cat(generate_fiss_chart(res_fiss, symbol_set = "unicode"))
# 
# # LaTeX (for PDF/journal submission)
# cat(generate_fiss_chart(res_fiss, symbol_set = "latex"))
# 
# # ASCII (maximum compatibility)
# cat(generate_fiss_chart(res_fiss, symbol_set = "ascii"))

## ----eval=FALSE---------------------------------------------------------------
# # Full report with Fiss four-symbol charts
# generate_report(
#   res_fiss,
#   output_file       = "TSQCA_Fiss_report.md",
#   format            = "full",
#   dat               = dat,
#   include_fiss_core = TRUE,       # activates four-symbol charts
#   chart_symbol_set  = "unicode",
#   include_raw_output = TRUE
# )

## ----eval=FALSE---------------------------------------------------------------
# # Access raw classification data frame for a given threshold
# classif_7 <- res_fiss$fiss_core[["7"]]$classification
# classif_7
# 
# # Filter core conditions only
# classif_7[classif_7$type == "core" & classif_7$status != "dontcare", ]
# 
# # Filter peripheral conditions only
# classif_7[classif_7$type == "peripheral" & classif_7$status != "dontcare", ]
# 
# # Parsimonious and intermediate expressions
# res_fiss$fiss_core[["7"]]$parsim_expression
# res_fiss$fiss_core[["7"]]$interm_expression

## -----------------------------------------------------------------------------
sessionInfo()

###############################################
# Helper functions for ThSQCA
###############################################

#' Sanitize a truth table before Boolean minimization
#'
#' Guards against two behaviors of \code{QCA::truthTable()} (observed with
#' QCA 3.25 / admisc 0.40) that can affect programmatic, repeated calls such
#' as threshold sweeps. For sparse truth tables the \code{incl}/\code{PRI}
#' columns may be returned as \code{character} rather than \code{numeric},
#' and logical-remainder rows (observed \code{n = 0}) may carry the string
#' \code{"-"}. Passing such a truth table to \code{QCA::minimize()} can, for
#' some truth table structures, cause the minimization to hang or return
#' misleading fit values. This helper coerces \code{incl}/\code{PRI} to
#' numeric and sets the remainder rows to 0.
#'
#' The \code{OUT} column is deliberately left untouched: \code{minimize()}
#' uses \code{OUT} (with the symbolic \code{"?"} for remainders) to identify
#' positive, negative, and remainder rows, so setting \code{incl}/\code{PRI}
#' to 0 on remainder rows does not alter remainder handling.
#'
#' @param tt A truth table object returned by \code{QCA::truthTable()}.
#'
#' @return The same object with numeric \code{incl}/\code{PRI} columns and
#'   no \code{NA} in those columns. Returns the input unchanged if it is
#'   \code{NULL} or lacks a \code{$tt} data frame.
#' @keywords internal
sanitize_truthtable <- function(tt) {
  if (is.null(tt) || is.null(tt$tt)) return(tt)
  if (!is.null(tt$tt$incl) && is.character(tt$tt$incl)) {
    tt$tt$incl <- suppressWarnings(as.numeric(tt$tt$incl))
  }
  if (!is.null(tt$tt$PRI) && is.character(tt$tt$PRI)) {
    tt$tt$PRI <- suppressWarnings(as.numeric(tt$tt$PRI))
  }
  if (!is.null(tt$tt$incl)) tt$tt$incl[is.na(tt$tt$incl)] <- 0
  if (!is.null(tt$tt$PRI))  tt$tt$PRI[is.na(tt$tt$PRI)]   <- 0
  tt
}

#' Escape special characters for Markdown
#'
#' Escapes asterisks and other special characters that have special
#' meaning in Markdown syntax.
#'
#' @param text Character. Text to escape.
#'
#' @return Character. Text with special characters escaped.
#' @keywords internal
escape_md <- function(text) {
  if (is.null(text) || length(text) == 0) return(text)
  # Escape * to \* for Markdown

  gsub("\\*", "\\\\*", text)
}


#' Extract all metrics from QCA solution object
#'
#' Safely extracts solution-level and term-level metrics from
#' various QCA solution object structures. Handles cases where
#' multiple solutions exist and sol.incl.cov may be NULL.
#'
#' @param IC IC object from QCA solution.
#' @param sol_obj Full solution object (optional, for fallback).
#'
#' @return List with elements:
#'   \itemize{
#'     \item \code{sol_inclS} — solution consistency (or NA)
#'     \item \code{sol_PRI} — solution PRI (or NA)
#'     \item \code{sol_covS} — solution coverage (or NA)
#'     \item \code{term_df} — data frame of per-term metrics (or NULL)
#'   }
#' @keywords internal
extract_all_metrics <- function(IC, sol_obj = NULL) {
  
  result <- list(
    sol_inclS = NA_real_,
    sol_PRI   = NA_real_,
    sol_covS  = NA_real_,
    term_df   = NULL
  )
  
  # Try to get from IC directly
  if (!is.null(IC)) {
    
    # Method 1: sol.incl.cov (single solution case)
    if ("sol.incl.cov" %in% names(IC)) {
      sol <- IC$sol.incl.cov
      if (!is.null(sol$inclS)) result$sol_inclS <- sol$inclS[1]
      if (!is.null(sol$PRI)) result$sol_PRI <- sol$PRI[1]
      if (!is.null(sol$covS)) result$sol_covS <- sol$covS[1]
    }
    
    # Method 3 (solution fit): individual (multiple solutions, FIRST solution).
    # This must run BEFORE the overall block below: with multiple minimal
    # solutions, the displayed formula is the first solution (M1), so its fit is
    # individual[[1]]$sol.incl.cov, not the overall (disjunction-of-all) aggregate
    # that overinflates coverage on fuzzy data. term_df is also taken from the
    # first individual solution.
    if ("individual" %in% names(IC)) {
      indiv <- IC$individual
      if (is.list(indiv) && length(indiv) > 0) {
        first_indiv <- indiv[[1]]
        if (is.list(first_indiv)) {
          if (is.na(result$sol_inclS) && "sol.incl.cov" %in% names(first_indiv)) {
            sol <- first_indiv$sol.incl.cov
            if (!is.null(sol$inclS)) result$sol_inclS <- sol$inclS[1]
            if (!is.null(sol$PRI)) result$sol_PRI <- sol$PRI[1]
            if (!is.null(sol$covS)) result$sol_covS <- sol$covS[1]
          }
          if (is.null(result$term_df) && "incl.cov" %in% names(first_indiv)) {
            result$term_df <- first_indiv$incl.cov
          }
        }
      }
    }

    # Method 2 (fallback): overall (disjunction of all solutions). Used only when
    # the first solution's own fit was unavailable above.
    if (is.na(result$sol_inclS) && "overall" %in% names(IC)) {
      overall <- IC$overall
      if (is.list(overall)) {
        if ("sol.incl.cov" %in% names(overall)) {
          sol <- overall$sol.incl.cov
          if (!is.null(sol$inclS)) result$sol_inclS <- sol$inclS[1]
          if (!is.null(sol$PRI)) result$sol_PRI <- sol$PRI[1]
          if (!is.null(sol$covS)) result$sol_covS <- sol$covS[1]
        }
        if (is.na(result$sol_inclS) && !is.null(overall$inclS)) {
          result$sol_inclS <- overall$inclS[1]
        }
        if (is.na(result$sol_PRI) && !is.null(overall$PRI)) {
          result$sol_PRI <- overall$PRI[1]
        }
        if (is.na(result$sol_covS) && !is.null(overall$covS)) {
          result$sol_covS <- overall$covS[1]
        }
      }
    }
    
    # Get per-term metrics (incl.cov)
    if (is.null(result$term_df) && "incl.cov" %in% names(IC)) {
      result$term_df <- IC$incl.cov
    }
    
    # If we got values, return
    if (!is.na(result$sol_inclS)) return(result)
  }
  
  # Fallback: try to get from sol_obj
  if (!is.null(sol_obj)) {
    
    # Method 4: sol_obj$IC
    if (!is.null(sol_obj$IC)) {
      IC <- sol_obj$IC
      
      # Try to get term_df from sol_obj$IC$incl.cov first
      if (is.null(result$term_df) && "incl.cov" %in% names(IC)) {
        result$term_df <- IC$incl.cov
      }
      
      # Method 4b (solution fit): individual first solution, BEFORE overall.
      if ("individual" %in% names(IC)) {
        indiv <- IC$individual
        if (is.list(indiv) && length(indiv) > 0) {
          first_indiv <- indiv[[1]]
          if (is.list(first_indiv)) {
            if (is.na(result$sol_inclS) && "sol.incl.cov" %in% names(first_indiv)) {
              sol <- first_indiv$sol.incl.cov
              if (!is.null(sol$inclS)) result$sol_inclS <- sol$inclS[1]
              if (!is.null(sol$PRI)) result$sol_PRI <- sol$PRI[1]
              if (!is.null(sol$covS)) result$sol_covS <- sol$covS[1]
            }
            if (is.null(result$term_df) && "incl.cov" %in% names(first_indiv)) {
              result$term_df <- first_indiv$incl.cov
            }
          }
        }
      }

      # overall as fallback only.
      if (is.na(result$sol_inclS) && "overall" %in% names(IC)) {
        overall <- IC$overall
        if (is.list(overall)) {
          if ("sol.incl.cov" %in% names(overall)) {
            sol <- overall$sol.incl.cov
            if (!is.null(sol$inclS)) result$sol_inclS <- sol$inclS[1]
            if (!is.null(sol$PRI)) result$sol_PRI <- sol$PRI[1]
            if (!is.null(sol$covS)) result$sol_covS <- sol$covS[1]
          }
          if (is.na(result$sol_inclS) && !is.null(overall$inclS)) {
            result$sol_inclS <- overall$inclS[1]
          }
          if (is.na(result$sol_PRI) && !is.null(overall$PRI)) {
            result$sol_PRI <- overall$PRI[1]
          }
          if (is.na(result$sol_covS) && !is.null(overall$covS)) {
            result$sol_covS <- overall$covS[1]
          }
        }
      }
    }
    
    # Method 5: i.sol (named list: C1P1, C1P2, etc.)
    # Check for term_df even if sol_inclS is already found
    if (!is.null(sol_obj$i.sol) && length(sol_obj$i.sol) > 0) {
      first_isol <- sol_obj$i.sol[[1]]
      if (!is.null(first_isol$IC)) {
        IC_isol <- first_isol$IC
        if (is.na(result$sol_inclS) && "sol.incl.cov" %in% names(IC_isol)) {
          sol <- IC_isol$sol.incl.cov
          if (!is.null(sol$inclS)) result$sol_inclS <- sol$inclS[1]
          if (!is.null(sol$PRI)) result$sol_PRI <- sol$PRI[1]
          if (!is.null(sol$covS)) result$sol_covS <- sol$covS[1]
        }
        if (is.null(result$term_df) && "incl.cov" %in% names(IC_isol)) {
          result$term_df <- IC_isol$incl.cov
        }
        # Also check individual within i.sol
        if (is.null(result$term_df) && "individual" %in% names(IC_isol)) {
          indiv_isol <- IC_isol$individual
          if (is.list(indiv_isol) && length(indiv_isol) > 0) {
            first_indiv_isol <- indiv_isol[[1]]
            if (is.list(first_indiv_isol) && "incl.cov" %in% names(first_indiv_isol)) {
              result$term_df <- first_indiv_isol$incl.cov
            }
          }
        }
      }
    }
  }
  
  return(result)
}


#' Convert data frame to Markdown table
#'
#' @param df Data frame to convert.
#' @param digits Number of decimal places for numeric columns.
#'
#' @return Character string of Markdown table.
#' @keywords internal
df_to_md_table <- function(df, digits = 3) {
  if (is.null(df) || nrow(df) == 0) return("(No data)\n")
  
  # Round numeric columns
  for (col in names(df)) {
    if (is.numeric(df[[col]])) {
      df[[col]] <- round(df[[col]], digits)
    }
  }
  
  # Convert all to character for consistent output
  df <- as.data.frame(lapply(df, as.character), stringsAsFactors = FALSE)
  
  # Escape special Markdown characters (especially * in QCA expressions)
  df <- as.data.frame(lapply(df, escape_md), stringsAsFactors = FALSE)
  
  # Build header
  header <- paste0("| ", paste(names(df), collapse = " | "), " |")
  separator <- paste0("|", paste(rep("---", ncol(df)), collapse = "|"), "|")
  
  # Build rows
  rows <- apply(df, 1, function(row) {
    paste0("| ", paste(row, collapse = " | "), " |")
  })
  
  paste(c(header, separator, rows), collapse = "\n")
}


#' Split solution expression into terms
#'
#' @param solution Character. Solution expression (e.g., "X1*X2 + X3").
#'
#' @return Character vector of terms.
#' @keywords internal
split_solution_terms <- function(solution) {
  if (is.null(solution) || is.na(solution) || solution == "" || 
      solution == "No solution" || solution == "No essential prime implicants") {
    return(character(0))
  }
  trimws(unlist(strsplit(solution, " \\+ ")))
}


#' Get all unique terms from multiple solutions
#'
#' @param sol_list List of solution character vectors from minimize().
#'
#' @return List with:
#'   \itemize{
#'     \item \code{all_terms} — all terms (with duplicates)
#'     \item \code{unique_terms} — unique terms
#'     \item \code{term_counts} — table of term frequencies
#'   }
#' @keywords internal
get_all_terms <- function(sol_list) {
  if (is.null(sol_list) || length(sol_list) == 0) {
    return(list(
      all_terms    = character(0),
      unique_terms = character(0),
      term_counts  = table(character(0))
    ))
  }
  
  # Extract terms from each solution
  all_terms <- unlist(lapply(sol_list, function(sol) {
    split_solution_terms(paste(sol, collapse = " + "))
  }))
  
  list(
    all_terms    = all_terms,
    unique_terms = unique(all_terms),
    term_counts  = table(all_terms)
  )
}


###############################################
# Solution formatting functions
###############################################

#' Format a single QCA term
#'
#' Inserts \code{*} between variables in a term where it may have been omitted.
#'
#' @param term Character. A single term (e.g., "KSPRVT" or "~KPR*PRD").
#' @param var_names Character vector. Variable names used in the analysis.
#' @param use_tilde Logical. If TRUE, negation is represented as \code{~VAR}.
#'   If FALSE, negation is represented as lowercase (e.g., \code{var}).
#'
#' @return Character. The formatted term with \code{*} between all variables.
#' @export
#'
#' @examples
#' var_names <- c("KSP", "KPR", "PRD", "RVT", "RCM")
#' format_qca_term("KSPRVTRCM", var_names)
#' # Returns: "KSP*RVT*RCM"
#' 
#' format_qca_term("~KPRPRD", var_names)
#' # Returns: "~KPR*PRD"
format_qca_term <- function(term, var_names, use_tilde = TRUE) {
  
  if (is.null(term) || is.na(term) || term == "") {
    return("")
  }
  
  # If already contains *, assume it's formatted
  if (grepl("\\*", term)) {
    return(term)
  }
  
  # Sort variable names by length (descending) to avoid partial matches
  var_names_sorted <- var_names[order(nchar(var_names), decreasing = TRUE)]
  
  # Build pattern for matching
  if (use_tilde) {
    # Match ~VAR or VAR
    patterns <- paste0("~?", var_names_sorted)
  } else {
    # Match VAR (uppercase) or var (lowercase for negation)
    patterns <- c(var_names_sorted, tolower(var_names_sorted))
  }
  
  pattern <- paste(patterns, collapse = "|")
  
  # Extract all matches
  matches <- regmatches(term, gregexpr(pattern, term, ignore.case = FALSE))[[1]]
  
  if (length(matches) == 0) {
    return(term)
  }
  
  # Join with *
  paste(matches, collapse = "*")
}


#' Format a QCA solution expression
#'
#' Formats a complete solution expression (multiple terms joined by +).
#'
#' @param solution Character. A solution expression (e.g., "KSPRVT + ~KPRPRD").
#' @param var_names Character vector. Variable names used in the analysis.
#' @param use_tilde Logical. If TRUE, negation is represented as \code{~VAR}.
#'
#' @return Character. The formatted solution expression.
#' @export
#'
#' @examples
#' var_names <- c("KSP", "KPR", "PRD", "RVT", "RCM")
#' format_qca_solution("KSPRVT + ~KPRPRD + RCM", var_names)
#' # Returns: "KSP*RVT + ~KPR*PRD + RCM"
format_qca_solution <- function(solution, var_names, use_tilde = TRUE) {
  
  if (is.null(solution) || is.na(solution) || solution == "") {
    return("")
  }
  
  if (solution == "No solution" || solution == "No essential prime implicants") {
    return(solution)
  }
  
  # Split by " + "
  terms <- trimws(unlist(strsplit(solution, " \\+ ")))
  
  # Format each term
  formatted_terms <- sapply(terms, format_qca_term, 
                            var_names = var_names, 
                            use_tilde = use_tilde,
                            USE.NAMES = FALSE)
  
  # Rejoin
  paste(formatted_terms, collapse = " + ")
}


#' Format multiple QCA solutions
#'
#' Formats a vector of solution expressions.
#'
#' @param solutions Character vector. Solution expressions from \code{minimize()}.
#' @param var_names Character vector. Variable names used in the analysis.
#' @param use_tilde Logical. If TRUE, negation is represented as \code{~VAR}.
#'
#' @return Character vector. Formatted solution expressions.
#' @export
#'
#' @examples
#' var_names <- c("KSP", "KPR", "PRD", "RVT", "RCM")
#' solutions <- c("KSPRVT + RCM", "~KPRPRD")
#' format_qca_solutions(solutions, var_names)
format_qca_solutions <- function(solutions, var_names, use_tilde = TRUE) {
  sapply(solutions, format_qca_solution,
         var_names = var_names,
         use_tilde = use_tilde,
         USE.NAMES = FALSE)
}


#' Extract and format terms from solutions
#'
#' Extracts individual terms from solution expressions and returns
#' formatted unique terms.
#'
#' @param solutions Character vector. Solution expressions.
#' @param var_names Character vector. Variable names used in the analysis.
#' @param use_tilde Logical. If TRUE, negation is represented as \code{~VAR}.
#'
#' @return List with:
#'   \itemize{
#'     \item \code{all_terms} — all terms (with duplicates)
#'     \item \code{unique_terms} — unique terms
#'     \item \code{n_total} — total term count
#'     \item \code{n_unique} — unique term count
#'   }
#' @export
#'
#' @examples
#' var_names <- c("X1", "X2", "X3")
#' solutions <- c("X1*X2 + X3", "X1*X2 + X1*X3")
#' extract_terms(solutions, var_names)
extract_terms <- function(solutions, var_names, use_tilde = TRUE) {
  
  if (is.null(solutions) || length(solutions) == 0) {
    return(list(
      all_terms    = character(0),
      unique_terms = character(0),
      n_total      = 0L,
      n_unique     = 0L
    ))
  }
  
  # Format solutions first
  formatted <- format_qca_solutions(solutions, var_names, use_tilde)
  
  # Split into terms
  all_terms <- unlist(lapply(formatted, function(sol) {
    if (sol == "" || sol == "No solution" || sol == "No essential prime implicants") {
      return(character(0))
    }
    trimws(unlist(strsplit(sol, " \\+ ")))
  }))
  
  list(
    all_terms    = all_terms,
    unique_terms = unique(all_terms),
    n_total      = length(all_terms),
    n_unique     = length(unique(all_terms))
  )
}

test_that("sanitize_truthtable coerces character incl/PRI to numeric", {
  # Mimic a QCA::truthTable() object whose incl/PRI came back as character
  # (observed with QCA 3.25 / admisc 0.40 on sparse truth tables), with
  # logical-remainder rows carrying the "-" string.
  fake_tt <- list(
    tt = data.frame(
      OUT  = c("1", "0", "?"),          # positive, negative, remainder
      n    = c(5, 3, 0),
      incl = c("0.900", "0.400", "-"),  # character, remainder = "-"
      PRI  = c("0.850", "0.300", "-"),
      stringsAsFactors = FALSE
    )
  )

  out <- sanitize_truthtable(fake_tt)

  expect_type(out$tt$incl, "double")
  expect_type(out$tt$PRI, "double")
  expect_false(anyNA(out$tt$incl))
  expect_false(anyNA(out$tt$PRI))

  # Observed rows keep their values; the remainder row ("-") becomes 0.
  expect_equal(out$tt$incl, c(0.900, 0.400, 0))
  expect_equal(out$tt$PRI, c(0.850, 0.300, 0))

  # OUT must be left untouched so minimize() can still identify remainders.
  expect_identical(out$tt$OUT, c("1", "0", "?"))
})

test_that("sanitize_truthtable is a no-op on already-numeric truth tables", {
  fake_tt <- list(
    tt = data.frame(
      OUT  = c("1", "0"),
      n    = c(5, 3),
      incl = c(0.9, 0.4),
      PRI  = c(0.85, 0.3),
      stringsAsFactors = FALSE
    )
  )
  out <- sanitize_truthtable(fake_tt)
  expect_equal(out$tt$incl, c(0.9, 0.4))
  expect_equal(out$tt$PRI, c(0.85, 0.3))
  expect_identical(out$tt$OUT, c("1", "0"))
})

test_that("sanitize_truthtable tolerates NULL and missing $tt", {
  expect_null(sanitize_truthtable(NULL))
  expect_identical(sanitize_truthtable(list(a = 1)), list(a = 1))
})

# End-to-end generate_report() regression test (v2.0.4).
# Fixture (.intermediate_construct_data) lives in helper-fixtures.R.
#
# This runs the ACTUAL generate_report() pipeline and inspects the emitted
# Markdown, rather than calling the internal extractors with hand-picked
# arguments. It exists because an earlier extractor-level test assumed
# generate_report() passed the intermediate IC to extract_all_metrics(), which
# was false for the "Solution Fit" and "Cross-Threshold Comparison" sections
# (they passed sol$IC, the parsimonious solution). Those sections therefore
# reported the parsimonious fit under the displayed intermediate formula. This
# test locks in the corrected, self-consistent report.

.read_report <- function(res, ...) {
  tmp <- tempfile(fileext = ".md")
  on.exit(unlink(tmp), add = TRUE)
  suppressWarnings(generate_report(res, output_file = tmp, format = "full",
                                   include_chart = FALSE, ...))
  readLines(tmp)
}

# Pull "0.911" style number out of a "| Label | 0.911 |" row that contains `key`.
.report_metric <- function(txt, key) {
  line <- grep(key, txt, value = TRUE)[1]
  if (is.na(line)) return(NA_real_)
  nums <- regmatches(line, gregexpr("[0-9]+\\.[0-9]+", line))[[1]]
  if (length(nums) == 0) return(NA_real_)
  as.numeric(nums[length(nums)])
}

test_that("generate_report Solution Fit reports the intermediate fit, not the parsimonious fit", {
  skip_if_not_installed("QCA")
  d <- .intermediate_construct_data()
  c3 <- c("A", "B", "C")
  emptyX <- stats::setNames(numeric(0), character(0))

  # Ground truth: the intermediate solution's own fit.
  dd <- d; dd$Y <- as.integer(d$Y >= 5)
  tt <- QCA::truthTable(dd, outcome = "Y", conditions = c3,
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "?", dir.exp = c(A = 1, B = 1, C = 1),
                       details = TRUE)
  isol_inclS <- sol$i.sol$C1P1$IC$sol.incl.cov$inclS[1]
  isol_covS  <- sol$i.sol$C1P1$IC$sol.incl.cov$covS[1]
  interm_expr <- paste(sol$i.sol$C1P1$solution[[1]], collapse = "*")  # "A*B*C"

  # Run the report under BOTH extract modes. The residual bug reports reproduced
  # with extract_mode = "all"; generate_report reads the stored minimize object
  # (mode-independent), so both must be correct.
  for (mode in c("all", "first")) {
    res <- otSweep(dat = d, outcome = "Y", conditions = c3, sweep_range = 5,
                   thrX = emptyX, pre_calibrated = c3,
                   dir.exp = c(A = 1, B = 1, C = 1), include = "?",
                   incl.cut = 0.80, n.cut = 1, extract_mode = mode,
                   return_details = TRUE)
    txt <- .read_report(res)

    # "Solution Fit" section: Consistency / Coverage must be the intermediate fit.
    rep_inclS <- .report_metric(txt, "Consistency \\(inclS\\)")
    rep_covS  <- .report_metric(txt, "Coverage \\(covS\\)")
    expect_equal(rep_inclS, isol_inclS, tolerance = 1e-3)
    expect_equal(rep_covS,  isol_covS,  tolerance = 1e-3)

    # The Per-Term Metrics row must describe the intermediate term (A*B*C), not a
    # parsimonious term (A*B / A*C / B*C).
    perterm_line <- grep("\\|.*\\\\?\\*.*\\|.*[0-9]\\.[0-9]", txt, value = TRUE)
    expect_true(any(grepl("A.*B.*C", perterm_line)))

    # Cross-Threshold Comparison row must also carry the intermediate fit.
    cross_line <- grep("thrY=5|thrY = 5", txt, value = TRUE)
    cross_line <- cross_line[grepl("[0-9]\\.[0-9]", cross_line)]
    expect_true(length(cross_line) >= 1)
    cross_nums <- as.numeric(regmatches(cross_line[1],
                               gregexpr("[0-9]+\\.[0-9]+", cross_line[1]))[[1]])
    expect_true(any(abs(cross_nums - isol_covS) < 1e-3))
    # And must NOT still be reporting the parsimonious aggregate.
    parsim_covS <- tryCatch(sol$IC$overall$sol.incl.cov$covS[1],
                            error = function(e) NA_real_)
    if (!is.na(parsim_covS) && abs(parsim_covS - isol_covS) > 1e-3) {
      expect_false(any(abs(cross_nums - parsim_covS) < 1e-4))
    }
  }
})

test_that("generate_report Summary Table and Solution Fit agree (self-consistency)", {
  skip_if_not_installed("QCA")
  d <- .intermediate_construct_data()
  c3 <- c("A", "B", "C")
  emptyX <- stats::setNames(numeric(0), character(0))
  res <- otSweep(dat = d, outcome = "Y", conditions = c3, sweep_range = 5,
                 thrX = emptyX, pre_calibrated = c3,
                 dir.exp = c(A = 1, B = 1, C = 1), include = "?",
                 incl.cut = 0.80, n.cut = 1, extract_mode = "all",
                 return_details = TRUE)
  txt <- .read_report(res)

  fit_covS <- .report_metric(txt, "Coverage \\(covS\\)")
  summ_line <- grep("A.*B.*C", grep("\\| 5 \\||thrY", txt, value = TRUE), value = TRUE)
  if (length(summ_line) == 0) summ_line <- grep("A\\\\?\\*B\\\\?\\*C", txt, value = TRUE)
  expect_true(length(summ_line) >= 1)
  summ_nums <- as.numeric(regmatches(summ_line[1],
                            gregexpr("[0-9]+\\.[0-9]+", summ_line[1]))[[1]])
  expect_true(any(abs(summ_nums - fit_covS) < 1e-3))
})

# Cross-extractor consistency stress test (v2.0.4).
#
# Scaled-down, fully deterministic version of the differential fuzzer used to
# find the fit-source bugs. The invariant, for every data set / solution-type /
# solution-count / calibration shape reached:
#
#   hand-computed fit of the DISPLAYED first solution
#     == qca_extract(sol, "first")
#     == extract_all_metrics(<matching IC>, sol)
#     == extract_solution_metrics_for_chart(sol, 1)
#
# The displayed solution is sol$i.sol$C1P1$solution[[1]] when dir.exp was given,
# else sol$solution[[1]]. Any regression that reattaches an overall/parsimonious
# fit to the displayed formula fails this test.

.stress_fit_of_terms <- function(terms, qd, y) {
  if (length(terms) == 0) return(c(NA_real_, NA_real_))
  ev <- function(t) {
    l <- strsplit(t, "\\*")[[1]]
    v <- sapply(l, function(p) {
      if (startsWith(p, "~")) 1 - qd[[substring(p, 2)]] else qd[[p]]
    })
    apply(as.matrix(v), 1, min)
  }
  X <- apply(as.matrix(sapply(terms, ev)), 1, max)
  c(sum(pmin(X, y)) / sum(X), sum(pmin(X, y)) / sum(y))
}

.stress_one <- function(qd, conds, include, dir_exp) {
  tt <- tryCatch(
    QCA::truthTable(qd, outcome = "Y", conditions = conds,
                    incl.cut = 0.80, n.cut = 1, complete = FALSE),
    error = function(e) NULL)
  if (is.null(tt)) return(NULL)
  sol <- tryCatch(
    if (is.null(dir_exp)) QCA::minimize(tt, include = include, details = TRUE)
    else QCA::minimize(tt, include = include, dir.exp = dir_exp, details = TRUE),
    error = function(e) NULL)
  if (is.null(sol)) return(NULL)

  is_inter <- !is.null(sol$i.sol) && length(sol$i.sol) > 0 &&
    !is.null(tryCatch(sol$i.sol$C1P1$solution, error = function(e) NULL))
  first_terms <- if (is_inter) {
    tryCatch(sol$i.sol$C1P1$solution[[1]], error = function(e) NULL)
  } else {
    tryCatch(sol$solution[[1]], error = function(e) NULL)
  }
  if (is.null(first_terms) || length(first_terms) == 0) return(NULL)

  oracle <- .stress_fit_of_terms(first_terms, qd, qd$Y)
  if (is.na(oracle[2])) return(NULL)

  ic_for_report <- if (is_inter) sol$i.sol$C1P1$IC else sol$IC
  list(
    oracle_covS = oracle[2],
    qe   = tryCatch(ThSQCA:::qca_extract(sol, "first")$covS, error = function(e) NA_real_),
    am   = tryCatch(ThSQCA:::extract_all_metrics(ic_for_report, sol)$sol_covS,
                    error = function(e) NA_real_),
    ch   = tryCatch(ThSQCA:::extract_solution_metrics_for_chart(sol, 1)$covS,
                    error = function(e) NA_real_)
  )
}

test_that("all three solution-level extractors match the displayed solution's fit", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D")
  dir_exp <- stats::setNames(rep(1, 4), conds)

  n_checked <- 0L
  for (crisp in c(TRUE, FALSE)) {
    for (seed in 1:4) {
      set.seed(seed * 13 + 4 * 97 + if (crisp) 0 else 4000)
      n <- 45
      if (crisp) {
        qd <- as.data.frame(lapply(conds, function(z) rbinom(n, 1, 0.5)))
        names(qd) <- conds
        qd$Y <- rbinom(n, 1, plogis(0.9 * qd$A + 0.8 * qd$B - 0.5))
      } else {
        qd <- as.data.frame(lapply(conds, function(z) round(plogis(rnorm(n, 0, 1.4)), 3)))
        names(qd) <- conds
        qd$Y <- round(plogis(
          qnorm(pmax(pmin(qd$A, .999), .001)) +
          0.9 * qnorm(pmax(pmin(qd$B, .999), .001)) + rnorm(n, 0, 1.2)), 3)
      }
      for (spec in list(list(inc = "", dx = NULL),
                        list(inc = "?", dx = NULL),
                        list(inc = "?", dx = dir_exp))) {
        r <- .stress_one(qd, conds, spec$inc, spec$dx)
        if (is.null(r)) next
        n_checked <- n_checked + 1L
        if (!is.na(r$qe)) expect_equal(r$qe, r$oracle_covS, tolerance = 2e-3)
        if (!is.na(r$am)) expect_equal(r$am, r$oracle_covS, tolerance = 2e-3)
        if (!is.na(r$ch)) expect_equal(r$ch, r$oracle_covS, tolerance = 2e-3)
      }
    }
  }
  # The loop must actually have exercised cells, otherwise the test is vacuous.
  expect_gt(n_checked, 5L)
})

###############################################
# Report generation for ThSQCA
###############################################

#' Generate Markdown Report for QCA Analysis
#'
#' Creates a markdown report from QCA analysis results.
#' Supports two formats: "full" (comprehensive) and "simple" (for manuscripts).
#'
#' @param result A result object from any Sweep function with
#'   \code{return_details = TRUE}.
#' @param output_file Character. Path to output markdown file.
#' @param format Character. Report format: \code{"full"} or \code{"simple"}.
#' @param title Character. Report title.
#' @param dat Optional data frame. Original data for descriptive statistics.
#' @param desc_vars Optional character vector. Variables for descriptive statistics.
#'   If NULL and dat is provided, uses Yvar and Xvars from params.
#' @param include_chart Logical. If TRUE (default), includes configuration charts
#'   (Fiss-style tables) in the report for each threshold.
#' @param chart_symbol_set Character. Symbol set for configuration charts:
#'   \code{"unicode"} (default), \code{"ascii"}, or \code{"latex"}.
#' @param chart_level Character. Chart aggregation level:
#'   \code{"term"} (default) produces solution-term level charts following Fiss (2011)
#'   notation, where each column represents one prime implicant (sufficient
#'   configuration). This format is recommended for academic publications.
#'   \code{"summary"} produces threshold-level summaries where each
#'   column represents one threshold, aggregating all configurations.
#' @param solution_note Logical. If TRUE (default), adds a note when multiple
#'   equivalent solutions exist explaining that M1 is shown.
#' @param solution_note_style Character. Style of solution note:
#'   \code{"simple"} (default) or \code{"detailed"} (includes EPIs).
#' @param solution_note_lang Character. Language for solution notes:
#'   \code{"en"} (default) or \code{"ja"}.
#' @param include_fiss_core Logical. If TRUE and \code{result$fiss_core} exists
#'   (i.e., \code{\link{compute_fiss_core}} has been run), the configuration
#'   charts use full Fiss (2011) four-symbol notation distinguishing core
#'   (present in both parsimonious and intermediate solutions) from peripheral
#'   conditions (intermediate only). If FALSE (default) or if \code{fiss_core}
#'   data is absent, the standard two-symbol chart is used.
#' @param include_raw_output Logical. If TRUE (default), includes the raw QCA
#'   package output (print(sol)) for each threshold for verification purposes.
#'
#' @return Invisibly returns the path to the generated report.
#' @export
#'
#' @examples
#' \dontrun{
#' data(sample_data)
#' thrX <- c(X1 = 7, X2 = 7, X3 = 7)
#' 
#' result <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = thrX,
#'   return_details = TRUE
#' )
#' 
#' # With descriptive statistics and configuration charts
#' generate_report(result, "my_report.md", format = "full", 
#'                 dat = sample_data, include_chart = TRUE)
#' 
#' # Without configuration charts
#' generate_report(result, "my_report.md", format = "simple",
#'                 include_chart = FALSE)
#' 
#' # With Fiss-style term-level charts (default, recommended for publications)
#' generate_report(result, "my_report.md", format = "full")
#' 
#' # With threshold-level summary charts
#' generate_report(result, "my_report.md", format = "full",
#'                 chart_level = "summary")
#' 
#' # With Fiss core/peripheral chart (requires compute_fiss_core)
#' res_fiss <- compute_fiss_core(result, conditions = c("X1", "X2", "X3"))
#' generate_report(res_fiss, "my_report.md", format = "full",
#'                 include_fiss_core = TRUE)
#' }
generate_report <- function(result,
                            output_file = "qca_report.md",
                            format = c("full", "simple"),
                            title = "QCA Analysis Report",
                            dat = NULL,
                            desc_vars = NULL,
                            include_chart = TRUE,
                            chart_symbol_set = c("unicode", "ascii", "latex"),
                            chart_level = c("term", "summary"),
                            solution_note = TRUE,
                            solution_note_style = c("simple", "detailed"),
                            solution_note_lang = c("en", "ja"),
                            include_fiss_core = FALSE,
                            include_raw_output = TRUE) {
  
  format <- match.arg(format)
  chart_symbol_set <- match.arg(chart_symbol_set)
  chart_level <- match.arg(chart_level)
  solution_note_style <- match.arg(solution_note_style)
  solution_note_lang <- match.arg(solution_note_lang)
  
  # Resolve fiss_core availability
  use_fiss <- isTRUE(include_fiss_core) && !is.null(result$fiss_core)
  
  # Validate input
  if (!is.list(result)) {
    stop("'result' must be a list object from a Sweep function.")
  }
  
  if (!"details" %in% names(result)) {
    stop("'result' must contain 'details'. Use return_details = TRUE in Sweep functions.")
  }
  
  # Open file connection
  con <- file(output_file, open = "w", encoding = "UTF-8")
  on.exit(close(con), add = TRUE)
  
  # Write header
  writeLines(paste0("# ", title, "\n"), con)
  writeLines("*(Auto-generated by ThSQCA package)*\n", con)
  writeLines(paste0("**Generated**: ", format(Sys.time(), "%Y-%m-%d %H:%M"), "\n"), con)
  writeLines("---\n", con)
  
  # Dispatch to appropriate format
  if (format == "full") {
    write_full_report(result, con, dat, desc_vars, include_chart, chart_symbol_set,
                      chart_level, solution_note, solution_note_style, solution_note_lang,
                      include_raw_output, use_fiss)
  } else {
    write_simple_report(result, con, include_chart, chart_symbol_set,
                        chart_level, solution_note, solution_note_style, solution_note_lang,
                        include_raw_output, use_fiss)
  }
  
  message("Report generated: ", output_file)
  invisible(output_file)
}


#' Write full report content
#' @keywords internal
write_full_report <- function(result, con, dat = NULL, desc_vars = NULL,
                              include_chart = TRUE, chart_symbol_set = "unicode",
                              chart_level = "term",
                              solution_note = TRUE, solution_note_style = "simple",
                              solution_note_lang = "en",
                              include_raw_output = TRUE,
                              use_fiss = FALSE) {
  
  summary_df <- result$summary
  details <- result$details
  params <- result$params
  
  # ============================================
  # 0. Analysis Overview
  # ============================================
  writeLines("## 0. Analysis Overview\n", con)
  writeLines("| Item | Value |", con)
  writeLines("|------|-------|", con)
  writeLines(paste0("| Analysis Date | ", format(Sys.time(), "%Y-%m-%d %H:%M"), " |"), con)
  
  if (!is.null(dat)) {
    writeLines(paste0("| Total N | ", nrow(dat), " |"), con)
  }
  
  if (!is.null(params)) {
    # Support both old (Yvar/Xvars) and new (outcome/conditions) parameter names
    outcome_var <- params$outcome
    if (is.null(outcome_var)) outcome_var <- params$Yvar
    conditions_var <- params$conditions
    if (is.null(conditions_var)) conditions_var <- params$Xvars
    
    if (!is.null(outcome_var)) {
      outcome_display <- outcome_var
      if (isTRUE(params$negate_outcome)) {
        outcome_display <- paste0(outcome_var, " (negated)")
      }
      writeLines(paste0("| Outcome Variable | ", outcome_display, " |"), con)
    }
    if (!is.null(conditions_var)) {
      writeLines(paste0("| Condition Variables | ", paste(conditions_var, collapse = ", "), " |"), con)
    }
    if (!is.null(params$pre_calibrated)) {
      pc_str <- paste(params$pre_calibrated, collapse = ", ")
      writeLines(paste0("| Pre-Calibrated Conditions | ", pc_str, " (passed through, no binarization) |"), con)
    }
    if (!is.null(params$thrX)) {
      thrX_str <- paste(names(params$thrX), params$thrX, sep = "=", collapse = ", ")
      writeLines(paste0("| X Thresholds | ", thrX_str, " |"), con)
    }
    if (!is.null(params$sweep_range)) {
      writeLines(paste0("| Y Sweep Range | ", min(params$sweep_range), "-", max(params$sweep_range), " |"), con)
    }
    if (!is.null(params$thrY)) {
      writeLines(paste0("| Y Threshold | ", params$thrY, " |"), con)
    }
    if (!is.null(params$incl.cut)) {
      writeLines(paste0("| Consistency Cutoff | ", params$incl.cut, " |"), con)
    }
    if (!is.null(params$n.cut)) {
      writeLines(paste0("| Frequency Cutoff (n.cut) | ", params$n.cut, " |"), con)
    }
    if (!is.null(params$include)) {
      writeLines(paste0("| Include | ", params$include, " |"), con)
    }
    if (!is.null(params$dir.exp)) {
      dir_str <- ifelse(all(params$dir.exp == 1), "positive (all)", 
                        paste(params$dir.exp, collapse = ", "))
      writeLines(paste0("| Directional Expectations | ", dir_str, " |"), con)
    }
    # Solution Type (determined from include and dir.exp)
    solution_type <- if (is.null(params$include) || params$include == "") {
      "Complex (Conservative)"
    } else if (!is.null(params$dir.exp)) {
      "Intermediate"
    } else {
      "Parsimonious"
    }
    writeLines(paste0("| **Solution Type** | **", solution_type, "** |"), con)
  }
  writeLines("\n---\n", con)
  
  # ============================================
  # 1. Descriptive Statistics (if dat provided)
  # ============================================
  if (!is.null(dat)) {
    writeLines("## 1. Descriptive Statistics\n", con)
    
    # Determine variables (support both old and new parameter names)
    if (is.null(desc_vars) && !is.null(params)) {
      outcome_var <- params$outcome
      if (is.null(outcome_var)) outcome_var <- params$Yvar
      conditions_var <- params$conditions
      if (is.null(conditions_var)) conditions_var <- params$Xvars
      
      # For negated outcome, use the cleaned variable name
      if (!is.null(outcome_var)) {
        outcome_clean <- sub("^~", "", outcome_var)
        desc_vars <- c(outcome_clean, conditions_var)
      } else {
        desc_vars <- conditions_var
      }
    }
    
    if (!is.null(desc_vars)) {
      desc_df <- data.frame(
        Variable = character(0),
        n = integer(0),
        Mean = numeric(0),
        SD = numeric(0),
        Min = numeric(0),
        Max = numeric(0),
        Skew = numeric(0),
        Kurtosis = numeric(0),
        stringsAsFactors = FALSE
      )
      
      for (var in desc_vars) {
        if (var %in% names(dat)) {
          x <- dat[[var]]
          x <- x[!is.na(x)]
          n <- length(x)
          m <- mean(x)
          s <- sd(x)
          
          # Skewness
          skew <- if (n > 2 && s > 0) {
            sum((x - m)^3) / (n * s^3)
          } else {
            NA
          }
          
          # Kurtosis (excess)
          kurt <- if (n > 3 && s > 0) {
            sum((x - m)^4) / (n * s^4) - 3
          } else {
            NA
          }
          
          desc_df <- rbind(desc_df, data.frame(
            Variable = var,
            n = n,
            Mean = round(m, 3),
            SD = round(s, 3),
            Min = round(min(x), 3),
            Max = round(max(x), 3),
            Skew = round(skew, 3),
            Kurtosis = round(kurt, 3),
            stringsAsFactors = FALSE
          ))
        }
      }
      
      if (nrow(desc_df) > 0) {
        writeLines(df_to_md_table(desc_df), con)
      }
    }
    writeLines("\n---\n", con)
  }
  
  # ============================================
  # 2. Summary Table
  # ============================================
  section_num <- if (!is.null(dat)) 2 else 1
  writeLines(paste0("## ", section_num, ". Summary Table\n"), con)
  writeLines(df_to_md_table(summary_df), con)
  writeLines("\n---\n", con)
  
  # ============================================
  # 3. Detailed Results per Threshold
  # ============================================
  section_num <- section_num + 1
  writeLines(paste0("## ", section_num, ". Detailed Results\n"), con)
  
  # Limit detailed output to avoid extremely long reports
  n_combinations <- length(details)
  MAX_DETAILS <- 27
  
  if (n_combinations > MAX_DETAILS) {
    writeLines(paste0("Due to the large number of threshold combinations (", 
                      n_combinations, "), detailed per-threshold results ",
                      "(necessity analysis, truth tables, per-term metrics) ",
                      "are omitted from this report.\n"), con)
    writeLines("", con)
    writeLines("To access details for specific threshold combinations, use:\n", con)
    writeLines("```r", con)
    writeLines("# List all threshold combinations", con)
    writeLines("names(result$details)", con)
    writeLines("", con)
    writeLines("# Access a specific combination (e.g., first one)", con)
    writeLines("key <- names(result$details)[1]", con)
    writeLines("det <- result$details[[key]]", con)
    writeLines("", con)
    writeLines("# Available components:", con)
    writeLines("det$truth_table$tt   # Truth table", con)
    writeLines("det$solution         # QCA solution object", con)
    writeLines("det$dat_bin          # Binarized data (for necessity analysis with QCA::pofind)", con)
    writeLines("det$thrX_vec         # X thresholds used", con)
    writeLines("det$thrY             # Y threshold used", con)
    writeLines("```\n", con)
  } else {
  
  for (key in names(details)) {
    det <- details[[key]]
    
    # Determine threshold label
    if (!is.null(det$thrY)) {
      writeLines(paste0("### thrY = ", det$thrY, "\n"), con)
    } else if (!is.null(det$threshold)) {
      writeLines(paste0("### Threshold = ", det$threshold, "\n"), con)
    } else if (!is.null(det$combo_id)) {
      writeLines(paste0("### Combination ", det$combo_id, "\n"), con)
    } else {
      writeLines(paste0("### ", key, "\n"), con)
    }
    
    # X thresholds
    if (!is.null(det$thrX_vec)) {
      thrX_str <- paste(names(det$thrX_vec), det$thrX_vec, sep = "=", collapse = ", ")
      writeLines(paste0("**X Thresholds**: ", thrX_str, "\n"), con)
    }
    
    # ---- Necessity Analysis ----
    dat_bin <- det$dat_bin
    if (!is.null(dat_bin) && !is.null(det$thrX_vec)) {
      Xvars <- names(det$thrX_vec)
      nec <- try(QCA::pofind(dat_bin, outcome = "Y", conditions = Xvars), silent = TRUE)
      if (!inherits(nec, "try-error") && !is.null(nec$incl.cov)) {
        writeLines("#### Necessity Analysis\n", con)
        nec_df <- nec$incl.cov
        nec_df <- cbind(Condition = rownames(nec_df), nec_df)
        rownames(nec_df) <- NULL
        writeLines(df_to_md_table(nec_df), con)
        writeLines("\n", con)
      }
    }
    
    # ---- Truth Table ----
    tt <- det$truth_table
    if (!is.null(tt) && !is.null(tt$tt)) {
      writeLines("#### Truth Table (observed configurations)\n", con)
      tt_df <- tt$tt
      tt_observed <- tt_df[tt_df$n > 0, , drop = FALSE]
      if (nrow(tt_observed) > 0) {
        tt_cols <- intersect(c(names(det$thrX_vec), "OUT", "n", "incl", "PRI"), names(tt_observed))
        tt_subset <- tt_observed[, tt_cols, drop = FALSE]
        tt_subset <- cbind(Row = rownames(tt_subset), tt_subset)
        rownames(tt_subset) <- NULL
        writeLines(df_to_md_table(tt_subset), con)
      } else {
        writeLines("*(No observed configurations)*", con)
      }
      writeLines("\n", con)
    }
    
    # ---- Solution ----
    sol <- det$solution
    if (is.null(sol)) {
      writeLines("#### Solution\n", con)
      writeLines("**No solution**\n", con)
    } else {
      n_sol <- get_n_solutions(sol)
      writeLines("#### Solution\n", con)
      writeLines(paste0("**Number of Solutions**: ", n_sol, "\n"), con)
      
      # Get solution list (i.sol first for true Intermediate when dir.exp
      # specified). Deduplicated across prime implicant charts; see
      # collect_unique_i_sol() for why naive concatenation double-counts.
      sol_list <- collect_unique_i_sol(sol)

      if (!is.null(sol_list) && length(sol_list) > 0) {
        writeLines("**Full Solutions**:\n", con)
        for (i in seq_along(sol_list)) {
          expr <- paste(sol_list[[i]], collapse = " + ")
          writeLines(paste0("- M", i, ": ", escape_md(expr), " -> Y\n"), con)
        }
        writeLines("\n", con)
        
        # Essential/Selective Prime Implicants (if multiple solutions)
        if (length(sol_list) > 1) {
          sol_terms <- lapply(sol_list, function(x) {
            if (is.character(x)) x else unlist(strsplit(paste(x, collapse = " + "), " \\+ "))
          })
          essential_terms <- Reduce(intersect, sol_terms)
          all_terms <- Reduce(union, sol_terms)
          selective_terms <- setdiff(all_terms, essential_terms)
          
          if (length(essential_terms) > 0) {
            writeLines(paste0("**Essential Prime Implicants**: ", 
                              escape_md(paste(essential_terms, collapse = " + ")), "\n"), con)
          } else {
            writeLines("**Essential Prime Implicants**: (none - solutions are disjoint)\n", con)
          }
          
          if (length(selective_terms) > 0) {
            writeLines(paste0("**Selective Prime Implicants**: ", 
                              escape_md(paste(selective_terms, collapse = " + ")), "\n"), con)
          }
          
          # Unique Terms
          unique_terms_list <- lapply(seq_along(sol_terms), function(i) {
            other_terms <- unique(unlist(sol_terms[-i]))
            setdiff(sol_terms[[i]], other_terms)
          })
          unique_terms_formatted <- sapply(seq_along(unique_terms_list), function(i) {
            if (length(unique_terms_list[[i]]) > 0) {
              paste0("M", i, ": ", escape_md(paste(unique_terms_list[[i]], collapse = " + ")))
            } else {
              NULL
            }
          })
          unique_terms_filtered <- unique_terms_formatted[!sapply(unique_terms_formatted, is.null)]
          if (length(unique_terms_filtered) > 0) {
            writeLines(paste0("**Unique Terms**: ", 
                              paste(unique_terms_filtered, collapse = "; "), "\n"), con)
          }
          writeLines("\n", con)
        }
      }
      
      # ---- Solution Fit ----
      writeLines("#### Solution Fit\n", con)
      # Report the fit of the DISPLAYED solution. When dir.exp yields an
      # intermediate solution, its fit lives in sol$i.sol$C1P1$IC; sol$IC
      # describes the parsimonious solution and must not be used here. This
      # mirrors the intermediate-aware call further below.
      ic_for_metrics <- if (!is.null(sol$i.sol) && length(sol$i.sol) > 0 &&
                            !is.null(sol$i.sol$C1P1$IC)) {
        sol$i.sol$C1P1$IC
      } else {
        sol$IC
      }
      metrics <- extract_all_metrics(ic_for_metrics, sol)
      writeLines("| Metric | Value |", con)
      writeLines("|--------|-------|", con)
      writeLines(paste0("| Consistency (inclS) | ", 
                        ifelse(is.na(metrics$sol_inclS), "N/A", round(metrics$sol_inclS, 3)), " |"), con)
      writeLines(paste0("| PRI | ", 
                        ifelse(is.na(metrics$sol_PRI), "N/A", round(metrics$sol_PRI, 3)), " |"), con)
      writeLines(paste0("| Coverage (covS) | ", 
                        ifelse(is.na(metrics$sol_covS), "N/A", round(metrics$sol_covS, 3)), " |"), con)
      writeLines("\n", con)
      
      # ---- Per-Term Metrics ----
      if (!is.null(metrics$term_df)) {
        writeLines("#### Per-Term Metrics\n", con)
        term_df <- metrics$term_df
        if ("cases" %in% names(term_df)) {
          term_df <- term_df[, !names(term_df) %in% "cases", drop = FALSE]
        }
        term_df <- cbind(Term = rownames(term_df), term_df)
        rownames(term_df) <- NULL
        writeLines(df_to_md_table(term_df), con)
        writeLines("\n", con)
      }
      
      # ---- Configuration Chart ----
      if (include_chart && !is.null(sol_list) && length(sol_list) > 0) {
        writeLines("#### Configuration Chart\n", con)
        
        # Fiss 4-symbol chart (per-threshold)
        thr_key <- as.character(key)
        if (use_fiss && !is.null(result$fiss_core[[thr_key]])) {
          fc <- result$fiss_core[[thr_key]]
          if (!is.null(fc$classification)) {
            writeLines(paste0(
              "*Fiss (2011) notation: core conditions (appear in both parsimonious ",
              "and intermediate solutions) are shown with large symbols; ",
              "peripheral conditions (intermediate only) with small symbols.*\n"
            ), con)
            writeLines(paste0(
              "- Parsimonious: `", fc$parsim_expression, "`\n"
            ), con)
            writeLines(paste0(
              "- Intermediate: `", fc$interm_expression, "`\n\n"
            ), con)
            
            conditions <- result$params$conditions
            interm_terms <- unique(fc$classification$term_expr)
            symbols_fiss <- SYMBOL_SETS_FISS[[chart_symbol_set]]
            thr_label <- paste0("thrY=", thr_key)
            
            mat <- build_fiss_matrix(
              interm_terms   = interm_terms,
              classification = fc$classification,
              conditions     = conditions,
              symbols        = symbols_fiss,
              thr_label      = thr_label
            )
            table_str <- config_matrix_to_md(mat, "Condition")
            legend <- if (solution_note_lang == "ja") symbols_fiss$note_ja else symbols_fiss$note_en
            writeLines(paste0(table_str, "\n\n*", legend, "*\n"), con)
          } else {
            writeLines("*No solution at this threshold.*\n", con)
          }
        } else {
          # Standard 2-symbol chart
          paths <- sol_list[[1]]
          if (!is.character(paths)) paths <- as.character(paths)
          
          if (length(paths) > 0) {
            epi_list <- NULL
            if (solution_note_style == "detailed" && length(sol_list) > 1) {
              epi_info <- identify_epi(sol_list)
              epi_list <- epi_info$epi
            }
            chart <- config_chart_from_paths(
              paths,
              symbol_set = chart_symbol_set,
              language = solution_note_lang,
              n_sol = length(sol_list),
              solution_note = solution_note,
              solution_note_style = solution_note_style,
              epi_list = epi_list
            )
            writeLines(chart, con)
            writeLines("\n", con)
          }
        }
      }
      
      # ---- QCA Package Output (for verification) ----
      if (include_raw_output) {
        writeLines("#### QCA Package Output (for verification)\n", con)
        writeLines("```", con)
        raw_output <- capture.output(print(sol))
        writeLines(raw_output, con)
        writeLines("```\n", con)
      }
    }
    
    # ---- Settings for Reproducibility ----
    writeLines("#### Settings (for reproducibility)\n", con)
    writeLines("```", con)
    if (!is.null(det$thrX_vec)) {
      writeLines(paste0("thrX: ", paste(det$thrX_vec, collapse = ", ")), con)
    }
    if (!is.null(det$thrY)) {
      writeLines(paste0("thrY: ", det$thrY), con)
    }
    if (!is.null(params$incl.cut)) {
      writeLines(paste0("incl.cut: ", params$incl.cut), con)
    }
    if (!is.null(params$dir.exp)) {
      writeLines(paste0("dir.exp: ", paste(params$dir.exp, collapse = ", ")), con)
    }
    writeLines("```\n", con)
    
    writeLines("---\n", con)
  }
  
  }  # End of if (n_combinations <= MAX_DETAILS)
  
  # ============================================
  # 4. Cross-Threshold Comparison
  # ============================================
  section_num <- section_num + 1
  writeLines(paste0("## ", section_num, ". Cross-Threshold Comparison\n"), con)
  
  if (n_combinations > MAX_DETAILS) {
    writeLines("Cross-threshold comparison table is designed for single-dimension sweeps ", con)
    writeLines("(otSweep, ctSweepS) with a smaller number of thresholds.\n", con)
    writeLines("", con)
    writeLines("For multi-dimensional sweeps with many combinations, ", con)
    writeLines("please refer to the Summary Table above to compare results across threshold settings.\n", con)
  } else {
  
  # Build comparison table
  comp_df <- data.frame(
    Threshold = character(0),
    inclS = numeric(0),
    PRI = numeric(0),
    covS = numeric(0),
    n_solutions = integer(0),
    N_Essential = integer(0),
    stringsAsFactors = FALSE
  )
  
  for (key in names(details)) {
    det <- details[[key]]
    sol <- det$solution
    
    # Threshold label
    thr_label <- if (!is.null(det$thrY)) {
      paste0("thrY=", det$thrY)
    } else if (!is.null(det$threshold)) {
      as.character(det$threshold)
    } else {
      key
    }
    
    if (is.null(sol)) {
      comp_df <- rbind(comp_df, data.frame(
        Threshold = thr_label,
        inclS = NA,
        PRI = NA,
        covS = NA,
        n_solutions = 0,
        N_Essential = 0,
        stringsAsFactors = FALSE
      ))
    } else {
      # Report the fit of the DISPLAYED solution (intermediate when dir.exp is
      # used); sol$IC is the parsimonious solution and would misreport it.
      ic_for_metrics <- if (!is.null(sol$i.sol) && length(sol$i.sol) > 0 &&
                            !is.null(sol$i.sol$C1P1$IC)) {
        sol$i.sol$C1P1$IC
      } else {
        sol$IC
      }
      metrics <- extract_all_metrics(ic_for_metrics, sol)
      n_sol <- get_n_solutions(sol)
      
      # Count essential prime implicants (i.sol first for true Intermediate).
      # Deduplicated across prime implicant charts; see collect_unique_i_sol().
      n_essential <- 0
      sol_list <- collect_unique_i_sol(sol)
      if (!is.null(sol_list) && length(sol_list) > 1) {
        sol_terms <- lapply(sol_list, function(x) {
          if (is.character(x)) x else unlist(strsplit(paste(x, collapse = " + "), " \\+ "))
        })
        essential_terms <- Reduce(intersect, sol_terms)
        n_essential <- length(essential_terms)
      } else if (!is.null(sol_list) && length(sol_list) == 1) {
        n_essential <- length(sol_list[[1]])
      }
      
      comp_df <- rbind(comp_df, data.frame(
        Threshold = thr_label,
        inclS = round(metrics$sol_inclS, 3),
        PRI = round(metrics$sol_PRI, 3),
        covS = round(metrics$sol_covS, 3),
        n_solutions = n_sol,
        N_Essential = n_essential,
        stringsAsFactors = FALSE
      ))
    }
  }
  
  writeLines(df_to_md_table(comp_df), con)
  
  }  # End of if (n_combinations <= MAX_DETAILS) for Section 4
  
  writeLines("\n---\n", con)
  
  # ============================================
  # 5. Cross-Threshold Configuration Chart
  # ============================================
  if (include_chart && n_combinations <= MAX_DETAILS) {
    section_num <- section_num + 1
    writeLines(paste0("## ", section_num, ". Cross-Threshold Configuration Chart\n"), con)
    
    # Get conditions from params
    conditions <- params$conditions
    if (is.null(conditions)) conditions <- params$Xvars
    
    if (!is.null(conditions) && length(conditions) > 0) {
      
      if (use_fiss) {
        # --- Fiss 4-symbol cross-threshold chart ---
        writeLines(paste0(
          "*Fiss (2011) four-symbol notation. ",
          "Large symbols (\u25CF/\u2297) = core conditions (parsimonious + intermediate). ",
          "Small symbols (\u2299/\u2298) = peripheral conditions (intermediate only). ",
          "Blank = don't care.*\n\n"
        ), con)
        chart <- generate_fiss_chart(
          result,
          conditions = conditions,
          symbol_set = chart_symbol_set,
          language   = solution_note_lang
        )
        writeLines(chart, con)
      } else {
        # --- Standard 2-symbol cross-threshold chart ---
        if (chart_level == "term") {
          writeLines("*Configuration chart at solution-term level (Fiss, 2011 notation).*\n", con)
          writeLines("*Each column represents one prime implicant (configuration).*\n\n", con)
        } else {
          writeLines("*Configuration chart at threshold-level summary.*\n", con)
          writeLines("*Each column aggregates all conditions that appear in any configuration at that threshold.*\n\n", con)
        }
        
        symbols <- SYMBOL_SETS[[chart_symbol_set]]
        chart <- if (chart_level == "term") {
          generate_term_level_chart(summary_df, conditions, symbols, solution_note_lang)
        } else {
          generate_threshold_level_chart(summary_df, conditions, symbols, solution_note_lang)
        }
        writeLines(chart, con)
      }
    } else {
      writeLines("*(Could not generate configuration chart - conditions not found)*\n", con)
    }
    
    writeLines("\n---\n", con)
  }
  
  # ============================================
  # 6. Notes
  # ============================================
  section_num <- section_num + 1
  writeLines(paste0("## ", section_num, ". Notes\n"), con)
  writeLines("- **Essential Prime Implicants (EPI)**: Terms that appear in ALL equivalent solutions (M1, M2, M3...).", con)
  writeLines("- **Selective Prime Implicants (SPI)**: Terms that appear in some but not all solutions.", con)
  writeLines("- **Unique Terms**: Terms that appear only in one specific solution.", con)
  writeLines("- **inclS**: Solution consistency (sufficiency).", con)
  writeLines("- **covS**: Solution coverage.", con)
  writeLines("- **PRI**: Proportional Reduction in Inconsistency.", con)
  writeLines("- **covU**: Unique coverage (coverage by this term alone).", con)
  writeLines("- **inclN**: Necessity consistency (>= 0.9 typically indicates necessary condition).", con)
  writeLines("- **RoN**: Relevance of Necessity.", con)
  writeLines("- **covN**: Necessity coverage.", con)
  if (use_fiss) {
    writeLines("", con)
    writeLines("**Fiss (2011) Core/Peripheral Classification:**", con)
    writeLines("- **Core condition** (\u25CF/\u2297): Appears in BOTH the parsimonious and intermediate solutions.", con)
    writeLines("- **Peripheral condition** (\u2299/\u2298): Appears in the intermediate solution ONLY.", con)
    writeLines("- Reference: Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach to typologies in organization research. *Academy of Management Journal*, 54(2), 393-420.", con)
  }
  
  writeLines("\n---\n", con)
  
  # ============================================
  # 7. Verification Recommendation
  # ============================================
  section_num <- section_num + 1
  writeLines(paste0("## ", section_num, ". Verification Recommendation\n"), con)
  writeLines("**For academic publications**, always verify ThSQCA results directly with the QCA package:\n", con)
  writeLines("```r", con)
  writeLines("library(QCA)", con)
  writeLines("tt <- truthTable(dat, outcome = \"Y\", conditions = c(...), incl.cut = 0.8)", con)
  writeLines("sol <- minimize(tt, include = \"?\", dir.exp = c(1, 1, ...))", con)
  writeLines("print(sol)  # Compare with ThSQCA output above", con)
  writeLines("```\n", con)
  writeLines("Ensure that solution expressions, consistency, and coverage values match before publishing.", con)
  writeLines("\n", con)
  writeLines("*Report generated by ThSQCA package (https://github.com/im-research-yt/ThSQCA)*", con)
}


#' Write simple report content
#' @keywords internal
write_simple_report <- function(result, con, include_chart = TRUE, 
                                chart_symbol_set = "unicode",
                                chart_level = "term",
                                solution_note = TRUE, solution_note_style = "simple",
                                solution_note_lang = "en",
                                include_raw_output = TRUE,
                                use_fiss = FALSE) {
  
  summary_df <- result$summary
  details <- result$details
  
  # 1. Summary Table
  writeLines("## Summary\n", con)
  writeLines(df_to_md_table(summary_df), con)
  writeLines("\n", con)
  writeLines("---\n", con)
  
  # 2. Solutions Overview
  writeLines("## Solutions Overview\n", con)
  
  for (key in names(details)) {
    det <- details[[key]]
    sol <- det$solution
    
    if (is.null(sol)) next
    
    # Determine threshold label
    if (!is.null(det$thrY)) {
      label <- paste0("Y >= ", det$thrY)
    } else if (!is.null(det$threshold)) {
      label <- paste0("threshold = ", det$threshold)
    } else {
      label <- key
    }
    
    n_sol <- get_n_solutions(sol)
    
    # Get solution list (i.sol first for true Intermediate when dir.exp
    # specified). Deduplicated across prime implicant charts; see
    # collect_unique_i_sol() for why naive concatenation double-counts.
    sol_list <- collect_unique_i_sol(sol)
    
    if (!is.null(sol_list) && length(sol_list) > 0) {
      writeLines(paste0("### ", label, "\n"), con)
      
      # Show solution formula
      if (length(sol_list) == 1) {
        expr <- paste(sol_list[[1]], collapse = " + ")
        writeLines(paste0("**Solution**: ", escape_md(expr), " -> Y\n"), con)
      } else {
        writeLines(paste0("**Number of Solutions**: ", length(sol_list), "\n"), con)
        
        # Essential prime implicants
        sol_terms <- lapply(sol_list, function(x) {
          if (is.character(x)) x else unlist(strsplit(paste(x, collapse = " + "), " \\+ "))
        })
        essential_terms <- Reduce(intersect, sol_terms)
        
        if (length(essential_terms) > 0) {
          writeLines(paste0("**Essential (EPI)**: ", escape_md(paste(essential_terms, collapse = " + ")), "\n"), con)
        }
        
        # List all solutions briefly
        for (i in seq_along(sol_list)) {
          expr <- paste(sol_list[[i]], collapse = " + ")
          writeLines(paste0("- M", i, ": ", escape_md(expr), "\n"), con)
        }
      }
      
      # Metrics (brief)
      metrics <- extract_all_metrics(sol$i.sol$C1P1$IC, sol)
      writeLines(paste0("*inclS = ", 
                        ifelse(is.na(metrics$sol_inclS), "N/A", round(metrics$sol_inclS, 3)),
                        ", covS = ",
                        ifelse(is.na(metrics$sol_covS), "N/A", round(metrics$sol_covS, 3)),
                        "*\n"), con)
      
      # ---- Configuration Chart ----
      if (include_chart) {
        writeLines("\n**Configuration Chart:**\n", con)
        
        thr_key <- as.character(key)
        if (use_fiss && !is.null(result$fiss_core[[thr_key]])) {
          fc <- result$fiss_core[[thr_key]]
          if (!is.null(fc$classification)) {
            writeLines(paste0(
              "*Fiss (2011) notation. ",
              "Parsimonious: `", fc$parsim_expression, "` / ",
              "Intermediate: `", fc$interm_expression, "`*\n\n"
            ), con)
            conditions <- result$params$conditions
            interm_terms <- unique(fc$classification$term_expr)
            symbols_fiss <- SYMBOL_SETS_FISS[[chart_symbol_set]]
            thr_label <- paste0("thrY=", thr_key)
            mat <- build_fiss_matrix(
              interm_terms   = interm_terms,
              classification = fc$classification,
              conditions     = conditions,
              symbols        = symbols_fiss,
              thr_label      = thr_label
            )
            table_str <- config_matrix_to_md(mat, "Condition")
            legend <- if (solution_note_lang == "ja") symbols_fiss$note_ja else symbols_fiss$note_en
            writeLines(paste0(table_str, "\n\n*", legend, "*\n"), con)
          }
        } else {
          paths <- sol_list[[1]]
          if (!is.character(paths)) paths <- as.character(paths)
          if (length(paths) > 0) {
            epi_list <- NULL
            if (solution_note_style == "detailed" && length(sol_list) > 1) {
              epi_info <- identify_epi(sol_list)
              epi_list <- epi_info$epi
            }
            chart <- config_chart_from_paths(
              paths,
              symbol_set = chart_symbol_set,
              language = solution_note_lang,
              n_sol = length(sol_list),
              solution_note = solution_note,
              solution_note_style = solution_note_style,
              epi_list = epi_list
            )
            writeLines(chart, con)
          }
        }
      }
      
      # ---- QCA Package Output (for verification) ----
      if (include_raw_output) {
        writeLines("\n**QCA Package Output (for verification):**\n", con)
        writeLines("```", con)
        raw_output <- capture.output(print(sol))
        writeLines(raw_output, con)
        writeLines("```\n", con)
      }
      
      writeLines("\n", con)
    }
  }
  
  # ============================================
  # Verification Recommendation
  # ============================================
  writeLines("---\n", con)
  writeLines("## Verification Recommendation\n", con)
  writeLines("**For academic publications**, always verify ThSQCA results directly with the QCA package:\n", con)
  writeLines("```r", con)
  writeLines("library(QCA)", con)
  writeLines("tt <- truthTable(dat, outcome = \"Y\", conditions = c(...), incl.cut = 0.8)", con)
  writeLines("sol <- minimize(tt, include = \"?\", dir.exp = c(1, 1, ...))", con)
  writeLines("print(sol)  # Compare with ThSQCA output above", con)
  writeLines("```\n", con)
  writeLines("Ensure that solution expressions, consistency, and coverage values match before publishing.", con)
  writeLines("\n", con)
  writeLines("*Report generated by ThSQCA package (https://github.com/im-research-yt/ThSQCA)*", con)
}

###############################################
# Configuration Chart Generator for ThSQCA
###############################################

#' Parse solution expression into individual terms
#'
#' Splits a solution expression (ORed terms) into individual prime implicants.
#'
#' @param expr Character. Solution expression (e.g., "X3 + X1*X2").
#'
#' @return Character vector of terms (e.g., c("X3", "X1*X2")), or NULL if
#'   no valid expression.
#'
#' @keywords internal
parse_solution_terms <- function(expr) {
  if (is.na(expr) || expr == "No solution" || expr == "" || is.null(expr)) {
    return(NULL)
  }
  # Split by + (OR operator), allowing spaces
  terms <- strsplit(expr, "\\s*\\+\\s*")[[1]]
  # Trim whitespace
  terms <- trimws(terms)
  # Remove empty strings
  terms <- terms[terms != ""]
  if (length(terms) == 0) return(NULL)
  return(terms)
}


#' Determine condition status in a term
#'
#' Checks whether a condition is present, absent (negated), or don't care
#' in a given term.
#'
#' @param term Character. Single term (e.g., "X1*X2", "~X3").
#' @param condition Character. Condition name (e.g., "X1").
#'
#' @return Character. One of "present", "absent", or "dontcare".
#'
#' @details Uses word boundary matching to avoid false positives when
#'   condition names are substrings of each other (e.g., X1 vs X10).
#'
#' @keywords internal
get_condition_status <- function(term, condition) {
  # Remove spaces from term for consistent matching
  term <- gsub("\\s+", "", term)
  
  # Escape special regex characters in condition name
  cond_escaped <- gsub("([\\[\\]\\(\\)\\{\\}\\^\\$\\.\\|\\?\\+\\\\])", 
                       "\\\\\\1", condition)
  
  # Negated condition check (~X1 or similar)
  negated_pattern <- paste0("~", cond_escaped, "(?![A-Za-z0-9_]|$)")
  # Also check for ~X1 at end of string
  negated_pattern_end <- paste0("~", cond_escaped, "$")
  if (grepl(negated_pattern, term, perl = TRUE) || 
      grepl(negated_pattern_end, term, perl = TRUE)) {
    return("absent")
  }
  
  # Positive condition check (X1 but not ~X1)
  # Pattern: X1 not preceded by ~ or alphanumeric, not followed by alphanumeric
  positive_pattern <- paste0("(?<![~A-Za-z0-9_])", cond_escaped, "(?![A-Za-z0-9_])")
  # Also check for condition at start of string
  positive_pattern_start <- paste0("^", cond_escaped, "(?![A-Za-z0-9_]|$)")
  positive_pattern_end <- paste0("(?<![~A-Za-z0-9_])", cond_escaped, "$")
  
  if (grepl(positive_pattern, term, perl = TRUE) ||
      grepl(positive_pattern_start, term, perl = TRUE) ||
      grepl(positive_pattern_end, term, perl = TRUE)) {
    return("present")
  }
  
  return("dontcare")
}


#' Symbol sets for configuration charts
#' @keywords internal
SYMBOL_SETS <- list(
  unicode = list(
    present = "\u25CF",     # ● (BLACK CIRCLE)
    absent  = "\u2297",     # ⊗ (CIRCLED TIMES)
    note_en = "\u25CF = presence, \u2297 = absence, blank = don't care",
    note_ja = "\u25CF = \u5b58\u5728, \u2297 = \u4e0d\u5728, \u7a7a\u6b04 = \u7121\u95a2\u4fc2"
  ),

  ascii = list(
    present = "O",
    absent  = "X",
    note_en = "O = presence, X = absence, blank = don't care",
    note_ja = "O = \u5b58\u5728, X = \u4e0d\u5728, \u7a7a\u6b04 = \u7121\u95a2\u4fc2"
  ),
  latex = list(
    present = "$\\bullet$",
    absent  = "$\\otimes$",
    note_en = "$\\bullet$ = presence, $\\otimes$ = absence, blank = don't care",
    note_ja = "$\\bullet$ = \u5b58\u5728, $\\otimes$ = \u4e0d\u5728, \u7a7a\u6b04 = \u7121\u95a2\u4fc2"
  )
)


#' Generate solution note for multiple solutions
#'
#' Creates a note explaining that multiple equivalent solutions exist
#' and that the displayed configuration is based on M1.
#'
#' @param n_sol Integer. Number of solutions.
#' @param epi_list Character vector. Essential prime implicants (NULL to omit).
#' @param style Character. \code{"simple"} or \code{"detailed"}.
#' @param language Character. \code{"en"} or \code{"ja"}.
#' @param format Character. \code{"markdown"} or \code{"latex"}.
#'
#' @return Character string of the note, or empty string if n_sol <= 1.
#'
#' @examples
#' # Simple note
#' generate_solution_note(2, style = "simple")
#' 
#' # Detailed note with EPIs
#' generate_solution_note(3, epi_list = c("A*B", "C"), style = "detailed")
#' 
#' # Japanese
#' generate_solution_note(2, style = "simple", language = "ja")
#'
#' @export
generate_solution_note <- function(n_sol,
                                    epi_list = NULL,
                                    style = c("simple", "detailed"),
                                    language = c("en", "ja"),
                                    format = c("markdown", "latex")) {
  
  # Return empty string for single solution
  if (is.null(n_sol) || n_sol <= 1) {
    return("")
  }

  
  style <- match.arg(style)
  language <- match.arg(language)
  format <- match.arg(format)
  
  labels <- get_config_labels(language)
  
  # Build note text
  if (language == "ja") {
    # Japanese version
    n_sol_text <- gsub("\\{n\\}", as.character(n_sol), labels$n_equiv_solutions)
    base_note <- paste0(n_sol_text, labels$table_based_on_m1)
    
    if (style == "detailed" && !is.null(epi_list) && length(epi_list) > 0) {
      # Format EPIs with appropriate connector
      if (format == "latex") {
        epi_str <- paste(epi_list, collapse = ", ")
        epi_str <- gsub("\\*", "$\\\\cdot$", epi_str)
      } else {
        epi_str <- paste(epi_list, collapse = ", ")
        epi_str <- gsub("\\*", "\u00B7", epi_str)  # middle dot
      }
      
      if (length(epi_list) == 1) {
        epi_text <- gsub("\\{epi\\}", epi_str, labels$all_share_epi_single)
      } else {
        epi_text <- gsub("\\{epi\\}", epi_str, labels$all_share_epi)
      }
      base_note <- paste0(base_note, epi_text)
    } else if (style == "detailed" && (is.null(epi_list) || length(epi_list) == 0)) {
      base_note <- paste0(base_note, labels$no_epi)
    }
    
  } else {
    # English version
    if (style == "detailed") {
      n_sol_text <- gsub("\\{n\\}", as.character(n_sol), labels$n_equiv_solutions_range)
    } else {
      n_sol_text <- gsub("\\{n\\}", as.character(n_sol), labels$n_equiv_solutions)
    }
    base_note <- paste0(n_sol_text, " ", labels$table_based_on_m1)
    
    if (style == "detailed" && !is.null(epi_list) && length(epi_list) > 0) {
      # Format EPIs with appropriate connector
      if (format == "latex") {
        epi_str <- paste(epi_list, collapse = " and ")
        epi_str <- gsub("\\*", "$\\\\cdot$", epi_str)
      } else {
        epi_str <- paste(epi_list, collapse = " and ")
        epi_str <- gsub("\\*", "\u00B7", epi_str)  # middle dot
      }
      
      if (length(epi_list) == 1) {
        epi_text <- gsub("\\{epi\\}", epi_str, labels$all_share_epi_single)
      } else {
        epi_text <- gsub("\\{epi\\}", epi_str, labels$all_share_epi)
      }
      base_note <- paste0(base_note, " ", epi_text)
    } else if (style == "detailed" && (is.null(epi_list) || length(epi_list) == 0)) {
      base_note <- paste0(base_note, " ", labels$no_epi)
    }
  }
  
  # Format output
  if (format == "latex") {
    note <- paste0("\\footnotesize{\\textit{", labels$note_prefix, "}: ", base_note, "}")
  } else {
    note <- paste0("*", labels$note_prefix, ": ", base_note, "*")
  }
  
  return(note)
}


#' Identify Essential Prime Implicants from multiple solutions
#'
#' Finds terms that appear in ALL solutions (EPIs) versus terms that
#' appear in only some solutions (SPIs).
#'
#' @param solutions List of solution vectors. Each element is a character
#'   vector of terms for one solution.
#'
#' @return List with:
#'   \itemize{
#'     \item \code{epi} — Essential prime implicants (in all solutions)
#'     \item \code{spi} — Selective prime implicants (in some solutions)
#'     \item \code{n_solutions} — Number of solutions
#'   }
#'
#' @examples
#' solutions <- list(
#'   c("A*B", "C", "D"),
#'   c("A*B", "C", "E"),
#'   c("A*B", "C", "F")
#' )
#' result <- identify_epi(solutions)
#' # result$epi = c("A*B", "C")
#' # result$spi = c("D", "E", "F")
#'
#' @export
identify_epi <- function(solutions) {
  
  if (is.null(solutions) || length(solutions) == 0) {
    return(list(epi = character(0), spi = character(0), n_solutions = 0L))
  }
  
  n_solutions <- length(solutions)
  
  if (n_solutions == 1) {
    # Single solution: all terms are "essential" by definition
    return(list(
      epi = solutions[[1]],
      spi = character(0),
      n_solutions = 1L
    ))
  }
  
  # Find intersection of all solutions (EPIs)
  epi <- Reduce(intersect, solutions)
  
  # Find union of all solutions
  all_terms <- Reduce(union, solutions)
  
  # SPIs are terms not in all solutions
spi <- setdiff(all_terms, epi)
  
  list(
    epi = epi,
    spi = spi,
    n_solutions = n_solutions
  )
}


#' Parse a single path/term into conditions
#'
#' @param path Character. A single path like "A*B*~C"
#' @return List with 'present' and 'absent' condition names
#' @keywords internal
parse_path_conditions <- function(path) {
  # Split by * (AND operator)
  terms <- unlist(strsplit(path, "\\*"))
  terms <- trimws(terms)
  
  # Separate present and absent conditions
  absent_terms <- grep("^~", terms, value = TRUE)
  present_terms <- setdiff(terms, absent_terms)
  
  # Remove ~ prefix from absent terms
  absent_conds <- gsub("^~", "", absent_terms)
  
  list(
    present = present_terms,
    absent  = absent_conds
  )
}


#' Extract all unique conditions from paths
#'
#' @param paths Character vector of paths
#' @return Character vector of unique condition names (without ~)
#' @keywords internal
extract_conditions_from_paths <- function(paths) {
  all_conds <- c()
  for (path in paths) {
    parsed <- parse_path_conditions(path)
    all_conds <- c(all_conds, parsed$present, parsed$absent)
  }
  unique(all_conds)
}


#' Build condition-path matrix for configuration chart
#'
#' @param paths Character vector of paths
#' @param conditions Character vector of condition names (optional)
#' @param symbols List with 'present' and 'absent' symbols
#' @return Matrix with conditions as rows, paths as columns
#' @keywords internal
build_config_matrix <- function(paths, conditions = NULL, symbols) {
  
  # Auto-detect conditions if not provided
  if (is.null(conditions)) {
    conditions <- extract_conditions_from_paths(paths)
  }
  
  n_paths <- length(paths)
  n_conds <- length(conditions)
  
  mat <- matrix("", nrow = n_conds, ncol = n_paths)
  rownames(mat) <- conditions
  # Columns are product terms (paths) within ONE solution, so they are labeled
  # T1, T2, ... "M1", "M2" are reserved for whole alternative minimal SOLUTIONS
  # (as in print(sol)); reusing M here made the two levels indistinguishable.
  colnames(mat) <- paste0("T", seq_len(n_paths))
  
  for (j in seq_along(paths)) {
    parsed <- parse_path_conditions(paths[j])
    for (cond in conditions) {
      if (cond %in% parsed$present) {
        mat[cond, j] <- symbols$present
      } else if (cond %in% parsed$absent) {
        mat[cond, j] <- symbols$absent
      }
      # else: leave blank (don't care)
    }
  }
  
  mat
}


#' Convert configuration matrix to Markdown table
#'
#' @param mat Matrix with rownames and colnames
#' @param row_header Character. Header for the row names column
#' @param center_align Logical. Whether to center-align columns
#' @return Character string of Markdown table
#' @keywords internal
config_matrix_to_md <- function(mat, row_header = "Condition", center_align = TRUE) {
  n_cols <- ncol(mat)
  
  # Header row
  header <- paste0("| ", row_header, " | ", 
                   paste(colnames(mat), collapse = " | "), " |")
  
  # Separator row
  if (center_align) {
    sep <- paste0("|", paste(rep(":--:", n_cols + 1), collapse = "|"), "|")
  } else {
    sep <- paste0("|", paste(rep("---", n_cols + 1), collapse = "|"), "|")
  }
  
  # Data rows
  rows <- sapply(seq_len(nrow(mat)), function(i) {
    paste0("| ", rownames(mat)[i], " | ", 
           paste(mat[i, ], collapse = " | "), " |")
  })
  
  paste(c(header, sep, rows), collapse = "\n")
}


#' Generate solution-term level chart (Fiss-style)
#'
#' Creates a configuration chart where each column represents a single
#' prime implicant (configuration), following Fiss (2011) notation.
#'
#' @param sum_df Data frame. Summary data frame from sweep results with
#'   expression column and threshold column(s).
#' @param conditions Character vector. Condition names for row ordering.
#' @param symbols List. Symbol set (present, absent) for the chart.
#' @param language Character. Language for labels ("en" or "ja").
#'
#' @return Character string containing Markdown-formatted table.
#'
#' @keywords internal
generate_term_level_chart <- function(sum_df, conditions, symbols, language = "en") {
  
  labels <- get_config_labels(language)
  
  # Step 1: Extract all terms from all thresholds
  terms_list <- lapply(seq_len(nrow(sum_df)), function(i) {
    expr <- sum_df$expression[i]
    
    # Identify threshold column (otSweep: thrY, ctSweepS: threshold, etc.)
    thr_col <- intersect(c("thrY", "threshold", "thrX"), names(sum_df))[1]
    if (is.na(thr_col) || is.null(thr_col)) {
      thr <- paste0("row", i)
    } else {
      thr <- sum_df[[thr_col]][i]
    }
    
    terms <- parse_solution_terms(expr)
    if (is.null(terms)) return(NULL)
    
    data.frame(
      thr = thr,
      term_num = seq_along(terms),
      term_expr = terms,
      stringsAsFactors = FALSE
    )
  })
  
  terms_df <- do.call(rbind, terms_list)
  
  # No solutions found
  if (is.null(terms_df) || nrow(terms_df) == 0) {
    return(paste0("*", labels$no_solution, "*\n"))
  }
  
  # Step 2: Generate column names (Fiss-style format)
  col_names <- paste0("thrY = ", terms_df$thr, " (C", terms_df$term_num, ")")
  
  # Step 3: Initialize matrix (blank = don't care)
  chart_matrix <- matrix(
    "",
    nrow = length(conditions),
    ncol = nrow(terms_df),
    dimnames = list(conditions, col_names)
  )
  
  # Step 4: Fill in cells
  for (j in seq_len(nrow(terms_df))) {
    term <- terms_df$term_expr[j]
    
    for (cond in conditions) {
      status <- get_condition_status(term, cond)
      
      chart_matrix[cond, j] <- switch(
        status,
        "present"  = symbols$present,
        "absent"   = symbols$absent,
        "dontcare" = ""
      )
    }
  }
  
  # Step 5: Convert to markdown
  table_str <- config_matrix_to_md(chart_matrix, labels$condition)
  
  # Add legend
  legend_note <- if (language == "ja") symbols$note_ja else symbols$note_en
  paste0(table_str, "\n\n*", legend_note, "*\n")
}


#' Generate threshold-level summary chart
#'
#' Creates a configuration chart where each column represents one threshold,
#' showing all conditions that appear in any configuration at that threshold.
#'
#' @param sum_df Data frame. Summary data frame from sweep results.
#' @param conditions Character vector. Condition names for row ordering.
#' @param symbols List. Symbol set (present, absent) for the chart.
#' @param language Character. Language for labels ("en" or "ja").
#'
#' @return Character string containing Markdown-formatted table.
#'
#' @keywords internal
generate_threshold_level_chart <- function(sum_df, conditions, symbols, language = "en") {
  
  labels <- get_config_labels(language)
  
  # Identify threshold column
  thr_col <- intersect(c("thrY", "threshold", "thrX"), names(sum_df))[1]
  if (is.na(thr_col) || is.null(thr_col)) {
    thr_col <- "row"
    sum_df[[thr_col]] <- seq_len(nrow(sum_df))
  }
  
  # Get unique thresholds
  thresholds <- unique(sum_df[[thr_col]])
  
  # Build column names
  col_names <- paste0("thrY=", thresholds)
  
  # Initialize matrix
  chart_matrix <- matrix(
    "",
    nrow = length(conditions),
    ncol = length(thresholds),
    dimnames = list(conditions, col_names)
  )
  
  # Fill in cells (aggregate across all terms at each threshold)
  for (j in seq_along(thresholds)) {
    thr <- thresholds[j]
    
    # Get all expressions at this threshold
    exprs <- sum_df$expression[sum_df[[thr_col]] == thr]
    
    # Parse all terms
    all_terms <- character(0)
    for (expr in exprs) {
      terms <- parse_solution_terms(expr)
      if (!is.null(terms)) {
        all_terms <- c(all_terms, terms)
      }
    }
    
    if (length(all_terms) == 0) next
    
    # Check each condition across all terms
    for (cond in conditions) {
      has_present <- FALSE
      has_absent <- FALSE
      
      for (term in all_terms) {
        status <- get_condition_status(term, cond)
        if (status == "present") has_present <- TRUE
        if (status == "absent") has_absent <- TRUE
      }
      
      # If both present and absent appear, show presence (it appears in some config)
      if (has_present) {
        chart_matrix[cond, j] <- symbols$present
      } else if (has_absent) {
        chart_matrix[cond, j] <- symbols$absent
      }
      # else: leave blank (don't care across all configs)
    }
  }
  
  # Convert to markdown
  table_str <- config_matrix_to_md(chart_matrix, labels$condition)
  
  # Add legend
  legend_note <- if (language == "ja") symbols$note_ja else symbols$note_en
  paste0(table_str, "\n\n*", legend_note, "*\n")
}


#' Generate cross-threshold configuration chart from sweep results
#'
#' Creates a configuration chart from threshold sweep results. Supports
#' two levels of aggregation: solution-term level (Fiss-style, default) and
#' threshold-level summary.
#'
#' @param result A result object from any Sweep function (otSweep, ctSweepS,
#'   ctSweepM, or dtSweep).
#' @param conditions Character vector. Condition names for row ordering.
#'   If NULL, automatically extracted from expressions.
#' @param symbol_set Character. One of \code{"unicode"}, \code{"ascii"}, 
#'   or \code{"latex"}. Default is \code{"unicode"}.
#' @param chart_level Character. Chart aggregation level:
#'   \code{"term"} (default) produces solution-term level charts following Fiss (2011)
#'   notation, where each column represents one prime implicant.
#'   \code{"summary"} produces threshold-level summaries where
#'   each column represents one threshold, aggregating all configurations.
#' @param language Character. \code{"en"} for English, \code{"ja"} for Japanese.
#'
#' @return Character string containing Markdown-formatted table.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' data(sample_data)
#' result <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = c(X1 = 7, X2 = 7, X3 = 7)
#' )
#' 
#' # Solution-term level, Fiss-style (default)
#' chart <- generate_cross_threshold_chart(result, c("X1", "X2", "X3"))
#' cat(chart)
#' 
#' # Threshold-level summary
#' chart <- generate_cross_threshold_chart(result, c("X1", "X2", "X3"),
#'                                          chart_level = "summary")
#' cat(chart)
#' }
generate_cross_threshold_chart <- function(result,
                                            conditions = NULL,
                                            symbol_set = c("unicode", "ascii", "latex"),
                                            chart_level = c("term", "summary"),
                                            language = c("en", "ja")) {
  
  symbol_set <- match.arg(symbol_set)
  chart_level <- match.arg(chart_level)
  language <- match.arg(language)
  symbols <- SYMBOL_SETS[[symbol_set]]
  
  # Get summary data frame
  if (is.data.frame(result)) {
    sum_df <- result
  } else if (is.list(result) && "summary" %in% names(result)) {
    sum_df <- result$summary
  } else {
    stop("'result' must be a data frame or a list with 'summary' element.")
  }
  
  # Auto-detect conditions from expressions if not provided
  if (is.null(conditions)) {
    all_terms <- character(0)
    for (expr in sum_df$expression) {
      terms <- parse_solution_terms(expr)
      if (!is.null(terms)) {
        all_terms <- c(all_terms, terms)
      }
    }
    conditions <- extract_conditions_from_paths(all_terms)
  }
  
  if (length(conditions) == 0) {
    return("*No conditions found.*\n")
  }
  
  # Dispatch to appropriate chart generator
  if (chart_level == "summary") {
    generate_threshold_level_chart(sum_df, conditions, symbols, language)
  } else {
    generate_term_level_chart(sum_df, conditions, symbols, language)
  }
}


#' Generate Configuration Chart from QCA Solution
#'
#' Creates a Markdown-formatted configuration chart (Fiss-style table)
#' from QCA minimization results. Supports single solution with multiple
#' paths, and multiple solutions (displayed as separate tables).
#'
#' @param sol A solution object returned by \code{QCA::minimize()}, or
#'   a list containing solution information.
#' @param symbol_set Character. One of \code{"unicode"}, \code{"ascii"}, 
#'   or \code{"latex"}. Default is \code{"unicode"}.
#' @param include_metrics Logical. Whether to include consistency/coverage
#'   metrics in the table. Default is TRUE.
#' @param language Character. \code{"en"} for English, \code{"ja"} for Japanese.
#'   Default is \code{"en"}.
#' @param condition_order Character vector. Optional ordering of conditions
#'   in the table rows. If NULL, conditions are ordered as they appear in paths.
#'
#' @return Character string containing Markdown-formatted table(s).
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # After running QCA::minimize()
#' library(QCA)
#' tt <- truthTable(data, outcome = "Y", conditions = c("A", "B", "C"))
#' sol <- minimize(tt, include = "?", details = TRUE)
#' 
#' # Generate configuration chart
#' chart <- generate_config_chart(sol)
#' cat(chart)
#'
#' # For LaTeX/PDF output (e.g., rticles)
#' chart <- generate_config_chart(sol, symbol_set = "latex")
#'
#' # ASCII for maximum compatibility
#' chart <- generate_config_chart(sol, symbol_set = "ascii")
#'
#' # Japanese labels
#' chart <- generate_config_chart(sol, language = "ja")
#' }
generate_config_chart <- function(sol,
                                   symbol_set = c("unicode", "ascii", "latex"),
                                   include_metrics = TRUE,
                                   language = c("en", "ja"),
                                   condition_order = NULL) {
  
  symbol_set <- match.arg(symbol_set)
  language <- match.arg(language)
  symbols <- SYMBOL_SETS[[symbol_set]]
  
  # Get labels based on language
  labels <- get_config_labels(language)
  
  # Get symbol note
  note <- if (language == "ja") symbols$note_ja else symbols$note_en
  
  # Extract solutions from QCA object
  sol_list <- extract_solution_list(sol)
  n_solutions <- length(sol_list)
  
  if (n_solutions == 0) {
    return(paste0("*", labels$no_solution, "*\n"))
  }
  
  # Single solution vs multiple solutions
  if (n_solutions == 1) {
    # Single solution: all paths in one table
    paths <- extract_paths_from_solution(sol_list[[1]])
    chart <- build_single_chart(
      paths = paths,
      sol = sol,
      symbols = symbols,
      labels = labels,
      include_metrics = include_metrics,
      condition_order = condition_order
    )
  } else {
    # Multiple solutions: separate tables with warning
    warning_msg <- paste0(
      "**", labels$note, ":** ", n_solutions, " ", labels$equiv_solutions, " ",
      labels$separate_tables, "\n\n"
    )
    
    charts <- lapply(seq_along(sol_list), function(i) {
      paths <- extract_paths_from_solution(sol_list[[i]])
      header <- paste0("### ", labels$solution, " M", i, "\n\n")
      table <- build_single_chart(
        paths = paths,
        sol = sol,
        symbols = symbols,
        labels = labels,
        include_metrics = include_metrics,
        condition_order = condition_order,
        solution_index = i
      )
      paste0(header, table)
    })
    
    chart <- paste0(warning_msg, paste(charts, collapse = "\n\n---\n\n"))
  }
  
  # Add symbol legend
  legend <- paste0("\n\n*", note, "*\n")
  
  paste0(chart, legend)
}


#' Get labels for configuration chart based on language
#' @keywords internal
get_config_labels <- function(language) {
  if (language == "ja") {
    list(
      condition = "\u6761\u4ef6",
      consistency = "\u4e00\u8cab\u6027",
      raw_coverage = "\u751f\u30ab\u30d0\u30ec\u30c3\u30b8",
      unique_coverage = "\u56fa\u6709\u30ab\u30d0\u30ec\u30c3\u30b8",
      solution_consistency = "\u89e3\u4e00\u8cab\u6027",
      solution_coverage = "\u89e3\u30ab\u30d0\u30ec\u30c3\u30b8",
      solution = "\u89e3",
      note = "\u6ce8\u610f",
      equiv_solutions = "\u500b\u306e\u7b49\u4fa1\u306a\u89e3\u304c\u5b58\u5728\u3057\u307e\u3059\u3002",
      separate_tables = "\u4ee5\u4e0b\u306b\u5225\u3005\u306e\u8868\u3092\u793a\u3057\u307e\u3059\u3002",
      no_solution = "\u89e3\u304c\u898b\u3064\u304b\u308a\u307e\u305b\u3093\u3067\u3057\u305f",
      # Solution note labels (Japanese)
      note_prefix = "\u6ce8",
      n_equiv_solutions = "\u8ad6\u7406\u7684\u306b\u7b49\u4fa1\u306a{n}\u3064\u306e\u89e3\u304c\u5f97\u3089\u308c\u305f\u3002",
      table_based_on_m1 = "\u672c\u8868\u306fM1\u306b\u57fa\u3065\u304f\u69cb\u6210\u3092\u793a\u3059\u3002",
      all_share_epi = "\u5168\u89e3\u306b\u5171\u901a\u3059\u308bEssential Prime Implicants: {epi}",
      all_share_epi_single = "\u5168\u89e3\u306b\u5171\u901a\u3059\u308bEssential Prime Implicant: {epi}",
      no_epi = "\u5168\u3066\u306e\u9805\u304cSelective Prime Implicants\u3067\u3042\u308b\u3002",
      # Fiss core/peripheral labels (Japanese)
      fiss_core       = "\u30b3\u30a2\u6761\u4ef6",
      fiss_peripheral = "\u5468\u8fba\u6761\u4ef6",
      fiss_parsim     = "\u7c21\u6f54\u89e3",
      fiss_interm     = "\u4e2d\u9593\u89e3",
      fiss_note       = paste0(
        "\u30b3\u30a2\u6761\u4ef6\uff1a\u7c21\u6f54\u89e3\u304b\u3064\u4e2d\u9593\u89e3\u306b\u5171\u901a\u3059\u308b\u6761\u4ef6\u3002",
        "\u5468\u8fba\u6761\u4ef6\uff1a\u4e2d\u9593\u89e3\u306e\u307f\u306b\u73fe\u308c\u308b\u6761\u4ef6\u3002",
        "Fiss (2011) \u306b\u57fa\u3065\u304f\u3002"
      )
    )
  } else {
    list(
      condition = "Condition",
      consistency = "Consistency",
      raw_coverage = "Raw Cov.",
      unique_coverage = "Uniq. Cov.",
      solution_consistency = "Solution Consistency",
      solution_coverage = "Solution Coverage",
      solution = "Solution",
      note = "Note",
      equiv_solutions = "equivalent solutions exist.",
      separate_tables = "Tables are shown separately below.",
      no_solution = "No solution found",
      # Solution note labels (English)
      note_prefix = "Note",
      n_equiv_solutions = "{n} logically equivalent solutions were identified.",
      n_equiv_solutions_range = "{n} logically equivalent solutions were identified (M1-M{n}).",
      table_based_on_m1 = "This table presents configurations based on M1.",
      all_share_epi = "All solutions share the essential prime implicants: {epi}.",
      all_share_epi_single = "All solutions share the essential prime implicant: {epi}.",
      no_epi = "Solutions differ in all prime implicants (no essential prime implicants).",
      # Fiss core/peripheral labels (English)
      fiss_core       = "Core condition",
      fiss_peripheral = "Peripheral condition",
      fiss_parsim     = "Parsimonious solution",
      fiss_interm     = "Intermediate solution",
      fiss_note       = paste0(
        "Core conditions appear in both the parsimonious and intermediate solutions. ",
        "Peripheral conditions appear in the intermediate solution only. ",
        "Based on Fiss (2011)."
      )
    )
  }
}


#' Extract solution list from QCA object
#' @note When dir.exp is specified, the true Intermediate solution is stored in
#'   sol$i.sol, not sol$solution (which contains the Parsimonious solution).
#' @keywords internal
extract_solution_list <- function(sol) {
  if (is.null(sol)) return(list())
  
  # Methods 1-2: QCA::minimize() result. collect_unique_i_sol() handles both the
  # i.sol structure (true Intermediate when dir.exp specified) and the
  # $solution fallback (Parsimonious / complex), and deduplicates models that
  # several prime implicant charts report identically. Counting those duplicates
  # separately previously made generate_config_chart() report an inflated number
  # of equivalent solutions and emit one identical table per duplicate.
  if (!is.null(sol$i.sol) || !is.null(sol$solution)) {
    solutions <- collect_unique_i_sol(sol)
    if (!is.null(solutions) && length(solutions) > 0) return(solutions)
  }
  
  # Method 3: If passed as simple character vector of paths
  if (is.character(sol)) {
    return(list(sol))
  }
  
  # Method 4: If passed as list of character vectors
  if (is.list(sol) && all(sapply(sol, is.character))) {
    return(sol)
  }
  
  list()
}


#' Extract paths from a single solution
#' @keywords internal
extract_paths_from_solution <- function(solution) {
  if (is.null(solution)) return(character(0))
  
  # If it's a character vector with multiple elements, those are paths
  if (is.character(solution) && length(solution) > 1) {
    return(solution)
  }
  
  # If it's a single string, check for " + " separator
  if (is.character(solution) && length(solution) == 1) {
    if (grepl(" \\+ ", solution)) {
      return(trimws(unlist(strsplit(solution, " \\+ "))))
    }
    return(solution)
  }
  
  character(0)
}


#' Build configuration chart for a single solution
#' @keywords internal
build_single_chart <- function(paths, sol, symbols, labels, 
                                include_metrics, condition_order = NULL,
                                solution_index = 1) {
  
  if (length(paths) == 0) {
    return(paste0("*", labels$no_solution, "*"))
  }
  
  # Determine condition order
  if (is.null(condition_order)) {
    conditions <- extract_conditions_from_paths(paths)
  } else {
    conditions <- condition_order
  }
  
  # Build matrix
  mat <- build_config_matrix(paths, conditions, symbols)
  
  # Convert to markdown
  table_str <- config_matrix_to_md(mat, labels$condition)
  
  # Add metrics if requested and available
  if (include_metrics && !is.null(sol)) {
    # Try to get per-path metrics
    path_metrics <- extract_path_metrics_for_chart(sol, solution_index)
    
    if (!is.null(path_metrics) && nrow(path_metrics) == length(paths)) {
      table_str <- add_metrics_rows(table_str, path_metrics, labels)
    }
    
    # Add solution-level metrics
    sol_metrics <- extract_solution_metrics_for_chart(sol, solution_index)
    
    if (!is.null(sol_metrics)) {
      table_str <- paste0(
        table_str, "\n\n",
        "**", labels$solution_consistency, "**: ", 
        format(round(sol_metrics$inclS, 3), nsmall = 3), "  \n",
        "**", labels$solution_coverage, "**: ", 
        format(round(sol_metrics$covS, 3), nsmall = 3)
      )
    }
  }
  
  table_str
}


#' Extract per-path metrics for configuration chart
#' @keywords internal
extract_path_metrics_for_chart <- function(sol, solution_index = 1) {
  if (is.null(sol)) return(NULL)

  # Mirror the displayed-paths source (i.sol first for intermediate), as in
  # extract_solution_metrics_for_chart.

  # Path A (intermediate): per-term table from the intermediate solution, BEFORE
  # sol$IC$incl.cov (which is the parsimonious per-term table). Handles both the
  # flat shape and the $individual shape (several intermediate solutions).
  if (!is.null(sol$i.sol) && length(sol$i.sol) > 0) {
    isol_ic <- try(sol$i.sol$C1P1$IC, silent = TRUE)
    if ((inherits(isol_ic, "try-error") || is.null(isol_ic)) &&
        length(sol$i.sol) >= solution_index) {
      isol_ic <- try(sol$i.sol[[solution_index]]$IC, silent = TRUE)
    }
    if (!inherits(isol_ic, "try-error") && !is.null(isol_ic)) {
      ic <- try(isol_ic$incl.cov, silent = TRUE)
      if (!inherits(ic, "try-error") && !is.null(ic)) return(ic)
      if (!is.null(isol_ic$individual) &&
          length(isol_ic$individual) >= solution_index) {
        ic <- try(isol_ic$individual[[solution_index]]$incl.cov, silent = TRUE)
        if (!inherits(ic, "try-error") && !is.null(ic)) return(ic)
      }
    }
  }
  # Path B: single solution without dir.exp.
  if (!is.null(sol$IC$incl.cov)) {
    return(sol$IC$incl.cov)
  }
  # Path C: multiple solutions - the displayed solution's per-term table.
  if (!is.null(sol$IC$individual)) {
    indiv <- sol$IC$individual
    if (length(indiv) >= solution_index) {
      if (!is.null(indiv[[solution_index]]$incl.cov)) {
        return(indiv[[solution_index]]$incl.cov)
      }
    }
  }

  NULL
}


#' Extract solution-level metrics for configuration chart
#' @keywords internal
extract_solution_metrics_for_chart <- function(sol, solution_index = 1) {
  if (is.null(sol)) return(NULL)
  mk <- function(ic) if (!is.null(ic)) list(inclS = ic$inclS[1], covS = ic$covS[1]) else NULL

  # The displayed paths come from get_solution_list(), which uses sol$i.sol first
  # (intermediate, when dir.exp was given) and only then sol$solution. The fit must
  # follow the same source, so:
  #
  # Path A (intermediate): read the intermediate solution's own fit. Checked BEFORE
  # sol$IC, whose sol.incl.cov / overall describe the PARSIMONIOUS solution.
  # The intermediate IC has the same two shapes as sol$IC: flat when the
  # intermediate solution is unique, and $individual/$overall when it has several
  # minimal solutions. Without the second shape this path would silently fall
  # through to the parsimonious values below.
  if (!is.null(sol$i.sol) && length(sol$i.sol) > 0) {
    isol_ic <- try(sol$i.sol$C1P1$IC, silent = TRUE)
    if ((inherits(isol_ic, "try-error") || is.null(isol_ic)) &&
        length(sol$i.sol) >= solution_index) {
      isol_ic <- try(sol$i.sol[[solution_index]]$IC, silent = TRUE)
    }
    if (!inherits(isol_ic, "try-error") && !is.null(isol_ic)) {
      ic <- try(isol_ic$sol.incl.cov, silent = TRUE)
      if (!inherits(ic, "try-error") && !is.null(ic)) return(mk(ic))
      if (!is.null(isol_ic$individual) &&
          length(isol_ic$individual) >= solution_index) {
        ic <- try(isol_ic$individual[[solution_index]]$sol.incl.cov, silent = TRUE)
        if (!inherits(ic, "try-error") && !is.null(ic)) return(mk(ic))
      }
      ic <- try(isol_ic$overall$sol.incl.cov, silent = TRUE)
      if (!inherits(ic, "try-error") && !is.null(ic)) return(mk(ic))
    }
  }
  # Path B (single parsimonious/complex solution).
  if (!is.null(sol$IC$sol.incl.cov)) return(mk(sol$IC$sol.incl.cov))
  # Path C (multiple solutions): the displayed solution's OWN fit, not the overall
  # (disjunction-of-all) aggregate.
  if (!is.null(sol$IC$individual) && length(sol$IC$individual) >= solution_index) {
    ic <- try(sol$IC$individual[[solution_index]]$sol.incl.cov, silent = TRUE)
    if (!inherits(ic, "try-error") && !is.null(ic)) return(mk(ic))
  }
  # Path D (last-resort fallback): overall aggregate.
  if (!is.null(sol$IC$overall$sol.incl.cov)) return(mk(sol$IC$overall$sol.incl.cov))

  NULL
}


#' Add metrics rows to markdown table
#' @keywords internal
add_metrics_rows <- function(table_str, metrics, labels) {
  
  n_paths <- nrow(metrics)
  
  # Add consistency row
  if ("inclS" %in% names(metrics)) {
    vals <- format(round(metrics$inclS, 3), nsmall = 3)
    row <- paste0("| **", labels$consistency, "** | ", 
                  paste(vals, collapse = " | "), " |")
    table_str <- paste0(table_str, "\n", row)
  }
  
  # Add raw coverage row
  if ("covS" %in% names(metrics)) {
    vals <- format(round(metrics$covS, 3), nsmall = 3)
    row <- paste0("| **", labels$raw_coverage, "** | ", 
                  paste(vals, collapse = " | "), " |")
    table_str <- paste0(table_str, "\n", row)
  }
  
  # Add unique coverage row
  if ("covU" %in% names(metrics)) {
    vals <- format(round(metrics$covU, 3), nsmall = 3)
    row <- paste0("| **", labels$unique_coverage, "** | ", 
                  paste(vals, collapse = " | "), " |")
    table_str <- paste0(table_str, "\n", row)
  }
  
  table_str
}


#' Generate configuration chart from paths (simple interface)
#'
#' A simpler interface for generating configuration charts when you have
#' paths directly (without a full QCA solution object).
#'
#' @param paths Character vector. Paths in QCA notation (e.g., "A*B*~C").
#' @param symbol_set Character. One of \code{"unicode"}, \code{"ascii"}, 
#'   or \code{"latex"}.
#' @param language Character. \code{"en"} for English, \code{"ja"} for Japanese.
#' @param condition_order Character vector. Optional ordering of conditions.
#' @param n_sol Integer. Number of equivalent solutions. If > 1, a note is added
#'   explaining that multiple solutions exist and M1 is shown. Default is 1.
#' @param solution_note Logical. Whether to add solution note when n_sol > 1.
#'   Default is TRUE.
#' @param solution_note_style Character. \code{"simple"} or \code{"detailed"}.
#'   Default is \code{"simple"}.
#' @param epi_list Character vector. Essential prime implicants for detailed notes.
#'   Only used when \code{solution_note_style = "detailed"}.
#'
#' @return Character string containing Markdown-formatted table.
#'
#' @export
#'
#' @examples
#' # Simple usage with paths
#' paths <- c("A*B", "A*C*~D", "B*E")
#' chart <- config_chart_from_paths(paths)
#' cat(chart)
#'
#' # With ASCII symbols
#' chart <- config_chart_from_paths(paths, symbol_set = "ascii")
#' cat(chart)
#'
#' # With multiple solution note
#' chart <- config_chart_from_paths(paths, n_sol = 2)
#' cat(chart)
#'
#' # With detailed note including EPIs
#' chart <- config_chart_from_paths(
#'   paths, n_sol = 2,
#'   solution_note_style = "detailed",
#'   epi_list = c("A*B")
#' )
#' cat(chart)
config_chart_from_paths <- function(paths,
                                     symbol_set = c("unicode", "ascii", "latex"),
                                     language = c("en", "ja"),
                                     condition_order = NULL,
                                     n_sol = 1L,
                                     solution_note = TRUE,
                                     solution_note_style = c("simple", "detailed"),
                                     epi_list = NULL) {
  
  symbol_set <- match.arg(symbol_set)
  language <- match.arg(language)
  solution_note_style <- match.arg(solution_note_style)
  symbols <- SYMBOL_SETS[[symbol_set]]
  labels <- get_config_labels(language)
  
  # Get symbol note
  legend_note <- if (language == "ja") symbols$note_ja else symbols$note_en
  
  # Determine format for solution note
  format_type <- if (symbol_set == "latex") "latex" else "markdown"
  
  # Determine conditions
  if (is.null(condition_order)) {
    conditions <- extract_conditions_from_paths(paths)
  } else {
    conditions <- condition_order
  }
  
  # Build matrix and table
  mat <- build_config_matrix(paths, conditions, symbols)
  table_str <- config_matrix_to_md(mat, labels$condition)
  
  # Add legend
  result <- paste0(table_str, "\n\n*", legend_note, "*")
  
  # Add solution note if multiple solutions exist
  if (solution_note && !is.null(n_sol) && n_sol > 1) {
    sol_note <- generate_solution_note(
      n_sol = n_sol,
      epi_list = epi_list,
      style = solution_note_style,
      language = language,
      format = format_type
    )
    result <- paste0(result, "\n\n", sol_note)
  }
  
  paste0(result, "\n")
}


#' Generate configuration chart for multiple solutions (simple interface)
#'
#' Generates separate configuration charts for multiple solutions.
#'
#' @param solutions List of character vectors. Each element is a vector of 
#'   paths for one solution.
#' @param symbol_set Character. One of \code{"unicode"}, \code{"ascii"}, 
#'   or \code{"latex"}.
#' @param language Character. \code{"en"} for English, \code{"ja"} for Japanese.
#' @param condition_order Character vector. Optional ordering of conditions.
#' @param show_epi Logical. Whether to identify and display Essential Prime
#'   Implicants (EPIs) in the note. Default is FALSE.
#'
#' @return Character string containing Markdown-formatted tables.
#'
#' @export
#'
#' @examples
#' # Multiple solutions
#' solutions <- list(
#'   c("A*B", "C"),
#'   c("A*B", "D"),
#'   c("A*C")
#' )
#' chart <- config_chart_multi_solutions(solutions)
#' cat(chart)
#'
#' # With EPI identification
#' chart <- config_chart_multi_solutions(solutions, show_epi = TRUE)
#' cat(chart)
config_chart_multi_solutions <- function(solutions,
                                          symbol_set = c("unicode", "ascii", "latex"),
                                          language = c("en", "ja"),
                                          condition_order = NULL,
                                          show_epi = FALSE) {
  
  symbol_set <- match.arg(symbol_set)
  language <- match.arg(language)
  symbols <- SYMBOL_SETS[[symbol_set]]
  labels <- get_config_labels(language)
  
  # Get symbol note
  legend_note <- if (language == "ja") symbols$note_ja else symbols$note_en
  
  n_solutions <- length(solutions)
  
  # Identify EPIs if requested
  epi_info <- NULL
  if (show_epi && n_solutions > 1) {
    epi_info <- identify_epi(solutions)
  }
  
  # Build note message
  if (show_epi && !is.null(epi_info) && length(epi_info$epi) > 0) {
    # Format EPIs
    epi_str <- paste(epi_info$epi, collapse = ", ")
    epi_str <- gsub("\\*", "\u00B7", epi_str)  # middle dot for markdown
    
    if (language == "ja") {
      note_msg <- paste0(
        "**", labels$note, ":** ", n_solutions, " ", labels$equiv_solutions, " ",
        labels$separate_tables, "\n\n",
        "**Essential Prime Implicants (EPI):** ", epi_str, "\n\n"
      )
    } else {
      note_msg <- paste0(
        "**", labels$note, ":** ", n_solutions, " ", labels$equiv_solutions, " ",
        labels$separate_tables, "\n\n",
        "**Essential Prime Implicants (EPI):** ", epi_str, "\n\n"
      )
    }
  } else if (show_epi && !is.null(epi_info) && length(epi_info$epi) == 0) {
    # No EPIs - all terms are SPIs
    if (language == "ja") {
      note_msg <- paste0(
        "**", labels$note, ":** ", n_solutions, " ", labels$equiv_solutions, " ",
        labels$separate_tables, "\n\n",
        "*", labels$no_epi, "*\n\n"
      )
    } else {
      note_msg <- paste0(
        "**", labels$note, ":** ", n_solutions, " ", labels$equiv_solutions, " ",
        labels$separate_tables, "\n\n",
        "*", labels$no_epi, "*\n\n"
      )
    }
  } else {
    # Simple note
    note_msg <- paste0(
      "**", labels$note, ":** ", n_solutions, " ", labels$equiv_solutions, " ",
      labels$separate_tables, "\n\n"
    )
  }
  
  # Generate chart for each solution
  charts <- lapply(seq_along(solutions), function(i) {
    paths <- solutions[[i]]
    
    # Determine conditions
    if (is.null(condition_order)) {
      conditions <- extract_conditions_from_paths(paths)
    } else {
      conditions <- condition_order
    }
    
    header <- paste0("### ", labels$solution, " M", i, "\n\n")
    mat <- build_config_matrix(paths, conditions, symbols)
    table_str <- config_matrix_to_md(mat, labels$condition)
    paste0(header, table_str)
  })
  
  # Combine
  paste0(note_msg, 
         paste(charts, collapse = "\n\n---\n\n"),
         "\n\n*", legend_note, "*\n")
}

# Regression tests for the report and chart fit extractors (v2.0.4).
# Fixtures live in helper-fixtures.R.
#
# extract_all_metrics() (report path) and extract_solution_metrics_for_chart()
# navigate the same polymorphic minimize() output as qca_extract() and had the
# same bug family independently: they returned sol$IC$overall (multiple
# solutions) or the parsimonious fit (intermediate) instead of the displayed
# solution's own fit. These tests pin the corrected behavior and assert the
# three solution-level extractors agree with each other.

test_that("report/chart extractors report individual[[1]], not overall (fuzzy multiple solutions)", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D")
  qd <- .multisol_fuzzy_data()
  tt <- QCA::truthTable(qd, outcome = "Y", conditions = conds,
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "", details = TRUE)
  skip_if(length(sol$solution) <= 1L)

  indiv1  <- sol$IC$individual[[1]]$sol.incl.cov$covS
  overall <- sol$IC$overall$sol.incl.cov$covS
  expect_gt(abs(overall - indiv1), 1e-3)

  am <- ThSQCA:::extract_all_metrics(sol$IC, sol)
  ch <- ThSQCA:::extract_solution_metrics_for_chart(sol, 1)
  expect_equal(am$sol_covS, indiv1, tolerance = 1e-6)
  expect_equal(ch$covS,     indiv1, tolerance = 1e-6)
  expect_false(isTRUE(all.equal(am$sol_covS, overall, tolerance = 1e-6)))
  expect_false(isTRUE(all.equal(ch$covS,     overall, tolerance = 1e-6)))
})

test_that("report/chart extractors report the intermediate fit, not the parsimonious fit", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D", "E")
  qd <- .intermediate_fuzzy_data()
  tt <- QCA::truthTable(qd, outcome = "Y", conditions = conds,
                        incl.cut = 0.75, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "?",
                       dir.exp = stats::setNames(rep(1, 5), conds),
                       details = TRUE)

  isol   <- sol$i.sol$C1P1$IC$sol.incl.cov$covS[1]
  parsim <- sol$IC$sol.incl.cov$covS[1]
  expect_gt(abs(isol - parsim), 1e-3)

  # Unit-level check: given the intermediate IC, the extractor returns the
  # intermediate fit. (Whether generate_report() actually hands it the
  # intermediate IC at each call site is verified end-to-end in
  # test-generate-report-e2e.R, which runs the full report pipeline.)
  am <- ThSQCA:::extract_all_metrics(sol$i.sol$C1P1$IC, sol)
  ch <- ThSQCA:::extract_solution_metrics_for_chart(sol, 1)
  expect_equal(am$sol_covS, isol, tolerance = 1e-6)
  expect_equal(ch$covS,     isol, tolerance = 1e-6)
  expect_false(isTRUE(all.equal(ch$covS, parsim, tolerance = 1e-6)))

  # Per-term (path) chart table must describe the intermediate solution: one row
  # per intermediate product term, not the parsimonious term table.
  pm <- ThSQCA:::extract_path_metrics_for_chart(sol, 1)
  expect_false(is.null(pm))
  expect_equal(nrow(pm), length(sol$i.sol$C1P1$solution[[1]]))
})

test_that("qca_extract, extract_all_metrics and the chart extractor agree (fuzzy multiple solutions)", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D")
  qd <- .multisol_fuzzy_data()
  tt <- QCA::truthTable(qd, outcome = "Y", conditions = conds,
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "", details = TRUE)
  skip_if(length(sol$solution) <= 1L)

  v_qe <- ThSQCA:::qca_extract(sol, "first")$covS
  v_am <- ThSQCA:::extract_all_metrics(sol$IC, sol)$sol_covS
  v_ch <- ThSQCA:::extract_solution_metrics_for_chart(sol, 1)$covS
  expect_equal(v_qe, v_am, tolerance = 1e-6)
  expect_equal(v_qe, v_ch, tolerance = 1e-6)
})

# End-to-end sweep-function regression tests built on a CONSTRUCTED data set
# with a guaranteed structure (v2.0.4).
#
# The data realize the classic cyclic prime-implicant chart on three conditions:
# positive configurations {000, 001, 010, 101, 110, 111}, negative {011, 100}.
# The coverage matrix is cyclic, so Boolean minimization has EXACTLY two
# equal-cost minimum covers,
#   M1 = ~A*~B + A*C + B*~C      M2 = ~A*~C + ~B*C + A*B,
# by construction (not by luck of a random seed). Conditions carry fuzzy
# memberships (jittered corners), so M1's and M2's memberships differ on the
# positive cases and the per-solution covS differs from the overall (union)
# aggregate. This is the exact pipeline shape of the original bug report
# (fuzzy conditions, outcome binarized by the sweep threshold), which makes
# these tests end-to-end: they check the numbers the user actually sees in the
# sweep table, not just the internal extractor.

.cyclic_ms_data <- function() {
  structure(list(
    A = c(0.216, 0.22, 0.116, 0.183, 0.143, 0.185, 0.146, 0.16, 0.215,
          0.915, 0.842, 0.904, 0.771, 0.803, 0.915, 0.923, 0.912, 0.872,
          0.076, 0.19, 0.178, 0.861, 0.906, 0.8),
    B = c(0.203, 0.173, 0.153, 0.22, 0.111, 0.144, 0.792, 0.928, 0.921,
          0.188, 0.2, 0.132, 0.868, 0.831, 0.84, 0.925, 0.869, 0.823,
          0.797, 0.812, 0.852, 0.113, 0.203, 0.181),
    C = c(0.188, 0.092, 0.175, 0.92, 0.927, 0.789, 0.083, 0.152, 0.132,
          0.88, 0.771, 0.903, 0.076, 0.226, 0.139, 0.825, 0.834, 0.896,
          0.878, 0.927, 0.892, 0.108, 0.077, 0.092),
    Y = c(8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
          2, 2, 2, 2, 2, 2)),
    row.names = c(NA, -24L), class = "data.frame")
}

# Hand computation of solution coverage against fuzzy condition columns and a
# crisp outcome vector, for a solution given as a "+"-joined expression string.
.covS_of_expression <- function(expr, qd, y) {
  terms <- trimws(unlist(strsplit(gsub("M[0-9]+:|->.*$", "", expr), "[+;]")))
  terms <- terms[nzchar(terms)]
  terms <- unique(terms[vapply(terms, function(t) {
    all(sub("^~", "", strsplit(t, "\\*")[[1]]) %in% names(qd))
  }, logical(1))])
  if (length(terms) == 0) return(NA_real_)
  ev <- function(t) {
    l <- strsplit(t, "\\*")[[1]]
    v <- sapply(l, function(p) {
      if (startsWith(p, "~")) 1 - qd[[substring(p, 2)]] else qd[[p]]
    })
    apply(as.matrix(v), 1, min)
  }
  X <- apply(as.matrix(sapply(terms, ev)), 1, max)
  sum(pmin(X, y)) / sum(y)
}

test_that("cyclic construction yields the guaranteed two minimal solutions", {
  skip_if_not_installed("QCA")
  d <- .cyclic_ms_data()
  dd <- d[c("A", "B", "C")]
  dd$Y <- as.integer(d$Y >= 5)
  tt <- QCA::truthTable(dd, outcome = "Y", conditions = c("A", "B", "C"),
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "", details = TRUE)
  # By construction (cyclic PI chart): exactly two minimal solutions.
  expect_equal(length(sol$solution), 2L)
  # And their fits genuinely differ from the overall aggregate.
  ind1 <- sol$IC$individual[[1]]$sol.incl.cov$covS
  ov   <- sol$IC$overall$sol.incl.cov$covS
  expect_gt(abs(ov - ind1), 2e-3)
})

test_that("otSweep end-to-end: displayed covS equals the displayed formula's covS", {
  skip_if_not_installed("QCA")
  d <- .cyclic_ms_data()
  emptyX <- stats::setNames(numeric(0), character(0))
  y <- as.integer(d$Y >= 5)

  r_first <- otSweep(dat = d, outcome = "Y", conditions = c("A", "B", "C"),
                     sweep_range = 5, thrX = emptyX,
                     pre_calibrated = c("A", "B", "C"),
                     include = "", incl.cut = 0.80, n.cut = 1,
                     extract_mode = "first", return_details = FALSE)
  hand_first <- .covS_of_expression(r_first$expression[1], d[c("A", "B", "C")], y)
  expect_equal(r_first$covS[1], hand_first, tolerance = 2e-3)

  r_all <- otSweep(dat = d, outcome = "Y", conditions = c("A", "B", "C"),
                   sweep_range = 5, thrX = emptyX,
                   pre_calibrated = c("A", "B", "C"),
                   include = "", incl.cut = 0.80, n.cut = 1,
                   extract_mode = "all", return_details = FALSE)
  # "all" reports the aggregate of all listed solutions: the covS must equal the
  # coverage of the union of all displayed terms.
  hand_all <- .covS_of_expression(r_all$expression[1], d[c("A", "B", "C")], y)
  expect_equal(r_all$covS[1], hand_all, tolerance = 2e-3)
  # And first != all on this construction (per-solution vs union).
  expect_gt(abs(r_all$covS[1] - r_first$covS[1]), 2e-3)
})

test_that("ctSweepS end-to-end shares the corrected extraction", {
  skip_if_not_installed("QCA")
  d <- .cyclic_ms_data()
  y <- as.integer(d$Y >= 5)
  r <- ctSweepS(dat = d, outcome = "Y", conditions = c("A", "B", "C"),
                sweep_var = "A", sweep_range = c(0.5), thrY = 5,
                pre_calibrated = c("B", "C"),
                include = "", incl.cut = 0.80, n.cut = 1,
                extract_mode = "first", return_details = FALSE)
  # Oracle must use ctSweepS's own calibration: A binarized at the swept 0.5.
  qd <- d[c("A", "B", "C")]
  qd$A <- as.numeric(qd$A >= 0.5)
  hand <- .covS_of_expression(r$expression[1], qd, y)
  expect_equal(r$covS[1], hand, tolerance = 2e-3)
})

test_that("multiple INTERMEDIATE solutions report a fit, not NA (regression)", {
  skip_if_not_installed("QCA")
  # Same cyclic construction, but with dir.exp: the intermediate solution itself
  # has several minimal solutions, so its IC is split into $individual/$overall
  # rather than a flat $sol.incl.cov. Reading only the flat slot returned NA.
  d <- .cyclic_ms_data()
  c3 <- c("A", "B", "C")
  dd <- d[c3]; dd$Y <- as.integer(d$Y >= 5)
  tt <- QCA::truthTable(dd, outcome = "Y", conditions = c3,
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "?", dir.exp = c(A = 1, B = 1, C = 1),
                       details = TRUE)

  isol <- sol$i.sol$C1P1
  skip_if(length(isol$solution) < 2)          # precondition: several intermediates
  expect_null(isol$IC$sol.incl.cov)           # precondition: split IC shape
  ind1 <- isol$IC$individual[[1]]$sol.incl.cov
  ov   <- isol$IC$overall$sol.incl.cov

  info <- ThSQCA:::qca_extract(sol, "first")
  expect_false(is.na(info$covS))              # the regression: this was NA
  expect_false(is.na(info$inclS))
  expect_equal(info$covS,  ind1$covS[1],  tolerance = 1e-6)
  expect_equal(info$inclS, ind1$inclS[1], tolerance = 1e-6)

  # "all" summarizes across the intermediate solutions, so it uses the aggregate.
  info_all <- ThSQCA:::qca_extract(sol, "all")
  expect_equal(info_all$covS, ov$covS[1], tolerance = 1e-6)

  # Chart extractors must also return the first intermediate solution's fit and
  # its per-term table (not the parsimonious ones).
  cm <- ThSQCA:::extract_solution_metrics_for_chart(sol, 1)
  expect_equal(cm$covS, ind1$covS[1], tolerance = 1e-6)
  pm <- ThSQCA:::extract_path_metrics_for_chart(sol, 1)
  expect_false(is.null(pm))
  expect_equal(nrow(pm), length(isol$solution[[1]]))

  # And end to end through the sweep table.
  emptyX <- stats::setNames(numeric(0), character(0))
  r <- otSweep(dat = d, outcome = "Y", conditions = c3, sweep_range = 5,
               thrX = emptyX, pre_calibrated = c3,
               dir.exp = c(A = 1, B = 1, C = 1), include = "?",
               incl.cut = 0.80, n.cut = 1, return_details = FALSE)
  expect_false(is.na(r$covS[1]))
  expect_equal(r$covS[1], ind1$covS[1], tolerance = 1e-6)
})

test_that("a solution with unreadable fit warns instead of returning a silent NA", {
  # Mock a result whose solution exists but whose IC carries no fit measures.
  sol <- list(solution = list(c("A", "B")), IC = list())
  expect_warning(info <- ThSQCA:::qca_extract(sol, "first"),
                 "could not be read")
  expect_true(is.na(info$covS))
  expect_equal(info$expression, "A + B")   # the expression is still reported
})

# Regression tests for the multi-prime-implicant-chart double-counting fix
# (collect_unique_i_sol(), added after v2.0.5).
#
# Bug: when include = "?" and dir.exp are specified and QCA::minimize()
# splits the intermediate solution across multiple prime implicant charts
# (print()'s "From C1P1, C2P1:"), sol$i.sol has one list entry per chart.
# Each entry's own $solution field is computed independently by
# QCA::minimize() and is not guaranteed to be chart-exclusive: on this
# fixture it already contains the full, resolved list of both M1 and M2 in
# BOTH chart entries. Naively concatenating every chart's $solution
# (the pre-fix behavior of get_n_solutions() and of two sol_list-building
# blocks in generate_report()) therefore double-counts: 2 charts x 2
# solutions each = 4, when only 2 unique models actually exist. This
# propagated a wrong n_solutions = 4 into res$summary and into the
# generated Markdown report's Summary Table.
#
# Fixture: .multichart_brand_data() (helper-fixtures.R), a real n = 336
# brand survey dataset. At thrY = 9, DLV = 8 (SPV = PRD = COM = 7 fixed,
# incl.cut = 0.85), this specific combination reliably produces the
# "C1P1, C2P1" two-chart structure (verified interactively; a hand-crafted
# small synthetic dataset only ever reproduced the different "C1P1, C1P2"
# single-chart/multiple-parsimonious-path case, so a real fixture is used).

test_that("get_n_solutions() does not double-count models across multiple prime implicant charts", {
  skip_if_not_installed("QCA")
  dat <- .multichart_brand_data()

  res <- ThSQCA::dtSweep(
    dat = dat, outcome = "INT", conditions = c("SPV", "PRD", "DLV", "COM"),
    sweep_range_Y = 6:9,
    sweep_list_X  = list(DLV = 6:9, SPV = 7, PRD = 7, COM = 7),
    incl.cut = 0.85, n.cut = 1,
    include = "?", dir.exp = c(1, 1, 1, "-"),
    extract_mode = "all", return_details = TRUE
  )

  idx <- which(res$summary$n_solutions > 1)
  skip_if(length(idx) == 0, "fixture no longer produces a multi-chart cell")

  sol <- res$details[[idx[1]]]$solution

  # Ground truth: print(sol) shows exactly two models, M1 and M2, spanning
  # two prime implicant charts.
  expect_true(!is.null(sol$i.sol))
  expect_gte(length(sol$i.sol), 2L)

  expect_equal(ThSQCA:::get_n_solutions(sol), 2L)
  expect_equal(res$summary$n_solutions[idx[1]], 2L)
})

test_that("collect_unique_i_sol() deduplicates by term-set content, not by chart identity", {
  skip_if_not_installed("QCA")
  dat <- .multichart_brand_data()

  res <- ThSQCA::dtSweep(
    dat = dat, outcome = "INT", conditions = c("SPV", "PRD", "DLV", "COM"),
    sweep_range_Y = 6:9,
    sweep_list_X  = list(DLV = 6:9, SPV = 7, PRD = 7, COM = 7),
    incl.cut = 0.85, n.cut = 1,
    include = "?", dir.exp = c(1, 1, 1, "-"),
    extract_mode = "all", return_details = TRUE
  )
  idx <- which(res$summary$n_solutions > 1)
  skip_if(length(idx) == 0, "fixture no longer produces a multi-chart cell")
  sol <- res$details[[idx[1]]]$solution

  uniq <- ThSQCA:::collect_unique_i_sol(sol)
  expect_length(uniq, 2L)

  # Each unique model must match one of the two chart-specific c.sol vectors
  # (the values print() actually displays as M1 and M2), independent of
  # which chart name it came from.
  chart_names <- names(sol$i.sol)
  csols <- lapply(chart_names, function(m) sol$i.sol[[m]]$c.sol)
  key <- function(x) paste(sort(as.character(x)), collapse = " | ")
  uniq_keys <- vapply(uniq, key, character(1))
  csol_keys <- vapply(csols, key, character(1))
  expect_setequal(uniq_keys, unique(csol_keys))
})

test_that("identify_epi() on the deduplicated list matches print()'s EPI/SPI split", {
  skip_if_not_installed("QCA")
  dat <- .multichart_brand_data()

  res <- ThSQCA::dtSweep(
    dat = dat, outcome = "INT", conditions = c("SPV", "PRD", "DLV", "COM"),
    sweep_range_Y = 6:9,
    sweep_list_X  = list(DLV = 6:9, SPV = 7, PRD = 7, COM = 7),
    incl.cut = 0.85, n.cut = 1,
    include = "?", dir.exp = c(1, 1, 1, "-"),
    extract_mode = "all", return_details = TRUE
  )
  idx <- which(res$summary$n_solutions > 1)
  skip_if(length(idx) == 0, "fixture no longer produces a multi-chart cell")
  sol <- res$details[[idx[1]]]$solution

  uniq <- ThSQCA:::collect_unique_i_sol(sol)
  epi_info <- identify_epi(uniq)

  expect_equal(epi_info$n_solutions, 2L)
  # Ground truth from print(sol): the two essential (non-parenthesized)
  # terms shared by both M1 and M2, and the two chart-specific optional terms.
  expect_setequal(epi_info$epi, c("~PRD*DLV*COM", "PRD*DLV*~COM"))
  expect_setequal(epi_info$spi, c("SPV*~PRD*DLV", "SPV*DLV*~COM"))
})

test_that("single-chart single-solution cells are unaffected by the dedup fix", {
  skip_if_not_installed("QCA")
  dat <- .multichart_brand_data()

  res <- ThSQCA::dtSweep(
    dat = dat, outcome = "INT", conditions = c("SPV", "PRD", "DLV", "COM"),
    sweep_range_Y = 6:9,
    sweep_list_X  = list(DLV = 6:9, SPV = 7, PRD = 7, COM = 7),
    incl.cut = 0.85, n.cut = 1,
    include = "?", dir.exp = c(1, 1, 1, "-"),
    extract_mode = "all", return_details = TRUE
  )

  single_idx <- which(res$summary$n_solutions == 1)
  expect_gt(length(single_idx), 0L)
  for (i in single_idx) {
    sol <- res$details[[i]]$solution
    expect_equal(ThSQCA:::get_n_solutions(sol), 1L)
    uniq <- ThSQCA:::collect_unique_i_sol(sol)
    expect_length(uniq, 1L)
  }

  no_sol_idx <- which(res$summary$n_solutions == 0)
  if (length(no_sol_idx) > 0) {
    for (i in no_sol_idx) {
      expect_null(res$details[[i]]$solution)
    }
  }
})

# The same duplication pattern existed in two further places, found in a
# follow-up audit of every sol$i.sol traversal in the package.

test_that("extract_solution_list() deduplicates, so config charts are not emitted twice", {
  skip_if_not_installed("QCA")
  dat <- .multichart_brand_data()

  res <- ThSQCA::dtSweep(
    dat = dat, outcome = "INT", conditions = c("SPV", "PRD", "DLV", "COM"),
    sweep_range_Y = 6:9,
    sweep_list_X  = list(DLV = 6:9, SPV = 7, PRD = 7, COM = 7),
    incl.cut = 0.85, n.cut = 1,
    include = "?", dir.exp = c(1, 1, 1, "-"),
    extract_mode = "all", return_details = TRUE
  )
  idx <- which(res$summary$n_solutions > 1)
  skip_if(length(idx) == 0, "fixture no longer produces a multi-chart cell")
  sol <- res$details[[idx[1]]]$solution

  sl <- ThSQCA:::extract_solution_list(sol)
  expect_length(sl, 2L)
  expect_equal(length(sl), ThSQCA:::get_n_solutions(sol))

  # generate_config_chart() emits one table per solution; with duplicates it
  # previously emitted the same table more than once and announced an inflated
  # number of equivalent solutions.
  chart <- ThSQCA:::generate_config_chart(sol)
  n_headings <- length(gregexpr("### Solution", chart, fixed = TRUE)[[1]])
  expect_equal(n_headings, 2L)
  expect_true(grepl("2 equivalent solutions", chart, fixed = TRUE))
  expect_false(grepl("4 equivalent solutions", chart, fixed = TRUE))
})

test_that("extract_sol_terms_by_model() deduplicates, so parsim_n_solutions is not inflated", {
  skip_if_not_installed("QCA")
  dat <- .multichart_brand_data()

  res <- ThSQCA::dtSweep(
    dat = dat, outcome = "INT", conditions = c("SPV", "PRD", "DLV", "COM"),
    sweep_range_Y = 6:9,
    sweep_list_X  = list(DLV = 6:9, SPV = 7, PRD = 7, COM = 7),
    incl.cut = 0.85, n.cut = 1,
    include = "?", dir.exp = c(1, 1, 1, "-"),
    extract_mode = "all", return_details = TRUE
  )
  idx <- which(res$summary$n_solutions > 1)
  skip_if(length(idx) == 0, "fixture no longer produces a multi-chart cell")
  sol <- res$details[[idx[1]]]$solution

  models <- ThSQCA:::extract_sol_terms_by_model(sol)
  expect_length(models, 2L)

  # No two returned models may share the same term set.
  keys <- vapply(models, function(x) paste(sort(x), collapse = " | "), character(1))
  expect_equal(length(unique(keys)), length(keys))
})

###############################################
# CTS–QCA (single X) and MCTS–QCA (multiple X)
# v0.3.0: QCA-compatible argument names + negated outcome support
###############################################

#' CTS–QCA: Single-condition threshold sweep
#'
#' Performs a threshold sweep for one focal condition X. For each threshold
#' in \code{sweep_range}, the outcome Y and all X variables are binarized
#' using user-specified thresholds, and a crisp-set QCA is executed.
#'
#' @param dat Data frame containing the outcome and condition variables.
#' @param outcome Character. Outcome variable name. Supports negation with
#'   tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.
#' @param conditions Character vector. Names of condition variables.
#' @param sweep_var Character. Name of the condition to be swept.
#'   Must be one of \code{conditions}.
#' @param sweep_range Numeric vector. Candidate thresholds for \code{sweep_var}.
#' @param thrY Numeric. Threshold for Y (fixed).
#' @param thrX_default Numeric. Default threshold for non-swept X variables.
#'   Variables listed in \code{pre_calibrated} do not require this threshold.
#' @param pre_calibrated Character vector or \code{NULL}. Names of condition
#'   variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
#'   and should be passed through to \code{QCA::truthTable()} without
#'   binarization. These variables must contain values in the \code{[0, 1]}
#'   range. Variables not listed here will be binarized using \code{thrX_default}
#'   as usual. Default is \code{NULL} (all variables binarized).
#'   It is recommended to sweep variables on their original (raw) scale rather
#'   than as pre-calibrated fuzzy values, because raw-scale thresholds are
#'   easier to interpret substantively.
#' @param dir.exp Directional expectations for \code{minimize}.
#'   If \code{NULL} (default), no directional expectations are applied.
#'   To compute the \strong{intermediate solution}, specify a numeric vector
#'   (1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
#'   for three conditions all expected to contribute positively.
#' @param include Inclusion rule for \code{minimize}. 
#'   \code{""} (default, QCA compatible) computes the \strong{complex solution}
#'   without logical remainders.
#'   Use \code{"?"} to include logical remainders for \strong{parsimonious}
#'   (with \code{dir.exp = NULL}) or \strong{intermediate} solutions
#'   (with \code{dir.exp} specified).
#' @param incl.cut Consistency cutoff for \code{\link[QCA]{truthTable}}.
#' @param n.cut Frequency cutoff for \code{truthTable}.
#' @param pri.cut PRI cutoff for \code{minimize}.
#' @param extract_mode Character. How to handle multiple solutions:
#'   \code{"first"} (default), \code{"all"}, or \code{"essential"}.
#'   See \code{\link{qca_extract}} for details.
#' @param return_details Logical. If \code{TRUE} (default), returns both
#'   summary and detailed objects for use with \code{generate_report()}.
#' @param Yvar Deprecated. Use \code{outcome} instead.
#' @param Xvars Deprecated. Use \code{conditions} instead.
#'
#' @return
#' If \code{return_details = FALSE}, a data frame with columns:
#' \itemize{
#'   \item \code{threshold} — swept threshold for \code{sweep_var}
#'   \item \code{expression} — minimized solution expression
#'   \item \code{inclS} — solution consistency
#'   \item \code{covS} — solution coverage
#'   \item (additional columns depending on \code{extract_mode})
#' }
#'
#' If \code{return_details = TRUE}, a list with:
#' \itemize{
#'   \item \code{summary} — the data frame above
#'   \item \code{details} — per-threshold list of
#'     \code{threshold}, \code{thrX_vec}, \code{truth_table}, \code{solution}
#' }
#'
#' Note that \code{return_details} changes the \emph{type} of the returned
#' object, not just its contents: with \code{TRUE} the summary table is at
#' \code{result$summary}, whereas with \code{FALSE} the summary table
#' \emph{is} the returned object and \code{result$summary} is \code{NULL}.
#' Code intended to work under both settings should branch on
#' \code{inherits(result, "data.frame")} (or simply always pass
#' \code{return_details = TRUE}) rather than assuming \code{result$summary}
#' exists.
#'
#' @importFrom QCA truthTable minimize
#' @export
#' @examples
#' # Load sample data
#' data(sample_data)
#' 
#' # === Three Types of QCA Solutions ===
#' 
#' # 1. Complex Solution (default, QCA compatible)
#' result_comp <- ctSweepS(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_var = "X3",
#'   sweep_range = 7,
#'   thrY = 7,
#'   thrX_default = 7
#'   # include = "" (default), dir.exp = NULL (default)
#' )
#' head(result_comp$summary)
#' 
#' # 2. Parsimonious Solution (include = "?")
#' result_pars <- ctSweepS(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_var = "X3",
#'   sweep_range = 7,
#'   thrY = 7,
#'   thrX_default = 7,
#'   include = "?"  # Include logical remainders
#' )
#' head(result_pars$summary)
#' 
#' # 3. Intermediate Solution (include = "?" + dir.exp)
#' result_int <- ctSweepS(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_var = "X3",
#'   sweep_range = 7,
#'   thrY = 7,
#'   thrX_default = 7,
#'   include = "?",
#'   dir.exp = c(1, 1, 1)  # All conditions expected positive
#' )
#' head(result_int$summary)
#' 
#' # === Threshold Sweep Example ===
#' 
#' # Run single condition threshold sweep on X3 (complex solutions by default)
#' result <- ctSweepS(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_var = "X3",
#'   sweep_range = 6:8,
#'   thrY = 7,
#'   thrX_default = 7
#' )
#' head(result$summary)
#' 
#' # Run with negated outcome (~Y)
#' result_neg <- ctSweepS(
#'   dat = sample_data,
#'   outcome = "~Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_var = "X3",
#'   sweep_range = 6:8,
#'   thrY = 7,
#'   thrX_default = 7
#' )
#' head(result_neg$summary)
ctSweepS <- function(dat, 
                     outcome = NULL, conditions = NULL,
                     sweep_var, sweep_range,
                     thrY, thrX_default = 7,
                     pre_calibrated = NULL,
                     dir.exp = NULL, include = "",
                     incl.cut = 0.8, n.cut = 1, pri.cut = 0,
                     extract_mode = c("first", "all", "essential"),
                     return_details = TRUE,
                     Yvar = NULL, Xvars = NULL) {
  
  # === Backward compatibility for deprecated arguments ===
  if (!is.null(Yvar) && is.null(outcome)) {
    outcome <- Yvar
    warning("Argument 'Yvar' is deprecated. Use 'outcome' instead.",
            call. = FALSE)
  }
  if (!is.null(Xvars) && is.null(conditions)) {
    conditions <- Xvars
    warning("Argument 'Xvars' is deprecated. Use 'conditions' instead.",
            call. = FALSE)
  }
  
  # === Validate required arguments ===
  if (is.null(outcome)) {
    stop("Argument 'outcome' is required.")
  }
  if (is.null(conditions)) {
    stop("Argument 'conditions' is required.")
  }
  
  # === Handle negated outcome ===
  negate_outcome <- grepl("^~", outcome)
  outcome_clean <- sub("^~", "", outcome)
  
  # Validate outcome variable exists
  if (!outcome_clean %in% names(dat)) {
    stop("Variable '", outcome_clean, "' not found in data.")
  }
  
  # Validate condition variables exist
  missing_conds <- setdiff(conditions, names(dat))
  if (length(missing_conds) > 0) {
    stop("Condition variable(s) not found in data: ", 
         paste(missing_conds, collapse = ", "))
  }
  
  extract_mode <- match.arg(extract_mode)
  
  if (!sweep_var %in% conditions) {
    stop("sweep_var must be one of conditions.")
  }
  
  # Initialize output data frame based on extract_mode
  df_out <- data.frame(
    threshold   = numeric(0),
    expression  = character(0),
    inclS       = numeric(0),
    covS        = numeric(0),
    n_solutions = integer(0),
    stringsAsFactors = FALSE
  )
  
  # Add columns based on extract_mode
  if (extract_mode == "essential") {
    df_out$selective_terms <- character(0)
    df_out$unique_terms     <- character(0)
  }
  
  details_list <- list()
  
  # Track thresholds with multiple solutions (for warning in "first" mode)
  multi_sol_thresholds <- c()
  
  # Handle dir.exp: scalar -> expand to vector; NULL is passed through
  # NULL -> parsimonious solution; c(1,1,...) -> intermediate solution
  local_dir.exp <- dir.exp
  if (!is.null(local_dir.exp) && length(local_dir.exp) == 1) {
    local_dir.exp <- rep(local_dir.exp[1], length(conditions))
    names(local_dir.exp) <- conditions
  } else if (!is.null(local_dir.exp) && is.null(names(local_dir.exp))) {
    names(local_dir.exp) <- conditions
  }

  # Validate pre_calibrated (once, before loop)
  validate_pre_calibrated(pre_calibrated, conditions, dat)

  for (thr in sweep_range) {
    
    # vector of thresholds for all X
    thrX_vec <- rep(thrX_default, length(conditions))
    names(thrX_vec) <- conditions
    thrX_vec[sweep_var] <- thr
    
    # Prepare data: binarize or pass through based on pre_calibrated
    dat_bin <- prepare_dat_bin(
      dat, outcome_clean, conditions,
      thrY, thrX_vec,
      pre_calibrated = pre_calibrated
    )
    
    # Determine outcome string for truthTable (with ~ if negated)
    outcome_tt <- if (negate_outcome) "~Y" else "Y"
    
    # Truth table (wrapped in try to handle errors)
    tt <- try(
      QCA::truthTable(
        dat_bin,
        outcome    = outcome_tt,
        conditions = conditions,
        show.cases = FALSE,
        incl.cut1  = incl.cut,
        n.cut      = n.cut,
        pri.cut    = pri.cut
      ),
      silent = TRUE
    )
    
    if (inherits(tt, "try-error")) {
      new_row <- data.frame(
        threshold   = thr,
        expression  = "No solution",
        inclS       = NA_real_,
        covS        = NA_real_,
        n_solutions = 0L,
        stringsAsFactors = FALSE
      )
      
      if (extract_mode == "essential") {
        new_row$selective_terms <- NA_character_
        new_row$unique_terms     <- NA_character_
      }
      
      df_out <- rbind(df_out, new_row)
      
      if (return_details) {
        details_list[[as.character(thr)]] <- list(
          threshold   = thr,
          thrX_vec    = thrX_vec,
          truth_table = NULL,
          solution    = NULL,
          dat_bin     = NULL
        )
      }
      next
    }
    
    # Minimize (wrapped in try to handle errors)
    tt <- sanitize_truthtable(tt)  # QCA 3.25 guard: char incl/PRI + '-' remainders
    sol <- try(
      QCA::minimize(
        tt,
        include    = include,
        dir.exp    = local_dir.exp,
        details    = TRUE,
        show.cases = FALSE,
        pri.cut    = pri.cut
      ),
      silent = TRUE
    )
    
    if (inherits(sol, "try-error")) {
      new_row <- data.frame(
        threshold   = thr,
        expression  = "No solution",
        inclS       = NA_real_,
        covS        = NA_real_,
        n_solutions = 0L,
        stringsAsFactors = FALSE
      )
      
      if (extract_mode == "essential") {
        new_row$selective_terms <- NA_character_
        new_row$unique_terms     <- NA_character_
      }
      
      df_out <- rbind(df_out, new_row)
      
      if (return_details) {
        details_list[[as.character(thr)]] <- list(
          threshold   = thr,
          thrX_vec    = thrX_vec,
          truth_table = tt,
          solution    = NULL,
          dat_bin     = dat_bin
        )
      }
      next
    }
    
    sol_info <- qca_extract(sol, extract_mode = extract_mode)
    
    # Track multiple solutions
    if (sol_info$n_solutions > 1) {
      multi_sol_thresholds <- c(multi_sol_thresholds, thr)
    }
    
    # Build result row based on extract_mode
    new_row <- data.frame(
      threshold   = thr,
      expression  = sol_info$expression,
      inclS       = sol_info$inclS,
      covS        = sol_info$covS,
      n_solutions = sol_info$n_solutions,
      stringsAsFactors = FALSE
    )
    
    if (extract_mode == "essential") {
      new_row$selective_terms <- sol_info$selective_terms
      new_row$unique_terms     <- sol_info$unique_terms
    }
    
    df_out <- rbind(df_out, new_row)
    
    if (return_details) {
      details_list[[as.character(thr)]] <- list(
        threshold   = thr,
        thrX_vec    = thrX_vec,
        truth_table = tt,
        solution    = sol,
        dat_bin     = dat_bin
      )
    }
  }
  
  df_out <- df_out[order(df_out$threshold), ]
  
  # Issue warning for multiple solutions
  if (length(multi_sol_thresholds) > 0) {
    warning(
      "Multiple intermediate solutions exist for threshold = ",
      paste(multi_sol_thresholds, collapse = ", "),
      " (n_solutions > 1). ",
      "Only the first solution (M1) and its fit metrics are shown. ",
      "Use generate_report() for full analysis.",
      call. = FALSE
    )
  }
  
  if (return_details) {
    result <- list(
      summary = df_out, 
      details = details_list,
      params = list(
        outcome = outcome,
        conditions = conditions,
        negate_outcome = negate_outcome,
        pre_calibrated = pre_calibrated,
        sweep_var = sweep_var,
        sweep_range = sweep_range,
        thrY = thrY,
        thrX_default = thrX_default,
        incl.cut = incl.cut,
        n.cut = n.cut,
        pri.cut = pri.cut,
        include = include,
        dir.exp = dir.exp  # Store original value for reproducibility
      )
    )
    class(result) <- c("ctSweepS_result", "tsqca_result", "list")
    return(result)
  }
  
  df_out
}


###############################################
# MCTS–QCA (multiple X)
###############################################

#' MCTS–QCA: Multi-condition threshold sweep
#'
#' Performs a grid search over thresholds of multiple X variables.
#' For each combination of thresholds in \code{sweep_list}, the outcome Y
#' and all X variables are binarized, and a crisp-set QCA is executed.
#'
#' @param dat Data frame containing the outcome and condition variables.
#' @param outcome Character. Outcome variable name. Supports negation with
#'   tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.
#' @param conditions Character vector. Names of condition variables.
#' @param sweep_list Named list. Each element is a numeric vector of
#'   candidate thresholds for the corresponding X. Names must match
#'   \code{conditions}. Variables listed in \code{pre_calibrated} do not
#'   need a \code{sweep_list} entry. Every condition must receive a threshold
#'   from one of \code{sweep_list}, \code{pre_calibrated}, or
#'   \code{thrX_default}; a condition covered by none of these raises an
#'   error. To hold a condition at a fixed value while sweeping others, give
#'   it a length-one entry (e.g. \code{list(SPV = 6:8, PRD = 7)}).
#' @param thrY Numeric. Threshold for Y (fixed).
#' @param thrX_default Numeric or \code{NULL}. Threshold used to binarize any
#'   condition that appears in neither \code{sweep_list} nor
#'   \code{pre_calibrated}, mirroring the argument of the same name in
#'   \code{\link{ctSweepS}}. Default is \code{NULL}, in which case such a
#'   condition is an error rather than being silently dropped.
#' @param pre_calibrated Character vector or \code{NULL}. Names of condition
#'   variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
#'   and should be passed through to \code{QCA::truthTable()} without
#'   binarization. These variables must contain values in the \code{[0, 1]}
#'   range. Variables not listed here will be binarized using \code{sweep_list}
#'   thresholds as usual. Default is \code{NULL} (all variables binarized).
#'   It is recommended to sweep variables on their original (raw) scale rather
#'   than as pre-calibrated fuzzy values, because raw-scale thresholds are
#'   easier to interpret substantively.
#' @param dir.exp Directional expectations for \code{minimize}.
#'   If \code{NULL} (default), no directional expectations are applied.
#'   To compute the \strong{intermediate solution}, specify a numeric vector
#'   (1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
#'   for three conditions all expected to contribute positively.
#' @param include Inclusion rule for \code{minimize}. 
#'   \code{""} (default, QCA compatible) computes the \strong{complex solution}
#'   without logical remainders.
#'   Use \code{"?"} to include logical remainders for \strong{parsimonious}
#'   (with \code{dir.exp = NULL}) or \strong{intermediate} solutions
#'   (with \code{dir.exp} specified).
#' @param incl.cut Consistency cutoff for \code{truthTable}.
#' @param n.cut Frequency cutoff for \code{truthTable}.
#' @param pri.cut PRI cutoff for \code{minimize}.
#' @param extract_mode Character. How to handle multiple solutions:
#'   \code{"first"} (default), \code{"all"}, or \code{"essential"}.
#'   See \code{\link{qca_extract}} for details.
#' @param return_details Logical. If \code{TRUE} (default), returns both
#'   summary and detailed objects for use with \code{generate_report()}.
#' @param Yvar Deprecated. Use \code{outcome} instead.
#' @param Xvars Deprecated. Use \code{conditions} instead.
#'
#' @return
#' If \code{return_details = FALSE}, a data frame with columns:
#' \itemize{
#'   \item \code{combo_id} — index of the threshold combination
#'   \item \code{threshold} — character string summarizing thresholds,
#'     e.g. \code{"X1=6, X2=7, X3=7"}
#'   \item \code{expression} — minimized solution expression
#'   \item \code{inclS} — solution consistency
#'   \item \code{covS} — solution coverage
#'   \item (additional columns depending on \code{extract_mode})
#' }
#'
#' If \code{return_details = TRUE}, a list with:
#' \itemize{
#'   \item \code{summary} — the data frame above
#'   \item \code{details} — per-combination list of
#'     \code{combo_id}, \code{thrX_vec}, \code{truth_table}, \code{solution}
#' }
#'
#' Note that \code{return_details} changes the \emph{type} of the returned
#' object, not just its contents: with \code{TRUE} the summary table is at
#' \code{result$summary}, whereas with \code{FALSE} the summary table
#' \emph{is} the returned object and \code{result$summary} is \code{NULL}.
#' Code intended to work under both settings should branch on
#' \code{inherits(result, "data.frame")} (or simply always pass
#' \code{return_details = TRUE}) rather than assuming \code{result$summary}
#' exists.
#'
#' @importFrom QCA truthTable minimize
#' @export
#' @examples
#' # Load sample data
#' data(sample_data)
#' 
#' # === Three Types of QCA Solutions ===
#' 
#' # Quick demonstration with 2 conditions
#' sweep_list <- list(X1 = 7, X2 = 7)
#' 
#' # 1. Complex Solution (default, QCA compatible)
#' result_comp <- ctSweepM(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list = sweep_list,
#'   thrY = 7
#'   # include = "" (default), dir.exp = NULL (default)
#' )
#' head(result_comp$summary)
#' 
#' # 2. Parsimonious Solution (include = "?")
#' result_pars <- ctSweepM(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list = sweep_list,
#'   thrY = 7,
#'   include = "?"  # Include logical remainders
#' )
#' head(result_pars$summary)
#' 
#' # 3. Intermediate Solution (include = "?" + dir.exp)
#' result_int <- ctSweepM(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list = sweep_list,
#'   thrY = 7,
#'   include = "?",
#'   dir.exp = c(1, 1)  # Positive expectations
#' )
#' head(result_int$summary)
#' 
#' # === Threshold Sweep Example ===
#' 
#' # Using 2 conditions and 2 threshold levels
#' sweep_list <- list(
#'   X1 = 6:7,
#'   X2 = 6:7
#' )
#' 
#' # Run multiple condition threshold sweep (complex solutions by default)
#' result_quick <- ctSweepM(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list = sweep_list,
#'   thrY = 7
#' )
#' head(result_quick$summary)
#' 
#' # Run with negated outcome (~Y)
#' result_neg <- ctSweepM(
#'   dat = sample_data,
#'   outcome = "~Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list = sweep_list,
#'   thrY = 7
#' )
#' head(result_neg$summary)
#' 
#' \donttest{
#' # Full multi-condition analysis (27 combinations)
#' sweep_list_full <- list(
#'   X1 = 6:8,
#'   X2 = 6:8,
#'   X3 = 6:8
#' )
#' 
#' result_full <- ctSweepM(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_list = sweep_list_full,
#'   thrY = 7
#' )
#' head(result_full$summary)
#' }
ctSweepM <- function(dat, 
                     outcome = NULL, conditions = NULL,
                     sweep_list, thrY,
                     thrX_default = NULL,
                     pre_calibrated = NULL,
                     dir.exp = NULL, include = "",
                     incl.cut = 0.8, n.cut = 1, pri.cut = 0,
                     extract_mode = c("first", "all", "essential"),
                     return_details = TRUE,
                     Yvar = NULL, Xvars = NULL) {
  
  # === Backward compatibility for deprecated arguments ===
  if (!is.null(Yvar) && is.null(outcome)) {
    outcome <- Yvar
    warning("Argument 'Yvar' is deprecated. Use 'outcome' instead.",
            call. = FALSE)
  }
  if (!is.null(Xvars) && is.null(conditions)) {
    conditions <- Xvars
    warning("Argument 'Xvars' is deprecated. Use 'conditions' instead.",
            call. = FALSE)
  }
  
  # === Validate required arguments ===
  if (is.null(outcome)) {
    stop("Argument 'outcome' is required.")
  }
  if (is.null(conditions)) {
    stop("Argument 'conditions' is required.")
  }
  
  # === Handle negated outcome ===
  negate_outcome <- grepl("^~", outcome)
  outcome_clean <- sub("^~", "", outcome)
  
  # Validate outcome variable exists
  if (!outcome_clean %in% names(dat)) {
    stop("Variable '", outcome_clean, "' not found in data.")
  }
  
  # Validate condition variables exist
  missing_conds <- setdiff(conditions, names(dat))
  if (length(missing_conds) > 0) {
    stop("Condition variable(s) not found in data: ", 
         paste(missing_conds, collapse = ", "))
  }
  
  extract_mode <- match.arg(extract_mode)
  
  # === Validate coverage of all conditions (Issue: silent "No solution") ===
  # Every condition must get a binarization threshold from somewhere: either
  # sweep_list (swept or held at a fixed value), pre_calibrated (already fuzzy),
  # or thrX_default. A condition covered by none of these previously had no
  # column created in dat_bin at all, which made truthTable() fail and produced
  # "No solution" in every cell with no error or warning.
  uncovered <- setdiff(conditions, c(names(sweep_list), pre_calibrated))
  if (length(uncovered) > 0 && is.null(thrX_default)) {
    stop("Condition(s) not covered by 'sweep_list', 'pre_calibrated', or ",
         "'thrX_default': ", paste(uncovered, collapse = ", "), ".\n",
         "Every condition needs a threshold. Either add them to 'sweep_list' ",
         "as fixed values (e.g. sweep_list = list(",
         paste(names(sweep_list)[1], " = ", 
               if (length(sweep_list) > 0) paste0(min(sweep_list[[1]]), ":", max(sweep_list[[1]])) else "6:8",
               sep = ""),
         ", ", paste(paste0(uncovered, " = 7"), collapse = ", "), ")), ",
         "or supply 'thrX_default' to binarize them all at one value.",
         call. = FALSE)
  }
  
  combo_mat <- expand.grid(
    sweep_list,
    KEEP.OUT.ATTRS  = FALSE,
    stringsAsFactors = FALSE
  )
  
  n_combos <- nrow(combo_mat)
  
  # Initialize output data frame based on extract_mode
  df_out <- data.frame(
    combo_id    = seq_len(n_combos),
    threshold   = NA_character_,
    expression  = NA_character_,
    inclS       = NA_real_,
    covS        = NA_real_,
    n_solutions = NA_integer_,
    stringsAsFactors = FALSE
  )
  
  # Add columns based on extract_mode
  if (extract_mode == "essential") {
    df_out$selective_terms <- NA_character_
    df_out$unique_terms     <- NA_character_
  }
  
  details_list <- list()
  
  # Track combinations with multiple solutions (for warning in "first" mode)
  multi_sol_combos <- c()
  
  # Handle dir.exp: scalar -> expand to vector; NULL is passed through
  # NULL -> parsimonious solution; c(1,1,...) -> intermediate solution
  local_dir.exp <- dir.exp
  if (!is.null(local_dir.exp) && length(local_dir.exp) == 1) {
    local_dir.exp <- rep(local_dir.exp[1], length(conditions))
    names(local_dir.exp) <- conditions
  } else if (!is.null(local_dir.exp) && is.null(names(local_dir.exp))) {
    names(local_dir.exp) <- conditions
  }

  # Validate pre_calibrated (once, before loop)
  validate_pre_calibrated(pre_calibrated, conditions, dat)

  # Warn if pre_calibrated variable is also a sweep target in sweep_list
  if (!is.null(pre_calibrated)) {
    swept_vars <- names(sweep_list)[
      sapply(sweep_list, function(x) length(x) > 1)
    ]
    conflict <- intersect(pre_calibrated, swept_vars)
    if (length(conflict) > 0) {
      warning("Variable(s) ", paste(conflict, collapse = ", "),
              " are both pre_calibrated and in sweep_list. ",
              "Pre-calibrated values will be used; sweep thresholds ignored. ",
              "It is recommended to sweep variables on their original (raw) scale, ",
              "not as pre-calibrated fuzzy values, because threshold values on a ",
              "raw scale (e.g., Likert 1-10) are easier to interpret substantively. ",
              "See the package vignette section 'Choosing Sweep Variables' for details.",
              call. = FALSE)
    }
  }

  for (i in seq_len(n_combos)) {
    
    thrX_vec <- as.numeric(combo_mat[i, ])
    names(thrX_vec) <- names(combo_mat)
    
    thrX_label <- paste(names(thrX_vec), thrX_vec,
                        sep = "=", collapse = ", ")
    
    # Prepare data: binarize or pass through based on pre_calibrated
    # Note: prepare_dat_bin uses conditions (all vars); thrX_vec covers sweep_list vars only.
    # For conditions not in sweep_list, they are handled via pre_calibrated or thrX_vec lookup.
    dat_bin <- data.frame(Y = qca_bin(dat[[outcome_clean]], thrY))
    for (x in conditions) {
      if (!is.null(pre_calibrated) && x %in% pre_calibrated) {
        dat_bin[[x]] <- dat[[x]]
      } else if (x %in% names(thrX_vec)) {
        dat_bin[[x]] <- qca_bin(dat[[x]], thrX_vec[x])
      } else {
        # Not swept and not pre-calibrated: fall back to thrX_default.
        # Guaranteed non-NULL here by the validation above.
        dat_bin[[x]] <- qca_bin(dat[[x]], thrX_default)
      }
    }
    
    # Determine outcome string for truthTable (with ~ if negated)
    outcome_tt <- if (negate_outcome) "~Y" else "Y"
    
    tt <- try(
      QCA::truthTable(
        dat_bin,
        outcome    = outcome_tt,
        conditions = conditions,
        show.cases = FALSE,
        incl.cut1  = incl.cut,
        n.cut      = n.cut,
        pri.cut    = pri.cut
      ),
      silent = TRUE
    )
    
    if (inherits(tt, "try-error")) {
      df_out$threshold[i]   <- thrX_label
      df_out$expression[i]  <- "No solution"
      df_out$inclS[i]       <- NA_real_
      df_out$covS[i]        <- NA_real_
      df_out$n_solutions[i] <- 0L
      
      if (extract_mode == "essential") {
        df_out$selective_terms[i] <- NA_character_
        df_out$unique_terms[i]     <- NA_character_
      }
      
      if (return_details) {
        details_list[[i]] <- list(
          combo_id    = i,
          thrX_vec    = thrX_vec,
          truth_table = NULL,
          solution    = NULL,
          dat_bin     = NULL
        )
      }
      next
    }
    
    tt <- sanitize_truthtable(tt)  # QCA 3.25 guard: char incl/PRI + '-' remainders
    sol <- try(
      QCA::minimize(
        tt,
        include    = include,
        dir.exp    = local_dir.exp,
        details    = TRUE,
        show.cases = FALSE,
        pri.cut    = pri.cut
      ),
      silent = TRUE
    )
    
    if (inherits(sol, "try-error")) {
      df_out$threshold[i]   <- thrX_label
      df_out$expression[i]  <- "No solution"
      df_out$inclS[i]       <- NA_real_
      df_out$covS[i]        <- NA_real_
      df_out$n_solutions[i] <- 0L
      
      if (extract_mode == "essential") {
        df_out$selective_terms[i] <- NA_character_
        df_out$unique_terms[i]     <- NA_character_
      }
      
      if (return_details) {
        details_list[[i]] <- list(
          combo_id    = i,
          thrX_vec    = thrX_vec,
          truth_table = tt,
          solution    = NULL,
          dat_bin     = dat_bin
        )
      }
      next
    }
    
    sol_info <- qca_extract(sol, extract_mode = extract_mode)
    
    # Track multiple solutions
    if (sol_info$n_solutions > 1) {
      multi_sol_combos <- c(multi_sol_combos, i)
    }
    
    df_out$threshold[i]   <- thrX_label
    df_out$expression[i]  <- sol_info$expression
    df_out$inclS[i]       <- sol_info$inclS
    df_out$covS[i]        <- sol_info$covS
    df_out$n_solutions[i] <- sol_info$n_solutions
    
    if (extract_mode == "essential") {
      df_out$selective_terms[i] <- sol_info$selective_terms
      df_out$unique_terms[i]     <- sol_info$unique_terms
    }
    
    if (return_details) {
      details_list[[i]] <- list(
        combo_id    = i,
        thrX_vec    = thrX_vec,
        truth_table = tt,
        solution    = sol,
        dat_bin     = dat_bin
      )
    }
  }
  
  # Issue warning for multiple solutions
  if (length(multi_sol_combos) > 0) {
    n_multi <- length(multi_sol_combos)
    warning(
      "Multiple intermediate solutions exist for ", n_multi, " combination(s) ",
      "(n_solutions > 1). ",
      "Only the first solution (M1) and its fit metrics are shown. ",
      "Use generate_report() for full analysis.",
      call. = FALSE
    )
  }
  
  if (return_details) {
    result <- list(
      summary = df_out, 
      details = details_list,
      params = list(
        outcome = outcome,
        conditions = conditions,
        negate_outcome = negate_outcome,
        pre_calibrated = pre_calibrated,
        sweep_list = sweep_list,
        thrY = thrY,
        incl.cut = incl.cut,
        n.cut = n.cut,
        pri.cut = pri.cut,
        include = include,
        dir.exp = dir.exp  # Store original value for reproducibility
      )
    )
    class(result) <- c("ctSweepM_result", "tsqca_result", "list")
    return(result)
  }
  
  df_out
}

test_that("ctSweepS runs and returns valid output", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data
  
  res <- ctSweepS(
    dat = dat,
    Yvar = "Y",
    Xvars = c("X1", "X2", "X3"),
    sweep_var = "X3",
    sweep_range = 6:8,
    thrY = 7,
    thrX_default = 7,
    return_details = FALSE
  )
  
  # 1. Output must be a data frame
  expect_s3_class(res, "data.frame")
  
  # 2. Must contain at least one row
  expect_true(nrow(res) > 0)
  
  # 3. Core columns must exist
  required_cols <- c("expression", "inclS", "covS")
  expect_true(all(required_cols %in% colnames(res)))
  
  # 4. Must include at least one threshold-related column
  thr_cols <- grep("thr|threshold", colnames(res), ignore.case = TRUE, value = TRUE)
  expect_true(length(thr_cols) >= 1)
})

test_that("ctSweepM runs and returns valid output", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data
  
  res <- ctSweepM(
    dat = dat,
    Yvar = "Y",
    Xvars = c("X1", "X2", "X3"),
    sweep_list = list(X1 = 6:7, X2 = 6:7, X3 = 7),
    thrY = 7,
    return_details = FALSE
  )
  
  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) > 0)
  required_cols <- c("expression", "inclS", "covS")
  expect_true(all(required_cols %in% colnames(res)))
  thr_cols <- grep("thr|threshold", colnames(res), ignore.case = TRUE, value = TRUE)
  expect_true(length(thr_cols) >= 1)
})

# tests/testthat/test-s3-methods.R

test_that("otSweep returns correct S3 class", {
  data(sample_data)
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)
  
  result <- otSweep(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX = thrX
  )
  
  expect_s3_class(result, "otSweep_result")
  expect_s3_class(result, "tsqca_result")
  expect_s3_class(result, "list")
})

test_that("dtSweep returns correct S3 class", {
  data(sample_data)
  
  result <- dtSweep(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_list_X = list(X1 = 7, X2 = 7, X3 = 7),
    sweep_range_Y = 7:8
  )
  
  expect_s3_class(result, "dtSweep_result")
  expect_s3_class(result, "tsqca_result")
  expect_s3_class(result, "list")
})

test_that("ctSweepS returns correct S3 class", {
  data(sample_data)
  
  result <- ctSweepS(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_var = "X3",
    sweep_range = 6:8,
    thrY = 7,
    thrX_default = 7
  )
  
  expect_s3_class(result, "ctSweepS_result")
  expect_s3_class(result, "tsqca_result")
  expect_s3_class(result, "list")
})

test_that("ctSweepM returns correct S3 class", {
  data(sample_data)
  
  result <- ctSweepM(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_list = list(X1 = 7, X2 = 7, X3 = 6:7),
    thrY = 7
  )
  
  expect_s3_class(result, "ctSweepM_result")
  expect_s3_class(result, "tsqca_result")
  expect_s3_class(result, "list")
})

test_that("print.otSweep_result works without error", {
  data(sample_data)
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)
  
  result <- otSweep(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX = thrX
  )
  
  expect_output(print(result), "OTS-QCA")
})

test_that("summary.otSweep_result works without error", {
  data(sample_data)
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)
  
  result <- otSweep(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX = thrX
  )
  
  expect_output(summary(result), "Summary")
})

test_that("$ accessor still works after S3 class assignment", {
  data(sample_data)
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)
  
  result <- otSweep(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX = thrX
  )
  
  # Existing access methods should still work

  expect_true(is.data.frame(result$summary))
  expect_true(is.list(result$details))
  expect_true(is.list(result$params))
})

test_that("return_details = FALSE does not add S3 class", {
  data(sample_data)
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)
  
  result <- otSweep(
    dat = sample_data,
    outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX = thrX,
    return_details = FALSE
  )
  
  # Should be a plain data.frame, not S3 class
  expect_true(is.data.frame(result))
  expect_false(inherits(result, "tsqca_result"))
})

test_that("print works with negated outcome", {
  data(sample_data)
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)
  
  result <- otSweep(
    dat = sample_data,
    outcome = "~Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX = thrX
  )
  
  expect_output(print(result), "negated")
})

# =============================================================================
# Tests for pre_calibrated parameter (ThSQCA v2.0.0)
# =============================================================================
# Test coverage:
#   [BC] Backward compatibility  — pre_calibrated = NULL produces v1.2.0 output
#   [T]  Functional tests        — T-01 through T-06 from design spec
#   [V]  Validation tests        — V-01 through V-04 from design spec
#   [C]  Consistency test        — manual binarization equivalence
#   [E]  extract_mode="essential" x pre_calibrated (confirmation item 2)
# =============================================================================

# ---------------------------------------------------------------------------
# Helper: build test data with one pre-calibrated (fuzzy) column
# ---------------------------------------------------------------------------
make_test_dat <- function() {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data
  # X2_fuzzy: manually fuzzy-calibrate X2 (range ~1-10) into [0,1]
  # simple linear rescale so values stay in [0,1]
  x2_raw <- dat$X2
  dat$X2_fuzzy <- (x2_raw - min(x2_raw)) / (max(x2_raw) - min(x2_raw))
  # X3_bin: hard binary (already 0/1-like after threshold)
  dat$X3_bin <- ifelse(dat$X3 >= 7, 1L, 0L)
  dat
}

# =============================================================================
# [BC] Backward Compatibility Tests
# =============================================================================

test_that("[BC] otSweep: pre_calibrated=NULL produces identical output to v1.2.0", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)

  res_default <- otSweep(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8, thrX = thrX, return_details = FALSE
  )
  res_null <- otSweep(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8, thrX = thrX,
    pre_calibrated = NULL, return_details = FALSE
  )

  expect_equal(res_default, res_null)
})

test_that("[BC] dtSweep: pre_calibrated=NULL produces identical output to v1.2.0", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data

  res_default <- dtSweep(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_list_X = list(X1 = 6:7, X2 = 7, X3 = 7),
    sweep_range_Y = 6:7, return_details = FALSE
  )
  res_null <- dtSweep(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_list_X = list(X1 = 6:7, X2 = 7, X3 = 7),
    sweep_range_Y = 6:7,
    pre_calibrated = NULL, return_details = FALSE
  )

  expect_equal(res_default, res_null)
})

test_that("[BC] ctSweepS: pre_calibrated=NULL produces identical output to v1.2.0", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data

  res_default <- ctSweepS(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_var = "X3", sweep_range = 6:8, thrY = 7, thrX_default = 7,
    return_details = FALSE
  )
  res_null <- ctSweepS(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_var = "X3", sweep_range = 6:8, thrY = 7, thrX_default = 7,
    pre_calibrated = NULL, return_details = FALSE
  )

  expect_equal(res_default, res_null)
})

test_that("[BC] ctSweepM: pre_calibrated=NULL produces identical output to v1.2.0", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data

  res_default <- ctSweepM(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_list = list(X1 = 6:7, X2 = 7, X3 = 7),
    thrY = 7, return_details = FALSE
  )
  res_null <- ctSweepM(
    dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
    sweep_list = list(X1 = 6:7, X2 = 7, X3 = 7),
    thrY = 7, pre_calibrated = NULL, return_details = FALSE
  )

  expect_equal(res_default, res_null)
})

# =============================================================================
# [T] Functional Tests (T-01 to T-06)
# =============================================================================

test_that("[T-01] otSweep: single fuzzy condition runs and returns valid output", {
  dat <- make_test_dat()

  res <- otSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_range = 6:8,
    thrX        = c(X1 = 7, X2_fuzzy = 0.5, X3 = 7),
    pre_calibrated = c("X2_fuzzy"),
    return_details = TRUE
  )

  expect_s3_class(res, "otSweep_result")
  expect_true(nrow(res$summary) > 0)
  expect_true(all(c("expression", "inclS", "covS") %in% names(res$summary)))
  # pre_calibrated stored in params
  expect_equal(res$params$pre_calibrated, c("X2_fuzzy"))
})

test_that("[T-01] otSweep: fuzzy condition produces different result from crisp binarization", {
  dat <- make_test_dat()
  thrX_crisp  <- c(X1 = 7, X2 = 7,   X3 = 7)
  thrX_fuzzy  <- c(X1 = 7, X2_fuzzy = 0.5, X3 = 7)

  res_crisp <- otSweep(
    dat = dat, outcome = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8, thrX = thrX_crisp, return_details = FALSE
  )
  res_fuzzy <- otSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_range = 6:8, thrX = thrX_fuzzy,
    pre_calibrated = c("X2_fuzzy"), return_details = FALSE
  )

  # Results should differ because X2_fuzzy retains graded membership
  # (at least one expression or fit metric will differ)
  expect_false(identical(res_crisp$expression, res_fuzzy$expression) &&
               identical(res_crisp$inclS,      res_fuzzy$inclS) &&
               identical(res_crisp$covS,        res_fuzzy$covS))
})

test_that("[T-02] otSweep: all conditions fuzzy runs without error", {
  dat <- make_test_dat()
  dat$X1_fuzzy <- (dat$X1 - min(dat$X1)) / (max(dat$X1) - min(dat$X1))

  res <- otSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1_fuzzy", "X2_fuzzy", "X3_bin"),
    sweep_range = 6:8,
    thrX        = c(X1_fuzzy = 0.5, X2_fuzzy = 0.5, X3_bin = 1),
    pre_calibrated = c("X1_fuzzy", "X2_fuzzy", "X3_bin"),
    return_details = FALSE
  )

  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) > 0)
})

test_that("[T-03] dtSweep: mixed fuzzy/crisp conditions run without error", {
  dat <- make_test_dat()

  res <- dtSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_list_X  = list(X1 = 6:7, X2_fuzzy = 0.5, X3 = 7),
    sweep_range_Y = 6:7,
    pre_calibrated = c("X2_fuzzy"),
    return_details = FALSE
  )

  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) > 0)
})

test_that("[T-04] ctSweepS: pre_calibrated non-swept conditions pass through", {
  dat <- make_test_dat()

  res <- ctSweepS(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_var   = "X1",
    sweep_range = 6:8,
    thrY        = 7, thrX_default = 7,
    pre_calibrated = c("X2_fuzzy"),
    return_details = TRUE
  )

  expect_s3_class(res, "ctSweepS_result")
  expect_true(nrow(res$summary) > 0)
  expect_equal(res$params$pre_calibrated, c("X2_fuzzy"))
})

test_that("[T-05] ctSweepM: pre_calibrated non-swept conditions pass through", {
  dat <- make_test_dat()

  res <- ctSweepM(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_list  = list(X1 = 6:7, X2_fuzzy = 0.5, X3 = 7),
    thrY        = 7,
    pre_calibrated = c("X2_fuzzy"),
    return_details = TRUE
  )

  expect_s3_class(res, "ctSweepM_result")
  expect_true(nrow(res$summary) > 0)
  expect_equal(res$params$pre_calibrated, c("X2_fuzzy"))
})

test_that("[T-06] Binary variable as pre_calibrated passes through unchanged", {
  dat <- make_test_dat()

  # X3_bin is already 0/1 — using pre_calibrated should give same result as
  # binarizing with thrX = 1
  res_pc <- otSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2", "X3_bin"),
    sweep_range = 6:8,
    thrX        = c(X1 = 7, X2 = 7, X3_bin = 1),
    pre_calibrated = c("X3_bin"),
    return_details = FALSE
  )
  res_thr <- otSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2", "X3_bin"),
    sweep_range = 6:8,
    thrX        = c(X1 = 7, X2 = 7, X3_bin = 1),
    return_details = FALSE
  )

  expect_equal(res_pc$expression, res_thr$expression)
  expect_equal(res_pc$inclS,      res_thr$inclS)
  expect_equal(res_pc$covS,       res_thr$covS)
})

# =============================================================================
# [V] Validation Tests (V-01 to V-04)
# =============================================================================

test_that("[V-01] Error when pre_calibrated variable not in conditions", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data

  expect_error(
    otSweep(
      dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
      sweep_range = 6:8, thrX = c(X1 = 7, X2 = 7, X3 = 7),
      pre_calibrated = c("X99")
    ),
    regexp = "not found in conditions"
  )
})

test_that("[V-02] Error when pre_calibrated variable has values outside [0, 1]", {
  data("sample_data", package = "ThSQCA")
  dat <- sample_data  # X1 has raw Likert values (e.g., 1-10)

  expect_error(
    otSweep(
      dat = dat, outcome = "Y", conditions = c("X1", "X2", "X3"),
      sweep_range = 6:8, thrX = c(X1 = 7, X2 = 7, X3 = 7),
      pre_calibrated = c("X1")  # X1 is raw scale, not [0,1]
    ),
    regexp = "outside \\[0, 1\\]"
  )
})

test_that("[V-03] Warning when pre_calibrated variable contains NA", {
  dat <- make_test_dat()
  dat$X2_fuzzy[1] <- NA  # inject NA

  expect_warning(
    otSweep(
      dat = dat, outcome = "Y",
      conditions  = c("X1", "X2_fuzzy", "X3"),
      sweep_range = 6:8,
      thrX        = c(X1 = 7, X2_fuzzy = 0.5, X3 = 7),
      pre_calibrated = c("X2_fuzzy")
    ),
    regexp = "contains NA"
  )
})

test_that("[V-04] Warning when pre_calibrated variable is also a sweep target in dtSweep", {
  dat <- make_test_dat()

  expect_warning(
    dtSweep(
      dat = dat, outcome = "Y",
      conditions  = c("X1", "X2_fuzzy", "X3"),
      sweep_list_X  = list(X1 = 6:7, X2_fuzzy = c(0.3, 0.5, 0.7), X3 = 7),
      sweep_range_Y = 6:7,
      pre_calibrated = c("X2_fuzzy")
    ),
    regexp = "sweep thresholds ignored"
  )
})

test_that("[V-04] Warning when pre_calibrated variable is also a sweep target in ctSweepM", {
  dat <- make_test_dat()

  expect_warning(
    ctSweepM(
      dat = dat, outcome = "Y",
      conditions  = c("X1", "X2_fuzzy", "X3"),
      sweep_list  = list(X1 = 6:7, X2_fuzzy = c(0.3, 0.5, 0.7), X3 = 7),
      thrY        = 7,
      pre_calibrated = c("X2_fuzzy")
    ),
    regexp = "sweep thresholds ignored"
  )
})

# =============================================================================
# [C] Consistency Test — manual binarization equivalence
# =============================================================================

test_that("[C] Manual binarization at 0.5 equals pre_calibrated=NULL with thrX=0.5", {
  dat <- make_test_dat()

  # Method A: pass fuzzy values, let qca_bin() binarize at 0.5 (default behavior)
  res_a <- otSweep(
    dat = dat, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_range = 6:8,
    thrX        = c(X1 = 7, X2_fuzzy = 0.5, X3 = 7),
    pre_calibrated = NULL,   # binarize X2_fuzzy at 0.5
    return_details = FALSE
  )

  # Method B: manually binarize X2_fuzzy at 0.5 before passing
  dat_b <- dat
  dat_b$X2_fuzzy <- ifelse(dat$X2_fuzzy >= 0.5, 1L, 0L)
  res_b <- otSweep(
    dat = dat_b, outcome = "Y",
    conditions  = c("X1", "X2_fuzzy", "X3"),
    sweep_range = 6:8,
    thrX        = c(X1 = 7, X2_fuzzy = 1, X3 = 7),
    pre_calibrated = c("X2_fuzzy"),  # already 0/1, pass through
    return_details = FALSE
  )

  expect_equal(res_a$expression, res_b$expression)
  expect_equal(res_a$inclS,      res_b$inclS,      tolerance = 1e-10)
  expect_equal(res_a$covS,       res_b$covS,        tolerance = 1e-10)
})

# =============================================================================
# [E] extract_mode="essential" x pre_calibrated (confirmation item 2)
# =============================================================================

test_that("[E] extract_mode='essential' works with pre_calibrated fuzzy condition", {
  dat <- make_test_dat()

  res <- otSweep(
    dat = dat, outcome = "Y",
    conditions   = c("X1", "X2_fuzzy", "X3"),
    sweep_range  = 6:8,
    thrX         = c(X1 = 7, X2_fuzzy = 0.5, X3 = 7),
    pre_calibrated = c("X2_fuzzy"),
    extract_mode = "essential",
    return_details = TRUE
  )

  expect_s3_class(res, "otSweep_result")
  expect_true("n_solutions" %in% names(res$summary))
  # expression column must exist and be character (EPI or "No essential prime implicants")
  expect_true(is.character(res$summary$expression))
})

test_that("[E] extract_mode='all' works with pre_calibrated fuzzy condition", {
  dat <- make_test_dat()

  res <- otSweep(
    dat = dat, outcome = "Y",
    conditions   = c("X1", "X2_fuzzy", "X3"),
    sweep_range  = 6:8,
    thrX         = c(X1 = 7, X2_fuzzy = 0.5, X3 = 7),
    pre_calibrated = c("X2_fuzzy"),
    extract_mode = "all",
    return_details = TRUE
  )

  expect_s3_class(res, "otSweep_result")
  expect_true("n_solutions" %in% names(res$summary))
})

test_that("[E] EPI identification unaffected by variable being fuzzy vs crisp", {
  dat <- make_test_dat()

  # essential mode with pre_calibrated fuzzy variable
  res_fuzzy <- otSweep(
    dat = dat, outcome = "Y",
    conditions   = c("X1", "X2_fuzzy", "X3"),
    sweep_range  = 6:8,
    thrX         = c(X1 = 7, X2_fuzzy = 0.5, X3 = 7),
    pre_calibrated = c("X2_fuzzy"),
    extract_mode = "essential",
    return_details = FALSE
  )

  # expression column must not be NA for all rows
  non_na_count <- sum(!is.na(res_fuzzy$expression))
  expect_true(non_na_count > 0)
  # selective_terms and unique_terms columns must exist
  expect_true("selective_terms" %in% names(res_fuzzy))
  expect_true("unique_terms"    %in% names(res_fuzzy))
})

###############################################
# Core utilities for ThSQCA
###############################################

#' Binary calibration helper for ThSQCA
#'
#' Converts a numeric vector into a crisp set (0/1) based on a threshold.
#'
#' @param x Numeric vector.
#' @param thr Numeric scalar. Cases with \code{x >= thr} are coded as 1,
#'   others as 0.
#'
#' @return Integer vector of 0/1 with the same length as \code{x}.
#' @keywords internal
qca_bin <- function(x, thr) {
  ifelse(x >= thr, 1L, 0L)
}

#' Prepare analysis data frame for QCA::truthTable()
#'
#' Constructs the data frame to be passed to \code{QCA::truthTable()}.
#' For pre-calibrated variables, the original values are passed through
#' without binarization. For all other variables, \code{qca_bin()} is applied.
#'
#' @param dat Original data frame.
#' @param outcome_clean Character. Outcome variable name (without \code{~}).
#' @param conditions Character vector. Condition variable names.
#' @param thrY Numeric. Threshold for outcome binarization.
#' @param thrX_vec Named numeric vector. Thresholds for conditions.
#' @param pre_calibrated Character vector or NULL. Names of pre-calibrated
#'   variables to pass through without binarization.
#'
#' @return Data frame with column \code{Y} (binarized outcome) and condition
#'   columns (binarized or passed through).
#' @keywords internal
prepare_dat_bin <- function(dat, outcome_clean, conditions,
                            thrY, thrX_vec,
                            pre_calibrated = NULL) {

  # Outcome: always binarize (threshold sweeping is the core purpose of TS-QCA)
  dat_bin <- data.frame(Y = qca_bin(dat[[outcome_clean]], thrY))

  # Conditions: binarize or pass through
  for (x in conditions) {
    if (!is.null(pre_calibrated) && x %in% pre_calibrated) {
      # Pass through: use original values (fuzzy membership or binary 0/1)
      dat_bin[[x]] <- dat[[x]]
    } else {
      # Binarize: apply threshold
      dat_bin[[x]] <- qca_bin(dat[[x]], thrX_vec[x])
    }
  }

  dat_bin
}

#' Validate the pre_calibrated parameter
#'
#' Checks that all names in \code{pre_calibrated} exist in \code{conditions}
#' and that the corresponding values in \code{dat} are within the \code{[0, 1]}
#' range required for fuzzy membership scores.
#'
#' @param pre_calibrated Character vector or NULL.
#' @param conditions Character vector. Valid condition variable names.
#' @param dat Data frame containing the variables.
#'
#' @return Invisible NULL. Raises errors or warnings as needed.
#' @keywords internal
validate_pre_calibrated <- function(pre_calibrated, conditions, dat) {
  if (is.null(pre_calibrated)) return(invisible(NULL))

  # All names must exist in conditions
  invalid <- setdiff(pre_calibrated, conditions)
  if (length(invalid) > 0) {
    stop("pre_calibrated variable(s) not found in conditions: ",
         paste(invalid, collapse = ", "),
         call. = FALSE)
  }

  # Values must be in [0, 1]
  for (v in pre_calibrated) {
    vals <- dat[[v]]
    if (any(is.na(vals))) {
      warning("pre_calibrated variable '", v, "' contains NA values.",
              call. = FALSE)
    }
    rng <- range(vals, na.rm = TRUE)
    if (rng[1] < 0 || rng[2] > 1) {
      stop("pre_calibrated variable '", v,
           "' has values outside [0, 1] range: [",
           round(rng[1], 4), ", ", round(rng[2], 4), "]. ",
           "Apply QCA::calibrate() before passing to the sweep function.",
           call. = FALSE)
    }
  }

  invisible(NULL)
}

#' Collect and deduplicate the solution models attached to a QCA::minimize()
#' result
#'
#' When \code{dir.exp} is specified and the minimization produces more than
#' one prime implicant chart (QCA's own \code{C1}, \code{C2}, ... indexing,
#' visible as \code{"From C1P1, C2P1:"} in \code{print()}), \code{sol$i.sol}
#' contains one list entry per chart/path combination (e.g. \code{C1P1},
#' \code{C2P1}). Each entry's own \code{$solution} field is computed
#' independently by \code{QCA::minimize()} (via its internal
#' \code{getSolution()} call) and is NOT guaranteed to hold only that chart's
#' own model: depending on the data, it may already enumerate the full,
#' cross-chart set of tied minimal models. Concatenating every chart's
#' \code{$solution} list (the previous approach used here and in
#' \code{generate_report()}) therefore risks double- or N-fold counting the
#' same model, inflating the reported number of solutions.
#'
#' This helper performs the same enumeration, then deduplicates by comparing
#' each model's term set (order-independent), so structurally identical
#' models are counted once regardless of which chart(s) produced them. This
#' mirrors the manual "remove duplicate Boolean expressions" step that is
#' standard practice when reporting multiple minimal solutions by hand.
#'
#' @param sol A solution object returned by \code{QCA::minimize()}.
#' @return A list of unique solution term-vectors, or \code{NULL} if none
#'   were found.
#' @note When dir.exp is specified, the true Intermediate solution is stored in
#'   sol$i.sol, not sol$solution (which contains the Parsimonious solution).
#' @keywords internal
collect_unique_i_sol <- function(sol) {
  if (is.null(sol)) return(NULL)

  raw <- list()

  # Priority 1: i.sol structure (contains true Intermediate solution when dir.exp specified)
  if (!is.null(sol$i.sol) && length(sol$i.sol) > 0) {
    for (model_name in names(sol$i.sol)) {
      model_sols <- sol$i.sol[[model_name]]$solution
      if (!is.null(model_sols) && length(model_sols) > 0) {
        for (s in model_sols) {
          raw <- c(raw, list(s))
        }
      }
    }
  }

  # Fallback: sol$solution (for Parsimonious or when dir.exp not specified)
  if (length(raw) == 0) {
    sol_list <- try(sol$solution, silent = TRUE)
    if (!inherits(sol_list, "try-error") && !is.null(sol_list) && length(sol_list) > 0) {
      raw <- sol_list
    }
  }

  if (length(raw) == 0) return(NULL)

  # Deduplicate by term-set content: same terms regardless of order/source
  # chart are the same model.
  keys <- vapply(raw, function(x) paste(sort(as.character(x)), collapse = " | "), character(1))
  raw[!duplicated(keys)]
}

#' Get the number of intermediate solutions
#'
#' @param sol A solution object returned by \code{QCA::minimize()}.
#' @return Integer. Number of intermediate solutions, or 0 if none.
#' @note When dir.exp is specified, the true Intermediate solution is stored in
#'   sol$i.sol, not sol$solution (which contains the Parsimonious solution).
#'   See \code{\link{collect_unique_i_sol}} for why deduplication is required
#'   when multiple prime implicant charts are present.
#' @keywords internal
get_n_solutions <- function(sol) {
  if (is.null(sol)) return(0L)

  uniq <- collect_unique_i_sol(sol)
  if (is.null(uniq)) return(0L)
  length(uniq)
}

#' Extract solution information from a QCA minimization result
#'
#' Internal helper to obtain the solution expression, consistency
#' (\code{inclS}) and coverage (\code{covS}) from an object returned by
#' \code{QCA::minimize()}.
#'
#' @param sol A solution object returned by \code{QCA::minimize()}.
#' @param extract_mode Character. How to handle multiple intermediate solutions:
#'   \itemize{
#'     \item \code{"first"} - return only the first solution (M1). Default.
#'     \item \code{"all"} - return all solutions concatenated.
#'     \item \code{"essential"} - return essential prime implicants (terms 
#'       common to all solutions), plus selective prime implicants and 
#'       solution count.
#'   }
#'
#' @return A list with elements depending on \code{extract_mode}.
#'
#'   For \code{"first"}: \code{expression}, \code{inclS}, \code{covS}.
#'
#'   For \code{"all"}: adds \code{n_solutions}.
#'
#'   For \code{"essential"}: adds \code{selective_terms}, \code{unique_terms},
#'   \code{n_solutions}.
#'
#'   If extraction fails, returns \code{"No solution"} and \code{NA_real_}
#'   for numeric values.
#' @keywords internal
qca_extract <- function(sol, extract_mode = c("first", "all", "essential")) {
  
  extract_mode <- match.arg(extract_mode)
  
  # Base null response
  null_response <- function(mode) {
    base <- list(
      expression   = "No solution",
      inclS        = NA_real_,
      covS         = NA_real_,
      n_solutions  = 0L
    )
    if (mode == "essential") {
      base$selective_terms <- NA_character_
      base$unique_terms     <- NA_character_
    }
    base
  }
  
  if (is.null(sol)) {
    return(null_response(extract_mode))
  }
  
  # === Priority 1: i.sol structure (true Intermediate solution when dir.exp specified) ===
  sol_list <- NULL
  # Track whether the displayed expression comes from the i.sol (dir.exp intermediate)
  # structure or from sol$solution (parsimonious/complex). The per-solution fit lives
  # in a different place for each, so this flag guards the fit lookup below.
  used_isol <- FALSE

  # Try i.sol first (contains true Intermediate solution when dir.exp specified)
  if (!is.null(sol$i.sol) && length(sol$i.sol) > 0) {
    sol_list <- try(sol$i.sol$C1P1$solution, silent = TRUE)
    if (inherits(sol_list, "try-error") || is.null(sol_list) || length(sol_list) == 0) {
      # Try first i.sol entry
      sol_list <- try(sol$i.sol[[1]]$solution, silent = TRUE)
      if (inherits(sol_list, "try-error")) sol_list <- NULL
    }
    if (!is.null(sol_list) && length(sol_list) > 0) used_isol <- TRUE
  }
  
  # Fallback: sol$solution (for Parsimonious or when dir.exp not specified)
  if (is.null(sol_list) || length(sol_list) == 0) {
    if (!is.null(sol$solution) && length(sol$solution) > 0) {
      sol_list <- sol$solution
      used_isol <- FALSE
    }
  }
  
  if (is.null(sol_list) || length(sol_list) == 0) {
    return(null_response(extract_mode))
  }
  
  # === Fit measures: source them from the SAME solution the expression shows ===
  #
  # The displayed expression (sol_list) is either the intermediate solution
  # (sol$i.sol, when dir.exp was given; used_isol == TRUE) or the
  # parsimonious/complex solution (sol$solution; used_isol == FALSE). The fit for
  # each lives in a different place, so we branch on used_isol and never let the
  # intermediate branch read sol$IC (which describes the parsimonious solution).
  inclS <- NA_real_
  covS <- NA_real_

  if (used_isol) {
    # --- Intermediate solution (dir.exp): fit comes from the intermediate itself.
    # sol$IC (sol.incl.cov / individual / overall) all describe the PARSIMONIOUS
    # solution, so reading them here attaches the parsimonious fit to the displayed
    # intermediate formula. The intermediate fit is under sol$i.sol$C1P1$IC, and no
    # sol$IC fallback is used.
    #
    # That intermediate IC has the SAME two shapes as sol$IC: flat
    # ($sol.incl.cov) when the intermediate solution is unique, and split into
    # $individual[[k]] / $overall when the intermediate solution itself has
    # several minimal solutions. Both shapes must be handled, otherwise the
    # multiple-intermediate-solution case yields no fit at all.
    isol_ic <- try(sol$i.sol$C1P1$IC, silent = TRUE)
    if (inherits(isol_ic, "try-error") || is.null(isol_ic)) {
      # Rare shape where C1P1 is absent: first i.sol entry.
      isol_ic <- try(sol$i.sol[[1]]$IC, silent = TRUE)
    }
    if (!inherits(isol_ic, "try-error") && !is.null(isol_ic)) {
      # Path A1: unique intermediate solution.
      ic_flat <- try(isol_ic$sol.incl.cov, silent = TRUE)
      if (!inherits(ic_flat, "try-error") && !is.null(ic_flat)) {
        if (!is.null(ic_flat$inclS)) inclS <- ic_flat$inclS[1]
        if (!is.null(ic_flat$covS))  covS  <- ic_flat$covS[1]
      }
      # Path A2: several intermediate solutions, extract_mode = "first". The
      # displayed expression is the first of them, so use its own fit.
      if (identical(extract_mode, "first") && (is.na(inclS) || is.na(covS)) &&
          !is.null(isol_ic$individual) && length(isol_ic$individual) >= 1L) {
        ic_ind <- try(isol_ic$individual[[1L]]$sol.incl.cov, silent = TRUE)
        if (!inherits(ic_ind, "try-error") && !is.null(ic_ind)) {
          if (is.na(inclS) && !is.null(ic_ind$inclS)) inclS <- ic_ind$inclS[1]
          if (is.na(covS)  && !is.null(ic_ind$covS))  covS  <- ic_ind$covS[1]
        }
      }
      # Path A3: aggregate across the intermediate solutions. Correct for
      # "all"/"essential" (which summarize across solutions) and a last resort.
      if (is.na(inclS) || is.na(covS)) {
        ic_ov <- try(isol_ic$overall$sol.incl.cov, silent = TRUE)
        if (!inherits(ic_ov, "try-error") && !is.null(ic_ov)) {
          if (is.na(inclS) && !is.null(ic_ov$inclS)) inclS <- ic_ov$inclS[1]
          if (is.na(covS)  && !is.null(ic_ov$covS))  covS  <- ic_ov$covS[1]
        }
      }
    }
  } else {
    # --- Parsimonious / complex solution: 2.0.3 behavior (includes the Bug A fix).

    # Path 1: sol$IC$sol.incl.cov (single solution).
    if (is.na(inclS)) {
      incl_val <- try(sol$IC$sol.incl.cov$inclS, silent = TRUE)
      if (!inherits(incl_val, "try-error") && !is.null(incl_val)) inclS <- incl_val
    }
    if (is.na(covS)) {
      cov_val <- try(sol$IC$sol.incl.cov$covS, silent = TRUE)
      if (!inherits(cov_val, "try-error") && !is.null(cov_val)) covS <- cov_val
    }

    # Path 1b: multiple solutions, extract_mode = "first". The displayed expression
    # is the first solution (M1), so its fit is sol$IC$individual[[1]]$sol.incl.cov,
    # NOT the overall aggregate (the disjunction of ALL solutions, which is >= any
    # single solution's coverage on fuzzy data). Crisp data: individual == overall.
    if (identical(extract_mode, "first") &&
        !is.null(sol$IC$individual) && length(sol$IC$individual) >= 1L) {
      ic_first <- try(sol$IC$individual[[1L]]$sol.incl.cov, silent = TRUE)
      if (!inherits(ic_first, "try-error") && !is.null(ic_first)) {
        if (is.na(inclS) && !is.null(ic_first$inclS)) inclS <- ic_first$inclS
        if (is.na(covS)  && !is.null(ic_first$covS))  covS  <- ic_first$covS
      }
    }

    # Path 2: sol$IC$overall. Appropriate for extract_mode = "all"/"essential",
    # which summarize across all solutions, and as a fallback.
    if (is.na(inclS)) {
      incl_val <- try(sol$IC$overall$sol.incl.cov$inclS, silent = TRUE)
      if (!inherits(incl_val, "try-error") && !is.null(incl_val)) inclS <- incl_val
    }
    if (is.na(covS)) {
      cov_val <- try(sol$IC$overall$sol.incl.cov$covS, silent = TRUE)
      if (!inherits(cov_val, "try-error") && !is.null(cov_val)) covS <- cov_val
    }
  }
  
  # Get total solution count (always use get_n_solutions for consistency)
  n_solutions <- get_n_solutions(sol)

  # A solution was found, but no fit measures could be located for it. This
  # should not happen for the shapes QCA currently produces; if a future version
  # stores them elsewhere, tell the user why the cell is empty rather than
  # returning a silent NA that looks like a computation failure.
  if (is.na(inclS) || is.na(covS)) {
    warning(
      "A solution was found, but its consistency/coverage could not be read ",
      "from the QCA result, so inclS/covS are reported as NA",
      if (used_isol) " (intermediate solution)" else "",
      ifelse(is.na(n_solutions) || n_solutions <= 1, "",
             paste0(" (", n_solutions, " minimal solutions)")),
      ". The solution expression itself is unaffected. Please report this ",
      "together with your QCA package version at ",
      "https://github.com/im-research-yt/ThSQCA/issues",
      call. = FALSE
    )
  }


  # Mode-specific processing
  if (extract_mode == "first") {
    expression <- paste(sol_list[[1]], collapse = " + ")
    return(list(
      expression  = expression,
      inclS       = inclS,
      covS        = covS,
      n_solutions = n_solutions
    ))
  }

  # For the modes that summarize ACROSS solutions ("all", "essential"), the
  # model list must be the same enumeration that n_solutions counts, otherwise
  # the reported count and the reported terms describe different things.
  #
  # sol_list above is deliberately chart-1-only (sol$i.sol$C1P1$solution),
  # which is right for "first" because C1P1's first entry is the displayed M1.
  # It is NOT a safe basis for cross-solution summaries: when the intermediate
  # solution spans several prime implicant charts, whether C1P1$solution
  # happens to enumerate every chart's model or only its own is an
  # implementation detail of QCA::minimize()'s internal getSolution() call, not
  # a documented guarantee. collect_unique_i_sol() enumerates every chart and
  # deduplicates by term-set content, and is what get_n_solutions() counts, so
  # using it here keeps the count and the terms consistent by construction.
  models <- collect_unique_i_sol(sol)
  if (is.null(models) || length(models) == 0) {
    models <- sol_list
  }

  if (extract_mode == "all") {
    all_exprs <- sapply(seq_along(models), function(i) {
      paste0("M", i, ": ", paste(models[[i]], collapse = " + "))
    })
    expression <- paste(all_exprs, collapse = "; ")
    return(list(
      expression  = expression,
      inclS       = inclS,
      covS        = covS,
      n_solutions = n_solutions
    ))
  }
  
  if (extract_mode == "essential") {
    # Split each solution into terms
    sol_terms <- lapply(models, function(x) {
      unlist(strsplit(paste(x, collapse = " + "), " \\+ "))
    })
    
    # Essential prime implicants: intersection of all solutions
    essential_terms <- Reduce(intersect, sol_terms)
    
    # All terms: union of all solutions
    all_terms <- Reduce(union, sol_terms)
    
    # Selective prime implicants: in some but not all solutions
    selective_terms <- setdiff(all_terms, essential_terms)
    
    # Unique terms: only in specific solution
    if (n_solutions > 1) {
      unique_terms_list <- lapply(seq_along(sol_terms), function(i) {
        other_terms <- unique(unlist(sol_terms[-i]))
        setdiff(sol_terms[[i]], other_terms)
      })
      unique_terms_formatted <- sapply(seq_along(unique_terms_list), function(i) {
        if (length(unique_terms_list[[i]]) > 0) {
          paste0("M", i, ":", paste(unique_terms_list[[i]], collapse = "+"))
        } else {
          NULL
        }
      })
      unique_terms_str <- paste(unique_terms_formatted[!sapply(unique_terms_formatted, is.null)],
                                collapse = "; ")
      if (unique_terms_str == "") unique_terms_str <- NA_character_
    } else {
      unique_terms_str <- NA_character_
    }
    
    # Essential expression
    expression <- if (length(essential_terms) > 0) {
      paste(essential_terms, collapse = " + ")
    } else {
      "No essential prime implicants"
    }
    
    # Selective expression
    selective_str <- if (length(selective_terms) > 0) {
      paste(selective_terms, collapse = " + ")
    } else {
      NA_character_
    }
    
    return(list(
      expression       = expression,
      inclS            = inclS,
      covS             = covS,
      selective_terms  = selective_str,
      unique_terms     = unique_terms_str,
      n_solutions      = n_solutions
    ))
  }
}

# Regression tests for the multiple-minimal-solution fit-measure fix (v2.0.3).
# Fixtures (.multisol_fuzzy_data, .hand_covS) live in helper-fixtures.R.
#
# With multiple minimal solutions, extract_mode = "first" displays M1; its fit
# must be sol$IC$individual[[1]]$sol.incl.cov, not sol$IC$overall (the
# disjunction of all solutions, which overinflates coverage on fuzzy data).

test_that("first-solution covS matches individual[[1]], not overall (fuzzy, multiple solutions)", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D")
  qd <- .multisol_fuzzy_data()

  tt <- QCA::truthTable(qd, outcome = "Y", conditions = conds,
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "", details = TRUE)

  expect_gt(length(sol$solution), 1L)
  indiv1_covS  <- sol$IC$individual[[1]]$sol.incl.cov$covS
  overall_covS <- sol$IC$overall$sol.incl.cov$covS
  expect_gt(abs(overall_covS - indiv1_covS), 1e-3)

  info <- ThSQCA:::qca_extract(sol, extract_mode = "first")
  expect_equal(info$covS, indiv1_covS, tolerance = 1e-6)
  hand <- .hand_covS(sol$solution[[1]], qd[conds], qd$Y)
  expect_equal(info$covS, hand, tolerance = 1e-3)
  expect_false(isTRUE(all.equal(info$covS, overall_covS, tolerance = 1e-6)))
})

test_that("single-solution cell is unaffected by the fix", {
  skip_if_not_installed("QCA")
  data("sample_data", package = "ThSQCA")
  res <- otSweep(dat = sample_data, outcome = "Y",
                 conditions = c("X1", "X2", "X3"),
                 sweep_range = 6:8, thrX = c(X1 = 7, X2 = 7, X3 = 7),
                 return_details = FALSE)
  ok <- !is.na(res$covS)
  expect_true(all(res$covS[ok] >= 0 & res$covS[ok] <= 1))
})

test_that("all/essential modes still report the overall aggregate", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D")
  qd <- .multisol_fuzzy_data()
  tt <- QCA::truthTable(qd, outcome = "Y", conditions = conds,
                        incl.cut = 0.80, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "", details = TRUE)
  skip_if(length(sol$solution) <= 1L)

  overall_covS <- sol$IC$overall$sol.incl.cov$covS
  info_all <- ThSQCA:::qca_extract(sol, extract_mode = "all")
  expect_equal(info_all$covS, overall_covS, tolerance = 1e-6)
})

# Regression tests for the intermediate-solution fit-measure fix (v2.0.4).
# Fixture (.intermediate_fuzzy_data) lives in helper-fixtures.R.
#
# With dir.exp, the displayed solution is the intermediate (sol$i.sol$C1P1);
# its fit must come from sol$i.sol$C1P1$IC$sol.incl.cov, not sol$IC (which
# describes the parsimonious solution).

test_that("intermediate (used_isol) fit comes from i.sol C1P1, not sol$IC", {
  sol <- list(
    solution = list(c("A", "B"), c("A", "C")),
    i.sol = list(C1P1 = list(
      solution = list(c("A", "B", "D")),
      IC = list(sol.incl.cov = data.frame(inclS = 0.811, covS = 0.172))
    )),
    IC = list(
      sol.incl.cov = NULL,
      individual = list(
        list(sol.incl.cov = data.frame(inclS = 0.90, covS = 0.30)),
        list(sol.incl.cov = data.frame(inclS = 0.90, covS = 0.30))
      ),
      overall = list(sol.incl.cov = data.frame(inclS = 0.455, covS = 0.177))
    )
  )
  for (mode in c("first", "all", "essential")) {
    qe <- ThSQCA:::qca_extract(sol, mode)
    expect_equal(qe$inclS, 0.811, tolerance = 1e-6)
    expect_equal(qe$covS,  0.172, tolerance = 1e-6)
    expect_false(isTRUE(all.equal(qe$covS, 0.177)))
  }
})

test_that("intermediate fit matches i.sol on real fuzzy data (single parsimonious solution)", {
  skip_if_not_installed("QCA")
  conds <- c("A", "B", "C", "D", "E")
  qd <- .intermediate_fuzzy_data()
  tt <- QCA::truthTable(qd, outcome = "Y", conditions = conds,
                        incl.cut = 0.75, n.cut = 1, complete = FALSE)
  sol <- QCA::minimize(tt, include = "?",
                       dir.exp = stats::setNames(rep(1, 5), conds),
                       details = TRUE)

  isol_covS   <- sol$i.sol$C1P1$IC$sol.incl.cov$covS[1]
  parsim_covS <- sol$IC$sol.incl.cov$covS[1]
  expect_true(!is.null(isol_covS) && !is.na(isol_covS))
  expect_gt(abs(isol_covS - parsim_covS), 1e-3)

  info <- ThSQCA:::qca_extract(sol, extract_mode = "first")
  expect_equal(info$inclS, sol$i.sol$C1P1$IC$sol.incl.cov$inclS[1], tolerance = 1e-6)
  expect_equal(info$covS,  isol_covS, tolerance = 1e-6)
  expect_false(isTRUE(all.equal(info$covS, parsim_covS, tolerance = 1e-6)))
})

###############################################
# OTS–QCA (Y sweep) and DTS–QCA (2D sweep)
# v0.3.0: QCA-compatible argument names + negated outcome support
###############################################

#' OTS–QCA: Outcome threshold sweep
#'
#' Sweeps the threshold of the outcome Y while keeping the thresholds of
#' all X conditions fixed.
#'
#' @param dat Data frame containing the outcome and condition variables.
#' @param outcome Character. Outcome variable name. Supports negation with
#'   tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.
#' @param conditions Character vector. Names of condition variables.
#' @param sweep_range Numeric vector. Candidate thresholds for Y.
#' @param thrX Named numeric vector. Fixed thresholds for X variables.
#'   Names must match the conditions that require binarization.
#'   Variables listed in \code{pre_calibrated} do not need a \code{thrX} entry.
#' @param pre_calibrated Character vector or \code{NULL}. Names of condition
#'   variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
#'   and should be passed through to \code{QCA::truthTable()} without
#'   binarization. These variables must contain values in the \code{[0, 1]}
#'   range. Variables not listed here will be binarized using \code{thrX}
#'   thresholds as usual. Default is \code{NULL} (all variables binarized).
#'   It is recommended to sweep variables on their original (raw) scale rather
#'   than as pre-calibrated fuzzy values, because raw-scale thresholds are
#'   easier to interpret substantively.
#' @param dir.exp Directional expectations for \code{minimize}.
#'   If \code{NULL} (default), no directional expectations are applied.
#'   To compute the \strong{intermediate solution}, specify a numeric vector
#'   (1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
#'   for three conditions all expected to contribute positively.
#' @param include Inclusion rule for \code{minimize}. 
#'   \code{""} (default, QCA compatible) computes the \strong{complex solution}
#'   without logical remainders.
#'   Use \code{"?"} to include logical remainders for \strong{parsimonious}
#'   (with \code{dir.exp = NULL}) or \strong{intermediate} solutions
#'   (with \code{dir.exp} specified).
#' @param incl.cut Consistency cutoff for \code{truthTable}.
#' @param n.cut Frequency cutoff for \code{truthTable}.
#' @param pri.cut PRI cutoff for \code{minimize}.
#' @param extract_mode Character. How to handle multiple solutions:
#'   \code{"first"} (default), \code{"all"}, or \code{"essential"}.
#'   See \code{\link{qca_extract}} for details.
#' @param return_details Logical. If \code{TRUE} (default), returns both
#'   summary and detailed objects for use with \code{generate_report()}.
#' @param Yvar Deprecated. Use \code{outcome} instead.
#' @param Xvars Deprecated. Use \code{conditions} instead.
#'
#' @return
#' If \code{return_details = FALSE}, a data frame with columns:
#' \itemize{
#'   \item \code{thrY} — threshold for Y
#'   \item \code{expression} — minimized solution expression
#'   \item \code{inclS} — solution consistency
#'   \item \code{covS} — solution coverage
#'   \item (additional columns depending on \code{extract_mode})
#' }
#'
#' If \code{return_details = TRUE}, a list with:
#' \itemize{
#'   \item \code{summary} — the data frame above
#'   \item \code{details} — per-Y-threshold list of
#'     \code{thrY}, \code{thrX_vec}, \code{truth_table}, \code{solution}
#' }
#'
#' Note that \code{return_details} changes the \emph{type} of the returned
#' object, not just its contents: with \code{TRUE} the summary table is at
#' \code{result$summary}, whereas with \code{FALSE} the summary table
#' \emph{is} the returned object and \code{result$summary} is \code{NULL}.
#' Code intended to work under both settings should branch on
#' \code{inherits(result, "data.frame")} (or simply always pass
#' \code{return_details = TRUE}) rather than assuming \code{result$summary}
#' exists.
#'
#' @importFrom QCA truthTable minimize
#' @export
#' @examples
#' # Load sample data
#' data(sample_data)
#' 
#' # Set fixed thresholds for conditions
#' thrX <- c(X1 = 7, X2 = 7, X3 = 7)
#' 
#' # === Three Types of QCA Solutions ===
#' 
#' # 1. Complex Solution (default, QCA compatible)
#' #    Does not use logical remainders (most conservative)
#' result_comp <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 7,
#'   thrX = thrX
#'   # include = "" (default), dir.exp = NULL (default)
#' )
#' head(result_comp$summary)
#' 
#' # 2. Parsimonious Solution (include = "?")
#' #    Uses logical remainders without directional expectations
#' result_pars <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 7,
#'   thrX = thrX,
#'   include = "?"  # Include logical remainders
#' )
#' head(result_pars$summary)
#' 
#' # 3. Intermediate Solution (include = "?" + dir.exp)
#' #    Uses logical remainders with directional expectations
#' result_int <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 7,
#'   thrX = thrX,
#'   include = "?",
#'   dir.exp = c(1, 1, 1)  # All conditions expected positive
#' )
#' head(result_int$summary)
#' 
#' # === Threshold Sweep Example ===
#' 
#' # Sweep with complex solutions (default)
#' result_sweep <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = thrX
#' )
#' head(result_sweep$summary)
#' 
#' # Run with negated outcome (~Y)
#' # Analyzes conditions for Y < threshold
#' result_neg <- otSweep(
#'   dat = sample_data,
#'   outcome = "~Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = thrX
#' )
#' head(result_neg$summary)
otSweep <- function(dat, 
                    outcome = NULL, conditions = NULL,
                    sweep_range, thrX,
                    pre_calibrated = NULL,
                    dir.exp = NULL, include = "",
                    incl.cut = 0.8, n.cut = 1, pri.cut = 0,
                    extract_mode = c("first", "all", "essential"),
                    return_details = TRUE,
                    Yvar = NULL, Xvars = NULL) {
  
  # === Backward compatibility for deprecated arguments ===
  if (!is.null(Yvar) && is.null(outcome)) {
    outcome <- Yvar
    warning("Argument 'Yvar' is deprecated. Use 'outcome' instead.",
            call. = FALSE)
  }
  if (!is.null(Xvars) && is.null(conditions)) {
    conditions <- Xvars
    warning("Argument 'Xvars' is deprecated. Use 'conditions' instead.",
            call. = FALSE)
  }
  
  # === Validate required arguments ===
  if (is.null(outcome)) {
    stop("Argument 'outcome' is required.")
  }
  if (is.null(conditions)) {
    stop("Argument 'conditions' is required.")
  }
  
  # === Handle negated outcome ===
  negate_outcome <- grepl("^~", outcome)
  outcome_clean <- sub("^~", "", outcome)
  
  # Validate outcome variable exists

  if (!outcome_clean %in% names(dat)) {
    stop("Variable '", outcome_clean, "' not found in data.")
  }
  
  # Validate condition variables exist
  missing_conds <- setdiff(conditions, names(dat))
  if (length(missing_conds) > 0) {
    stop("Condition variable(s) not found in data: ", 
         paste(missing_conds, collapse = ", "))
  }
  
  extract_mode <- match.arg(extract_mode)
  
  # Initialize output data frame based on extract_mode
  df_out <- data.frame(
    thrY       = numeric(0),
    expression = character(0),
    inclS      = numeric(0),
    covS       = numeric(0),
    n_solutions = integer(0),
    stringsAsFactors = FALSE
  )
  
  # Add columns based on extract_mode
  if (extract_mode == "essential") {
    df_out$selective_terms <- character(0)
    df_out$unique_terms     <- character(0)
  }
  
  details_list <- list()
  
  # Track thresholds with multiple solutions (for warning in "first" mode)
  multi_sol_thresholds <- c()
  
  # Handle dir.exp: scalar -> expand to vector; NULL is passed through

  # NULL -> parsimonious solution; c(1,1,...) -> intermediate solution
  local_dir.exp <- dir.exp
  if (!is.null(local_dir.exp) && length(local_dir.exp) == 1) {
    local_dir.exp <- rep(local_dir.exp[1], length(conditions))
    names(local_dir.exp) <- conditions
  } else if (!is.null(local_dir.exp) && is.null(names(local_dir.exp))) {
    names(local_dir.exp) <- conditions
  }

  # Validate pre_calibrated (once, before loop)
  validate_pre_calibrated(pre_calibrated, conditions, dat)

  for (thrY in sweep_range) {
    
    # Prepare data: binarize or pass through based on pre_calibrated
    dat_bin <- prepare_dat_bin(
      dat, outcome_clean, conditions,
      thrY, thrX,
      pre_calibrated = pre_calibrated
    )
    
    # Determine outcome string for truthTable (with ~ if negated)
    outcome_tt <- if (negate_outcome) "~Y" else "Y"
    
    tt <- try(
      QCA::truthTable(
        dat_bin,
        outcome    = outcome_tt,
        conditions = conditions,
        show.cases = FALSE,
        incl.cut1  = incl.cut,
        n.cut      = n.cut,
        pri.cut    = pri.cut
      ),
      silent = TRUE
    )
    
    if (inherits(tt, "try-error")) {
      new_row <- data.frame(
        thrY        = thrY,
        expression  = "No solution",
        inclS       = NA_real_,
        covS        = NA_real_,
        n_solutions = 0L,
        stringsAsFactors = FALSE
      )
      
      if (extract_mode == "essential") {
        new_row$selective_terms <- NA_character_
        new_row$unique_terms     <- NA_character_
      }
      
      df_out <- rbind(df_out, new_row)
      
      if (return_details) {
        details_list[[as.character(thrY)]] <- list(
          thrY        = thrY,
          thrX_vec    = thrX,
          truth_table = NULL,
          solution    = NULL,
          dat_bin     = NULL
        )
      }
      next
    }
    
    tt <- sanitize_truthtable(tt)  # QCA 3.25 guard: char incl/PRI + '-' remainders
    sol <- try(
      QCA::minimize(
        tt,
        include    = include,
        dir.exp    = local_dir.exp,
        details    = TRUE,
        show.cases = FALSE,
        pri.cut    = pri.cut
      ),
      silent = TRUE
    )
    
    if (inherits(sol, "try-error")) {
      new_row <- data.frame(
        thrY        = thrY,
        expression  = "No solution",
        inclS       = NA_real_,
        covS        = NA_real_,
        n_solutions = 0L,
        stringsAsFactors = FALSE
      )
      
      if (extract_mode == "essential") {
        new_row$selective_terms <- NA_character_
        new_row$unique_terms     <- NA_character_
      }
      
      df_out <- rbind(df_out, new_row)
      
      if (return_details) {
        details_list[[as.character(thrY)]] <- list(
          thrY        = thrY,
          thrX_vec    = thrX,
          truth_table = tt,
          solution    = NULL,
          dat_bin     = dat_bin
        )
      }
      next
    }
    
    info <- qca_extract(sol, extract_mode = extract_mode)
    
    # Track multiple solutions (for warning)
    if (info$n_solutions > 1) {
      multi_sol_thresholds <- c(multi_sol_thresholds, thrY)
    }
    
    # Build result row based on extract_mode
    new_row <- data.frame(
      thrY        = thrY,
      expression  = info$expression,
      inclS       = info$inclS,
      covS        = info$covS,
      n_solutions = info$n_solutions,
      stringsAsFactors = FALSE
    )
    
    if (extract_mode == "essential") {
      new_row$selective_terms <- info$selective_terms
      new_row$unique_terms     <- info$unique_terms
    }
    
    df_out <- rbind(df_out, new_row)
    
    if (return_details) {
      details_list[[as.character(thrY)]] <- list(
        thrY        = thrY,
        thrX_vec    = thrX,
        truth_table = tt,
        solution    = sol,
        dat_bin     = dat_bin
      )
    }
  }
  
  # Issue warning for multiple solutions
  if (length(multi_sol_thresholds) > 0) {
    warning(
      "Multiple intermediate solutions exist for thrY = ",
      paste(multi_sol_thresholds, collapse = ", "),
      " (n_solutions > 1). ",
      "Only the first solution (M1) and its fit metrics are shown. ",
      "Use generate_report() for full analysis.",
      call. = FALSE
    )
  }
  
  if (return_details) {
    result <- list(
      summary = df_out, 
      details = details_list,
      params = list(
        outcome = outcome,
        conditions = conditions,
        negate_outcome = negate_outcome,
        pre_calibrated = pre_calibrated,
        thrX = thrX,
        sweep_range = sweep_range,
        incl.cut = incl.cut,
        n.cut = n.cut,
        pri.cut = pri.cut,
        include = include,
        dir.exp = dir.exp  # Store original value for reproducibility
      )
    )
    class(result) <- c("otSweep_result", "tsqca_result", "list")
    return(result)
  }
  
  df_out
}


###############################################
# DTS–QCA (2D sweep)
###############################################

#' DTS–QCA: Two-dimensional X–Y threshold sweep
#'
#' Sweeps thresholds for multiple X variables and the outcome Y jointly.
#' For each combination of X thresholds and each candidate Y threshold, the
#' data are binarized and a crisp-set QCA is executed.
#'
#' @param dat Data frame containing the outcome and condition variables.
#' @param outcome Character. Outcome variable name. Supports negation with
#'   tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.
#' @param conditions Character vector. Names of condition variables.
#' @param sweep_list_X Named list. Each element is a numeric vector of
#'   candidate thresholds for the corresponding X. Variables listed in
#'   \code{pre_calibrated} do not need a \code{sweep_list_X} entry.
#' @param sweep_range_Y Numeric vector. Candidate thresholds for Y.
#' @param pre_calibrated Character vector or \code{NULL}. Names of condition
#'   variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
#'   and should be passed through to \code{QCA::truthTable()} without
#'   binarization. These variables must contain values in the \code{[0, 1]}
#'   range. Variables not listed here will be binarized using \code{sweep_list_X}
#'   thresholds as usual. Default is \code{NULL} (all variables binarized).
#'   It is recommended to sweep variables on their original (raw) scale rather
#'   than as pre-calibrated fuzzy values, because raw-scale thresholds are
#'   easier to interpret substantively.
#' @param dir.exp Directional expectations for \code{minimize}.
#'   If \code{NULL} (default), no directional expectations are applied.
#'   To compute the \strong{intermediate solution}, specify a numeric vector
#'   (1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
#'   for three conditions all expected to contribute positively.
#' @param include Inclusion rule for \code{minimize}. 
#'   \code{""} (default, QCA compatible) computes the \strong{complex solution}
#'   without logical remainders.
#'   Use \code{"?"} to include logical remainders for \strong{parsimonious}
#'   (with \code{dir.exp = NULL}) or \strong{intermediate} solutions
#'   (with \code{dir.exp} specified).
#' @param incl.cut Consistency cutoff for \code{truthTable}.
#' @param n.cut Frequency cutoff for \code{truthTable}.
#' @param pri.cut PRI cutoff for \code{minimize}.
#' @param extract_mode Character. How to handle multiple solutions:
#'   \code{"first"} (default), \code{"all"}, or \code{"essential"}.
#'   See \code{\link{qca_extract}} for details.
#' @param return_details Logical. If \code{TRUE} (default), returns both
#'   summary and detailed objects for use with \code{generate_report()}.
#' @param Yvar Deprecated. Use \code{outcome} instead.
#' @param Xvars Deprecated. Use \code{conditions} instead.
#'
#' @return
#' If \code{return_details = FALSE}, a data frame with columns:
#' \itemize{
#'   \item \code{combo_id} — index of threshold combination
#'   \item \code{thrY} — threshold for Y
#'   \item \code{thrX} — character summary of X thresholds
#'   \item \code{expression} — minimized solution expression
#'   \item \code{inclS} — solution consistency
#'   \item \code{covS} — solution coverage
#'   \item (additional columns depending on \code{extract_mode})
#' }
#'
#' If \code{return_details = TRUE}, a list with:
#' \itemize{
#'   \item \code{summary} — the data frame above
#'   \item \code{details} — list of runs with
#'     \code{combo_id}, \code{thrY}, \code{thrX_vec},
#'     \code{truth_table}, \code{solution}
#' }
#'
#' Note that \code{return_details} changes the \emph{type} of the returned
#' object, not just its contents: with \code{TRUE} the summary table is at
#' \code{result$summary}, whereas with \code{FALSE} the summary table
#' \emph{is} the returned object and \code{result$summary} is \code{NULL}.
#' Code intended to work under both settings should branch on
#' \code{inherits(result, "data.frame")} (or simply always pass
#' \code{return_details = TRUE}) rather than assuming \code{result$summary}
#' exists.
#'
#' @importFrom QCA truthTable minimize
#' @examples
#' # Load sample data
#' data(sample_data)
#' 
#' # === Three Types of QCA Solutions ===
#' 
#' # Quick demonstration with 2 conditions
#' sweep_list_X <- list(X1 = 7, X2 = 7)
#' sweep_range_Y <- 7
#' 
#' # 1. Complex Solution (default, QCA compatible)
#' result_comp <- dtSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list_X = sweep_list_X,
#'   sweep_range_Y = sweep_range_Y
#'   # include = "" (default), dir.exp = NULL (default)
#' )
#' head(result_comp$summary)
#' 
#' # 2. Parsimonious Solution (include = "?")
#' result_pars <- dtSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list_X = sweep_list_X,
#'   sweep_range_Y = sweep_range_Y,
#'   include = "?"  # Include logical remainders
#' )
#' head(result_pars$summary)
#' 
#' # 3. Intermediate Solution (include = "?" + dir.exp)
#' result_int <- dtSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list_X = sweep_list_X,
#'   sweep_range_Y = sweep_range_Y,
#'   include = "?",
#'   dir.exp = c(1, 1)  # Positive expectations
#' )
#' head(result_int$summary)
#' 
#' # === Threshold Sweep Example ===
#' 
#' # Using 2 conditions and 2 threshold levels
#' sweep_list_X <- list(
#'   X1 = 6:7,
#'   X2 = 6:7
#' )
#' sweep_range_Y <- 6:7
#' 
#' # Run dual threshold sweep (complex solutions by default)
#' result_quick <- dtSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2"),
#'   sweep_list_X = sweep_list_X,
#'   sweep_range_Y = sweep_range_Y
#' )
#' head(result_quick$summary)
#' 
#' \donttest{
#' # Full analysis with 3 conditions (81 combinations)
#' sweep_list_X_full <- list(
#'   X1 = 6:8,
#'   X2 = 6:8,
#'   X3 = 6:8
#' )
#' sweep_range_Y_full <- 6:8
#' 
#' result_full <- dtSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_list_X = sweep_list_X_full,
#'   sweep_range_Y = sweep_range_Y_full
#' )
#' head(result_full$summary)
#' }
#' @export
dtSweep <- function(dat, 
                    outcome = NULL, conditions = NULL,
                    sweep_list_X, sweep_range_Y,
                    pre_calibrated = NULL,
                    dir.exp = NULL, include = "",
                    incl.cut = 0.8, n.cut = 1, pri.cut = 0,
                    extract_mode = c("first", "all", "essential"),
                    return_details = TRUE,
                    Yvar = NULL, Xvars = NULL) {
  
  # === Backward compatibility for deprecated arguments ===
  if (!is.null(Yvar) && is.null(outcome)) {
    outcome <- Yvar
    warning("Argument 'Yvar' is deprecated. Use 'outcome' instead.",
            call. = FALSE)
  }
  if (!is.null(Xvars) && is.null(conditions)) {
    conditions <- Xvars
    warning("Argument 'Xvars' is deprecated. Use 'conditions' instead.",
            call. = FALSE)
  }
  
  # === Validate required arguments ===
  if (is.null(outcome)) {
    stop("Argument 'outcome' is required.")
  }
  if (is.null(conditions)) {
    stop("Argument 'conditions' is required.")
  }
  
  # === Handle negated outcome ===
  negate_outcome <- grepl("^~", outcome)
  outcome_clean <- sub("^~", "", outcome)
  
  # Validate outcome variable exists
  if (!outcome_clean %in% names(dat)) {
    stop("Variable '", outcome_clean, "' not found in data.")
  }
  
  # Validate condition variables exist
  missing_conds <- setdiff(conditions, names(dat))
  if (length(missing_conds) > 0) {
    stop("Condition variable(s) not found in data: ", 
         paste(missing_conds, collapse = ", "))
  }
  
  extract_mode <- match.arg(extract_mode)
  
  combo_X <- expand.grid(
    sweep_list_X,
    KEEP.OUT.ATTRS  = FALSE,
    stringsAsFactors = FALSE
  )
  
  # Initialize output data frame based on extract_mode
  df_out <- data.frame(
    combo_id    = integer(0),
    thrY        = numeric(0),
    thrX        = character(0),
    expression  = character(0),
    inclS       = numeric(0),
    covS        = numeric(0),
    n_solutions = integer(0),
    stringsAsFactors = FALSE
  )
  
  # Add columns based on extract_mode
  if (extract_mode == "essential") {
    df_out$selective_terms <- character(0)
    df_out$unique_terms     <- character(0)
  }
  
  details_list <- list()
  
  # Handle dir.exp: scalar -> expand to vector; NULL is passed through
  # NULL -> parsimonious solution; c(1,1,...) -> intermediate solution
  local_dir.exp <- dir.exp
  if (!is.null(local_dir.exp) && length(local_dir.exp) == 1) {
    local_dir.exp <- rep(local_dir.exp[1], length(conditions))
    names(local_dir.exp) <- conditions
  } else if (!is.null(local_dir.exp) && is.null(names(local_dir.exp))) {
    names(local_dir.exp) <- conditions
  }

  # Validate pre_calibrated (once, before loop)
  validate_pre_calibrated(pre_calibrated, conditions, dat)

  # Warn if pre_calibrated variable is also a sweep target in sweep_list_X
  if (!is.null(pre_calibrated)) {
    swept_vars <- names(sweep_list_X)[
      sapply(sweep_list_X, function(x) length(x) > 1)
    ]
    conflict <- intersect(pre_calibrated, swept_vars)
    if (length(conflict) > 0) {
      warning("Variable(s) ", paste(conflict, collapse = ", "),
              " are both pre_calibrated and in sweep_list_X. ",
              "Pre-calibrated values will be used; sweep thresholds ignored. ",
              "It is recommended to sweep variables on their original (raw) scale, ",
              "not as pre-calibrated fuzzy values, because threshold values on a ",
              "raw scale (e.g., Likert 1-10) are easier to interpret substantively. ",
              "See the package vignette section 'Choosing Sweep Variables' for details.",
              call. = FALSE)
    }
  }

  # Track combinations with multiple solutions (for warning in "first" mode)
  multi_sol_combos <- c()
  
  combo_id <- 1L
  
  for (i in seq_len(nrow(combo_X))) {
    
    thrX_vec <- as.numeric(combo_X[i, ])
    names(thrX_vec) <- names(combo_X)
    thrX_label <- paste(names(thrX_vec), thrX_vec,
                        sep = "=", collapse = ", ")
    
    for (thrY in sweep_range_Y) {
      
      # Prepare data: binarize or pass through based on pre_calibrated
      dat_bin <- prepare_dat_bin(
        dat, outcome_clean, conditions,
        thrY, thrX_vec,
        pre_calibrated = pre_calibrated
      )
      
      # Determine outcome string for truthTable (with ~ if negated)
      outcome_tt <- if (negate_outcome) "~Y" else "Y"
      
      tt <- try(
        QCA::truthTable(
          dat_bin,
          outcome    = outcome_tt,
          conditions = conditions,
          show.cases = FALSE,
          incl.cut1  = incl.cut,
          n.cut      = n.cut,
          pri.cut    = pri.cut
        ),
        silent = TRUE
      )
      
      if (inherits(tt, "try-error")) {
        new_row <- data.frame(
          combo_id    = combo_id,
          thrY        = thrY,
          thrX        = thrX_label,
          expression  = "No solution",
          inclS       = NA_real_,
          covS        = NA_real_,
          n_solutions = 0L,
          stringsAsFactors = FALSE
        )
        
        if (extract_mode == "essential") {
          new_row$selective_terms <- NA_character_
          new_row$unique_terms     <- NA_character_
        }
        
        df_out <- rbind(df_out, new_row)
        
        if (return_details) {
          details_list[[length(details_list) + 1L]] <- list(
            combo_id    = combo_id,
            thrY        = thrY,
            thrX_vec    = thrX_vec,
            truth_table = NULL,
            solution    = NULL,
            dat_bin     = NULL
          )
        }
        
        next
      }
      
      tt <- sanitize_truthtable(tt)  # QCA 3.25 guard: char incl/PRI + '-' remainders
      sol <- try(
        QCA::minimize(
          tt,
          include    = include,
          dir.exp    = local_dir.exp,
          details    = TRUE,
          show.cases = FALSE,
          pri.cut    = pri.cut
        ),
        silent = TRUE
      )
      
      if (inherits(sol, "try-error")) {
        new_row <- data.frame(
          combo_id    = combo_id,
          thrY        = thrY,
          thrX        = thrX_label,
          expression  = "No solution",
          inclS       = NA_real_,
          covS        = NA_real_,
          n_solutions = 0L,
          stringsAsFactors = FALSE
        )
        
        if (extract_mode == "essential") {
          new_row$selective_terms <- NA_character_
          new_row$unique_terms     <- NA_character_
        }
        
        df_out <- rbind(df_out, new_row)
        
        if (return_details) {
          details_list[[length(details_list) + 1L]] <- list(
            combo_id    = combo_id,
            thrY        = thrY,
            thrX_vec    = thrX_vec,
            truth_table = tt,
            solution    = NULL,
            dat_bin     = dat_bin
          )
        }
        
        next
      }
      
      info <- qca_extract(sol, extract_mode = extract_mode)
      
      # Track multiple solutions
      if (info$n_solutions > 1) {
        multi_sol_combos <- c(multi_sol_combos, 
                              paste0("combo_id=", combo_id, ", thrY=", thrY))
      }
      
      # Build result row based on extract_mode
      new_row <- data.frame(
        combo_id    = combo_id,
        thrY        = thrY,
        thrX        = thrX_label,
        expression  = info$expression,
        inclS       = info$inclS,
        covS        = info$covS,
        n_solutions = info$n_solutions,
        stringsAsFactors = FALSE
      )
      
      if (extract_mode == "essential") {
        new_row$selective_terms <- info$selective_terms
        new_row$unique_terms     <- info$unique_terms
      }
      
      df_out <- rbind(df_out, new_row)
      
      if (return_details) {
        details_list[[length(details_list) + 1L]] <- list(
          combo_id    = combo_id,
          thrY        = thrY,
          thrX_vec    = thrX_vec,
          truth_table = tt,
          solution    = sol,
          dat_bin     = dat_bin
        )
      }
    }
    
    combo_id <- combo_id + 1L
  }
  
  # Issue warning for multiple solutions
  if (length(multi_sol_combos) > 0) {
    n_multi <- length(multi_sol_combos)
    warning(
      "Multiple intermediate solutions exist for ", n_multi, " combination(s) ",
      "(n_solutions > 1). ",
      "Only the first solution (M1) and its fit metrics are shown. ",
      "Use generate_report() for full analysis.",
      call. = FALSE
    )
  }
  
  if (return_details) {
    result <- list(
      summary = df_out, 
      details = details_list,
      params = list(
        outcome = outcome,
        conditions = conditions,
        negate_outcome = negate_outcome,
        pre_calibrated = pre_calibrated,
        sweep_list_X = sweep_list_X,
        sweep_range_Y = sweep_range_Y,
        incl.cut = incl.cut,
        n.cut = n.cut,
        pri.cut = pri.cut,
        include = include,
        dir.exp = dir.exp
      )
    )
    class(result) <- c("dtSweep_result", "tsqca_result", "list")
    return(result)
  }
  
  df_out
}

# Degenerate-input robustness tests (v2.0.4).
#
# These pin the CURRENT (verified-safe) behavior on edge inputs: the sweep
# functions must not crash, and must return a well-formed data.frame with the
# expected number of rows. They intentionally do not assert specific solution
# strings (which may legitimately vary with QCA versions); they assert
# structural sanity only.

.edge_frame <- function(n, y, seed = 1) {
  set.seed(seed)
  d <- data.frame(X1 = sample(0:10, n, TRUE),
                  X2 = sample(0:10, n, TRUE),
                  X3 = sample(0:10, n, TRUE))
  d$Y <- y
  d
}

.expect_sweep_ok <- function(res, n_rows) {
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), n_rows)
  expect_true(all(c("expression", "inclS", "covS") %in% colnames(res)))
  ok <- !is.na(res$covS)
  expect_true(all(res$covS[ok] >= 0 & res$covS[ok] <= 1))
  expect_true(all(res$inclS[ok & !is.na(res$inclS)] >= 0 &
                  res$inclS[ok & !is.na(res$inclS)] <= 1))
}

test_that("otSweep survives all-positive and all-negative outcomes", {
  skip_if_not_installed("QCA")
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)

  d_hi <- .edge_frame(30, rep(10, 30))
  r_hi <- otSweep(dat = d_hi, outcome = "Y", conditions = c("X1", "X2", "X3"),
                  sweep_range = 6:8, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_hi, 3L)

  d_lo <- .edge_frame(30, rep(0, 30))
  r_lo <- otSweep(dat = d_lo, outcome = "Y", conditions = c("X1", "X2", "X3"),
                  sweep_range = 6:8, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_lo, 3L)
  # No positive case at any threshold: no cell may claim a solution with fit.
  expect_true(all(is.na(r_lo$covS) | grepl("No solution", r_lo$expression)))
})

test_that("otSweep survives constant conditions, tiny n, and duplicated rows", {
  skip_if_not_installed("QCA")
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)

  d_const <- .edge_frame(30, sample(0:10, 30, TRUE), seed = 2)
  d_const$X3 <- 5
  r_const <- otSweep(dat = d_const, outcome = "Y", conditions = c("X1", "X2", "X3"),
                     sweep_range = 6:8, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_const, 3L)

  d_one <- .edge_frame(1, 8, seed = 3)
  r_one <- otSweep(dat = d_one, outcome = "Y", conditions = c("X1", "X2", "X3"),
                   sweep_range = 6:8, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_one, 3L)

  d_dup <- .edge_frame(4, c(8, 8, 2, 2), seed = 4)
  d_dup <- d_dup[rep(1:4, each = 10), ]
  r_dup <- otSweep(dat = d_dup, outcome = "Y", conditions = c("X1", "X2", "X3"),
                   sweep_range = 6:8, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_dup, 3L)
})

test_that("otSweep survives out-of-range sweeps, huge n.cut, and NA in a condition", {
  skip_if_not_installed("QCA")
  thrX <- c(X1 = 7, X2 = 7, X3 = 7)

  d <- .edge_frame(30, sample(0:10, 30, TRUE), seed = 5)
  r_oor <- otSweep(dat = d, outcome = "Y", conditions = c("X1", "X2", "X3"),
                   sweep_range = 50:52, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_oor, 3L)
  # Nothing can clear a threshold above the data maximum.
  expect_true(all(is.na(r_oor$covS) | grepl("No solution", r_oor$expression)))

  r_ncut <- otSweep(dat = d, outcome = "Y", conditions = c("X1", "X2", "X3"),
                    sweep_range = 6:8, thrX = thrX, n.cut = 999,
                    return_details = FALSE)
  .expect_sweep_ok(r_ncut, 3L)

  d_na <- d; d_na$X1[3] <- NA
  r_na <- otSweep(dat = d_na, outcome = "Y", conditions = c("X1", "X2", "X3"),
                  sweep_range = 6:8, thrX = thrX, return_details = FALSE)
  .expect_sweep_ok(r_na, 3L)
})

test_that("otSweep accepts a descending sweep_range", {
  skip_if_not_installed("QCA")
  d <- .edge_frame(30, sample(0:10, 30, TRUE), seed = 6)
  r <- otSweep(dat = d, outcome = "Y", conditions = c("X1", "X2", "X3"),
               sweep_range = 8:6, thrX = c(X1 = 7, X2 = 7, X3 = 7),
               return_details = FALSE)
  .expect_sweep_ok(r, 3L)
})

###############################################
# S3 Methods for ThSQCA Result Objects
# v0.4.0: print() and summary() methods
###############################################

# ============================================
# PRINT METHODS
# ============================================

#' Print method for ThSQCA results
#'
#' Displays a concise overview of ThSQCA analysis results.
#'
#' @param x A ThSQCA result object returned by one of the sweep functions.
#' @param ... Additional arguments (ignored).
#'
#' @return Invisibly returns \code{x}.
#'
#' @examples
#' \donttest{
#' data(sample_data)
#' result <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = c(X1 = 7, X2 = 7, X3 = 7)
#' )
#' print(result)
#' }
#'
#' @export
print.tsqca_result <- function(x, ...) {
  
  # Determine analysis type from class
  analysis_type <- class(x)[1]
  type_label <- switch(analysis_type,
    "otSweep_result" = "OTS-QCA (Outcome Threshold Sweep)",
    "dtSweep_result" = "DTS-QCA (Dual Threshold Sweep)",
    "ctSweepS_result" = "CTS-QCA (Condition Threshold Sweep - Single)",
    "ctSweepM_result" = "MCTS-QCA (Multi-Condition Threshold Sweep)",
    "ThSQCA Analysis"
  )
  
  cat(type_label, "Result\n")
  cat(strrep("=", nchar(type_label) + 7), "\n\n")
  
  # Extract params
  p <- x$params
  
  # Outcome info
  if (!is.null(p$outcome)) {
    outcome_str <- p$outcome
    if (isTRUE(p$negate_outcome)) {
      outcome_str <- paste0(outcome_str, " (negated)")
    }
    cat("Outcome:", outcome_str, "\n")
  }
  
  # Conditions info
  if (!is.null(p$conditions)) {
    cat("Conditions:", paste(p$conditions, collapse = ", "), "\n")
  }
  
  # Threshold info (varies by analysis type)
  if (analysis_type == "otSweep_result") {
    if (!is.null(p$sweep_range)) {
      cat("Y thresholds swept:", paste(p$sweep_range, collapse = ", "), "\n")
    }
    if (!is.null(p$thrX)) {
      thrX_str <- paste(names(p$thrX), p$thrX, sep = "=", collapse = ", ")
      cat("Fixed X thresholds:", thrX_str, "\n")
    }
  } else if (analysis_type == "dtSweep_result") {
    if (!is.null(p$sweep_range_Y)) {
      cat("Y thresholds swept:", paste(p$sweep_range_Y, collapse = ", "), "\n")
    }
    if (!is.null(p$sweep_list_X)) {
      cat("X thresholds swept:", length(p$sweep_list_X), "condition(s)\n")
    }
  } else if (analysis_type == "ctSweepS_result") {
    if (!is.null(p$sweep_var)) {
      cat("Swept condition:", p$sweep_var, "\n")
    }
    if (!is.null(p$sweep_range)) {
      cat("Thresholds swept:", paste(p$sweep_range, collapse = ", "), "\n")
    }
    if (!is.null(p$thrY)) {
      cat("Fixed Y threshold:", p$thrY, "\n")
    }
  } else if (analysis_type == "ctSweepM_result") {
    if (!is.null(p$sweep_list)) {
      cat("Conditions swept:", length(p$sweep_list), "\n")
    }
    if (!is.null(p$thrY)) {
      cat("Fixed Y threshold:", p$thrY, "\n")
    }
  }
  
  cat("\n")
  
  # Summary statistics
  df <- x$summary
  n_total <- nrow(df)
  n_valid <- sum(df$expression != "No solution", na.rm = TRUE)
  n_multi <- sum(df$n_solutions > 1, na.rm = TRUE)
  
  cat("Threshold settings analyzed:", n_total, "\n")
  cat("  - Valid solutions:", n_valid, "\n")
  cat("  - No solution:", n_total - n_valid, "\n")
  
  if (n_multi > 0) {
    cat("  - Multiple solutions:", n_multi, "\n")
  }
  
  cat("\n")
  cat("Use summary() for detailed results table\n")
  cat("Use generate_report() for full markdown report\n")
  
  invisible(x)
}

#' @rdname print.tsqca_result
#' @export
print.otSweep_result <- function(x, ...) {
  NextMethod()
}

#' @rdname print.tsqca_result
#' @export
print.dtSweep_result <- function(x, ...) {
  NextMethod()
}

#' @rdname print.tsqca_result
#' @export
print.ctSweepS_result <- function(x, ...) {
  NextMethod()
}

#' @rdname print.tsqca_result
#' @export
print.ctSweepM_result <- function(x, ...) {
  NextMethod()
}


# ============================================
# SUMMARY METHODS
# ============================================

#' Summary method for ThSQCA results
#'
#' Displays detailed results table with solution formulas and fit measures.
#'
#' @param object A ThSQCA result object returned by one of the sweep functions.
#' @param ... Additional arguments (ignored).
#'
#' @return Invisibly returns \code{object}.
#'
#' @examples
#' \donttest{
#' data(sample_data)
#' result <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = c(X1 = 7, X2 = 7, X3 = 7)
#' )
#' summary(result)
#' }
#'
#' @export
summary.tsqca_result <- function(object, ...) {
  
  # Determine analysis type from class
  analysis_type <- class(object)[1]
  type_label <- switch(analysis_type,
    "otSweep_result" = "OTS-QCA",
    "dtSweep_result" = "DTS-QCA",
    "ctSweepS_result" = "CTS-QCA",
    "ctSweepM_result" = "MCTS-QCA",
    "ThSQCA"
  )
  
  cat(type_label, "Summary\n")
  cat(strrep("=", nchar(type_label) + 8), "\n\n")
  
  # Analysis Parameters
  cat("Analysis Parameters:\n")
  p <- object$params
  
  if (!is.null(p$outcome)) {
    outcome_str <- p$outcome
    if (isTRUE(p$negate_outcome)) {
      outcome_str <- paste0(outcome_str, " (negated)")
    }
    cat("  Outcome:", outcome_str, "\n")
  }
  if (!is.null(p$conditions)) {
    cat("  Conditions:", paste(p$conditions, collapse = ", "), "\n")
  }
  if (!is.null(p$incl.cut)) {
    cat("  Consistency cutoff:", p$incl.cut, "\n")
  }
  if (!is.null(p$n.cut)) {
    cat("  Frequency cutoff:", p$n.cut, "\n")
  }
  
  cat("\n")
  
  # Results Table
  cat("Results by Threshold:\n\n")
  
  df <- object$summary
  
  # Select columns to display based on what's available
  display_cols <- intersect(
    c("thrY", "threshold", "combo_id", "thrX", 
      "expression", "inclS", "covS", "n_solutions"),
    names(df)
  )
  
  df_display <- df[, display_cols, drop = FALSE]
  
  # Round numeric columns
  for (col in names(df_display)) {
    if (is.numeric(df_display[[col]]) && col %in% c("inclS", "covS")) {
      df_display[[col]] <- round(df_display[[col]], 3)
    }
  }
  
  # Print as formatted table
  print(df_display, row.names = FALSE)
  
  cat("\n")
  
  # Notes
  n_multi <- sum(df$n_solutions > 1, na.rm = TRUE)
  if (n_multi > 0) {
    cat("Note:", n_multi, "threshold setting(s) had multiple solutions.\n")
    cat("      Use generate_report() for full details.\n")
  }
  
  invisible(object)
}

#' @rdname summary.tsqca_result
#' @export
summary.otSweep_result <- function(object, ...) {
  NextMethod()
}

#' @rdname summary.tsqca_result
#' @export
summary.dtSweep_result <- function(object, ...) {
  NextMethod()
}

#' @rdname summary.tsqca_result
#' @export
summary.ctSweepS_result <- function(object, ...) {
  NextMethod()
}

#' @rdname summary.tsqca_result
#' @export
summary.ctSweepM_result <- function(object, ...) {
  NextMethod()
}

#' Sample dataset for ThSQCA examples
#'
#' A small artificial dataset with variables:
#' \describe{
#'   \item{Y}{Outcome (numeric)}
#'   \item{X1}{Condition 1}
#'   \item{X2}{Condition 2}
#'   \item{X3}{Condition 3}
#' }
#'
#' @format A data frame with 80 rows and 4 variables.
"sample_data"

#' @keywords internal
#' @importFrom utils capture.output
"_PACKAGE"

#' ThSQCA: Threshold-Sweep QCA
#'
#' @description
#' The ThSQCA package provides a systematic framework for analyzing threshold 
#' dependency in Qualitative Comparative Analysis (QCA). Rather than relying 
#' on a single calibration threshold, ThSQCA systematically explores how 
#' sufficient conditions change across different threshold levels, revealing 
#' the dynamic structure of causal configurations.
#'
#' @details
#' ## Overview
#' 
#' ThSQCA extends existing QCA methodology by transforming the calibration 
#' stage from a fixed prerequisite into an analytical object. The package 
#' utilizes existing QCA packages (particularly the \pkg{QCA} package's 
#' \code{truthTable()} and \code{minimize()} functions) internally, and 
#' focuses on the systematic exploration of threshold parameter space.
#' 
#' ## Core Philosophy
#' 
#' Traditional QCA assumes researchers select a single calibration threshold 
#' exogenously and extract sufficient conditions based on that threshold. 
#' ThSQCA restructures this process by:
#' 
#' \itemize{
#'   \item Executing QCA repeatedly across multiple threshold values
#'   \item Systematically analyzing changes in obtained solutions
#'   \item Extracting solution stability, critical points, and hierarchical 
#'         causal structures
#' }
#' 
#' This makes ThSQCA a complementary meta-analytical framework that sits 
#' above existing QCA, not a replacement for it.
#' 
#' ## Four Core Methods
#' 
#' \describe{
#'   \item{\strong{OTS-QCA} (Outcome Threshold Sweep)}{
#'     Varies the outcome threshold (e.g., Y >= 6, 7, 8, 9) to identify 
#'     how sufficient conditions change with different target levels of 
#'     the outcome.
#'   }
#'   \item{\strong{CTS-QCA} (Condition Threshold Sweep)}{
#'     Varies a single condition's threshold (e.g., X >= 5, 6, 7, 8) to 
#'     identify the "onset point" where the condition begins to demonstrate 
#'     causal efficacy.
#'   }
#'   \item{\strong{MCTS-QCA} (Multi-dimensional Condition Threshold Sweep)}{
#'     Simultaneously explores threshold combinations across multiple 
#'     conditions (Cartesian product space), visualizing regions of stable 
#'     solutions and critical boundaries where causal structures shift.
#'   }
#'   \item{\strong{DTS-QCA} (Dual Threshold Sweep)}{
#'     Simultaneously varies both outcome and condition thresholds in a 
#'     two-dimensional sweep, enabling analysis of how target outcome 
#'     levels and condition improvement levels interact.
#'   }
#' }
#' 
#' ## Key Advantages
#' 
#' \itemize{
#'   \item \strong{Addresses Threshold Dependency}: Makes calibration 
#'         uncertainty explicit rather than hidden
#'   \item \strong{Reveals Hierarchical Causality}: Identifies how 
#'         conditions operate at different threshold levels
#'   \item \strong{Detects Critical Points}: Locates threshold tipping 
#'         points where causal structures transform
#'   \item \strong{Enhances Robustness}: Tests solution stability across 
#'         threshold ranges
#'   \item \strong{Supports Theory Building}: Generates theoretical 
#'         insights from threshold variation patterns
#' }
#' 
#' ## Relationship with QCA Package
#' 
#' ThSQCA is built on top of the \pkg{QCA} package (Duşa, 2024). All threshold 
#' sweep functions use \code{\link[QCA]{truthTable}} and \code{\link[QCA]{minimize}} 
#' internally. Function arguments such as \code{incl.cut}, \code{n.cut}, 
#' \code{pri.cut}, \code{include}, and \code{dir.exp} follow QCA package 
#' conventions. For detailed explanations of these parameters, please refer to 
#' the QCA package documentation:
#' 
#' \itemize{
#'   \item \code{\link[QCA]{truthTable}} - For threshold and frequency cutoffs 
#'         (\code{incl.cut}, \code{n.cut}, \code{pri.cut})
#'   \item \code{\link[QCA]{minimize}} - For minimization parameters 
#'         (\code{include}, \code{dir.exp})
#' }
#' 
#' This design ensures:
#' 
#' \itemize{
#'   \item \strong{QCA package's role}: Truth table generation, logical 
#'         minimization, consistency/coverage calculation
#'   \item \strong{ThSQCA's role}: Systematic threshold exploration, 
#'         stability analysis, critical point detection
#'   \item \strong{Integration}: ThSQCA calls QCA functions internally; 
#'         it does not reimplement core QCA algorithms
#'   \item \strong{Compatibility}: Works seamlessly with established QCA 
#'         workflows while adding new analytical capabilities
#' }
#' 
#' ## Three Types of QCA Solutions
#' 
#' As of version 1.1.0, ThSQCA uses the same defaults as \code{QCA::minimize()}:
#' 
#' \describe{
#'   \item{\strong{Complex Solution} (default)}{
#'     \code{include = ""}, \code{dir.exp = NULL}. 
#'     Does not use logical remainders. Most conservative interpretation.
#'   }
#'   \item{\strong{Parsimonious Solution}}{
#'     \code{include = "?"}, \code{dir.exp = NULL}. 
#'     Uses all logical remainders. Most simplified form.
#'   }
#'   \item{\strong{Intermediate Solution}}{
#'     \code{include = "?"}, \code{dir.exp = c(1, 1, ...)}. 
#'     Uses only theory-consistent remainders. Most common in publications.
#'   }
#' }
#' 
#' ## Typical Workflow
#' 
#' \enumerate{
#'   \item Prepare data with continuous or ordinal variables
#'   \item Define threshold sequences for conditions and/or outcomes
#'   \item Apply appropriate sweep method (OTS/CTS/MCTS/DTS)
#'   \item Analyze threshold-dependent solution changes
#'   \item Visualize stability regions and critical transitions
#'   \item Interpret hierarchical causal structures
#' }
#' 
#' ## Application Domains
#' 
#' ThSQCA is particularly valuable in fields where:
#' 
#' \itemize{
#'   \item Continuous indicators are common (marketing, organizational research)
#'   \item Threshold choices lack strong theoretical foundation
#'   \item Exploratory theory building is the goal
#'   \item KPI-level thresholds have practical significance
#'   \item Solution robustness is critical
#' }
#'
#' @section Main Functions:
#' 
#' \describe{
#'   \item{\code{\link{otSweep}}}{Execute OTS-QCA (Outcome Threshold Sweep)}
#'   \item{\code{\link{ctSweepS}}}{Execute CTS-QCA (Condition Threshold Sweep)}
#'   \item{\code{\link{ctSweepM}}}{Execute MCTS-QCA (Multi-dimensional CTS)}
#'   \item{\code{\link{dtSweep}}}{Execute DTS-QCA (Dual Threshold Sweep)}
#'   \item{\code{\link{compute_fiss_core}}}{Compute Fiss (2011) core/peripheral classification}
#'   \item{\code{\link{generate_fiss_chart}}}{Generate four-symbol Fiss configuration chart}
#'   \item{\code{\link{print_fiss_summary}}}{Print core/peripheral summary for a threshold}
#'   \item{\code{\link{generate_report}}}{Generate comprehensive Markdown report}
#' }
#'
#' @references 
#' Ragin, C. C. (2008). \emph{Redesigning Social Inquiry: Fuzzy Sets and Beyond}. 
#' Chicago: University of Chicago Press.
#' 
#' Dusa, A. (2024). \emph{QCA: Qualitative Comparative Analysis}. 
#' R package version 3.22. \url{https://CRAN.R-project.org/package=QCA}
#' 
#' Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
#' to typologies in organization research. \emph{Academy of Management Journal},
#' 54(2), 393-420. \doi{10.5465/amj.2011.60263120}
#'
#' @seealso 
#' \itemize{
#'   \item GitHub repository: https://github.com/im-research-yt/ThSQCA}
#'   \item Bug reports: https://github.com/im-research-yt/ThSQCA/issues
#'   \item \pkg{QCA} package for standard QCA analysis
#' }
#' 
#' @examples
#' \dontrun{
#' # Load package
#' library(ThSQCA)
#' data(sample_data)
#' 
#' # Define thresholds
#' thrX <- c(X1 = 7, X2 = 7, X3 = 7)
#' 
#' # Example 1: Complex solution (default, most conservative)
#' result_comp <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = thrX
#' )
#' 
#' # Example 2: Parsimonious solution (uses all logical remainders)
#' result_pars <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = thrX,
#'   include = "?"
#' )
#' 
#' # Example 3: Intermediate solution (most common in publications)
#' result_int <- otSweep(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = thrX,
#'   include = "?",
#'   dir.exp = c(1, 1, 1)
#' )
#' 
#' # Example 4: CTS-QCA with single condition threshold sweep
#' result_cts <- ctSweepS(
#'   dat = sample_data,
#'   outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_var = "X1",
#'   sweep_range = 5:8,
#'   thrY = 7,
#'   thrX_default = 7,
#'   include = "?",
#'   dir.exp = c(1, 1, 1)
#' )
#' 
#' # See vignettes for detailed tutorials
#' vignette("ThSQCA_Tutorial_EN", package = "ThSQCA")
#' }
#'

###############################################
# Fiss Core/Peripheral Classification for ThSQCA
#
# Implements Fiss (2011) core/peripheral distinction:
#   Core condition    : appears in BOTH parsimonious AND intermediate solution
#   Peripheral condition: appears in intermediate solution ONLY
#
# Reference:
#   Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
#   to typologies in organization research. Academy of Management Journal,
#   54(2), 393-420.
###############################################


# ============================================================
# Symbol sets (4-symbol: core present/absent, peripheral present/absent)
# ============================================================

#' Fiss-style symbol sets (4 symbols)
#' @keywords internal
SYMBOL_SETS_FISS <- list(

  unicode = list(
    core_present   = "\u25CF",   # ● BLACK CIRCLE          (large, filled)
    core_absent    = "\u2297",   # ⊗ CIRCLED TIMES         (large, X)
    periph_present = "\u2299",   # ⊙ CIRCLED DOT OPERATOR  (small, dot)
    periph_absent  = "\u2298",   # ⊘ CIRCLED DIVISION SLASH(small, slash)
    note_en = paste0(
      "\u25CF = core presence, \u2297 = core absence, ",
      "\u2299 = peripheral presence, \u2298 = peripheral absence, ",
      "blank = don't care"
    ),
    note_ja = paste0(
      "\u25CF = \u30b3\u30a2\u5b58\u5728, \u2297 = \u30b3\u30a2\u4e0d\u5728, ",
      "\u2299 = \u5468\u8fba\u5b58\u5728, \u2298 = \u5468\u8fba\u4e0d\u5728, ",
      "\u7a7a\u6b04 = \u7121\u95a2\u4fc2"
    )
  ),

  latex = list(
    core_present   = "$\\bullet$",
    core_absent    = "$\\otimes$",
    periph_present = "$\\odot$",
    periph_absent  = "$\\oslash$",
    note_en = paste0(
      "$\\bullet$ = core presence, $\\otimes$ = core absence, ",
      "$\\odot$ = peripheral presence, $\\oslash$ = peripheral absence, ",
      "blank = don't care"
    ),
    note_ja = paste0(
      "$\\bullet$ = \u30b3\u30a2\u5b58\u5728, $\\otimes$ = \u30b3\u30a2\u4e0d\u5728, ",
      "$\\odot$ = \u5468\u8fba\u5b58\u5728, $\\oslash$ = \u5468\u8fba\u4e0d\u5728, ",
      "\u7a7a\u6b04 = \u7121\u95a2\u4fc2"
    )
  ),

  ascii = list(
    core_present   = "O",    # large / core
    core_absent    = "X",
    periph_present = "o",    # small / peripheral
    periph_absent  = "x",
    note_en = paste0(
      "O = core presence, X = core absence, ",
      "o = peripheral presence, x = peripheral absence, ",
      "blank = don't care"
    ),
    note_ja = paste0(
      "O = \u30b3\u30a2\u5b58\u5728, X = \u30b3\u30a2\u4e0d\u5728, ",
      "o = \u5468\u8fba\u5b58\u5728, x = \u5468\u8fba\u4e0d\u5728, ",
      "\u7a7a\u6b04 = \u7121\u95a2\u4fc2"
    )
  )
)


# ============================================================
# Internal helpers
# ============================================================

#' Extract all (condition, status) pairs present in a set of solution terms
#'
#' @param terms Character vector of solution terms (e.g. c("X1*X2", "~X3"))
#' @param conditions Character vector of all condition names
#'
#' @return Named list: condition name -> character vector of statuses
#'   ("present" and/or "absent") found in any term
#'
#' @keywords internal
extract_cond_status_map <- function(terms, conditions) {
  map <- setNames(vector("list", length(conditions)), conditions)
  for (cond in conditions) {
    statuses <- character(0)
    for (t in terms) {
      s <- get_condition_status(t, cond)
      if (s != "dontcare") statuses <- c(statuses, s)
    }
    map[[cond]] <- unique(statuses)
  }
  map
}


#' Classify each (condition, status) pair in an intermediate term as
#' core or peripheral
#'
#' A condition is **core** when it appears with the same presence/absence
#' status in at least one term of the parsimonious solution.
#' A condition is **peripheral** when it appears in the intermediate term
#' but NOT (with the same status) in any parsimonious term.
#'
#' @param interm_term  Character. Single intermediate-solution term.
#' @param parsim_map   Named list returned by \code{extract_cond_status_map()}
#'   for the parsimonious solution.
#' @param conditions   Character vector of all condition names.
#'
#' @return Data frame with columns:
#'   \code{condition}, \code{status} ("present"/"absent"/"dontcare"),
#'   \code{type} ("core"/"peripheral"/"dontcare").
#'
#' @keywords internal
classify_term_conditions <- function(interm_term, parsim_map, conditions) {

  n <- length(conditions)
  result <- data.frame(
    condition = conditions,
    status    = character(n),
    type      = character(n),
    stringsAsFactors = FALSE
  )

  for (i in seq_len(n)) {
    cond     <- conditions[i]
    i_status <- get_condition_status(interm_term, cond)

    if (i_status == "dontcare") {
      result$status[i] <- "dontcare"
      result$type[i]   <- "dontcare"
      next
    }

    result$status[i] <- i_status

    # Core if parsimonious solution also contains this condition with same status
    p_statuses <- parsim_map[[cond]]
    result$type[i] <- if (!is.null(p_statuses) && i_status %in% p_statuses) {
      "core"
    } else {
      "peripheral"
    }
  }

  result
}


#' Run parsimonious QCA minimization on a stored truth table
#'
#' @param truth_table  Truth table object (from QCA::truthTable())
#' @param conditions   Character vector of condition names
#'
#' @return QCA solution object, or NULL on error / no solution
#'
#' @keywords internal
run_parsimonious <- function(truth_table, conditions) {
  truth_table <- sanitize_truthtable(truth_table)  # QCA 3.25 guard
  sol <- try(
    QCA::minimize(
      truth_table,
      include    = "?",
      dir.exp    = NULL,    # parsimonious: no directional expectations
      details    = TRUE,
      show.cases = FALSE
    ),
    silent = TRUE
  )
  if (inherits(sol, "try-error")) return(NULL)
  sol
}


#' Extract solution terms from a QCA solution object (intermediate or parsim)
#'
#' @param sol  QCA solution object
#'
#' @return Character vector of terms (prime implicants), or character(0)
#'
#' @keywords internal
extract_sol_terms <- function(sol) {
  models <- extract_sol_terms_by_model(sol)
  if (length(models) == 0) return(character(0))
  unique(unlist(models, use.names = FALSE))
}


#' Extract solution terms separately for each minimal solution
#'
#' Same sources as \code{extract_sol_terms()}, but the terms of each minimal
#' solution (M1, M2, ...) are kept apart instead of being pooled. This is what
#' the core/peripheral classification needs: pooling first makes a condition
#' look as if it occurred with both polarities in "the" parsimonious solution,
#' when in fact one minimal solution had it present and another had it absent.
#'
#' @param sol QCA solution object.
#'
#' @return List of character vectors, one per minimal solution. Empty list when
#'   no solution is available.
#'
#' @keywords internal
extract_sol_terms_by_model <- function(sol) {
  if (is.null(sol)) return(list())

  # Deduplicate models by term-set content. When the solution spans several
  # prime implicant charts, each chart's $solution may report the same model,
  # so a plain chart-by-chart loop counts one model several times. The
  # core/peripheral classification itself is agreement-based and so is
  # insensitive to duplicates, but the reported parsim_n_solutions (and the
  # warning derived from it) would otherwise be inflated.
  dedup_models <- function(models) {
    if (length(models) <= 1L) return(models)
    keys <- vapply(models, function(x) paste(sort(as.character(x)), collapse = " | "),
                   character(1))
    models[!duplicated(keys)]
  }

  # Intermediate solution stored in i.sol
  if (!is.null(sol$i.sol) && length(sol$i.sol) > 0) {
    models <- list()
    for (entry in sol$i.sol) {
      if (!is.null(entry$solution) && length(entry$solution) > 0) {
        for (s in entry$solution) {
          parsed <- parse_solution_terms(paste(s, collapse = " + "))
          if (length(parsed) > 0) models[[length(models) + 1L]] <- unique(parsed)
        }
      }
    }
    if (length(models) > 0) return(dedup_models(models))
  }

  # Parsimonious / complex solution in $solution
  if (!is.null(sol$solution) && length(sol$solution) > 0) {
    models <- list()
    for (s in sol$solution) {
      parsed <- parse_solution_terms(paste(s, collapse = " + "))
      if (length(parsed) > 0) models[[length(models) + 1L]] <- unique(parsed)
    }
    return(dedup_models(models))
  }

  list()
}


#' Build the parsimonious condition-status map used for core classification
#'
#' A condition counts as core only for a status it holds in EVERY minimal
#' parsimonious solution. With a single minimal solution this is identical to
#' reading that solution directly, so ordinary results are unaffected.
#'
#' With several tied minimal solutions, pooling their terms first (the previous
#' behavior) meant that a condition present in M1 and absent in M2 ended up with
#' both statuses, so an intermediate term matched whichever polarity it used and
#' the condition was classified core either way. Requiring agreement across all
#' minimal solutions removes that bias: what cannot be asserted regardless of
#' which minimal solution is selected is not treated as core.
#'
#' @param models_terms List of character vectors, one per minimal solution
#'   (from \code{extract_sol_terms_by_model()}).
#' @param conditions Character vector of all condition names.
#'
#' @return Named list mapping each condition to the statuses it holds in every
#'   minimal solution (possibly \code{character(0)}).
#'
#' @keywords internal
build_parsim_status_map <- function(models_terms, conditions) {
  if (length(models_terms) == 0) {
    return(setNames(rep(list(character(0)), length(conditions)), conditions))
  }
  per_model <- lapply(models_terms, extract_cond_status_map, conditions = conditions)
  map <- setNames(vector("list", length(conditions)), conditions)
  for (cond in conditions) {
    sets <- lapply(per_model, function(m) m[[cond]])
    map[[cond]] <- Reduce(intersect, sets)
  }
  map
}


# ============================================================
# Public API
# ============================================================

#' Compute Fiss Core/Peripheral Classification for Sweep Results
#'
#' Takes an existing threshold-sweep result object (produced by
#' \code{\link{otSweep}}, \code{\link{ctSweepS}}, \code{\link{ctSweepM}},
#' or \code{\link{dtSweep}}) and augments it with Fiss (2011)
#' core/peripheral classification.
#'
#' The classification requires that:
#' \itemize{
#'   \item The sweep was run with \code{include = "?"} (to allow parsimonious
#'         computation)
#'   \item \code{return_details = TRUE} was used (truth tables must be stored)
#'   \item \code{dir.exp} was specified (i.e., the sweep produced intermediate
#'         solutions — core/peripheral is only meaningful when comparing
#'         parsimonious vs intermediate)
#' }
#'
#' For each threshold in the result, this function:
#' \enumerate{
#'   \item Retrieves the intermediate solution already stored in
#'         \code{result$details}.
#'   \item Re-runs \code{QCA::minimize()} on the same truth table with
#'         \code{dir.exp = NULL} to obtain the parsimonious solution.
#'   \item Compares the two solutions: conditions appearing in both are
#'         \strong{core}; conditions appearing only in the intermediate
#'         solution are \strong{peripheral}.
#' }
#'
#' @param result  A sweep result object with \code{$details} and
#'   \code{$settings} slots (e.g., from \code{otSweep(..., return_details = TRUE)}).
#' @param conditions  Character vector. Condition names (used for consistent
#'   row ordering in charts). If \code{NULL}, extracted automatically.
#'
#' @return The original \code{result} object with an additional
#'   \code{$fiss_core} slot: a named list keyed by threshold (character),
#'   each entry containing:
#'   \itemize{
#'     \item \code{parsim_expression} — parsimonious solution expression
#'     \item \code{interm_expression} — intermediate solution expression
#'     \item \code{classification}    — data frame with columns
#'       \code{term_idx}, \code{term_expr}, \code{condition},
#'       \code{status}, \code{type}
#'   }
#'
#' @references
#' Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
#' to typologies in organization research. \emph{Academy of Management Journal},
#' 54(2), 393-420.
#'
#' @seealso \code{\link{generate_fiss_chart}}
#'
#' @examples
#' \dontrun{
#' library(ThSQCA)
#' data(sample_data)
#'
#' # Step 1: Run intermediate sweep (dir.exp required)
#' res <- otSweep(
#'   dat        = sample_data,
#'   outcome    = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX       = c(X1 = 7, X2 = 7, X3 = 7),
#'   include    = "?",
#'   dir.exp    = c(1, 1, 1),
#'   return_details = TRUE
#' )
#'
#' # Step 2: Augment with Fiss core/peripheral classification
#' res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))
#'
#' # Step 3: Generate Fiss-style chart
#' cat(generate_fiss_chart(res_fiss, symbol_set = "unicode"))
#' }
#'
#' @export
compute_fiss_core <- function(result, conditions = NULL) {

  # --- Guard: details must be present ---
  if (is.null(result$details) || length(result$details) == 0) {
    stop(
      "No details found in result. ",
      "Re-run the sweep with return_details = TRUE."
    )
  }

  # --- Guard: sweep must have used include = "?" ---
  stored_include <- result$params$include
  if (!is.null(stored_include) && stored_include != "?") {
    stop(
      "Fiss core/peripheral classification requires include = \"?\". ",
      "Re-run the sweep with include = \"?\" and dir.exp specified."
    )
  }

  # --- Guard: dir.exp must have been specified ---
  stored_direxp <- result$params$dir.exp
  if (is.null(stored_direxp)) {
    stop(
      "Fiss core/peripheral classification requires dir.exp to be specified ",
      "(intermediate solution). Re-run the sweep with dir.exp = c(1, 1, ...)."
    )
  }

  # --- Resolve conditions ---
  if (is.null(conditions)) {
    conditions <- result$params$conditions
  }
  if (is.null(conditions)) {
    stop("Could not determine condition names. Please supply conditions argument.")
  }

  # --- Process each threshold ---
  fiss_list <- list()

  for (thr_key in names(result$details)) {

    detail <- result$details[[thr_key]]
    tt     <- detail$truth_table
    interm_sol <- detail$solution

    # Skip if no truth table or no intermediate solution
    if (is.null(tt) || is.null(interm_sol)) {
      fiss_list[[thr_key]] <- list(
        parsim_expression = NA_character_,
        interm_expression = NA_character_,
        classification    = NULL
      )
      next
    }

    # Extract intermediate solution terms
    interm_terms <- extract_sol_terms(interm_sol)

    # Build intermediate expression string
    interm_expr <- if (length(interm_terms) > 0) {
      paste(interm_terms, collapse = " + ")
    } else {
      "No solution"
    }

    # Compute parsimonious solution on same truth table
    parsim_sol    <- run_parsimonious(tt, conditions)
    parsim_models <- extract_sol_terms_by_model(parsim_sol)
    parsim_terms  <- if (length(parsim_models) > 0) {
      unique(unlist(parsim_models, use.names = FALSE))
    } else {
      character(0)
    }
    parsim_expr  <- if (length(parsim_terms) > 0) {
      paste(parsim_terms, collapse = " + ")
    } else {
      "No solution"
    }

    # Build parsimonious condition-status map. A status counts only when it
    # holds in EVERY minimal parsimonious solution, so that tied solutions
    # cannot make a condition core under both polarities at once.
    parsim_map <- build_parsim_status_map(parsim_models, conditions)

    n_parsim_sol <- length(parsim_models)
    if (n_parsim_sol > 1L) {
      warning(
        "The parsimonious solution for ", thr_key, " has ", n_parsim_sol,
        " tied minimal solutions. A condition is treated as core only where ",
        "all of them agree, so some conditions may be reported as peripheral.",
        call. = FALSE
      )
    }

    # Classify each intermediate term
    if (length(interm_terms) == 0) {
      fiss_list[[thr_key]] <- list(
        parsim_expression   = parsim_expr,
        interm_expression   = interm_expr,
        parsim_n_solutions  = n_parsim_sol,
        classification      = NULL
      )
      next
    }

    classif_rows <- lapply(seq_along(interm_terms), function(j) {
      row_df <- classify_term_conditions(
        interm_term = interm_terms[j],
        parsim_map  = parsim_map,
        conditions  = conditions
      )
      row_df$term_idx  <- j
      row_df$term_expr <- interm_terms[j]
      row_df[, c("term_idx", "term_expr", "condition", "status", "type")]
    })

    classif_df <- do.call(rbind, classif_rows)
    rownames(classif_df) <- NULL

    fiss_list[[thr_key]] <- list(
      parsim_expression   = parsim_expr,
      interm_expression   = interm_expr,
      parsim_n_solutions  = n_parsim_sol,
      classification      = classif_df
    )
  }

  # Attach to result
  result$fiss_core <- fiss_list
  result
}


# ============================================================
# Chart generation
# ============================================================

#' Build a Fiss-style configuration matrix (4-symbol)
#'
#' @param interm_terms    Character vector of intermediate-solution terms
#' @param classification  Data frame from \code{compute_fiss_core()} for one
#'   threshold (the \code{$classification} element)
#' @param conditions      Character vector of condition names (row order)
#' @param symbols         Fiss symbol set (one element of \code{SYMBOL_SETS_FISS})
#' @param thr_label       Character. Label prefix for column headers (e.g. "thrY=7")
#'
#' @return Character matrix with conditions as rows, terms as columns
#'
#' @keywords internal
build_fiss_matrix <- function(interm_terms, classification,
                               conditions, symbols, thr_label) {
  n_terms <- length(interm_terms)
  n_conds <- length(conditions)

  # Terms are labeled T1, T2, ... rather than M1, M2, ...: M is reserved for
  # whole alternative minimal solutions elsewhere in the package.
  col_names <- paste0(thr_label, " (T", seq_len(n_terms), ")")

  mat <- matrix(
    "",
    nrow = n_conds,
    ncol = n_terms,
    dimnames = list(conditions, col_names)
  )

  for (j in seq_len(n_terms)) {
    rows_j <- classification[classification$term_idx == j, , drop = FALSE]

    for (cond in conditions) {
      row_cond <- rows_j[rows_j$condition == cond, , drop = FALSE]

      if (nrow(row_cond) == 0) next

      status <- row_cond$status[1]
      type   <- row_cond$type[1]

      mat[cond, j] <- if (status == "dontcare" || type == "dontcare") {
        ""
      } else if (type == "core" && status == "present") {
        symbols$core_present
      } else if (type == "core" && status == "absent") {
        symbols$core_absent
      } else if (type == "peripheral" && status == "present") {
        symbols$periph_present
      } else if (type == "peripheral" && status == "absent") {
        symbols$periph_absent
      } else {
        ""
      }
    }
  }

  mat
}


#' Generate Fiss-Style Configuration Chart from Sweep Results
#'
#' Produces a Markdown-formatted configuration chart following Fiss (2011),
#' using four symbols to distinguish core conditions (present in both
#' parsimonious and intermediate solutions) from peripheral conditions
#' (present in intermediate solution only).
#'
#' Call \code{\link{compute_fiss_core}} first to augment the sweep result.
#'
#' @param result     Sweep result augmented by \code{\link{compute_fiss_core}}.
#' @param conditions Character vector. Condition names (row order).
#'   If \code{NULL}, extracted from stored settings.
#' @param symbol_set Character. One of \code{"unicode"} (default),
#'   \code{"ascii"}, or \code{"latex"}.
#' @param language   Character. \code{"en"} (default) or \code{"ja"}.
#'
#' @return Character string: Markdown-formatted Fiss configuration chart.
#'
#' @references
#' Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
#' to typologies in organization research. \emph{Academy of Management Journal},
#' 54(2), 393-420.
#'
#' @seealso \code{\link{compute_fiss_core}}
#'
#' @examples
#' \dontrun{
#' data(sample_data)
#' res <- otSweep(
#'   dat = sample_data, outcome = "Y",
#'   conditions = c("X1", "X2", "X3"),
#'   sweep_range = 6:8,
#'   thrX = c(X1 = 7, X2 = 7, X3 = 7),
#'   include = "?", dir.exp = c(1, 1, 1),
#'   return_details = TRUE
#' )
#' res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))
#' cat(generate_fiss_chart(res_fiss))
#' cat(generate_fiss_chart(res_fiss, symbol_set = "latex"))
#' cat(generate_fiss_chart(res_fiss, language = "ja"))
#' }
#'
#' @export
generate_fiss_chart <- function(result,
                                 conditions = NULL,
                                 symbol_set = c("unicode", "ascii", "latex"),
                                 language   = c("en", "ja")) {

  symbol_set <- match.arg(symbol_set)
  language   <- match.arg(language)
  symbols    <- SYMBOL_SETS_FISS[[symbol_set]]

  # --- Guard: fiss_core must be computed ---
  if (is.null(result$fiss_core)) {
    stop(
      "No fiss_core data found. ",
      "Run compute_fiss_core(result) first."
    )
  }

  # --- Resolve conditions ---
  if (is.null(conditions)) {
    conditions <- result$params$conditions
  }
  if (is.null(conditions)) {
    stop("Could not determine condition names. Please supply conditions argument.")
  }

  # --- Determine threshold label column ---
  sum_df  <- result$summary
  thr_col <- intersect(c("thrY", "threshold", "thrX"), names(sum_df))[1]

  # --- Build one matrix per threshold ---
  all_matrices <- list()

  for (thr_key in names(result$fiss_core)) {
    fc <- result$fiss_core[[thr_key]]

    if (is.null(fc$classification)) next

    interm_terms <- unique(fc$classification$term_expr)
    if (length(interm_terms) == 0) next

    thr_label <- paste0("thrY=", thr_key)

    mat <- build_fiss_matrix(
      interm_terms   = interm_terms,
      classification = fc$classification,
      conditions     = conditions,
      symbols        = symbols,
      thr_label      = thr_label
    )

    all_matrices[[thr_key]] <- mat
  }

  if (length(all_matrices) == 0) {
    return("*No solution found across all thresholds.*\n")
  }

  # --- Combine all matrices into one wide matrix ---
  combined <- do.call(cbind, all_matrices)

  # --- Render as Markdown table ---
  table_str <- config_matrix_to_md(combined, "Condition")

  # --- Legend ---
  legend <- if (language == "ja") symbols$note_ja else symbols$note_en

  paste0(table_str, "\n\n*", legend, "*\n")
}


#' Print Fiss core/peripheral summary for a single threshold
#'
#' Displays which conditions are core and which are peripheral
#' at a given threshold, in a human-readable format.
#'
#' @param result   Sweep result augmented by \code{\link{compute_fiss_core}}.
#' @param thr_key  Character or numeric. Threshold key (e.g., "7" or 7).
#' @param language Character. \code{"en"} or \code{"ja"}.
#'
#' @return Invisibly returns the classification data frame for \code{thr_key}.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' res_fiss <- compute_fiss_core(res)
#' print_fiss_summary(res_fiss, thr_key = "7")
#' }
print_fiss_summary <- function(result, thr_key = NULL, language = c("en", "ja")) {

  language <- match.arg(language)

  if (is.null(result$fiss_core)) {
    stop("No fiss_core data. Run compute_fiss_core() first.")
  }

  # thr_key omitted: summarize every threshold level rather than erroring out
  # with R's generic "argument is missing" message. Callers previously had to
  # inspect names(result$fiss_core) themselves to discover valid keys.
  if (is.null(thr_key)) {
    keys <- names(result$fiss_core)
    if (length(keys) == 0) {
      stop("No threshold levels found in result$fiss_core.", call. = FALSE)
    }
    out <- list()
    for (k in keys) {
      out[[k]] <- print_fiss_summary(result, thr_key = k, language = language)
    }
    return(invisible(out))
  }

  thr_key <- as.character(thr_key)

  if (!thr_key %in% names(result$fiss_core)) {
    available <- paste(names(result$fiss_core), collapse = ", ")
    stop("thr_key '", thr_key, "' not found. Available: ", available)
  }

  fc <- result$fiss_core[[thr_key]]

  if (language == "ja") {
    cat("=== Fiss \u30b3\u30a2/\u5468\u8fba\u5206\u985e (thrY =", thr_key, ") ===\n")
    cat("\u3010\u7c21\u6f54\u89e3\u3011", fc$parsim_expression, "\n")
    cat("\u3010\u4e2d\u9593\u89e3\u3011", fc$interm_expression, "\n\n")
  } else {
    cat("=== Fiss Core/Peripheral Classification (thrY =", thr_key, ") ===\n")
    cat("Parsimonious :", fc$parsim_expression, "\n")
    cat("Intermediate :", fc$interm_expression, "\n\n")
  }

  if (is.null(fc$classification)) {
    cat(if (language == "ja") "\u89e3\u306a\u3057\n" else "No solution\n")
    return(invisible(NULL))
  }

  classif <- fc$classification
  classif_active <- classif[classif$status != "dontcare", , drop = FALSE]

  if (nrow(classif_active) == 0) {
    cat(if (language == "ja") "\u6761\u4ef6\u306a\u3057\n" else "No conditions\n")
    return(invisible(classif))
  }

  # Summary by term
  terms <- unique(classif_active$term_expr)
  for (j in seq_along(terms)) {
    term <- terms[j]
    rows_j <- classif_active[classif_active$term_expr == term, , drop = FALSE]

    core_pres  <- rows_j$condition[rows_j$type == "core"       & rows_j$status == "present"]
    core_abs   <- rows_j$condition[rows_j$type == "core"       & rows_j$status == "absent"]
    periph_pres <- rows_j$condition[rows_j$type == "peripheral" & rows_j$status == "present"]
    periph_abs  <- rows_j$condition[rows_j$type == "peripheral" & rows_j$status == "absent"]

    # Term numbering uses "T" (not "M"): in print(sol) and elsewhere in this
    # package, M1/M2 denote whole SOLUTIONS (alternative minimal solutions),
    # so reusing M for product terms within one solution is ambiguous.
    cat(if (language == "ja") paste0("[\u9805 T", j, "] ") else paste0("[Term T", j, "] "),
        term, "\n")

    if (length(core_pres) > 0)
      cat(if (language == "ja") "  \u30b3\u30a2\u5b58\u5728  : " else "  Core present    : ",
          paste(core_pres, collapse = ", "), "\n")
    if (length(core_abs) > 0)
      cat(if (language == "ja") "  \u30b3\u30a2\u4e0d\u5728  : " else "  Core absent     : ",
          paste(core_abs, collapse = ", "), "\n")
    if (length(periph_pres) > 0)
      cat(if (language == "ja") "  \u5468\u8fba\u5b58\u5728  : " else "  Periph. present : ",
          paste(periph_pres, collapse = ", "), "\n")
    if (length(periph_abs) > 0)
      cat(if (language == "ja") "  \u5468\u8fba\u4e0d\u5728  : " else "  Periph. absent  : ",
          paste(periph_abs, collapse = ", "), "\n")
    cat("\n")
  }

  invisible(classif)
}

test_that("compute_fiss_core returns augmented result with fiss_core slot", {
  data("sample_data", package = "ThSQCA")

  res <- otSweep(
    dat        = sample_data,
    outcome    = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX       = c(X1 = 7, X2 = 7, X3 = 7),
    include    = "?",
    dir.exp    = c(1, 1, 1),
    return_details = TRUE
  )

  res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))

  expect_true(!is.null(res_fiss$fiss_core))
  expect_true(is.list(res_fiss$fiss_core))
  # Keys should match sweep_range
  expect_true(all(c("6", "7", "8") %in% names(res_fiss$fiss_core)))
})

test_that("compute_fiss_core classification has correct structure", {
  data("sample_data", package = "ThSQCA")

  res <- otSweep(
    dat        = sample_data,
    outcome    = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 7,
    thrX       = c(X1 = 7, X2 = 7, X3 = 7),
    include    = "?",
    dir.exp    = c(1, 1, 1),
    return_details = TRUE
  )

  res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))
  fc <- res_fiss$fiss_core[["7"]]

  expect_true(!is.null(fc))
  expect_true("parsim_expression" %in% names(fc))
  expect_true("interm_expression" %in% names(fc))

  # If a solution exists, classification should be a data frame
  if (!is.null(fc$classification)) {
    expect_s3_class(fc$classification, "data.frame")
    expect_true(all(c("term_idx", "term_expr", "condition",
                      "status", "type") %in% names(fc$classification)))
    # type must be one of core/peripheral/dontcare
    expect_true(all(fc$classification$type %in%
                      c("core", "peripheral", "dontcare")))
    # status must be one of present/absent/dontcare
    expect_true(all(fc$classification$status %in%
                      c("present", "absent", "dontcare")))
  }
})

test_that("compute_fiss_core errors without return_details", {
  data("sample_data", package = "ThSQCA")

  res <- otSweep(
    dat        = sample_data,
    outcome    = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 7,
    thrX       = c(X1 = 7, X2 = 7, X3 = 7),
    include    = "?",
    dir.exp    = c(1, 1, 1),
    return_details = FALSE  # no details
  )

  expect_error(compute_fiss_core(res), "return_details")
})

test_that("compute_fiss_core errors without dir.exp", {
  data("sample_data", package = "ThSQCA")

  res <- otSweep(
    dat        = sample_data,
    outcome    = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 7,
    thrX       = c(X1 = 7, X2 = 7, X3 = 7),
    include    = "?",
    # dir.exp = NULL  (not specified)
    return_details = TRUE
  )

  expect_error(compute_fiss_core(res), "dir.exp")
})

test_that("generate_fiss_chart returns character string", {
  data("sample_data", package = "ThSQCA")

  res <- otSweep(
    dat        = sample_data,
    outcome    = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 6:8,
    thrX       = c(X1 = 7, X2 = 7, X3 = 7),
    include    = "?",
    dir.exp    = c(1, 1, 1),
    return_details = TRUE
  )
  res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))

  chart_uni   <- generate_fiss_chart(res_fiss, symbol_set = "unicode")
  chart_latex <- generate_fiss_chart(res_fiss, symbol_set = "latex")
  chart_ascii <- generate_fiss_chart(res_fiss, symbol_set = "ascii")

  expect_type(chart_uni,   "character")
  expect_type(chart_latex, "character")
  expect_type(chart_ascii, "character")

  # Should contain table markup
  expect_true(grepl("\\|", chart_uni))
})

test_that("generate_fiss_chart errors without fiss_core", {
  data("sample_data", package = "ThSQCA")

  res <- otSweep(
    dat        = sample_data,
    outcome    = "Y",
    conditions = c("X1", "X2", "X3"),
    sweep_range = 7,
    thrX       = c(X1 = 7, X2 = 7, X3 = 7),
    include    = "?",
    dir.exp    = c(1, 1, 1),
    return_details = TRUE
  )
  # No compute_fiss_core call
  expect_error(generate_fiss_chart(res), "fiss_core")
})

test_that("SYMBOL_SETS_FISS has correct structure", {
  for (fmt in c("unicode", "latex", "ascii")) {
    ss <- SYMBOL_SETS_FISS[[fmt]]
    expect_true(!is.null(ss$core_present))
    expect_true(!is.null(ss$core_absent))
    expect_true(!is.null(ss$periph_present))
    expect_true(!is.null(ss$periph_absent))
    expect_true(!is.null(ss$note_en))
    expect_true(!is.null(ss$note_ja))
  }
})

# Regression tests for the core/peripheral polarity-pooling fix (v2.0.5).
#
# When the parsimonious solution has several tied minimal solutions, the terms
# used to used to be pooled across all of them before the condition-status map
# was built. A condition present in M1 and absent in M2 therefore ended up
# holding both statuses, so an intermediate term matched whichever polarity it
# used and the condition was classified "core" either way: a structural bias
# toward core. A status now counts only where every minimal solution agrees.

test_that("extract_sol_terms_by_model keeps minimal solutions apart", {
  sol <- list(solution = list(c("A*C", "B"), c("A*~C", "B")))
  models <- extract_sol_terms_by_model(sol)
  expect_equal(length(models), 2L)
  expect_true(all(c("A*C", "B") %in% models[[1]]))
  expect_true(all(c("A*~C", "B") %in% models[[2]]))
  # The pooled helper still returns the union, for the displayed expression.
  expect_true(all(c("A*C", "A*~C", "B") %in% extract_sol_terms(sol)))
})

test_that("a condition with conflicting polarity across minimal solutions is not core", {
  conds <- c("A", "B", "C")
  models <- list(c("A*C", "B"), c("A*~C", "B"))   # C present in M1, absent in M2

  pooled <- extract_cond_status_map(unique(unlist(models)), conds)
  expect_setequal(pooled[["C"]], c("present", "absent"))   # the old, biased map

  map <- build_parsim_status_map(models, conds)
  expect_equal(length(map[["C"]]), 0L)   # no status survives the disagreement
  # Conditions the solutions agree on are unaffected.
  expect_true("present" %in% map[["A"]])
  expect_true("present" %in% map[["B"]])

  # Consequently an intermediate term using either polarity of C is peripheral.
  for (term in c("A*C*B", "A*~C*B")) {
    cl <- classify_term_conditions(term, map, conds)
    expect_equal(cl$type[cl$condition == "C"], "peripheral")
    expect_equal(cl$type[cl$condition == "A"], "core")
  }
})

test_that("with a single minimal solution the map is unchanged", {
  conds <- c("A", "B", "C")
  single <- list(c("A*C", "B"))
  old <- extract_cond_status_map(unlist(single), conds)
  new <- build_parsim_status_map(single, conds)
  expect_equal(lapply(old, sort), lapply(new, sort))
})

test_that("a condition absent from one minimal solution is not core", {
  conds <- c("A", "B", "C")
  # C is present in M1 and does not occur at all in M2.
  models <- list(c("A*C"), c("A*B"))
  map <- build_parsim_status_map(models, conds)
  expect_equal(length(map[["C"]]), 0L)
  expect_true("present" %in% map[["A"]])
  cl <- classify_term_conditions("A*C", map, conds)
  expect_equal(cl$type[cl$condition == "C"], "peripheral")
})

test_that("compute_fiss_core records the number of tied parsimonious solutions", {
  skip_if_not_installed("QCA")
  data("sample_data", package = "ThSQCA")
  res <- otSweep(dat = sample_data, outcome = "Y",
                 conditions = c("X1", "X2", "X3"),
                 sweep_range = 6:8, thrX = c(X1 = 7, X2 = 7, X3 = 7),
                 include = "?", dir.exp = c(X1 = 1, X2 = 1, X3 = 1),
                 return_details = TRUE)
  f <- suppressWarnings(compute_fiss_core(res))
  entries <- Filter(function(e) !is.null(e$classification), f$fiss_core)
  skip_if(length(entries) == 0)
  for (e in entries) {
    expect_true(!is.null(e$parsim_n_solutions))
    expect_true(e$parsim_n_solutions >= 1)
  }
})

---
title: "ThSQCA Tutorial (English)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ThSQCA Tutorial (English)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

Threshold-Sweep QCA (TS-QCA) is a framework for systematically exploring how different threshold settings affect the results of crisp-set Qualitative Comparative Analysis (QCA).  
In crisp-set QCA, the researcher must choose thresholds to binarize:

- the outcome **Y**, and
- the conditions **X**.

Small changes in these thresholds may lead to substantial differences in truth tables and minimized solutions.

TS-QCA provides:

- a **systematic** way to vary thresholds,
- a **transparent** way to evaluate sensitivity,
- and a **reproducible** workflow for robustness assessment.

The `ThSQCA` package implements four sweep methods:

| Method | What varies | What stays fixed | Purpose |
|--------|-------------|------------------|----------|
| **CTS–QCA** | One X threshold | Y + other Xs | Evaluate influence of a single condition |
| **MCTS–QCA** | Multiple X thresholds | Y | Explore combinations of X thresholds |
| **OTS–QCA** | Y threshold | All Xs | Assess robustness to Y calibration |
| **DTS–QCA** | X and Y thresholds | None | Full 2D sensitivity analysis |

> **Scope:** This package focuses on **sufficiency analysis**—identifying condition combinations that are sufficient for an outcome. Necessity analysis (whether a condition is required for an outcome) involves different logical structures and evaluation metrics, and is planned for future versions.

> **Important (v1.2.0)**: Version 1.2.0 fixed a critical bug where intermediate solutions (using `dir.exp`) were incorrectly extracted. If you used intermediate solutions in previous versions, please re-run your analyses. Reports now also display the Solution Type and include optional QCA package output for verification.

---

# Three Types of QCA Solutions (New in v1.1.0)

## Overview

QCA minimization can produce three types of solutions depending on how **logical remainders** (unobserved configurations) are handled:

| Solution Type | `include` | `dir.exp` | Logical Remainders | Description |
|--------------|-----------|-----------|-------------------|-------------|
| **Complex** (default) | `""` | `NULL` | Not used | Most conservative; only observed configurations |
| **Parsimonious** | `"?"` | `NULL` | All used | Most simplified; uses all remainders |
| **Intermediate** | `"?"` | `c(1,1,...)` | Theory-guided | Uses remainders consistent with theory |

## QCA-Compatible Defaults (v1.1.0)

As of v1.1.0, ThSQCA uses the **same defaults** as `QCA::minimize()`:

- `include = ""` → Does not include logical remainders
- `dir.exp = NULL` → No directional expectations

This produces the **complex (conservative) solution** by default.

## Example: Computing All Three Solutions

```{r}
library(ThSQCA)
data(sample_data)
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

# 1. Complex Solution (default, most conservative)
result_comp <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX
  # include = "" (default), dir.exp = NULL (default)
)
cat("Complex Solution:", result_comp$summary$expression, "\n")

# 2. Parsimonious Solution (uses all logical remainders)
result_pars <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?"  # Include logical remainders
)
cat("Parsimonious Solution:", result_pars$summary$expression, "\n")

# 3. Intermediate Solution (theory-guided remainders)
result_int <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?",
  dir.exp = c(1, 1, 1)  # All conditions expected to contribute positively
)
cat("Intermediate Solution:", result_int$summary$expression, "\n")
```

## When to Use Each Solution

| Solution | When to Use | Pros | Cons |
|----------|-------------|------|------|
| **Complex** | Exploratory analysis, maximum caution | No assumptions about unobserved cases | Often too complex for interpretation |
| **Parsimonious** | Robustness checks, simplicity | Most simplified form | May rely on implausible assumptions |
| **Intermediate** | Theory-driven analysis, publications | Balances parsimony and theory | Requires theoretical justification |

> **Note:** The **intermediate solution** is most commonly reported in QCA publications (Ragin, 2008; Fiss, 2011), as it incorporates theoretical knowledge while avoiding implausible simplifying assumptions.

## Migration from v1.0.0

If you were using ThSQCA v1.0.0, note that the default behavior has changed:

```{r, eval=FALSE}
# v1.0.0 (intermediate solution by default due to bug)
result <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX)

# v1.1.0: To get the same result as v1.0.0, explicitly specify:
result <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?",
  dir.exp = c(1, 1, 1)  # Explicit intermediate solution
)
```

---

# Data Requirements

TS-QCA assumes:

- TS-QCA handles any numeric data (continuous, interval, or ordinal scales). 
  Pre-calibration is NOT required because raw scores (e.g., 1–85 scale) are 
  binarized directly at specified thresholds.
- Ordinal data can be analyzed directly, enabling analyses that are difficult 
  with fsQCA (which requires continuous membership values).
- Thresholds can be any numeric value (integer or real).
- Missing values should be handled before running sweeps.

Example dataset structure:
```{r}
library(ThSQCA)
data("sample_data")
dat <- sample_data
str(dat)
```

Define outcome and conditions:
```{r}
outcome  <- "Y"
conditions <- c("X1", "X2", "X3")
```

---

# Working with Mixed Data Types

## Handling Binary and Continuous Variables

In real-world social science research, datasets often contain **both binary variables** (e.g., gender, yes/no responses) **and continuous variables** (e.g., sales, satisfaction scores). When using ThSQCA with such mixed data, special attention is required.

### Key Principles

1. **Do NOT sweep binary variables** — they are already binarized (0/1)
2. **Use threshold = 1 for binary variables** to preserve their original values
3. **Explicitly specify thresholds** for each variable in `sweep_list`

### Why Threshold = 1 for Binary Variables?

The internal `qca_bin()` function uses the rule `x >= thr` for binarization:

- If `x = 0`: `0 >= 1` → FALSE → **0** (preserved)
- If `x = 1`: `1 >= 1` → TRUE → **1** (preserved)

This ensures that binary variables remain unchanged during the binarization process.

### Practical Example: Mixed Data

Suppose your dataset has:

- **X1**: Gender (0 = Male, 1 = Female) — binary variable
- **X2**: Product Quality Score (0-10) — continuous variable
- **X3**: Store Atmosphere Score (0-10) — continuous variable

When using `ctSweepM()`:

```{r, eval=FALSE}
# CORRECT: Specify threshold explicitly for each variable
sweep_list <- list(
  X1 = 1,      # Binary variable: use threshold 1
  X2 = 6:8,    # Continuous: sweep thresholds
  X3 = 6:8     # Continuous: sweep thresholds
)

# Using intermediate solution (most common in publications)
res_mixed <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1)     # Directional expectations (intermediate)
)
```

This explores 1 × 3 × 3 = **9 threshold combinations**, treating X1 as a fixed binary condition while sweeping X2 and X3.

### Common Mistake

```{r, eval=FALSE}
# WRONG: Using sweep range for binary variables
sweep_list <- list(
  X1 = 6:8,    # All values become 0 (since 0 < 6 and 1 < 6)
  X2 = 6:8,
  X3 = 6:8
)
```

If you accidentally specify `X1 = 6:8`, both 0 and 1 will fail the `>= 6` condition, making all X1 values become 0. This destroys the information in your binary variable.

### Best Practice

Always examine your data structure before setting up threshold sweeps:

```{r, eval=FALSE}
# Check variable ranges
summary(dat[, c("X1", "X2", "X3")])

# Identify binary variables (only 0 and 1)
sapply(dat[, c("X1", "X2", "X3")], function(x) {
  unique_vals <- sort(unique(x))
  if (length(unique_vals) == 2 && all(unique_vals == c(0, 1))) {
    "Binary (use threshold = 1)"
  } else {
    paste("Continuous (range:", min(x), "-", max(x), ")")
  }
})
```

## Using Pre-Calibrated Fuzzy Conditions (`pre_calibrated`)

Version 1.3.0 introduces the `pre_calibrated` argument to all four sweep
functions (`otSweep`, `ctSweepS`, `ctSweepM`, `dtSweep`). This argument
enables a **mixed crisp/fuzzy** workflow: some conditions can be passed as
fuzzy-set membership scores (values in [0, 1] produced by
`QCA::calibrate()`), while other conditions are swept on their raw scale
as usual.

### How it works

- Variables listed in `pre_calibrated` are passed directly to
  `QCA::truthTable()` **without binarization**.
- All other conditions are binarized using `qca_bin()` at the thresholds
  specified in `thrX`.
- Pre-calibrated variables do not require a `thrX` entry, since no
  binarization is applied to them.

### Validation

The function raises an error if:

- A variable listed in `pre_calibrated` is not found in `conditions`.
- Its values fall outside the [0, 1] range (indicating that a raw-scale
  variable was passed by mistake).

A warning is issued if the variable contains `NA` values.

### Sweep variables vs. pre-calibrated variables

Sweep variables should be kept on their **raw (original) scale**. When you
sweep a variable on its raw scale (e.g., Likert scores 6, 7, 8), each
threshold carries direct substantive meaning. Pre-calibrating a sweep
variable into fuzzy values and then sweeping over those values (e.g., 0.3,
0.5, 0.7) makes interpretation indirect and harder to justify.

If a variable appears in both `pre_calibrated` and a sweep list, ThSQCA
issues a warning and uses the pre-calibrated values, ignoring the sweep
thresholds.

### Recommended practice

| Variable type | Recommended approach |
|---|---|
| Conditions you want to sweep | Keep on raw scale; use `qca_bin()` via `thrX` |
| Conditions with theoretically grounded fuzzy calibration | Pre-calibrate and list in `pre_calibrated` |
| Binary conditions (0/1) | Use `thrX = 1`; optionally list in `pre_calibrated` |

*A worked example with sample data will be added in a future release.*

---

# CTS–QCA: Single-Condition Sweep (`ctSweepS`)

CTS–QCA varies the threshold for **one X condition**, keeping the others fixed.

## Default: Complex Solution
```{r, error=TRUE}
sweep_var   <- "X3"   # Condition (X) whose threshold is swept
sweep_range <- 6:9    # Candidate threshold values to evaluate

thrY         <- 7     # Outcome (Y) threshold (fixed)
thrX_default <- 7     # Threshold for other X conditions (fixed)

# Default: Complex solution (include = "", dir.exp = NULL)
res_cts <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = sweep_var,
  sweep_range    = sweep_range,
  thrY           = thrY,
  thrX_default   = thrX_default,
  return_details = TRUE
)

summary(res_cts)
```

## Intermediate Solution (Theory-Driven)
```{r, error=TRUE}
# Intermediate solution: specify include = "?" and dir.exp
res_cts_int <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = sweep_var,
  sweep_range    = sweep_range,
  thrY           = thrY,
  thrX_default   = thrX_default,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1),    # Directional expectations
  return_details = TRUE
)

summary(res_cts_int)
```

---

# MCTS–QCA: Multiple X Sweep (`ctSweepM`)

MCTS–QCA evaluates all combinations of thresholds for multiple X conditions.

## Default: Complex Solution
```{r, error=TRUE}
# Create a sweep list specifying thresholds for each condition
sweep_list <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

# Default: Complex solution
res_mcts <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  return_details = TRUE
)

summary(res_mcts)
```

## Intermediate Solution (Theory-Driven)
```{r, error=TRUE}
# Intermediate solution: specify include = "?" and dir.exp
res_mcts_int <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_mcts_int)
```

---

# OTS–QCA: Outcome Sweep (`otSweep`)

OTS–QCA varies only the threshold of **Y**, keeping X thresholds fixed.

## Default: Complex Solution
```{r}
# Default: Complex solution
res_ots <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = c(X1 = 7, X2 = 7, X3 = 7),
  return_details = TRUE
)

summary(res_ots)
```

## Intermediate Solution (Theory-Driven)
```{r}
# Intermediate solution: specify include = "?" and dir.exp
res_ots_int <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = c(X1 = 7, X2 = 7, X3 = 7),
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_ots_int)
```

---

# DTS–QCA: Two-Dimensional Sweep (`dtSweep`)

DTS–QCA varies both **X thresholds** and **Y thresholds**, creating a full 2D grid.

## Default: Complex Solution
```{r}
sweep_list_dts_X <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

sweep_range_dts_Y <- 6:7

# Default: Complex solution
res_dts <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  return_details = TRUE
)

summary(res_dts)
```

## Intermediate Solution (Theory-Driven)
```{r}
# Intermediate solution: specify include = "?" and dir.exp
res_dts_int <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_dts_int)
```

---

# Understanding the Output

Each sweep result contains:

- threshold values tested,
- minimized solution expression,
- solution consistency (`inclS`),
- solution coverage (`covS`).

General guidance:

- **Consistency ≥ 0.8** is typically required for sufficiency.
- **Coverage** indicates empirical relevance; higher is better.
- A solution sensitive to small threshold changes suggests low robustness.

---

# Handling Multiple Solutions (New in v0.2.0)

## Why Multiple Solutions Matter

When QCA minimization produces multiple equivalent intermediate solutions, researchers face a methodological challenge: which solution should be reported? Traditional approaches often report only the first solution (M1), but this may miss important causal heterogeneity.

ThSQCA v0.2.0 addresses this by:

1. **Detecting** when multiple solutions exist
2. **Extracting** essential prime implicants (terms common to all solutions)
3. **Identifying** selective prime implicants and unique terms

## Essential vs. Selective Prime Implicants

Understanding the distinction between essential and selective prime implicants is essential for robust QCA interpretation:

| Type | Definition | Interpretation |
|------|------------|----------------|
| **Essential prime implicants** | Present in ALL solutions | Robust findings; essential causal factors |
| **Selective prime implicants** | Present in SOME but not all solutions | Context-dependent; may vary across cases |
| **Unique terms** | Present in only ONE specific solution | Solution-specific; least robust |

## Using `extract_mode`

The `extract_mode` parameter controls how solutions are extracted:

### Mode: "first" (Default)
```{r, eval=FALSE}
# Returns only the first solution (M1)
# Backward compatible with v0.1.x
result <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7),
  extract_mode = "first"  # Default
)
```

### Mode: "all"
```{r, eval=FALSE}
# Returns all solutions concatenated
# Useful for seeing all equivalent solutions
result_all <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7),
  extract_mode = "all"
)

# Output includes n_solutions column
head(result_all$summary)
# expression column shows: "M1: A*B + C; M2: A*B + D; M3: ..."
```

### Mode: "essential"
```{r, eval=FALSE}
# Returns essential prime implicants (terms common to all solutions)
# Best for identifying robust findings
result_essential <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7),
  extract_mode = "essential"
)

# Output includes:
# - expression: essential prime implicants
# - selective_terms: terms in some but not all solutions
# - unique_terms: solution-specific terms
# - n_solutions: number of equivalent solutions
```

## Practical Example

Consider a scenario where QCA produces three equivalent solutions:

- M1: `A*B + C → Y`
- M2: `A*B + D → Y`
- M3: `A*B + E → Y`

The analysis reveals:

| Component | Terms | Interpretation |
|-----------|-------|----------------|
| Essential (EPI) | `A*B` | Present in all three solutions; robust finding |
| Selective (SPI) | `C, D, E` | Each appears in one solution; context-dependent |
| Unique (M1) | `C` | Only in solution 1 |
| Unique (M2) | `D` | Only in solution 2 |
| Unique (M3) | `E` | Only in solution 3 |

**Recommendation**: Report the essential prime implicants (`A*B → Y`) as your main finding, and discuss the selective prime implicants as alternative pathways in your discussion section.

---

# Generating Reports (New in v0.2.0)

## Overview

The `generate_report()` function creates comprehensive markdown reports from your analysis results. This automates the documentation process and ensures reproducibility.

## Report Formats

### Full Report

Contains comprehensive analysis details:

- Analysis settings (for reproducibility)
- Summary table across all thresholds
- Per-threshold detailed results
- All solution formulas
- Essential and selective prime implicants
- Fit measures (consistency, coverage, PRI)
- Configuration charts (New in v0.5.0)

```{r, eval=FALSE}
# Generate full report
generate_report(res_ots, "my_analysis_full.md", dat = dat, format = "full")
```

### Simple Report

Designed for journal manuscripts:

- Condensed format
- Essential information only
- Ready for supplementary materials

```{r, eval=FALSE}
# Generate simple report
generate_report(res_ots, "my_analysis_simple.md", dat = dat, format = "simple")
```

## Using Reports Effectively

### For Research Papers

1. Run analysis with `return_details = TRUE` (default in v0.2.0)
2. Generate a **simple report** for the main manuscript
3. Generate a **full report** for supplementary materials

```{r, eval=FALSE}
# Complete workflow
result <- otSweep(
  dat = mydata,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)

# For main text
generate_report(result, "manuscript_results.md", dat = mydata, format = "simple")

# For supplementary materials
generate_report(result, "supplementary_full.md", dat = mydata, format = "full")
```

### Accessing Analysis Parameters

All analysis parameters are stored in `result$params` for reproducibility:

```{r, eval=FALSE}
# View stored parameters
result$params

# Includes:
# - outcome, conditions: variable names
# - thrX, thrY: threshold values
# - incl.cut, n.cut, pri.cut: QCA parameters
# - dir.exp, include: minimization settings
```

---

# Best Practices

## Start Small, Then Expand

When using sweep functions, the number of QCA analyses grows quickly. A systematic approach prevents wasted computation:

### Step 1: Single Value Test
```{r, eval=FALSE}
# Test with a single threshold first
result <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,  # Single value
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
```

### Step 2: Small Range
```{r, eval=FALSE}
# Expand to a small range
result <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:7,  # Small range
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
```

### Step 3: Full Analysis
```{r, eval=FALSE}
# Run full analysis only after confirming it works
result <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 5:9,  # Full range
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
```

## Computational Complexity

Understanding computational complexity helps plan your analysis:

| Function | Complexity | Example | Analyses |
|----------|------------|---------|----------|
| `otSweep()` | O(n) | 5 Y thresholds | 5 |
| `ctSweepS()` | O(n) | 5 X thresholds | 5 |
| `ctSweepM()` | O(m^k) | 3 thresholds × 3 conditions | 27 |
| `dtSweep()` | O(n × m^k) | 3 Y × 3^3 X | 81 |

### Managing Large Sweeps

For `dtSweep()` and `ctSweepM()`, reduce conditions first:

```{r, eval=FALSE}
# Manageable: 2 × 2 × 2 = 8 combinations
sweep_list <- list(X1 = 6:7, X2 = 6:7, X3 = 6:7)

# Caution: 5 × 5 × 5 = 125 combinations
sweep_list <- list(X1 = 5:9, X2 = 5:9, X3 = 5:9)

# Avoid: 5 × 5 × 5 × 5 × 5 = 3125 combinations!
sweep_list <- list(X1 = 5:9, X2 = 5:9, X3 = 5:9, X4 = 5:9, X5 = 5:9)
```

## Interpreting Results

### When Solutions Are Stable

If the same solution appears across multiple thresholds, your findings are robust:

```
thrY | expression    | inclS | covS
-----|---------------|-------|------
6    | A*B + C → Y   | 0.85  | 0.72
7    | A*B + C → Y   | 0.88  | 0.68
8    | A*B + C → Y   | 0.91  | 0.65
```

**Interpretation**: The solution `A*B + C` is robust across threshold variations.

### When Solutions Change

If solutions vary significantly, investigate the threshold sensitivity:

```
thrY | expression    | inclS | covS
-----|---------------|-------|------
6    | A*B + C → Y   | 0.82  | 0.75
7    | A*B → Y       | 0.89  | 0.62
8    | B*D → Y       | 0.93  | 0.45
```

**Interpretation**: Results are threshold-sensitive. Consider reporting the most theoretically justified threshold with appropriate caveats.

---

# Negated Outcome Analysis (New in v0.3.0)

## Why Analyze Negated Outcomes?

In QCA, researchers typically analyze conditions sufficient for the **presence** of an outcome (Y = 1). However, understanding conditions for the **absence** of an outcome (~Y) can provide equally valuable insights:

- What conditions lead to project **failure** (not success)?
- What factors contribute to customer **dissatisfaction** (not satisfaction)?
- What explains policy **non-adoption** (not adoption)?

## Using the `~` Notation

ThSQCA v0.3.0 supports the QCA package's tilde (`~`) notation for negated outcomes:

```{r, eval=FALSE}
# Analyze conditions for Y >= threshold (standard)
result_Y <- otSweep(
  dat = dat,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)

# Analyze conditions for Y < threshold (negated)
result_negY <- otSweep(
  dat = dat,
  outcome = "~Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
```

## Interpreting Negated Results

When using `~Y`, the solution shows conditions sufficient for the **absence** of the outcome:

| Analysis | Solution Example | Interpretation |
|----------|------------------|----------------|
| `outcome = "Y"` | `X1*X2 + X3` | High X1 AND X2, OR high X3 → High Y |
| `outcome = "~Y"` | `~X1*~X3 + ~X2*~X3` | Low X1 AND X3, OR low X2 AND X3 → Low Y |

**Note**: Negated conditions (`~X1`) in the solution mean the **absence** of that condition (below threshold).

## All Sweep Functions Support Negation

```{r, eval=FALSE}
# ctSweepS with negated outcome
result <- ctSweepS(
  dat = dat,
  outcome = "~Y",
  conditions = c("X1", "X2", "X3"),
  sweep_var = "X3",
  sweep_range = 6:8,
  thrY = 7,
  thrX_default = 7
)

# ctSweepM with negated outcome
result <- ctSweepM(
  dat = dat,
  outcome = "~Y",
  conditions = c("X1", "X2"),
  sweep_list = list(X1 = 6:7, X2 = 6:7),
  thrY = 7
)

# dtSweep with negated outcome
result <- dtSweep(
  dat = dat,
  outcome = "~Y",
  conditions = c("X1", "X2"),
  sweep_list_X = list(X1 = 6:7, X2 = 7),
  sweep_range_Y = 6:8
)
```

## Checking Negation in Results

The `params` object stores whether negation was used:

```{r, eval=FALSE}
result <- otSweep(dat = dat, outcome = "~Y", ...)

# Check if negated
result$params$negate_outcome
# [1] TRUE

result$params$outcome
# [1] "~Y"
```

---

# Fiss (2011) Core/Peripheral Classification (New in v1.3.2)

## What Is the Core/Peripheral Distinction?

Fiss (2011) introduced a critical refinement to QCA configuration tables
that is now standard in the *Academy of Management Journal* and other top
management journals. Beyond simply indicating whether a condition is present
or absent in a solution, Fiss distinguishes between:

| Type | Definition | Symbol |
|------|------------|--------|
| **Core condition** | Appears in BOTH the parsimonious AND intermediate solutions | ● (large) |
| **Peripheral condition** | Appears in the intermediate solution ONLY | ⊙ (small) |

The intuition is straightforward: conditions that survive even the most
aggressive simplification (parsimonious solution) are more central to the
causal story than conditions that only appear when theoretical constraints
are applied (intermediate solution).

> **Why this matters:** Prior to v1.3.2, ThSQCA configuration charts treated
> all conditions as equivalent, using only two symbols. This is technically
> incorrect per Fiss (2011) and may raise concerns from reviewers at
> journals that require Fiss notation.

## The Four-Symbol Set

```
● = core condition present      (large filled circle)
⊗ = core condition absent       (large circle-times)
⊙ = peripheral condition present (small circled dot)
⊘ = peripheral condition absent  (small circled slash)
  = don't care (blank)
```

For LaTeX / academic PDF output: `$\bullet$`, `$\otimes$`, `$\odot$`, `$\oslash$`.

## Workflow

`compute_fiss_core()` requires three conditions from the original sweep:

1. `include = "?"` — logical remainders must be enabled
2. `dir.exp` specified — intermediate solution must have been requested
3. `return_details = TRUE` — truth tables must be stored

```{r, eval=FALSE}
library(ThSQCA)
data(sample_data)

# Step 1: Run intermediate sweep (all three conditions above are met)
res <- otSweep(
  dat        = sample_data,
  outcome    = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX       = c(X1 = 7, X2 = 7, X3 = 7),
  include    = "?",           # required
  dir.exp    = c(1, 1, 1),   # required
  return_details = TRUE       # required
)

# Step 2: Compute Fiss core/peripheral classification
# For each threshold, compute_fiss_core() re-runs QCA::minimize()
# with dir.exp = NULL to obtain the parsimonious solution,
# then compares it to the stored intermediate solution.
res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))

# Step 3: Inspect a specific threshold
print_fiss_summary(res_fiss, thr_key = "7")

# Step 4: Generate Fiss-style four-symbol chart
cat(generate_fiss_chart(res_fiss, symbol_set = "unicode"))

# For academic papers (LaTeX)
cat(generate_fiss_chart(res_fiss, symbol_set = "latex"))

# Step 5: Include in report
generate_report(
  res_fiss,
  output_file      = "fiss_report.md",
  format           = "full",
  include_fiss_core = TRUE,          # activates four-symbol charts
  chart_symbol_set  = "unicode"
)
```

## Reading the Output

`print_fiss_summary()` shows the classification term-by-term:

```
=== Fiss Core/Peripheral Classification (thrY = 7) ===
Parsimonious : X1*X2
Intermediate : X3 + X1*X2

[Term M1] X3
  Periph. present : X3

[Term M2] X1*X2
  Core present    : X1, X2
```

**Interpretation:** X1 and X2 are core (they appear in the parsimonious
solution `X1*X2`). X3 is peripheral — it only appears when theory-guided
directional expectations are applied (intermediate solution), suggesting
it plays a supporting rather than essential causal role.

## What If All Conditions Are Core?

When the parsimonious and intermediate solutions are identical, all
conditions are core and no conditions are peripheral. This indicates
maximum theoretical parsimony.

## Reference

Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
to typologies in organization research. *Academy of Management Journal*,
54(2), 393–420. doi:10.5465/amj.2011.60263120

---

# Configuration Charts (New in v0.5.0)

ThSQCA can generate Fiss-style configuration charts (Table 5 format) commonly used in QCA publications.

## Automatic Inclusion in Reports

Configuration charts are now automatically included in reports generated by `generate_report()`:

```{r eval=FALSE}
# Generate report with configuration charts (default)
generate_report(result, "my_report.md", dat = dat, format = "full")

# Disable charts if needed
generate_report(result, "my_report.md", dat = dat, include_chart = FALSE)

# Use LaTeX symbols for academic papers
generate_report(result, "my_report.md", dat = dat, chart_symbol_set = "latex")

# Include QCA package raw output for verification (default is TRUE)
generate_report(result, "my_report.md", dat = dat, include_raw_output = TRUE)

# Disable raw output if not needed
generate_report(result, "my_report.md", dat = dat, include_raw_output = FALSE)
```

**Note (v1.2.0)**: Reports now display the **Solution Type** (Complex/Parsimonious/Intermediate) in the Analysis Overview section, and optionally include raw QCA package output for verification purposes.

## Standalone Chart Functions

You can also generate configuration charts directly:

```{r}
# From path strings
paths <- c("A*B*~C", "A*D", "B*E")
chart <- config_chart_from_paths(paths)
cat(chart)
```

### Symbol Sets

Three symbol sets are available:

```{r}
# ASCII (maximum compatibility)
cat(config_chart_from_paths(paths, symbol_set = "ascii"))
```

```{r eval=FALSE}
# LaTeX (for PDF/academic papers)
cat(config_chart_from_paths(paths, symbol_set = "latex"))
# Output: $\bullet$ for presence, $\otimes$ for absence
```

### Multiple Solutions

When you have multiple equivalent solutions:

```{r}
solutions <- list(
  c("A*B", "C*D"),
  c("A*B", "C*E")
)
chart <- config_chart_multi_solutions(solutions)
cat(chart)
```

---

# Conclusion

ThSQCA provides a structured and reproducible way to evaluate  
how threshold choices influence QCA results.

Using CTS, MCTS, OTS, and DTS sweeps, researchers can:

- assess robustness,
- identify stable causal patterns,
- detect threshold-sensitive relationships,
- and strengthen QCA validity.

**New in v1.3.2:**

- `compute_fiss_core()`: Augments sweep results with Fiss (2011) core/peripheral classification
- `generate_fiss_chart()`: Four-symbol configuration charts (●/⊗/⊙/⊘) for academic publications
- `print_fiss_summary()`: Human-readable core/peripheral breakdown per threshold
- `generate_report(..., include_fiss_core = TRUE)`: Reports with full Fiss notation

**New in v1.3.0:**

- `pre_calibrated` argument: mixed crisp/fuzzy workflow for all four sweep functions

**New in v1.1.0:**

- QCA-compatible defaults: `include = ""` and `dir.exp = NULL` (complex solution)
- Clear documentation of three solution types (complex, parsimonious, intermediate)
- Fixed critical bug in `dir.exp` handling

**New in v0.5.0:**

- Configuration charts automatically included in reports
- New parameters: `include_chart`, `chart_symbol_set`
- Standalone chart functions: `config_chart_from_paths()`, `config_chart_multi_solutions()`

**New in v0.3.0:**

- QCA-compatible argument names (`outcome`, `conditions`)
- Negated outcome support (`~Y` notation)
- Backward compatibility with deprecation warnings

**New in v0.2.0:**

- Detect and analyze multiple equivalent solutions
- Extract essential prime implicants for robust findings
- Generate comprehensive reports automatically
- Access stored parameters for reproducibility

---

# Verification for Academic Publications

**For academic publications**, we strongly recommend verifying ThSQCA results directly with the QCA package before submission. This ensures accuracy and reproducibility.

## Recommended Verification Workflow

```{r eval=FALSE}
library(QCA)
library(ThSQCA)

# Step 1: Run ThSQCA analysis
data(sample_data)
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

result <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = thrX,
  include = "?",
  dir.exp = c(1, 1, 1),
  return_details = TRUE
)

# Step 2: Generate report with QCA output for verification
generate_report(result, "my_analysis.md", dat = sample_data, 
                include_raw_output = TRUE)  # Default is TRUE

# Step 3: Verify key results directly with QCA package
# For each important threshold, run QCA::minimize() directly
dat_bin <- sample_data
dat_bin$Y_bin <- ifelse(sample_data$Y >= 7, 1, 0)
dat_bin$X1_bin <- ifelse(sample_data$X1 >= 7, 1, 0)
dat_bin$X2_bin <- ifelse(sample_data$X2 >= 7, 1, 0)
dat_bin$X3_bin <- ifelse(sample_data$X3 >= 7, 1, 0)

tt <- truthTable(dat_bin, outcome = "Y_bin", 
                 conditions = c("X1_bin", "X2_bin", "X3_bin"),
                 incl.cut = 0.8)
sol <- minimize(tt, include = "?", dir.exp = c(1, 1, 1))
print(sol)  # Compare with ThSQCA report
```

## Why Verification Matters

1. **Accuracy**: Confirms ThSQCA extraction matches QCA package output
2. **Reproducibility**: Provides independent verification for reviewers
3. **Transparency**: Documents the analysis pipeline clearly

## What to Check

- Solution expressions match exactly
- Consistency and coverage values match
- Number of solutions (M1, M2, ...) is correct

> **Tip**: The "QCA Package Output" section in ThSQCA reports (enabled by `include_raw_output = TRUE`) shows the direct `print(sol)` output for easy comparison.


## References

For more information on TS-QCA methodology, see:

- Ragin, C. C. (2008). *Redesigning Social Inquiry: Fuzzy Sets and Beyond*. University of Chicago Press. DOI: [10.7208/chicago/9780226702797.001.0001](https://doi.org/10.7208/chicago/9780226702797.001.0001)
- Duşa, A. (2019). *QCA with R: A Comprehensive Resource*. Springer. DOI: [10.1007/978-3-319-75668-4](https://doi.org/10.1007/978-3-319-75668-4)
- Oana, I.-E., & Schneider, C. Q. (2024). A Robustness Test Protocol for Applied QCA: Theory and R Software Application. *Sociological Methods & Research*, 53(1), 57–88. DOI: [10.1177/00491241211036158](https://doi.org/10.1177/00491241211036158)



# Session Info
```{r}
sessionInfo()
```

---
title: "ThSQCA Reproducible Code (English)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ThSQCA Reproducible Code (English)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This document provides **reproducible code** for ThSQCA analyses.  
It is designed to be inserted directly into a research article's appendix  
or provided as supplementary analysis documentation.
All code is fully executable and ordered for reproducibility.

> **Note (v1.3.2)**: New Section 12 provides complete reproducible code for
> Fiss (2011) core/peripheral classification using `compute_fiss_core()`,
> `generate_fiss_chart()`, and `print_fiss_summary()`.

> **Note (v1.2.0)**: If you previously used intermediate solutions (with `dir.exp`), please re-run your analyses. Version 1.2.0 fixed a bug where intermediate solutions were incorrectly extracted. Reports now also display the Solution Type and include optional QCA package output for verification.

---

# 1. Load Packages
```{r}
library(ThSQCA)
library(QCA)
```

---

# 2. Load Data
```{r}
# Adjust the file name as needed
library(ThSQCA)
data("sample_data")
dat <- sample_data

# Outcome and conditions
outcome  <- "Y"
conditions <- c("X1", "X2", "X3")

# Quick inspection
str(dat)
summary(dat)
```

---

# 3. Baseline Thresholds
```{r}
thrY_base <- 7
thrX_base <- 7

# Fixed X thresholds (for OTS–QCA)
thrX_vec <- c(
  X1 = thrX_base,
  X2 = thrX_base,
  X3 = thrX_base
)
thrX_vec
```

---

# 3.5. Three Types of QCA Solutions (New in v1.1.0)

As of v1.1.0, ThSQCA defaults match the QCA package:

| Solution Type | `include` | `dir.exp` | When to Use |
|--------------|-----------|-----------|-------------|
| **Complex** (default) | `""` | `NULL` | Conservative analysis |
| **Parsimonious** | `"?"` | `NULL` | Maximum simplification |
| **Intermediate** | `"?"` | `c(1,1,...)` | Theory-driven (most common) |

```{r}
# Example: Three solution types at single threshold
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

# Complex (default)
res_comp <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX)
cat("Complex:", res_comp$summary$expression, "\n")

# Parsimonious
res_pars <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX,
                    include = "?")
cat("Parsimonious:", res_pars$summary$expression, "\n")

# Intermediate
res_int <- otSweep(dat, "Y", c("X1", "X2", "X3"), sweep_range = 7, thrX = thrX,
                   include = "?", dir.exp = c(1, 1, 1))
cat("Intermediate:", res_int$summary$expression, "\n")
```

---

# 4. CTS–QCA (ctSweepS)

Sweep a **single condition X** (example: X3).

## Default: Complex Solution
```{r, error=TRUE}
sweep_var   <- "X3"   # Condition (X) whose threshold is swept
sweep_range <- 6:9    # Candidate threshold values to evaluate
thrY         <- 7     # Outcome (Y) threshold (fixed)
thrX_default <- 7     # Threshold for other X conditions (fixed)

# Default: Complex solution (include = "", dir.exp = NULL)
res_cts <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = "X3",
  sweep_range    = 6:9,
  thrY           = 7,
  thrX_default   = 7,
  return_details = TRUE
)

summary(res_cts)
```

## Intermediate Solution (Theory-Driven)
```{r, error=TRUE}
# Intermediate solution: specify include = "?" and dir.exp
res_cts_int <- ctSweepS(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_var      = "X3",
  sweep_range    = 6:9,
  thrY           = 7,
  thrX_default   = 7,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_cts_int)
```

Export:
```{r, eval=FALSE}
write.csv(res_cts$summary, file = "TSQCA_CTS_results.csv", row.names = FALSE)
```

---

# 5. MCTS–QCA (ctSweepM)

Sweep **multiple X thresholds simultaneously**.

## Default: Complex Solution
```{r, error=TRUE}
# Create a sweep list specifying thresholds for each condition
sweep_list <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

# Default: Complex solution
res_mcts <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  return_details = TRUE
)

summary(res_mcts)
```

## Intermediate Solution (Theory-Driven)
```{r, error=TRUE}
# Intermediate solution: specify include = "?" and dir.exp
res_mcts_int <- ctSweepM(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list     = sweep_list,
  thrY           = 7,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_mcts_int)
```

Export:
```{r, eval=FALSE}
write.csv(res_mcts$summary, file = "TSQCA_MCTS_results.csv", row.names = FALSE)
```

---

# 6. OTS–QCA (otSweep)

Sweep **only the outcome threshold (Y)**.

## Default: Complex Solution
```{r}
sweep_range_ots <- 6:8

# Default: Complex solution
res_ots <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = sweep_range_ots,
  thrX           = thrX_vec,
  return_details = TRUE
)

summary(res_ots)
```

## Intermediate Solution (Theory-Driven)
```{r}
# Intermediate solution: specify include = "?" and dir.exp
res_ots_int <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = sweep_range_ots,
  thrX           = thrX_vec,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_ots_int)
```

Export:
```{r, eval=FALSE}
write.csv(res_ots$summary, file = "TSQCA_OTS_results.csv", row.names = FALSE)
```

---

# 7. DTS–QCA (dtSweep)

Two-dimensional sweep: **X thresholds × Y thresholds**.

## Default: Complex Solution
```{r}
sweep_list_dts_X <- list(
  X1 = 6:7,
  X2 = 6:7,
  X3 = 6:7
)

sweep_range_dts_Y <- 6:7

# Default: Complex solution
res_dts <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_dts)
```

## Intermediate Solution (Theory-Driven)
```{r}
# Intermediate solution: specify include = "?" and dir.exp
res_dts_int <- dtSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_list_X   = sweep_list_dts_X,
  sweep_range_Y  = sweep_range_dts_Y,
  include        = "?",
  dir.exp        = c(1, 1, 1),
  return_details = TRUE
)

summary(res_dts_int)
```

Export:
```{r, eval=FALSE}
write.csv(res_dts$summary, file = "TSQCA_DTS_results.csv", row.names = FALSE)
```

---

# 8. Using extract_mode (New in v0.2.0)

Extract all solutions or essential prime implicants for robustness analysis.

## Extract All Solutions
```{r, eval=FALSE}
res_all <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = thrX_vec,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1),    # Intermediate solution
  extract_mode   = "all",
  return_details = TRUE
)

# View results with n_solutions column
head(res_all$summary)
```

## Extract Essential Prime Implicants
```{r, eval=FALSE}
res_essential <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = thrX_vec,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1),    # Intermediate solution
  extract_mode   = "essential",
  return_details = TRUE
)

# View results with essential prime implicants, selective terms, and unique terms
head(res_essential$summary)
```

---

# 9. Generating Reports (New in v0.2.0)

Create comprehensive markdown reports for documentation.

## Full Report
```{r, eval=FALSE}
generate_report(res_ots, "TSQCA_OTS_report_full.md", dat = dat, format = "full")
```

## Simple Report (for manuscripts)
```{r, eval=FALSE}
generate_report(res_ots, "TSQCA_OTS_report_simple.md", dat = dat, format = "simple")
```

---

# 10. Negated Outcome Analysis (New in v0.3.0)

Analyze conditions sufficient for the **absence** of the outcome.

## Standard vs Negated Outcome
```{r, eval=FALSE}
# Standard: conditions for Y >= threshold (intermediate solution)
res_Y <- otSweep(
  dat            = dat,
  outcome        = "Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = thrX_vec,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1)     # Intermediate solution
)

# Negated: conditions for Y < threshold (intermediate solution)
res_negY <- otSweep(
  dat            = dat,
  outcome        = "~Y",
  conditions     = c("X1", "X2", "X3"),
  sweep_range    = 6:8,
  thrX           = thrX_vec,
  include        = "?",           # Include logical remainders
  dir.exp        = c(1, 1, 1)     # Intermediate solution
)

# Compare results
res_Y$summary
res_negY$summary

# Check negation flag
res_negY$params$negate_outcome
# [1] TRUE
```

---

# 11. Accessing Analysis Parameters

All parameters are stored for reproducibility.
```{r, eval=FALSE}
# View stored parameters
res_ots$params

# Example output:
# $outcome
# [1] "Y"
# $conditions
# [1] "X1" "X2" "X3"
# $thrX
# X1 X2 X3 
#  7  7  7 
# $incl.cut
# [1] 0.8
# $n.cut
# [1] 1
# $pri.cut
# [1] 0
```

---

## Configuration Charts (New in v0.5.0)

Configuration charts are now automatically included in reports. You can also generate them separately:

```{r}
# From path strings
paths <- c("A*B*~C", "A*D")
chart <- config_chart_from_paths(paths)
cat(chart)
```

For reports:
```{r, eval=FALSE}
# Charts are included by default
generate_report(result, "report.md", dat = dat, format = "full")

# Use LaTeX symbols for academic papers
generate_report(result, "report.md", dat = dat, chart_symbol_set = "latex")
```

---

## References

For more information on TS-QCA methodology, see:

- Ragin, C. C. (2008). *Redesigning Social Inquiry: Fuzzy Sets and Beyond*. University of Chicago Press. DOI: [10.7208/chicago/9780226702797.001.0001](https://doi.org/10.7208/chicago/9780226702797.001.0001)
- Duşa, A. (2019). *QCA with R: A Comprehensive Resource*. Springer. DOI: [10.1007/978-3-319-75668-4](https://doi.org/10.1007/978-3-319-75668-4)
- Oana, I.-E., & Schneider, C. Q. (2024). A Robustness Test Protocol for Applied QCA: Theory and R Software Application. *Sociological Methods & Research*, 53(1), 57–88. DOI: [10.1177/00491241211036158](https://doi.org/10.1177/00491241211036158)



---

# 12. Fiss (2011) Core/Peripheral Classification (New in v1.3.2)

This section provides fully reproducible code for Fiss-style four-symbol
configuration charts. Run sections 1–6 first to ensure `res_ots_int` is available.

## 12.1 Compute Core/Peripheral Classification

```{r, eval=FALSE}
# Requires: include = "?", dir.exp specified, return_details = TRUE
# res_ots_int was created in Section 6 above.

res_fiss <- compute_fiss_core(
  res_ots_int,
  conditions = c("X1", "X2", "X3")
)

# Inspect available thresholds
names(res_fiss$fiss_core)
```

## 12.2 Print Per-Threshold Summary

```{r, eval=FALSE}
# English
print_fiss_summary(res_fiss, thr_key = "7")

# Japanese
print_fiss_summary(res_fiss, thr_key = "7", language = "ja")
```

Example output:

```
=== Fiss Core/Peripheral Classification (thrY = 7) ===
Parsimonious : X1*X2
Intermediate : X3 + X1*X2

[Term M1] X3
  Periph. present : X3

[Term M2] X1*X2
  Core present    : X1, X2
```

## 12.3 Generate Four-Symbol Chart

```{r, eval=FALSE}
# Unicode (Markdown / screen)
cat(generate_fiss_chart(res_fiss, symbol_set = "unicode"))

# LaTeX (for PDF/journal submission)
cat(generate_fiss_chart(res_fiss, symbol_set = "latex"))

# ASCII (maximum compatibility)
cat(generate_fiss_chart(res_fiss, symbol_set = "ascii"))
```

Symbol legend:

| Symbol (unicode) | LaTeX | ASCII | Meaning |
|-----------------|-------|-------|---------|
| ● | `$\bullet$` | O | Core condition present |
| ⊗ | `$\otimes$` | X | Core condition absent |
| ⊙ | `$\odot$` | o | Peripheral condition present |
| ⊘ | `$\oslash$` | x | Peripheral condition absent |
| (blank) | (blank) | (blank) | Don't care |

## 12.4 Include in Report

```{r, eval=FALSE}
# Full report with Fiss four-symbol charts
generate_report(
  res_fiss,
  output_file       = "TSQCA_Fiss_report.md",
  format            = "full",
  dat               = dat,
  include_fiss_core = TRUE,       # activates four-symbol charts
  chart_symbol_set  = "unicode",
  include_raw_output = TRUE
)
```

## 12.5 Accessing the Classification Data

```{r, eval=FALSE}
# Access raw classification data frame for a given threshold
classif_7 <- res_fiss$fiss_core[["7"]]$classification
classif_7

# Filter core conditions only
classif_7[classif_7$type == "core" & classif_7$status != "dontcare", ]

# Filter peripheral conditions only
classif_7[classif_7$type == "peripheral" & classif_7$status != "dontcare", ]

# Parsimonious and intermediate expressions
res_fiss$fiss_core[["7"]]$parsim_expression
res_fiss$fiss_core[["7"]]$interm_expression
```

## 12.6 Note on Computational Cost

`compute_fiss_core()` runs `QCA::minimize()` once per threshold (to obtain
the parsimonious solution). The total computation time is therefore
approximately double that of the original sweep. For large sweeps
(many thresholds or conditions), this is worth keeping in mind.

---

# 13. Session Information
```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{split_solution_terms}
\alias{split_solution_terms}
\title{Split solution expression into terms}
\usage{
split_solution_terms(solution)
}
\arguments{
\item{solution}{Character. Solution expression (e.g., "X1*X2 + X3").}
}
\value{
Character vector of terms.
}
\description{
Split solution expression into terms
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{get_config_labels}
\alias{get_config_labels}
\title{Get labels for configuration chart based on language}
\usage{
get_config_labels(language)
}
\description{
Get labels for configuration chart based on language
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{sanitize_truthtable}
\alias{sanitize_truthtable}
\title{Sanitize a truth table before Boolean minimization}
\usage{
sanitize_truthtable(tt)
}
\arguments{
\item{tt}{A truth table object returned by \code{QCA::truthTable()}.}
}
\value{
The same object with numeric \code{incl}/\code{PRI} columns and
  no \code{NA} in those columns. Returns the input unchanged if it is
  \code{NULL} or lacks a \code{$tt} data frame.
}
\description{
Guards against two behaviors of \code{QCA::truthTable()} (observed with
QCA 3.25 / admisc 0.40) that can affect programmatic, repeated calls such
as threshold sweeps. For sparse truth tables the \code{incl}/\code{PRI}
columns may be returned as \code{character} rather than \code{numeric},
and logical-remainder rows (observed \code{n = 0}) may carry the string
\code{"-"}. Passing such a truth table to \code{QCA::minimize()} can, for
some truth table structures, cause the minimization to hang or return
misleading fit values. This helper coerces \code{incl}/\code{PRI} to
numeric and sets the remainder rows to 0.
}
\details{
The \code{OUT} column is deliberately left untouched: \code{minimize()}
uses \code{OUT} (with the symbolic \code{"?"} for remainders) to identify
positive, negative, and remainder rows, so setting \code{incl}/\code{PRI}
to 0 on remainder rows does not alter remainder handling.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{config_chart_multi_solutions}
\alias{config_chart_multi_solutions}
\title{Generate configuration chart for multiple solutions (simple interface)}
\usage{
config_chart_multi_solutions(
  solutions,
  symbol_set = c("unicode", "ascii", "latex"),
  language = c("en", "ja"),
  condition_order = NULL,
  show_epi = FALSE
)
}
\arguments{
\item{solutions}{List of character vectors. Each element is a vector of 
paths for one solution.}

\item{symbol_set}{Character. One of \code{"unicode"}, \code{"ascii"}, 
or \code{"latex"}.}

\item{language}{Character. \code{"en"} for English, \code{"ja"} for Japanese.}

\item{condition_order}{Character vector. Optional ordering of conditions.}

\item{show_epi}{Logical. Whether to identify and display Essential Prime
Implicants (EPIs) in the note. Default is FALSE.}
}
\value{
Character string containing Markdown-formatted tables.
}
\description{
Generates separate configuration charts for multiple solutions.
}
\examples{
# Multiple solutions
solutions <- list(
  c("A*B", "C"),
  c("A*B", "D"),
  c("A*C")
)
chart <- config_chart_multi_solutions(solutions)
cat(chart)

# With EPI identification
chart <- config_chart_multi_solutions(solutions, show_epi = TRUE)
cat(chart)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_report.R
\name{generate_report}
\alias{generate_report}
\title{Generate Markdown Report for QCA Analysis}
\usage{
generate_report(
  result,
  output_file = "qca_report.md",
  format = c("full", "simple"),
  title = "QCA Analysis Report",
  dat = NULL,
  desc_vars = NULL,
  include_chart = TRUE,
  chart_symbol_set = c("unicode", "ascii", "latex"),
  chart_level = c("term", "summary"),
  solution_note = TRUE,
  solution_note_style = c("simple", "detailed"),
  solution_note_lang = c("en", "ja"),
  include_fiss_core = FALSE,
  include_raw_output = TRUE
)
}
\arguments{
\item{result}{A result object from any Sweep function with
\code{return_details = TRUE}.}

\item{output_file}{Character. Path to output markdown file.}

\item{format}{Character. Report format: \code{"full"} or \code{"simple"}.}

\item{title}{Character. Report title.}

\item{dat}{Optional data frame. Original data for descriptive statistics.}

\item{desc_vars}{Optional character vector. Variables for descriptive statistics.
If NULL and dat is provided, uses Yvar and Xvars from params.}

\item{include_chart}{Logical. If TRUE (default), includes configuration charts
(Fiss-style tables) in the report for each threshold.}

\item{chart_symbol_set}{Character. Symbol set for configuration charts:
\code{"unicode"} (default), \code{"ascii"}, or \code{"latex"}.}

\item{chart_level}{Character. Chart aggregation level:
\code{"term"} (default) produces solution-term level charts following Fiss (2011)
notation, where each column represents one prime implicant (sufficient
configuration). This format is recommended for academic publications.
\code{"summary"} produces threshold-level summaries where each
column represents one threshold, aggregating all configurations.}

\item{solution_note}{Logical. If TRUE (default), adds a note when multiple
equivalent solutions exist explaining that M1 is shown.}

\item{solution_note_style}{Character. Style of solution note:
\code{"simple"} (default) or \code{"detailed"} (includes EPIs).}

\item{solution_note_lang}{Character. Language for solution notes:
\code{"en"} (default) or \code{"ja"}.}

\item{include_fiss_core}{Logical. If TRUE and \code{result$fiss_core} exists
(i.e., \code{\link{compute_fiss_core}} has been run), the configuration
charts use full Fiss (2011) four-symbol notation distinguishing core
(present in both parsimonious and intermediate solutions) from peripheral
conditions (intermediate only). If FALSE (default) or if \code{fiss_core}
data is absent, the standard two-symbol chart is used.}

\item{include_raw_output}{Logical. If TRUE (default), includes the raw QCA
package output (print(sol)) for each threshold for verification purposes.}
}
\value{
Invisibly returns the path to the generated report.
}
\description{
Creates a markdown report from QCA analysis results.
Supports two formats: "full" (comprehensive) and "simple" (for manuscripts).
}
\examples{
\dontrun{
data(sample_data)
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

result <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = thrX,
  return_details = TRUE
)

# With descriptive statistics and configuration charts
generate_report(result, "my_report.md", format = "full", 
                dat = sample_data, include_chart = TRUE)

# Without configuration charts
generate_report(result, "my_report.md", format = "simple",
                include_chart = FALSE)

# With Fiss-style term-level charts (default, recommended for publications)
generate_report(result, "my_report.md", format = "full")

# With threshold-level summary charts
generate_report(result, "my_report.md", format = "full",
                chart_level = "summary")

# With Fiss core/peripheral chart (requires compute_fiss_core)
res_fiss <- compute_fiss_core(result, conditions = c("X1", "X2", "X3"))
generate_report(res_fiss, "my_report.md", format = "full",
                include_fiss_core = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{generate_cross_threshold_chart}
\alias{generate_cross_threshold_chart}
\title{Generate cross-threshold configuration chart from sweep results}
\usage{
generate_cross_threshold_chart(
  result,
  conditions = NULL,
  symbol_set = c("unicode", "ascii", "latex"),
  chart_level = c("term", "summary"),
  language = c("en", "ja")
)
}
\arguments{
\item{result}{A result object from any Sweep function (otSweep, ctSweepS,
ctSweepM, or dtSweep).}

\item{conditions}{Character vector. Condition names for row ordering.
If NULL, automatically extracted from expressions.}

\item{symbol_set}{Character. One of \code{"unicode"}, \code{"ascii"}, 
or \code{"latex"}. Default is \code{"unicode"}.}

\item{chart_level}{Character. Chart aggregation level:
\code{"term"} (default) produces solution-term level charts following Fiss (2011)
notation, where each column represents one prime implicant.
\code{"summary"} produces threshold-level summaries where
each column represents one threshold, aggregating all configurations.}

\item{language}{Character. \code{"en"} for English, \code{"ja"} for Japanese.}
}
\value{
Character string containing Markdown-formatted table.
}
\description{
Creates a configuration chart from threshold sweep results. Supports
two levels of aggregation: solution-term level (Fiss-style, default) and
threshold-level summary.
}
\examples{
\dontrun{
data(sample_data)
result <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)

# Solution-term level, Fiss-style (default)
chart <- generate_cross_threshold_chart(result, c("X1", "X2", "X3"))
cat(chart)

# Threshold-level summary
chart <- generate_cross_threshold_chart(result, c("X1", "X2", "X3"),
                                         chart_level = "summary")
cat(chart)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{classify_term_conditions}
\alias{classify_term_conditions}
\title{Classify each (condition, status) pair in an intermediate term as
core or peripheral}
\usage{
classify_term_conditions(interm_term, parsim_map, conditions)
}
\arguments{
\item{interm_term}{Character. Single intermediate-solution term.}

\item{parsim_map}{Named list returned by \code{extract_cond_status_map()}
for the parsimonious solution.}

\item{conditions}{Character vector of all condition names.}
}
\value{
Data frame with columns:
  \code{condition}, \code{status} ("present"/"absent"/"dontcare"),
  \code{type} ("core"/"peripheral"/"dontcare").
}
\description{
A condition is **core** when it appears with the same presence/absence
status in at least one term of the parsimonious solution.
A condition is **peripheral** when it appears in the intermediate term
but NOT (with the same status) in any parsimonious term.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{add_metrics_rows}
\alias{add_metrics_rows}
\title{Add metrics rows to markdown table}
\usage{
add_metrics_rows(table_str, metrics, labels)
}
\description{
Add metrics rows to markdown table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_report.R
\name{write_simple_report}
\alias{write_simple_report}
\title{Write simple report content}
\usage{
write_simple_report(
  result,
  con,
  include_chart = TRUE,
  chart_symbol_set = "unicode",
  chart_level = "term",
  solution_note = TRUE,
  solution_note_style = "simple",
  solution_note_lang = "en",
  include_raw_output = TRUE,
  use_fiss = FALSE
)
}
\description{
Write simple report content
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{generate_solution_note}
\alias{generate_solution_note}
\title{Generate solution note for multiple solutions}
\usage{
generate_solution_note(
  n_sol,
  epi_list = NULL,
  style = c("simple", "detailed"),
  language = c("en", "ja"),
  format = c("markdown", "latex")
)
}
\arguments{
\item{n_sol}{Integer. Number of solutions.}

\item{epi_list}{Character vector. Essential prime implicants (NULL to omit).}

\item{style}{Character. \code{"simple"} or \code{"detailed"}.}

\item{language}{Character. \code{"en"} or \code{"ja"}.}

\item{format}{Character. \code{"markdown"} or \code{"latex"}.}
}
\value{
Character string of the note, or empty string if n_sol <= 1.
}
\description{
Creates a note explaining that multiple equivalent solutions exist
and that the displayed configuration is based on M1.
}
\examples{
# Simple note
generate_solution_note(2, style = "simple")

# Detailed note with EPIs
generate_solution_note(3, epi_list = c("A*B", "C"), style = "detailed")

# Japanese
generate_solution_note(2, style = "simple", language = "ja")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ThSQCA-package.R
\docType{package}
\name{ThSQCA-package}
\alias{ThSQCA}
\alias{ThSQCA-package}
\title{ThSQCA: Threshold-Sweep QCA}
\description{
Provides threshold sweep methods for Qualitative Comparative Analysis (QCA). Implements Condition Threshold Sweep-Single (CTS-S), Condition Threshold Sweep-Multiple (CTS-M), Outcome Threshold Sweep (OTS), and Dual Threshold Sweep (DTS) for systematic exploration of threshold calibration effects on crisp-set QCA results. These methods extend traditional robustness approaches by treating threshold variation as an exploratory tool for discovering causal structures. Also provides Fiss (2011) \doi{10.5465/amj.2011.60263120} core/peripheral condition classification via compute_fiss_core() and generate_fiss_chart(), enabling four-symbol configuration charts that distinguish core conditions (present in both parsimonious and intermediate solutions) from peripheral conditions (intermediate only). Built on top of the 'QCA' package by Dusa (2019) \doi{10.1007/978-3-319-75668-4}, with function arguments following 'QCA' conventions. Based on set-theoretic methods by Ragin (2008) \doi{10.7208/chicago/9780226702797.001.0001} and established robustness protocols by Oana and Schneider (2024) \doi{10.1177/00491241211036158}. This package supersedes 'TSQCA'; see the NEWS file for migration guidance.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/im-research-yt/ThSQCA}
  \item \doi{10.5281/zenodo.17899390}
  \item Report bugs at \url{https://github.com/im-research-yt/ThSQCA/issues}
}

}
\author{
\strong{Maintainer}: Yuki Toyoda \email{yuki.toyoda.ds@hosei.ac.jp}

Authors:
\itemize{
  \item Yuki Toyoda \email{yuki.toyoda.ds@hosei.ac.jp}
}

Other contributors:
\itemize{
  \item Japan Society for the Promotion of Science (KAKENHI Grant Number JP20K01998) [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{extract_sol_terms}
\alias{extract_sol_terms}
\title{Extract solution terms from a QCA solution object (intermediate or parsim)}
\usage{
extract_sol_terms(sol)
}
\arguments{
\item{sol}{QCA solution object}
}
\value{
Character vector of terms (prime implicants), or character(0)
}
\description{
Extract solution terms from a QCA solution object (intermediate or parsim)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{format_qca_term}
\alias{format_qca_term}
\title{Format a single QCA term}
\usage{
format_qca_term(term, var_names, use_tilde = TRUE)
}
\arguments{
\item{term}{Character. A single term (e.g., "KSPRVT" or "~KPR*PRD").}

\item{var_names}{Character vector. Variable names used in the analysis.}

\item{use_tilde}{Logical. If TRUE, negation is represented as \code{~VAR}.
If FALSE, negation is represented as lowercase (e.g., \code{var}).}
}
\value{
Character. The formatted term with \code{*} between all variables.
}
\description{
Inserts \code{*} between variables in a term where it may have been omitted.
}
\examples{
var_names <- c("KSP", "KPR", "PRD", "RVT", "RCM")
format_qca_term("KSPRVTRCM", var_names)
# Returns: "KSP*RVT*RCM"

format_qca_term("~KPRPRD", var_names)
# Returns: "~KPR*PRD"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{extract_terms}
\alias{extract_terms}
\title{Extract and format terms from solutions}
\usage{
extract_terms(solutions, var_names, use_tilde = TRUE)
}
\arguments{
\item{solutions}{Character vector. Solution expressions.}

\item{var_names}{Character vector. Variable names used in the analysis.}

\item{use_tilde}{Logical. If TRUE, negation is represented as \code{~VAR}.}
}
\value{
List with:
  \itemize{
    \item \code{all_terms} — all terms (with duplicates)
    \item \code{unique_terms} — unique terms
    \item \code{n_total} — total term count
    \item \code{n_unique} — unique term count
  }
}
\description{
Extracts individual terms from solution expressions and returns
formatted unique terms.
}
\examples{
var_names <- c("X1", "X2", "X3")
solutions <- c("X1*X2 + X3", "X1*X2 + X1*X3")
extract_terms(solutions, var_names)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{format_qca_solutions}
\alias{format_qca_solutions}
\title{Format multiple QCA solutions}
\usage{
format_qca_solutions(solutions, var_names, use_tilde = TRUE)
}
\arguments{
\item{solutions}{Character vector. Solution expressions from \code{minimize()}.}

\item{var_names}{Character vector. Variable names used in the analysis.}

\item{use_tilde}{Logical. If TRUE, negation is represented as \code{~VAR}.}
}
\value{
Character vector. Formatted solution expressions.
}
\description{
Formats a vector of solution expressions.
}
\examples{
var_names <- c("KSP", "KPR", "PRD", "RVT", "RCM")
solutions <- c("KSPRVT + RCM", "~KPRPRD")
format_qca_solutions(solutions, var_names)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{df_to_md_table}
\alias{df_to_md_table}
\title{Convert data frame to Markdown table}
\usage{
df_to_md_table(df, digits = 3)
}
\arguments{
\item{df}{Data frame to convert.}

\item{digits}{Number of decimal places for numeric columns.}
}
\value{
Character string of Markdown table.
}
\description{
Convert data frame to Markdown table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample_data.R
\docType{data}
\name{sample_data}
\alias{sample_data}
\title{Sample dataset for ThSQCA examples}
\format{
A data frame with 80 rows and 4 variables.
}
\usage{
sample_data
}
\description{
A small artificial dataset with variables:
\describe{
  \item{Y}{Outcome (numeric)}
  \item{X1}{Condition 1}
  \item{X2}{Condition 2}
  \item{X3}{Condition 3}
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{get_condition_status}
\alias{get_condition_status}
\title{Determine condition status in a term}
\usage{
get_condition_status(term, condition)
}
\arguments{
\item{term}{Character. Single term (e.g., "X1*X2", "~X3").}

\item{condition}{Character. Condition name (e.g., "X1").}
}
\value{
Character. One of "present", "absent", or "dontcare".
}
\description{
Checks whether a condition is present, absent (negated), or don't care
in a given term.
}
\details{
Uses word boundary matching to avoid false positives when
  condition names are substrings of each other (e.g., X1 vs X10).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{build_single_chart}
\alias{build_single_chart}
\title{Build configuration chart for a single solution}
\usage{
build_single_chart(
  paths,
  sol,
  symbols,
  labels,
  include_metrics,
  condition_order = NULL,
  solution_index = 1
)
}
\description{
Build configuration chart for a single solution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_cts.R
\name{ctSweepM}
\alias{ctSweepM}
\title{MCTS–QCA: Multi-condition threshold sweep}
\usage{
ctSweepM(
  dat,
  outcome = NULL,
  conditions = NULL,
  sweep_list,
  thrY,
  thrX_default = NULL,
  pre_calibrated = NULL,
  dir.exp = NULL,
  include = "",
  incl.cut = 0.8,
  n.cut = 1,
  pri.cut = 0,
  extract_mode = c("first", "all", "essential"),
  return_details = TRUE,
  Yvar = NULL,
  Xvars = NULL
)
}
\arguments{
\item{dat}{Data frame containing the outcome and condition variables.}

\item{outcome}{Character. Outcome variable name. Supports negation with
tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.}

\item{conditions}{Character vector. Names of condition variables.}

\item{sweep_list}{Named list. Each element is a numeric vector of
candidate thresholds for the corresponding X. Names must match
\code{conditions}. Variables listed in \code{pre_calibrated} do not
need a \code{sweep_list} entry. Every condition must receive a threshold
from one of \code{sweep_list}, \code{pre_calibrated}, or
\code{thrX_default}; a condition covered by none of these raises an
error. To hold a condition at a fixed value while sweeping others, give
it a length-one entry (e.g. \code{list(SPV = 6:8, PRD = 7)}).}

\item{thrY}{Numeric. Threshold for Y (fixed).}

\item{thrX_default}{Numeric or \code{NULL}. Threshold used to binarize any
condition that appears in neither \code{sweep_list} nor
\code{pre_calibrated}, mirroring the argument of the same name in
\code{\link{ctSweepS}}. Default is \code{NULL}, in which case such a
condition is an error rather than being silently dropped.}

\item{pre_calibrated}{Character vector or \code{NULL}. Names of condition
variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
and should be passed through to \code{QCA::truthTable()} without
binarization. These variables must contain values in the \code{[0, 1]}
range. Variables not listed here will be binarized using \code{sweep_list}
thresholds as usual. Default is \code{NULL} (all variables binarized).
It is recommended to sweep variables on their original (raw) scale rather
than as pre-calibrated fuzzy values, because raw-scale thresholds are
easier to interpret substantively.}

\item{dir.exp}{Directional expectations for \code{minimize}.
If \code{NULL} (default), no directional expectations are applied.
To compute the \strong{intermediate solution}, specify a numeric vector
(1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
for three conditions all expected to contribute positively.}

\item{include}{Inclusion rule for \code{minimize}. 
\code{""} (default, QCA compatible) computes the \strong{complex solution}
without logical remainders.
Use \code{"?"} to include logical remainders for \strong{parsimonious}
(with \code{dir.exp = NULL}) or \strong{intermediate} solutions
(with \code{dir.exp} specified).}

\item{incl.cut}{Consistency cutoff for \code{truthTable}.}

\item{n.cut}{Frequency cutoff for \code{truthTable}.}

\item{pri.cut}{PRI cutoff for \code{minimize}.}

\item{extract_mode}{Character. How to handle multiple solutions:
\code{"first"} (default), \code{"all"}, or \code{"essential"}.
See \code{\link{qca_extract}} for details.}

\item{return_details}{Logical. If \code{TRUE} (default), returns both
summary and detailed objects for use with \code{generate_report()}.}

\item{Yvar}{Deprecated. Use \code{outcome} instead.}

\item{Xvars}{Deprecated. Use \code{conditions} instead.}
}
\value{
If \code{return_details = FALSE}, a data frame with columns:
\itemize{
  \item \code{combo_id} — index of the threshold combination
  \item \code{threshold} — character string summarizing thresholds,
    e.g. \code{"X1=6, X2=7, X3=7"}
  \item \code{expression} — minimized solution expression
  \item \code{inclS} — solution consistency
  \item \code{covS} — solution coverage
  \item (additional columns depending on \code{extract_mode})
}

If \code{return_details = TRUE}, a list with:
\itemize{
  \item \code{summary} — the data frame above
  \item \code{details} — per-combination list of
    \code{combo_id}, \code{thrX_vec}, \code{truth_table}, \code{solution}
}

Note that \code{return_details} changes the \emph{type} of the returned
object, not just its contents: with \code{TRUE} the summary table is at
\code{result$summary}, whereas with \code{FALSE} the summary table
\emph{is} the returned object and \code{result$summary} is \code{NULL}.
Code intended to work under both settings should branch on
\code{inherits(result, "data.frame")} (or simply always pass
\code{return_details = TRUE}) rather than assuming \code{result$summary}
exists.
}
\description{
Performs a grid search over thresholds of multiple X variables.
For each combination of thresholds in \code{sweep_list}, the outcome Y
and all X variables are binarized, and a crisp-set QCA is executed.
}
\examples{
# Load sample data
data(sample_data)

# === Three Types of QCA Solutions ===

# Quick demonstration with 2 conditions
sweep_list <- list(X1 = 7, X2 = 7)

# 1. Complex Solution (default, QCA compatible)
result_comp <- ctSweepM(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2"),
  sweep_list = sweep_list,
  thrY = 7
  # include = "" (default), dir.exp = NULL (default)
)
head(result_comp$summary)

# 2. Parsimonious Solution (include = "?")
result_pars <- ctSweepM(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2"),
  sweep_list = sweep_list,
  thrY = 7,
  include = "?"  # Include logical remainders
)
head(result_pars$summary)

# 3. Intermediate Solution (include = "?" + dir.exp)
result_int <- ctSweepM(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2"),
  sweep_list = sweep_list,
  thrY = 7,
  include = "?",
  dir.exp = c(1, 1)  # Positive expectations
)
head(result_int$summary)

# === Threshold Sweep Example ===

# Using 2 conditions and 2 threshold levels
sweep_list <- list(
  X1 = 6:7,
  X2 = 6:7
)

# Run multiple condition threshold sweep (complex solutions by default)
result_quick <- ctSweepM(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2"),
  sweep_list = sweep_list,
  thrY = 7
)
head(result_quick$summary)

# Run with negated outcome (~Y)
result_neg <- ctSweepM(
  dat = sample_data,
  outcome = "~Y",
  conditions = c("X1", "X2"),
  sweep_list = sweep_list,
  thrY = 7
)
head(result_neg$summary)

\donttest{
# Full multi-condition analysis (27 combinations)
sweep_list_full <- list(
  X1 = 6:8,
  X2 = 6:8,
  X3 = 6:8
)

result_full <- ctSweepM(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_list = sweep_list_full,
  thrY = 7
)
head(result_full$summary)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{generate_config_chart}
\alias{generate_config_chart}
\title{Generate Configuration Chart from QCA Solution}
\usage{
generate_config_chart(
  sol,
  symbol_set = c("unicode", "ascii", "latex"),
  include_metrics = TRUE,
  language = c("en", "ja"),
  condition_order = NULL
)
}
\arguments{
\item{sol}{A solution object returned by \code{QCA::minimize()}, or
a list containing solution information.}

\item{symbol_set}{Character. One of \code{"unicode"}, \code{"ascii"}, 
or \code{"latex"}. Default is \code{"unicode"}.}

\item{include_metrics}{Logical. Whether to include consistency/coverage
metrics in the table. Default is TRUE.}

\item{language}{Character. \code{"en"} for English, \code{"ja"} for Japanese.
Default is \code{"en"}.}

\item{condition_order}{Character vector. Optional ordering of conditions
in the table rows. If NULL, conditions are ordered as they appear in paths.}
}
\value{
Character string containing Markdown-formatted table(s).
}
\description{
Creates a Markdown-formatted configuration chart (Fiss-style table)
from QCA minimization results. Supports single solution with multiple
paths, and multiple solutions (displayed as separate tables).
}
\examples{
\dontrun{
# After running QCA::minimize()
library(QCA)
tt <- truthTable(data, outcome = "Y", conditions = c("A", "B", "C"))
sol <- minimize(tt, include = "?", details = TRUE)

# Generate configuration chart
chart <- generate_config_chart(sol)
cat(chart)

# For LaTeX/PDF output (e.g., rticles)
chart <- generate_config_chart(sol, symbol_set = "latex")

# ASCII for maximum compatibility
chart <- generate_config_chart(sol, symbol_set = "ascii")

# Japanese labels
chart <- generate_config_chart(sol, language = "ja")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{extract_sol_terms_by_model}
\alias{extract_sol_terms_by_model}
\title{Extract solution terms separately for each minimal solution}
\usage{
extract_sol_terms_by_model(sol)
}
\arguments{
\item{sol}{QCA solution object.}
}
\value{
List of character vectors, one per minimal solution. Empty list when
  no solution is available.
}
\description{
Same sources as \code{extract_sol_terms()}, but the terms of each minimal
solution (M1, M2, ...) are kept apart instead of being pooled. This is what
the core/peripheral classification needs: pooling first makes a condition
look as if it occurred with both polarities in "the" parsimonious solution,
when in fact one minimal solution had it present and another had it absent.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_methods.R
\name{print.tsqca_result}
\alias{print.tsqca_result}
\alias{print.otSweep_result}
\alias{print.dtSweep_result}
\alias{print.ctSweepS_result}
\alias{print.ctSweepM_result}
\title{Print method for ThSQCA results}
\usage{
\method{print}{tsqca_result}(x, ...)

\method{print}{otSweep_result}(x, ...)

\method{print}{dtSweep_result}(x, ...)

\method{print}{ctSweepS_result}(x, ...)

\method{print}{ctSweepM_result}(x, ...)
}
\arguments{
\item{x}{A ThSQCA result object returned by one of the sweep functions.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a concise overview of ThSQCA analysis results.
}
\examples{
\donttest{
data(sample_data)
result <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
print(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{config_chart_from_paths}
\alias{config_chart_from_paths}
\title{Generate configuration chart from paths (simple interface)}
\usage{
config_chart_from_paths(
  paths,
  symbol_set = c("unicode", "ascii", "latex"),
  language = c("en", "ja"),
  condition_order = NULL,
  n_sol = 1L,
  solution_note = TRUE,
  solution_note_style = c("simple", "detailed"),
  epi_list = NULL
)
}
\arguments{
\item{paths}{Character vector. Paths in QCA notation (e.g., "A*B*~C").}

\item{symbol_set}{Character. One of \code{"unicode"}, \code{"ascii"}, 
or \code{"latex"}.}

\item{language}{Character. \code{"en"} for English, \code{"ja"} for Japanese.}

\item{condition_order}{Character vector. Optional ordering of conditions.}

\item{n_sol}{Integer. Number of equivalent solutions. If > 1, a note is added
explaining that multiple solutions exist and M1 is shown. Default is 1.}

\item{solution_note}{Logical. Whether to add solution note when n_sol > 1.
Default is TRUE.}

\item{solution_note_style}{Character. \code{"simple"} or \code{"detailed"}.
Default is \code{"simple"}.}

\item{epi_list}{Character vector. Essential prime implicants for detailed notes.
Only used when \code{solution_note_style = "detailed"}.}
}
\value{
Character string containing Markdown-formatted table.
}
\description{
A simpler interface for generating configuration charts when you have
paths directly (without a full QCA solution object).
}
\examples{
# Simple usage with paths
paths <- c("A*B", "A*C*~D", "B*E")
chart <- config_chart_from_paths(paths)
cat(chart)

# With ASCII symbols
chart <- config_chart_from_paths(paths, symbol_set = "ascii")
cat(chart)

# With multiple solution note
chart <- config_chart_from_paths(paths, n_sol = 2)
cat(chart)

# With detailed note including EPIs
chart <- config_chart_from_paths(
  paths, n_sol = 2,
  solution_note_style = "detailed",
  epi_list = c("A*B")
)
cat(chart)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{extract_solution_list}
\alias{extract_solution_list}
\title{Extract solution list from QCA object}
\usage{
extract_solution_list(sol)
}
\description{
Extract solution list from QCA object
}
\note{
When dir.exp is specified, the true Intermediate solution is stored in
  sol$i.sol, not sol$solution (which contains the Parsimonious solution).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{parse_path_conditions}
\alias{parse_path_conditions}
\title{Parse a single path/term into conditions}
\usage{
parse_path_conditions(path)
}
\arguments{
\item{path}{Character. A single path like "A*B*~C"}
}
\value{
List with 'present' and 'absent' condition names
}
\description{
Parse a single path/term into conditions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_core.R
\name{prepare_dat_bin}
\alias{prepare_dat_bin}
\title{Prepare analysis data frame for QCA::truthTable()}
\usage{
prepare_dat_bin(
  dat,
  outcome_clean,
  conditions,
  thrY,
  thrX_vec,
  pre_calibrated = NULL
)
}
\arguments{
\item{dat}{Original data frame.}

\item{outcome_clean}{Character. Outcome variable name (without \code{~}).}

\item{conditions}{Character vector. Condition variable names.}

\item{thrY}{Numeric. Threshold for outcome binarization.}

\item{thrX_vec}{Named numeric vector. Thresholds for conditions.}

\item{pre_calibrated}{Character vector or NULL. Names of pre-calibrated
variables to pass through without binarization.}
}
\value{
Data frame with column \code{Y} (binarized outcome) and condition
  columns (binarized or passed through).
}
\description{
Constructs the data frame to be passed to \code{QCA::truthTable()}.
For pre-calibrated variables, the original values are passed through
without binarization. For all other variables, \code{qca_bin()} is applied.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_ots_dts.R
\name{otSweep}
\alias{otSweep}
\title{OTS–QCA: Outcome threshold sweep}
\usage{
otSweep(
  dat,
  outcome = NULL,
  conditions = NULL,
  sweep_range,
  thrX,
  pre_calibrated = NULL,
  dir.exp = NULL,
  include = "",
  incl.cut = 0.8,
  n.cut = 1,
  pri.cut = 0,
  extract_mode = c("first", "all", "essential"),
  return_details = TRUE,
  Yvar = NULL,
  Xvars = NULL
)
}
\arguments{
\item{dat}{Data frame containing the outcome and condition variables.}

\item{outcome}{Character. Outcome variable name. Supports negation with
tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.}

\item{conditions}{Character vector. Names of condition variables.}

\item{sweep_range}{Numeric vector. Candidate thresholds for Y.}

\item{thrX}{Named numeric vector. Fixed thresholds for X variables.
Names must match the conditions that require binarization.
Variables listed in \code{pre_calibrated} do not need a \code{thrX} entry.}

\item{pre_calibrated}{Character vector or \code{NULL}. Names of condition
variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
and should be passed through to \code{QCA::truthTable()} without
binarization. These variables must contain values in the \code{[0, 1]}
range. Variables not listed here will be binarized using \code{thrX}
thresholds as usual. Default is \code{NULL} (all variables binarized).
It is recommended to sweep variables on their original (raw) scale rather
than as pre-calibrated fuzzy values, because raw-scale thresholds are
easier to interpret substantively.}

\item{dir.exp}{Directional expectations for \code{minimize}.
If \code{NULL} (default), no directional expectations are applied.
To compute the \strong{intermediate solution}, specify a numeric vector
(1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
for three conditions all expected to contribute positively.}

\item{include}{Inclusion rule for \code{minimize}. 
\code{""} (default, QCA compatible) computes the \strong{complex solution}
without logical remainders.
Use \code{"?"} to include logical remainders for \strong{parsimonious}
(with \code{dir.exp = NULL}) or \strong{intermediate} solutions
(with \code{dir.exp} specified).}

\item{incl.cut}{Consistency cutoff for \code{truthTable}.}

\item{n.cut}{Frequency cutoff for \code{truthTable}.}

\item{pri.cut}{PRI cutoff for \code{minimize}.}

\item{extract_mode}{Character. How to handle multiple solutions:
\code{"first"} (default), \code{"all"}, or \code{"essential"}.
See \code{\link{qca_extract}} for details.}

\item{return_details}{Logical. If \code{TRUE} (default), returns both
summary and detailed objects for use with \code{generate_report()}.}

\item{Yvar}{Deprecated. Use \code{outcome} instead.}

\item{Xvars}{Deprecated. Use \code{conditions} instead.}
}
\value{
If \code{return_details = FALSE}, a data frame with columns:
\itemize{
  \item \code{thrY} — threshold for Y
  \item \code{expression} — minimized solution expression
  \item \code{inclS} — solution consistency
  \item \code{covS} — solution coverage
  \item (additional columns depending on \code{extract_mode})
}

If \code{return_details = TRUE}, a list with:
\itemize{
  \item \code{summary} — the data frame above
  \item \code{details} — per-Y-threshold list of
    \code{thrY}, \code{thrX_vec}, \code{truth_table}, \code{solution}
}

Note that \code{return_details} changes the \emph{type} of the returned
object, not just its contents: with \code{TRUE} the summary table is at
\code{result$summary}, whereas with \code{FALSE} the summary table
\emph{is} the returned object and \code{result$summary} is \code{NULL}.
Code intended to work under both settings should branch on
\code{inherits(result, "data.frame")} (or simply always pass
\code{return_details = TRUE}) rather than assuming \code{result$summary}
exists.
}
\description{
Sweeps the threshold of the outcome Y while keeping the thresholds of
all X conditions fixed.
}
\examples{
# Load sample data
data(sample_data)

# Set fixed thresholds for conditions
thrX <- c(X1 = 7, X2 = 7, X3 = 7)

# === Three Types of QCA Solutions ===

# 1. Complex Solution (default, QCA compatible)
#    Does not use logical remainders (most conservative)
result_comp <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX
  # include = "" (default), dir.exp = NULL (default)
)
head(result_comp$summary)

# 2. Parsimonious Solution (include = "?")
#    Uses logical remainders without directional expectations
result_pars <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?"  # Include logical remainders
)
head(result_pars$summary)

# 3. Intermediate Solution (include = "?" + dir.exp)
#    Uses logical remainders with directional expectations
result_int <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 7,
  thrX = thrX,
  include = "?",
  dir.exp = c(1, 1, 1)  # All conditions expected positive
)
head(result_int$summary)

# === Threshold Sweep Example ===

# Sweep with complex solutions (default)
result_sweep <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = thrX
)
head(result_sweep$summary)

# Run with negated outcome (~Y)
# Analyzes conditions for Y < threshold
result_neg <- otSweep(
  dat = sample_data,
  outcome = "~Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = thrX
)
head(result_neg$summary)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{build_config_matrix}
\alias{build_config_matrix}
\title{Build condition-path matrix for configuration chart}
\usage{
build_config_matrix(paths, conditions = NULL, symbols)
}
\arguments{
\item{paths}{Character vector of paths}

\item{conditions}{Character vector of condition names (optional)}

\item{symbols}{List with 'present' and 'absent' symbols}
}
\value{
Matrix with conditions as rows, paths as columns
}
\description{
Build condition-path matrix for configuration chart
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{generate_fiss_chart}
\alias{generate_fiss_chart}
\title{Generate Fiss-Style Configuration Chart from Sweep Results}
\usage{
generate_fiss_chart(
  result,
  conditions = NULL,
  symbol_set = c("unicode", "ascii", "latex"),
  language = c("en", "ja")
)
}
\arguments{
\item{result}{Sweep result augmented by \code{\link{compute_fiss_core}}.}

\item{conditions}{Character vector. Condition names (row order).
If \code{NULL}, extracted from stored settings.}

\item{symbol_set}{Character. One of \code{"unicode"} (default),
\code{"ascii"}, or \code{"latex"}.}

\item{language}{Character. \code{"en"} (default) or \code{"ja"}.}
}
\value{
Character string: Markdown-formatted Fiss configuration chart.
}
\description{
Produces a Markdown-formatted configuration chart following Fiss (2011),
using four symbols to distinguish core conditions (present in both
parsimonious and intermediate solutions) from peripheral conditions
(present in intermediate solution only).
}
\details{
Call \code{\link{compute_fiss_core}} first to augment the sweep result.
}
\examples{
\dontrun{
data(sample_data)
res <- otSweep(
  dat = sample_data, outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7),
  include = "?", dir.exp = c(1, 1, 1),
  return_details = TRUE
)
res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))
cat(generate_fiss_chart(res_fiss))
cat(generate_fiss_chart(res_fiss, symbol_set = "latex"))
cat(generate_fiss_chart(res_fiss, language = "ja"))
}

}
\references{
Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
to typologies in organization research. \emph{Academy of Management Journal},
54(2), 393-420.
}
\seealso{
\code{\link{compute_fiss_core}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_methods.R
\name{summary.tsqca_result}
\alias{summary.tsqca_result}
\alias{summary.otSweep_result}
\alias{summary.dtSweep_result}
\alias{summary.ctSweepS_result}
\alias{summary.ctSweepM_result}
\title{Summary method for ThSQCA results}
\usage{
\method{summary}{tsqca_result}(object, ...)

\method{summary}{otSweep_result}(object, ...)

\method{summary}{dtSweep_result}(object, ...)

\method{summary}{ctSweepS_result}(object, ...)

\method{summary}{ctSweepM_result}(object, ...)
}
\arguments{
\item{object}{A ThSQCA result object returned by one of the sweep functions.}

\item{...}{Additional arguments (ignored).}
}
\value{
Invisibly returns \code{object}.
}
\description{
Displays detailed results table with solution formulas and fit measures.
}
\examples{
\donttest{
data(sample_data)
result <- otSweep(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX = c(X1 = 7, X2 = 7, X3 = 7)
)
summary(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{config_matrix_to_md}
\alias{config_matrix_to_md}
\title{Convert configuration matrix to Markdown table}
\usage{
config_matrix_to_md(mat, row_header = "Condition", center_align = TRUE)
}
\arguments{
\item{mat}{Matrix with rownames and colnames}

\item{row_header}{Character. Header for the row names column}

\item{center_align}{Logical. Whether to center-align columns}
}
\value{
Character string of Markdown table
}
\description{
Convert configuration matrix to Markdown table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{get_all_terms}
\alias{get_all_terms}
\title{Get all unique terms from multiple solutions}
\usage{
get_all_terms(sol_list)
}
\arguments{
\item{sol_list}{List of solution character vectors from minimize().}
}
\value{
List with:
  \itemize{
    \item \code{all_terms} — all terms (with duplicates)
    \item \code{unique_terms} — unique terms
    \item \code{term_counts} — table of term frequencies
  }
}
\description{
Get all unique terms from multiple solutions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_core.R
\name{validate_pre_calibrated}
\alias{validate_pre_calibrated}
\title{Validate the pre_calibrated parameter}
\usage{
validate_pre_calibrated(pre_calibrated, conditions, dat)
}
\arguments{
\item{pre_calibrated}{Character vector or NULL.}

\item{conditions}{Character vector. Valid condition variable names.}

\item{dat}{Data frame containing the variables.}
}
\value{
Invisible NULL. Raises errors or warnings as needed.
}
\description{
Checks that all names in \code{pre_calibrated} exist in \code{conditions}
and that the corresponding values in \code{dat} are within the \code{[0, 1]}
range required for fuzzy membership scores.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_core.R
\name{qca_extract}
\alias{qca_extract}
\title{Extract solution information from a QCA minimization result}
\usage{
qca_extract(sol, extract_mode = c("first", "all", "essential"))
}
\arguments{
\item{sol}{A solution object returned by \code{QCA::minimize()}.}

\item{extract_mode}{Character. How to handle multiple intermediate solutions:
\itemize{
  \item \code{"first"} - return only the first solution (M1). Default.
  \item \code{"all"} - return all solutions concatenated.
  \item \code{"essential"} - return essential prime implicants (terms 
    common to all solutions), plus selective prime implicants and 
    solution count.
}}
}
\value{
A list with elements depending on \code{extract_mode}.

  For \code{"first"}: \code{expression}, \code{inclS}, \code{covS}.

  For \code{"all"}: adds \code{n_solutions}.

  For \code{"essential"}: adds \code{selective_terms}, \code{unique_terms},
  \code{n_solutions}.

  If extraction fails, returns \code{"No solution"} and \code{NA_real_}
  for numeric values.
}
\description{
Internal helper to obtain the solution expression, consistency
(\code{inclS}) and coverage (\code{covS}) from an object returned by
\code{QCA::minimize()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{build_fiss_matrix}
\alias{build_fiss_matrix}
\title{Build a Fiss-style configuration matrix (4-symbol)}
\usage{
build_fiss_matrix(interm_terms, classification, conditions, symbols, thr_label)
}
\arguments{
\item{interm_terms}{Character vector of intermediate-solution terms}

\item{classification}{Data frame from \code{compute_fiss_core()} for one
threshold (the \code{$classification} element)}

\item{conditions}{Character vector of condition names (row order)}

\item{symbols}{Fiss symbol set (one element of \code{SYMBOL_SETS_FISS})}

\item{thr_label}{Character. Label prefix for column headers (e.g. "thrY=7")}
}
\value{
Character matrix with conditions as rows, terms as columns
}
\description{
Build a Fiss-style configuration matrix (4-symbol)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{extract_conditions_from_paths}
\alias{extract_conditions_from_paths}
\title{Extract all unique conditions from paths}
\usage{
extract_conditions_from_paths(paths)
}
\arguments{
\item{paths}{Character vector of paths}
}
\value{
Character vector of unique condition names (without ~)
}
\description{
Extract all unique conditions from paths
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{format_qca_solution}
\alias{format_qca_solution}
\title{Format a QCA solution expression}
\usage{
format_qca_solution(solution, var_names, use_tilde = TRUE)
}
\arguments{
\item{solution}{Character. A solution expression (e.g., "KSPRVT + ~KPRPRD").}

\item{var_names}{Character vector. Variable names used in the analysis.}

\item{use_tilde}{Logical. If TRUE, negation is represented as \code{~VAR}.}
}
\value{
Character. The formatted solution expression.
}
\description{
Formats a complete solution expression (multiple terms joined by +).
}
\examples{
var_names <- c("KSP", "KPR", "PRD", "RVT", "RCM")
format_qca_solution("KSPRVT + ~KPRPRD + RCM", var_names)
# Returns: "KSP*RVT + ~KPR*PRD + RCM"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_core.R
\name{get_n_solutions}
\alias{get_n_solutions}
\title{Get the number of intermediate solutions}
\usage{
get_n_solutions(sol)
}
\arguments{
\item{sol}{A solution object returned by \code{QCA::minimize()}.}
}
\value{
Integer. Number of intermediate solutions, or 0 if none.
}
\description{
Get the number of intermediate solutions
}
\note{
When dir.exp is specified, the true Intermediate solution is stored in
  sol$i.sol, not sol$solution (which contains the Parsimonious solution).
  See \code{\link{collect_unique_i_sol}} for why deduplication is required
  when multiple prime implicant charts are present.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{identify_epi}
\alias{identify_epi}
\title{Identify Essential Prime Implicants from multiple solutions}
\usage{
identify_epi(solutions)
}
\arguments{
\item{solutions}{List of solution vectors. Each element is a character
vector of terms for one solution.}
}
\value{
List with:
  \itemize{
    \item \code{epi} — Essential prime implicants (in all solutions)
    \item \code{spi} — Selective prime implicants (in some solutions)
    \item \code{n_solutions} — Number of solutions
  }
}
\description{
Finds terms that appear in ALL solutions (EPIs) versus terms that
appear in only some solutions (SPIs).
}
\examples{
solutions <- list(
  c("A*B", "C", "D"),
  c("A*B", "C", "E"),
  c("A*B", "C", "F")
)
result <- identify_epi(solutions)
# result$epi = c("A*B", "C")
# result$spi = c("D", "E", "F")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{extract_cond_status_map}
\alias{extract_cond_status_map}
\title{Extract all (condition, status) pairs present in a set of solution terms}
\usage{
extract_cond_status_map(terms, conditions)
}
\arguments{
\item{terms}{Character vector of solution terms (e.g. c("X1*X2", "~X3"))}

\item{conditions}{Character vector of all condition names}
}
\value{
Named list: condition name -> character vector of statuses
  ("present" and/or "absent") found in any term
}
\description{
Extract all (condition, status) pairs present in a set of solution terms
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{escape_md}
\alias{escape_md}
\title{Escape special characters for Markdown}
\usage{
escape_md(text)
}
\arguments{
\item{text}{Character. Text to escape.}
}
\value{
Character. Text with special characters escaped.
}
\description{
Escapes asterisks and other special characters that have special
meaning in Markdown syntax.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{extract_path_metrics_for_chart}
\alias{extract_path_metrics_for_chart}
\title{Extract per-path metrics for configuration chart}
\usage{
extract_path_metrics_for_chart(sol, solution_index = 1)
}
\description{
Extract per-path metrics for configuration chart
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{print_fiss_summary}
\alias{print_fiss_summary}
\title{Print Fiss core/peripheral summary for a single threshold}
\usage{
print_fiss_summary(result, thr_key = NULL, language = c("en", "ja"))
}
\arguments{
\item{result}{Sweep result augmented by \code{\link{compute_fiss_core}}.}

\item{thr_key}{Character or numeric. Threshold key (e.g., "7" or 7).}

\item{language}{Character. \code{"en"} or \code{"ja"}.}
}
\value{
Invisibly returns the classification data frame for \code{thr_key}.
}
\description{
Displays which conditions are core and which are peripheral
at a given threshold, in a human-readable format.
}
\examples{
\dontrun{
res_fiss <- compute_fiss_core(res)
print_fiss_summary(res_fiss, thr_key = "7")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{compute_fiss_core}
\alias{compute_fiss_core}
\title{Compute Fiss Core/Peripheral Classification for Sweep Results}
\usage{
compute_fiss_core(result, conditions = NULL)
}
\arguments{
\item{result}{A sweep result object with \code{$details} and
\code{$settings} slots (e.g., from \code{otSweep(..., return_details = TRUE)}).}

\item{conditions}{Character vector. Condition names (used for consistent
row ordering in charts). If \code{NULL}, extracted automatically.}
}
\value{
The original \code{result} object with an additional
  \code{$fiss_core} slot: a named list keyed by threshold (character),
  each entry containing:
  \itemize{
    \item \code{parsim_expression} — parsimonious solution expression
    \item \code{interm_expression} — intermediate solution expression
    \item \code{classification}    — data frame with columns
      \code{term_idx}, \code{term_expr}, \code{condition},
      \code{status}, \code{type}
  }
}
\description{
Takes an existing threshold-sweep result object (produced by
\code{\link{otSweep}}, \code{\link{ctSweepS}}, \code{\link{ctSweepM}},
or \code{\link{dtSweep}}) and augments it with Fiss (2011)
core/peripheral classification.
}
\details{
The classification requires that:
\itemize{
  \item The sweep was run with \code{include = "?"} (to allow parsimonious
        computation)
  \item \code{return_details = TRUE} was used (truth tables must be stored)
  \item \code{dir.exp} was specified (i.e., the sweep produced intermediate
        solutions — core/peripheral is only meaningful when comparing
        parsimonious vs intermediate)
}

For each threshold in the result, this function:
\enumerate{
  \item Retrieves the intermediate solution already stored in
        \code{result$details}.
  \item Re-runs \code{QCA::minimize()} on the same truth table with
        \code{dir.exp = NULL} to obtain the parsimonious solution.
  \item Compares the two solutions: conditions appearing in both are
        \strong{core}; conditions appearing only in the intermediate
        solution are \strong{peripheral}.
}
}
\examples{
\dontrun{
library(ThSQCA)
data(sample_data)

# Step 1: Run intermediate sweep (dir.exp required)
res <- otSweep(
  dat        = sample_data,
  outcome    = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_range = 6:8,
  thrX       = c(X1 = 7, X2 = 7, X3 = 7),
  include    = "?",
  dir.exp    = c(1, 1, 1),
  return_details = TRUE
)

# Step 2: Augment with Fiss core/peripheral classification
res_fiss <- compute_fiss_core(res, conditions = c("X1", "X2", "X3"))

# Step 3: Generate Fiss-style chart
cat(generate_fiss_chart(res_fiss, symbol_set = "unicode"))
}

}
\references{
Fiss, P. C. (2011). Building better causal theories: A fuzzy set approach
to typologies in organization research. \emph{Academy of Management Journal},
54(2), 393-420.
}
\seealso{
\code{\link{generate_fiss_chart}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{parse_solution_terms}
\alias{parse_solution_terms}
\title{Parse solution expression into individual terms}
\usage{
parse_solution_terms(expr)
}
\arguments{
\item{expr}{Character. Solution expression (e.g., "X3 + X1*X2").}
}
\value{
Character vector of terms (e.g., c("X3", "X1*X2")), or NULL if
  no valid expression.
}
\description{
Splits a solution expression (ORed terms) into individual prime implicants.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{SYMBOL_SETS_FISS}
\alias{SYMBOL_SETS_FISS}
\title{Fiss-style symbol sets (4 symbols)}
\usage{
SYMBOL_SETS_FISS
}
\description{
Fiss-style symbol sets (4 symbols)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{generate_term_level_chart}
\alias{generate_term_level_chart}
\title{Generate solution-term level chart (Fiss-style)}
\usage{
generate_term_level_chart(sum_df, conditions, symbols, language = "en")
}
\arguments{
\item{sum_df}{Data frame. Summary data frame from sweep results with
expression column and threshold column(s).}

\item{conditions}{Character vector. Condition names for row ordering.}

\item{symbols}{List. Symbol set (present, absent) for the chart.}

\item{language}{Character. Language for labels ("en" or "ja").}
}
\value{
Character string containing Markdown-formatted table.
}
\description{
Creates a configuration chart where each column represents a single
prime implicant (configuration), following Fiss (2011) notation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_helpers.R
\name{extract_all_metrics}
\alias{extract_all_metrics}
\title{Extract all metrics from QCA solution object}
\usage{
extract_all_metrics(IC, sol_obj = NULL)
}
\arguments{
\item{IC}{IC object from QCA solution.}

\item{sol_obj}{Full solution object (optional, for fallback).}
}
\value{
List with elements:
  \itemize{
    \item \code{sol_inclS} — solution consistency (or NA)
    \item \code{sol_PRI} — solution PRI (or NA)
    \item \code{sol_covS} — solution coverage (or NA)
    \item \code{term_df} — data frame of per-term metrics (or NULL)
  }
}
\description{
Safely extracts solution-level and term-level metrics from
various QCA solution object structures. Handles cases where
multiple solutions exist and sol.incl.cov may be NULL.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{extract_solution_metrics_for_chart}
\alias{extract_solution_metrics_for_chart}
\title{Extract solution-level metrics for configuration chart}
\usage{
extract_solution_metrics_for_chart(sol, solution_index = 1)
}
\description{
Extract solution-level metrics for configuration chart
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_core.R
\name{qca_bin}
\alias{qca_bin}
\title{Binary calibration helper for ThSQCA}
\usage{
qca_bin(x, thr)
}
\arguments{
\item{x}{Numeric vector.}

\item{thr}{Numeric scalar. Cases with \code{x >= thr} are coded as 1,
others as 0.}
}
\value{
Integer vector of 0/1 with the same length as \code{x}.
}
\description{
Converts a numeric vector into a crisp set (0/1) based on a threshold.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_core.R
\name{collect_unique_i_sol}
\alias{collect_unique_i_sol}
\title{Collect and deduplicate the solution models attached to a QCA::minimize()
result}
\usage{
collect_unique_i_sol(sol)
}
\arguments{
\item{sol}{A solution object returned by \code{QCA::minimize()}.}
}
\value{
A list of unique solution term-vectors, or \code{NULL} if none
  were found.
}
\description{
When \code{dir.exp} is specified and the minimization produces more than
one prime implicant chart (QCA's own \code{C1}, \code{C2}, ... indexing,
visible as \code{"From C1P1, C2P1:"} in \code{print()}), \code{sol$i.sol}
contains one list entry per chart/path combination (e.g. \code{C1P1},
\code{C2P1}). Each entry's own \code{$solution} field is computed
independently by \code{QCA::minimize()} (via its internal
\code{getSolution()} call) and is NOT guaranteed to hold only that chart's
own model: depending on the data, it may already enumerate the full,
cross-chart set of tied minimal models. Concatenating every chart's
\code{$solution} list (the previous approach used here and in
\code{generate_report()}) therefore risks double- or N-fold counting the
same model, inflating the reported number of solutions.
}
\details{
This helper performs the same enumeration, then deduplicates by comparing
each model's term set (order-independent), so structurally identical
models are counted once regardless of which chart(s) produced them. This
mirrors the manual "remove duplicate Boolean expressions" step that is
standard practice when reporting multiple minimal solutions by hand.
}
\note{
When dir.exp is specified, the true Intermediate solution is stored in
  sol$i.sol, not sol$solution (which contains the Parsimonious solution).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{run_parsimonious}
\alias{run_parsimonious}
\title{Run parsimonious QCA minimization on a stored truth table}
\usage{
run_parsimonious(truth_table, conditions)
}
\arguments{
\item{truth_table}{Truth table object (from QCA::truthTable())}

\item{conditions}{Character vector of condition names}
}
\value{
QCA solution object, or NULL on error / no solution
}
\description{
Run parsimonious QCA minimization on a stored truth table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{extract_paths_from_solution}
\alias{extract_paths_from_solution}
\title{Extract paths from a single solution}
\usage{
extract_paths_from_solution(solution)
}
\description{
Extract paths from a single solution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{SYMBOL_SETS}
\alias{SYMBOL_SETS}
\title{Symbol sets for configuration charts}
\usage{
SYMBOL_SETS
}
\description{
Symbol sets for configuration charts
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_fiss_core.R
\name{build_parsim_status_map}
\alias{build_parsim_status_map}
\title{Build the parsimonious condition-status map used for core classification}
\usage{
build_parsim_status_map(models_terms, conditions)
}
\arguments{
\item{models_terms}{List of character vectors, one per minimal solution
(from \code{extract_sol_terms_by_model()}).}

\item{conditions}{Character vector of all condition names.}
}
\value{
Named list mapping each condition to the statuses it holds in every
  minimal solution (possibly \code{character(0)}).
}
\description{
A condition counts as core only for a status it holds in EVERY minimal
parsimonious solution. With a single minimal solution this is identical to
reading that solution directly, so ordinary results are unaffected.
}
\details{
With several tied minimal solutions, pooling their terms first (the previous
behavior) meant that a condition present in M1 and absent in M2 ended up with
both statuses, so an intermediate term matched whichever polarity it used and
the condition was classified core either way. Requiring agreement across all
minimal solutions removes that bias: what cannot be asserted regardless of
which minimal solution is selected is not treated as core.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_report.R
\name{write_full_report}
\alias{write_full_report}
\title{Write full report content}
\usage{
write_full_report(
  result,
  con,
  dat = NULL,
  desc_vars = NULL,
  include_chart = TRUE,
  chart_symbol_set = "unicode",
  chart_level = "term",
  solution_note = TRUE,
  solution_note_style = "simple",
  solution_note_lang = "en",
  include_raw_output = TRUE,
  use_fiss = FALSE
)
}
\description{
Write full report content
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_config_chart.R
\name{generate_threshold_level_chart}
\alias{generate_threshold_level_chart}
\title{Generate threshold-level summary chart}
\usage{
generate_threshold_level_chart(sum_df, conditions, symbols, language = "en")
}
\arguments{
\item{sum_df}{Data frame. Summary data frame from sweep results.}

\item{conditions}{Character vector. Condition names for row ordering.}

\item{symbols}{List. Symbol set (present, absent) for the chart.}

\item{language}{Character. Language for labels ("en" or "ja").}
}
\value{
Character string containing Markdown-formatted table.
}
\description{
Creates a configuration chart where each column represents one threshold,
showing all conditions that appear in any configuration at that threshold.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsqca_cts.R
\name{ctSweepS}
\alias{ctSweepS}
\title{CTS–QCA: Single-condition threshold sweep}
\usage{
ctSweepS(
  dat,
  outcome = NULL,
  conditions = NULL,
  sweep_var,
  sweep_range,
  thrY,
  thrX_default = 7,
  pre_calibrated = NULL,
  dir.exp = NULL,
  include = "",
  incl.cut = 0.8,
  n.cut = 1,
  pri.cut = 0,
  extract_mode = c("first", "all", "essential"),
  return_details = TRUE,
  Yvar = NULL,
  Xvars = NULL
)
}
\arguments{
\item{dat}{Data frame containing the outcome and condition variables.}

\item{outcome}{Character. Outcome variable name. Supports negation with
tilde prefix (e.g., \code{"~Y"}) following QCA package conventions.}

\item{conditions}{Character vector. Names of condition variables.}

\item{sweep_var}{Character. Name of the condition to be swept.
Must be one of \code{conditions}.}

\item{sweep_range}{Numeric vector. Candidate thresholds for \code{sweep_var}.}

\item{thrY}{Numeric. Threshold for Y (fixed).}

\item{thrX_default}{Numeric. Default threshold for non-swept X variables.
Variables listed in \code{pre_calibrated} do not require this threshold.}

\item{pre_calibrated}{Character vector or \code{NULL}. Names of condition
variables that have been pre-calibrated (e.g., via \code{QCA::calibrate()})
and should be passed through to \code{QCA::truthTable()} without
binarization. These variables must contain values in the \code{[0, 1]}
range. Variables not listed here will be binarized using \code{thrX_default}
as usual. Default is \code{NULL} (all variables binarized).
It is recommended to sweep variables on their original (raw) scale rather
than as pre-calibrated fuzzy values, because raw-scale thresholds are
easier to interpret substantively.}

\item{dir.exp}{Directional expectations for \code{minimize}.
If \code{NULL} (default), no directional expectations are applied.
To compute the \strong{intermediate solution}, specify a numeric vector
(1, 0, or -1 for each condition). Example: \code{dir.exp = c(1, 1, 1)}
for three conditions all expected to contribute positively.}

\item{include}{Inclusion rule for \code{minimize}. 
\code{""} (default, QCA compatible) computes the \strong{complex solution}
without logical remainders.
Use \code{"?"} to include logical remainders for \strong{parsimonious}
(with \code{dir.exp = NULL}) or \strong{intermediate} solutions
(with \code{dir.exp} specified).}

\item{incl.cut}{Consistency cutoff for \code{\link[QCA]{truthTable}}.}

\item{n.cut}{Frequency cutoff for \code{truthTable}.}

\item{pri.cut}{PRI cutoff for \code{minimize}.}

\item{extract_mode}{Character. How to handle multiple solutions:
\code{"first"} (default), \code{"all"}, or \code{"essential"}.
See \code{\link{qca_extract}} for details.}

\item{return_details}{Logical. If \code{TRUE} (default), returns both
summary and detailed objects for use with \code{generate_report()}.}

\item{Yvar}{Deprecated. Use \code{outcome} instead.}

\item{Xvars}{Deprecated. Use \code{conditions} instead.}
}
\value{
If \code{return_details = FALSE}, a data frame with columns:
\itemize{
  \item \code{threshold} — swept threshold for \code{sweep_var}
  \item \code{expression} — minimized solution expression
  \item \code{inclS} — solution consistency
  \item \code{covS} — solution coverage
  \item (additional columns depending on \code{extract_mode})
}

If \code{return_details = TRUE}, a list with:
\itemize{
  \item \code{summary} — the data frame above
  \item \code{details} — per-threshold list of
    \code{threshold}, \code{thrX_vec}, \code{truth_table}, \code{solution}
}

Note that \code{return_details} changes the \emph{type} of the returned
object, not just its contents: with \code{TRUE} the summary table is at
\code{result$summary}, whereas with \code{FALSE} the summary table
\emph{is} the returned object and \code{result$summary} is \code{NULL}.
Code intended to work under both settings should branch on
\code{inherits(result, "data.frame")} (or simply always pass
\code{return_details = TRUE}) rather than assuming \code{result$summary}
exists.
}
\description{
Performs a threshold sweep for one focal condition X. For each threshold
in \code{sweep_range}, the outcome Y and all X variables are binarized
using user-specified thresholds, and a crisp-set QCA is executed.
}
\examples{
# Load sample data
data(sample_data)

# === Three Types of QCA Solutions ===

# 1. Complex Solution (default, QCA compatible)
result_comp <- ctSweepS(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_var = "X3",
  sweep_range = 7,
  thrY = 7,
  thrX_default = 7
  # include = "" (default), dir.exp = NULL (default)
)
head(result_comp$summary)

# 2. Parsimonious Solution (include = "?")
result_pars <- ctSweepS(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_var = "X3",
  sweep_range = 7,
  thrY = 7,
  thrX_default = 7,
  include = "?"  # Include logical remainders
)
head(result_pars$summary)

# 3. Intermediate Solution (include = "?" + dir.exp)
result_int <- ctSweepS(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_var = "X3",
  sweep_range = 7,
  thrY = 7,
  thrX_default = 7,
  include = "?",
  dir.exp = c(1, 1, 1)  # All conditions expected positive
)
head(result_int$summary)

# === Threshold Sweep Example ===

# Run single condition threshold sweep on X3 (complex solutions by default)
result <- ctSweepS(
  dat = sample_data,
  outcome = "Y",
  conditions = c("X1", "X2", "X3"),
  sweep_var = "X3",
  sweep_range = 6:8,
  thrY = 7,
  thrX_default = 7
)
head(result$summary)

# Run with negated outcome (~Y)
result_neg <- ctSweepS(
  dat = sample_data,
  outcome = "~Y",
  conditions = c("X1", "X2", "X3"),
  sweep_var = "X3",
  sweep_range = 6:8,
  thrY = 7,
  thrX_default = 7
)
head(result_neg$summary)
}
